package org.ffd2.skeletonx.application;


import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;

import mccaffrey.base.AboutDetails;
import mccaffrey.base.InterfaceUser;
import mccaffrey.library.MainInterfaceGUI;

import org.ffd2.skeletonx.compile.impl.FileRoot;
import org.ffd2.skeletonx.compile.library.ParserControl;
import org.ffd2.solar.cli.CommandLineParser;
import org.ffd2.solar.cli.CommandLineParser.HelpTarget;
import org.ffd2.solar.compile.FileBasedGeneralParser;
import org.ffd2.solar.compile.ParserBase;
import org.ffd2.solar.compile.ParserStateCreator;
import org.ffd2.solar.compile.FileBasedGeneralParser.ParserInstance;
import org.ffd2.solar.general.SimpleLogger;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.io.FileLocator;
import org.ffd2.solar.io.FileLocatorUtils;
import org.ffd2.solar.language.BasicErrorTracker;


public class ASkeletonXParser implements ParserInstance {
  private final BasicErrorTracker errorTracker_;
  private final ParserBase<FileRoot> parserBase_;
	private final ParserControl	parserControl_;

  
  public ASkeletonXParser(FileLocator source,FileLocator library) {
		this.errorTracker_ = new BasicErrorTracker();
    parserBase_ = new ParserBase<FileRoot>(source,library,this.errorTracker_);
    this.parserControl_ = new ParserControl(this.parserBase_, this.errorTracker_);
  }

  /*
	   * public BaseBuilder formNewBuilder(PackageSource path) {
	    BaseBuilder result = new BaseBuilder(path,this.outputContextGlobalBase_);
	    this.baseBuilders_.addElement(result);
	    return result;
	  }
//	  public VariableGraph getOutputContextGlobalBase() {
//	    return this.outputContextGlobalBase_;
//	  }

	  public ParserToHubConnection<FileRoot> getSourceHubConnection(){
	    return this.baseConnection_;
	  }
	  public GlobalEnvironment getDefinitionEnvironment() {
	    return this.definitionEnvironment_;
	  }
	  public GlobalEnvironment getImplementationEnvironment() {
	    return this.implementationEnvironment_;
	  }
	  
	  public void finish(FileLocator target) throws IOException {
	    if(!this.errorTracker_.isHasHadErrors()) {
	    	completeParsing();
	    }
	    if(!this.errorTracker_.isHasHadErrors()) {
	      for(BaseBuilder base : this.baseBuilders_) {
	        base.outputStubs(target);
	      }
	      Debug.println("Outputing all!");
	    }
	  }

	   */
  public boolean compile()  {
  	return this.parserControl_.completeParsing();
  }
  public void output(FileLocator target, String[] basePackage, boolean cleanAll) throws IOException {
  	this.parserControl_.output(target, basePackage, cleanAll);

  }
  public void addFile(String fileToCompile)  {
    try {
    	this.parserBase_.checkFile(fileToCompile,  new ParserStateCreator<FileRoot>() {
    
	
				public FileRoot generateParser(String fileName, boolean doOutput) {
	
					return new FileRoot(fileName, ASkeletonXParser.this.parserControl_,true);
				}
	    },true);
    } catch(FileNotFoundException e) {
    	this.errorTracker_.addUserError("File not found", "File \""+fileToCompile+"\" not found by system.");
    }catch(IOException e) {
    	this.errorTracker_.addUserError("General IO error", "Error = "+e);
    }
   }
  public void finish(FileLocator target, boolean cleanAll)  {
    try {
    	this.parserControl_.finish(target);
    	
    	this.errorTracker_.outputAll(new PrintWriter(System.out,true));
    } catch(IOException e) {
    	this.errorTracker_.addUserError("General IO error", "Error = "+e);
    }
  }
  

	public void getErrors(SimpleLogger target) {
    this.errorTracker_.outputAll(target);		
	}
  public static final void doSimpleParse(FileLocator source,FileLocator library, FileLocator target, String ... filesToCompile){
    ASkeletonXParser b = new ASkeletonXParser(source,library);
    for(String fileToCompile : filesToCompile) {
    	b.addFile(fileToCompile);
    }
    b.finish(target,false);
  }
//  public static final void main(String[] args) {
//    doSimpleParse(
//        FileLocatorUtils.getDirectoryBased("/storage/home/matt/Programming/source/SkeletonX/"),
//        FileLocatorUtils.NO_FILES_LOCATOR,
//        FileLocatorUtils.getDirectoryBased("/storage/home/matt/Programming/source/TestSkeleton2/"),
//        "TestSkeletonX.txt"
//    );
//  }

  private static final class CLPUser implements CommandLineParser.User {
		private final SimpleVector<String> files_ = new SimpleVector<String>();
		private FileLocator source_ = FileLocatorUtils.getDefault();
		private FileLocator library_ = null;
		private FileLocator target_ = FileLocatorUtils.getDefault();
		private boolean cleanAll_ = false;
		public CLPUser() {
			String systemLibrary = System.getenv("AUSTEN_LIBRARY_PATH");
			if(systemLibrary!=null) {
				library_ = FileLocatorUtils.getDirectoryBased(systemLibrary);
			} else {
				library_ = FileLocatorUtils.getDefault();
			}
		}
  	public void addOptionWithValue(String option, String value) {
  		if(option.equals("library")) {
  			library_ = FileLocatorUtils.getDirectoryBased(value);
  		} else if(option.equals("user")) {
  			source_ = FileLocatorUtils.getDirectoryBased(value);
  		} else if(option.equals("output")) {
  			target_ = FileLocatorUtils.getDirectoryBased(value);
  		} else {
  			System.out.println("Uknown option: "+option);	
  		}
		}
		
		public void addOption(String option) {
			if(option.equals("clean")) {
				this.cleanAll_ = true;
			} else {
				System.out.println("Uknown option: "+option);
			}
		}
		
		public void addGeneral(String text) {
			files_.addElement(text);
		}

		public String getGeneralType() {
			return "files";
		}

		public String getCommandName() {
			return "SkeletonX";
		}

		public String getDescriptionBlurb() {
			return "A code-generator code generator for JAVA.";
		}

		public void addRequestHelp(HelpTarget target) {
			target.addOption("user", "user path", "Sets alternative path for user files", true);
			target.addOption("output", "output path", "Sets alternative path for the output", true);
			target.addOption("library", "library path", "Sets alternative path for library files", true);
			target.addOption("clean", "clear target directories before outputing build results");

			target.finished();
		}
		public void doParse() {
			
			ASkeletonXParser b = new ASkeletonXParser(this.source_,this.library_);
			if(this.files_.isEmpty()) {
				System.out.println("No files specified");
			} else {
			
	  		for(String s : this.files_) {
	  			b.addFile(s);
	  		}
		  	b.finish(this.target_,this.cleanAll_);
				
			}
		}
	}
  
  public static final FileBasedGeneralParser createGeneralParser() {
  	return new FileBasedGeneralParser() {
			public ParserInstance createInstance(FileLocator source, FileLocator library) {
				return new ASkeletonXParser(source, library);
			}
  	};
  }
  public static final void main(String[] args) throws IOException{
  	if(args.length==0||args[0].equals("GUI")) {
	 		 MainInterfaceGUI.createApplication(
	 				 new AboutDetails("SkeletonX Generator", "#FFB8E3","#9E45AF")
	 				 	.addAuthor("Matthew Goode","matthew@scratchy.org.nz")
	 				 	.loadLicence("Licence.txt")
	 				 	.setAboutMainText("SkeletonX is an tool for generating code generators for use (currently) with " +
	 				 			"the Java programming language.")
	 				 	.setCopyrightYear(2014)
	 				 	.setWebsite("scratchy.org.nz")
	 				 	
	 				 , 
	 				 "skeletonx.prefs",new String[] { ".skeletonx", ".skeleton" }, new InterfaceUser() { }, createGeneralParser());
	 		
	 	} else {
			
			System.out.println("Skeleto X parser generator");
	  	System.out.println("(c) Matthew Goode, Scratchy Labs (matthew@scratchy.org.nz)");
	
	  	CommandLineParser clp = CommandLineParser.Utils.getUnixStyle();
	  	CLPUser user = new CLPUser();
	  	if(clp.parseLine(args, user))  {
	  		user.doParse(); 
	  	}
 		}
  }
//  private static final class ParserStateCreatorImpl implements ParserStateCreator<FileRoot>,DesignRoot<AustenDataBlock>  {
//  	private final AustenParser	parent_;
//  	private final AustenDesignRoot dataDesignRoot_;
//  	public ParserStateCreatorImpl(AustenParser parent) {
//			this.parent_ = parent;
//  		this.dataDesignRoot_ = new AustenDesignRoot(new AustenIDTracker());
//
//  	}
//  	public AustenDataBlock addDataBlock( String[] basePackageNameElements) {
//			return this.dataDesignRoot_.addDataBlock(false, basePackageNameElements);
//		}
//  		
//		public GeneralBuilder doBuild() {
//			AustenMainJavaBuilder xBuilder = new AustenMainJavaBuilder();
//			this.dataDesignRoot_.build(xBuilder);
//			return xBuilder;
//  	}
//
//		public FileRoot generateParser(String fileName,  boolean doOutput) {
//			FileRoot root = new FileRoot(fileName,parent_.packageTracker_,parent_.parserBase_, this,parent_.environment_, doOutput);
//
//		//	FileRoot root = new FileRoot(fileNam,relatedHubConnection,parent_.packageTracker_,parent_.environment_);
//			return root;
//		}
//  	
//  }
	
}
