package org.ffd2.skeletonx.compile.impl;

import org.ffd2.skeletonx.austenx.peg.base.BlockElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.ChainElementPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.DesignTypePatternPeer;
import org.ffd2.skeletonx.compile.design.DesignType;
import org.ffd2.skeletonx.compile.design.DesignType.ChainLinkSkeletonSource;
import org.ffd2.skeletonx.compile.design.DesignTypeAccess;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.ChainBlockFormHolder;
import org.ffd2.skeletonx.skeleton.ChainBlockFormHolder.ChainBlockFormBlock;
import org.ffd2.skeletonx.skeleton.DesignFormHolder;
import org.ffd2.skeletonx.skeleton.IDTrackerFormHolder;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder.TargetBaseFormBlock.ChainDataBlock.LinkRefDataBlock;
import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class ChainDefinitionBlock implements BlockElementPeer.ChainPatternPeer {
	private final SpecificCommonErrorOutput error_;
	private AccessStoreX<Link> links_;
	private final SimpleVector<Link> orderedLinks_ = new SimpleVector<Link>();
	private final DesignTypeAccess typeAccess_;
	private final GlobalEnvironment environment_;
	private final BaseBuilder baseBuilder_;
	private final String chainName_;

	private final TargetBaseFormHolder.TargetBaseFormBlock.ChainDataBlock targetPeerSkeleton_;
	private final IDTrackerFormHolder.IDTrackerFormBlock.ChainDataBlock idSkeleton_;

	private final ChainBlockFormHolder.ChainBlockFormBlock chainBlockFormSkeleton_;
	private final DesignFormHolder.DesignFormBlock.ChainDataBlock designChainSkeleton_;
	private final int chainTypeID_; 

	public ChainDefinitionBlock(
					String chainName, int chainIndex, DesignTypeAccess typeAccess, GlobalEnvironment environment, DesignBridgeComponents bridgeComponents,
					BaseBuilder baseBuilder,DesignFormHolder.DesignFormBlock designSkeleton,  
					IDTrackerFormHolder.IDTrackerFormBlock.ChainDataBlock idSkeleton, 
					TargetBaseFormHolder.TargetBaseFormBlock.ChainDataBlock targetPeerSkeleton,
					SpecificCommonErrorOutput error) {
		this.chainName_ = chainName;
		this.chainTypeID_ = chainIndex;
		this.typeAccess_ = typeAccess;
		this.environment_ = environment;
		this.baseBuilder_ = baseBuilder;
		this.error_ = error;
		this.idSkeleton_ = idSkeleton;
		this.targetPeerSkeleton_ = targetPeerSkeleton;
		this.links_ = environment.createEntityStore("link");
		this.chainBlockFormSkeleton_ = baseBuilder.createChainBlockChild(chainName, this.idSkeleton_, this.targetPeerSkeleton_);
		this.designChainSkeleton_ = designSkeleton.addChain(this.chainBlockFormSkeleton_, this.idSkeleton_, this.targetPeerSkeleton_);
	}
	public void resolutionPass() {
		for(Link l : links_) {
			l.buildSkeletons();
		}
	}
	
	public int getChainIndex() { return this.chainTypeID_; }

	public DesignFormHolder.DesignFormBlock.ChainDataBlock getDesignChainSkeleton() {
		return this.designChainSkeleton_;
	}
	
	public ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.ChainDataBlock addParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock paramaterSkeleton) {
		ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.ChainDataBlock result = paramaterSkeleton.addChain(this.chainBlockFormSkeleton_);
		return result;
	}
	public JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock.ParameterDataBlock.ChainRecordDataBlock addJavaParameter(JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock.ParameterDataBlock paramaterSkeleton) {
	//	BaseBuilder.JavaParameterSettingsFormBlock.ParameterDataBlock.ChainRecordDataBlock result = paramaterSkeleton.addChainRecord(this.chainBlockFormSkeleton_);
//		return result;
		throw Debug.finishMeMarker();
	}
	public void updateImplementation(DesignBaseImplementationTarget target) {
		ChainImplementationTarget newChain = target.newChain(this.chainTypeID_, this.chainName_, this,this.targetPeerSkeleton_);
		for(Link e : this.orderedLinks_) {
			e.updateChain(newChain);
		}
	}


	public void end() { }

	
	public ChainElementPatternPeer addChainElementForElement(String linkName,
			int nameLineNumber, int nameCharacterNumber) {
		SpecificCommonErrorOutput subError = this.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
		LinkRefDataBlock linkRefBlock = targetPeerSkeleton_.addLinkRef(linkName);
		int linkIndex = this.orderedLinks_.size();
		
		Link link = new Link(linkName,linkIndex,this.typeAccess_,this.environment_,linkRefBlock,this,subError);
		try  {
			this.links_.put(linkName, link);
		} catch(ItemAlreadyExistsException e) {
			subError.repeatedObjectError("link", linkName);
		}
		orderedLinks_.addElement(link);
		return link;
	}


	// ========================================================================================
	private final static class Link implements ChainElementPatternPeer{
		private final String name_;
		private final SpecificCommonErrorOutput myError_;
		private final DesignTypeBlock type_;
		private final LinkRefDataBlock linkRefBlock_;
		private final ChainDefinitionBlock parent_;
		private final int linkIndex_;
		private final DesignTypeAccess	typeAccess_;
		public Link(String name,int linkIndex,DesignTypeAccess typeAccess, GlobalEnvironment environment,LinkRefDataBlock linkRefBlock,ChainDefinitionBlock parent, SpecificCommonErrorOutput myError) {
			this.name_ = name;
			this.linkIndex_ = linkIndex;
			this.typeAccess_ = typeAccess;
			this.linkRefBlock_ = linkRefBlock;
			this.parent_ = parent;
			this.myError_ = myError;		
			this.type_ = new DesignTypeBlock(environment,myError);
		}
		public void updateChain(ChainImplementationTarget target) {
			DesignType resolved = this.type_.buildResolvedQuiet(this.typeAccess_);
			if(resolved!=null) {
				resolved.doAddChainLink(target,this.name_, this.linkIndex_);
			}
		}

		public void end() { }
		public void buildSkeletons() {
			DesignType designType = this.type_.buildResolvedQuiet(this.typeAccess_);
			if(designType!=null) {
				try {
					designType.addChainLink(this.name_, new ChainLinkSkeletonSource() {

						public ChainBlockFormBlock getChainBlockSkeleton() {
							return parent_.chainBlockFormSkeleton_;
						}

						public LinkRefDataBlock getLinkRefSkeleton() {
							return linkRefBlock_;
						}
						
					});
				} catch(ParsingException e) {
					e.updateError(this.myError_);
				}
			}
		}

		public DesignTypePatternPeer addDesignTypeForType() {
			return type_;
		}
	} // End of class Link


}
