package org.ffd2.skeletonx.compile.impl;

import org.ffd2.skeletonx.austenx.peg.base.DesignTypePatternPeer;
import org.ffd2.skeletonx.compile.design.DesignType;
import org.ffd2.skeletonx.compile.design.DesignTypeAccess;
import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class DesignTypeBlock implements DesignTypePatternPeer{
  private final SimpleVector<TypeElement> typeElements_ = new SimpleVector<TypeElement>();
  private DesignType resolvedType_;
  private boolean resolutionAttempted_ = false;
	private final SpecificCommonErrorOutput error_;
	private TypeAugmenter arrayAugmenter_ = NO_AUGMENTATION; 
	private final static TypeAugmenter NO_AUGMENTATION = new TypeAugmenter() {
		public DesignType augment(DesignType base) { return base; }
	};
	
  public DesignTypeBlock( GlobalEnvironment environment, SpecificCommonErrorOutput error) { 
		this.error_ = error;
  }
  public void setIsArray(final SpecificCommonErrorOutput relatedError) { 
  	this.arrayAugmenter_ = new TypeAugmenter() {
			public DesignType augment(DesignType base) {
				try {
      		return base.getAsArrayType();
      	} catch(ParsingException e) {
      		e.updateError(relatedError);
      		return base;
      	}
			}
  	};
  }
 
  public void addTypeElement(String id, SpecificCommonErrorOutput error) { 
    this.typeElements_.addElement(new TypeElement(id,error)); 
  }
  public void addTypeElement(String value, int lineNumber, int characterNumber) {
  	addTypeElement(value, this.error_.createAdjusted(lineNumber, characterNumber));
	}
	public void end() { }
	
  /**
   * Get the resolved type
   * @return the resolved type, or null if type not resolved
   * @note hopefully the null part won't be an issue, as an error will be triggered in the 
   * entity resolving pass, causing subsequent passes not to occur (but still, note the null bit)
   */
  public DesignType buildResolvedQuiet(DesignTypeAccess typeAccess) {
  	if(!resolutionAttempted_) {
  		if(this.typeElements_.isEmpty()) {
  			this.error_.systemError("Type block", "No elements found!");
  		} else {
	      DesignType resolved  = this.typeElements_.firstElement().getResolved(typeAccess);
	      int index = 1;
	      final int numberOfElements = this.typeElements_.size();
	      while(index<numberOfElements&&resolved!=null) {
	        resolved = this.typeElements_.get(index++).getSubResolved(resolved);
	      }
	      resolved = arrayAugmenter_.augment(resolved);
      
	      this.resolvedType_ = resolved;
  		}
	    this.resolutionAttempted_ = true;
    }
  	return this.resolvedType_;
  }
  private final static class TypeElement {
    private final SpecificCommonErrorOutput error_;
    private final String id_;

    public TypeElement(String id, SpecificCommonErrorOutput error) {
      this.id_ = id;
      this.error_ = error;
    }
    /**
     * Return sub type, or null if not resolved
     * @param typeAccess
     * @return
     */
    public DesignType getResolved(DesignTypeAccess typeAccess) {
      try {
        return typeAccess.getDesignType(this.id_);
      } catch(ParsingException e) {
        e.updateError(this.error_);
        return null;
      }
    }

    public DesignType getSubResolved(DesignType typeAccess) {
      try {
        return typeAccess.getDirectSubDesignType(this.id_);
      } catch(ParsingException e) {
        e.updateError(this.error_);
        return null;
      }
    }
  }
  
  private static interface TypeAugmenter {
  	public DesignType augment(DesignType base);
  }
	
}
