package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.SimpleVector;

import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;

public class AccessPool {
	 private final SimpleVector<AccessPair> environments_ = new SimpleVector<AccessPair>();
   public AccessPool() { }
   public void addElementAccess(TargetElementAccess elementAccess,BaseTrackers.JavaVariablePath path) {
     this.environments_.addElement(new AccessPair(elementAccess,path));
   }
   /**
    * 
    * @param variableTargetType
    * @param variableName
    * @param variablePath the resulting path to the target variables record (THIS MUST REMAIN CONSTANT AFTER 
    * THE CALL TO THIS METHOD!)
    * @return
    * @throws ParsingException
    */
   public TargetVariable getVariable(TargetType variableTargetType, String variableName, BaseTrackers.JavaVariablePath variablePath) throws ParsingException {
     final int environmentCount = this.environments_.size();
     for(int i = 0 ; i < environmentCount ; i++) {
       AccessPair env = this.environments_.get(i);
       variablePath.clear();
       TargetVariable result = env.getTargetVariableQuiet(false, variableTargetType, variableName, variablePath);
       if(result!=null) {

        // this.environments_.addElement(new AccessPair(result.getTargetTypeReferenceQuietFinal(),variablePath));
         return result;
       }
     }
     throw new ItemNotFoundException(variableName, "target variable");
   }
   public TargetType getTargetTypeQuiet(String typeName, BaseTrackers.JavaVariablePath variablePath) {
   	final int environmentCount = this.environments_.size();
     for(int i = 0 ; i < environmentCount ; i++) {
       AccessPair env = this.environments_.get(i);
       variablePath.clear();
       TargetType result = env.getTargetTypeQuiet(typeName, variablePath);
       return result;
     }
   	return null;
   }
   /**
    * 
    * @param variableTargetType
    * @param variableName
    * @param variablePathPostFixed the resulting path to the target variables record (THIS MUST REMAIN CONSTANT AFTER 
    * THE CALL TO THIS METHOD!)
    * @return
    * @throws ParsingException
    */
   public TargetVariable getFirstVariable(boolean allowLocal, CodeBlockEnvironment baseEnvironment, String variableName, BaseTrackers.JavaVariablePath variablePathPostFixed) throws ParsingException {
     TargetVariable result = baseEnvironment.getTargetVariableQuiet(allowLocal, variableName,variablePathPostFixed);
   	if(result!=null) {
   		TargetTypeReference targetTypeReference = result.getTargetTypeReferenceQuietFinal();
			targetTypeReference.updateAccessPool(this,variablePathPostFixed);
      // this.environments_.addElement(new AccessPair(result.getElementAccess(),variablePathPostFixed));
       return result;
     } else {
    	 throw new ItemNotFoundException(variableName, "target variable");
     }
   }
//   
   public TargetMethod getFirstMethod( CodeBlockEnvironment baseEnvironment, String methodName, BaseTrackers.JavaVariablePath variablePathPostFixed) throws ParsingException {
     TargetMethod result = baseEnvironment.getTargetMethodQuiet(methodName,variablePathPostFixed);
   	if(result!=null) {
      // this.environments_.addElement(new AccessPair(result.getElementAccess(),variablePathPostFixed));
       return result;
     }
     throw new ItemNotFoundException(methodName, "target variable");
   }
   /**
    * 
    * @param methodTargetType
    * @param methodName
    * @param variablePath the resulting path to the target variables record (THIS MUST REMAIN CONSTANT AFTER 
    * THE CALL TO THIS METHOD!)
    * @return
    * @throws ParsingException
    */
   public TargetMethod getMethod(TargetType methodTargetType, String methodName, BaseTrackers.JavaVariablePath variablePath) throws ParsingException {
     final int environmentCount = this.environments_.size();
     for(int i = 0 ; i < environmentCount ; i++) {
       AccessPair env = this.environments_.get(i);
       variablePath.clear();
       TargetMethod result = env.getTargetMethodQuiet(methodTargetType, methodName, variablePath);
       if(result!=null) {
         //this.environments_.addElement(new AccessPair(result.getElementAccess(),variablePath));
         return result;
       }
     }
     throw new ItemNotFoundException(methodName, "target method");
   }
   // ================================================================================
   private final static class AccessPair {
     private final JavaVariablePath path_;
     private final TargetElementAccess elementAccess_;

     public AccessPair(TargetElementAccess elementAccess, BaseTrackers.JavaVariablePath path) {
       this.elementAccess_ = elementAccess;
       this.path_ = path;
     }
     public TargetVariable getTargetVariableQuiet(boolean allowLocal,TargetType targetType, String name, BaseTrackers.JavaVariablePath pathToDeclaration) {
       this.path_.transferTo(pathToDeclaration);
       return elementAccess_.getTargetVariableQuiet(allowLocal, targetType, name, pathToDeclaration);
     }
     public TargetType getTargetTypeQuiet(String typeName, BaseTrackers.JavaVariablePath pathToDeclaration ) {
     	this.path_.transferTo(pathToDeclaration);
       return elementAccess_.getTargetTypeQuiet(typeName, pathToDeclaration);
     	
     }
     
     public TargetMethod getTargetMethodQuiet(TargetType targetType, String name, BaseTrackers.JavaVariablePath pathToDeclaration) {
       this.path_.transferTo(pathToDeclaration);
       return elementAccess_.getTargetMethodQuiet(targetType, name, pathToDeclaration);      
     }

   } //end of class AccessPair
} // end of class AccessPool
