package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.BuilderVariableChainPatternPeer;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseTrackers;

public class BuilderVariableChainBlock implements BuilderVariableChainPatternPeer {

  private final SpecificCommonErrorOutput baseError_;
  private final SimpleVector<Element> elements_ = new SimpleVector<Element>();
 // private final BaseTrackers.JavaVariablePath variablePath_ = new BaseTrackers.JavaVariablePath();
  public BuilderVariableChainBlock( SpecificCommonErrorOutput baseError) {
    this.baseError_ = baseError;
  }
  public void addValue(String value, int lineNumber, int characterNumber) {
    SpecificCommonErrorOutput error = this.baseError_.createAdjusted(lineNumber, characterNumber);
    this.elements_.add(new Element(value,error));
  }
  public void end() { }
  public String toString() { return elements_.toString(); }
  // ==========================================================================
  public DataBlockEnvironment resolveRecordQuiet(DataBlockEnvironment baseEnvironment,BaseTrackers.JavaVariablePath variablePath, boolean includeFinalVariableInPath) {
  	return this.resolveRecordQuiet(baseEnvironment, variablePath, includeFinalVariableInPath, null);
  }

  /**
   * Call during final pass to resolve references to variables.
   * @return true if succssful in resolving all variables in chain, false otherwise
   */
  public DataBlockEnvironment resolveRecordQuiet(DataBlockEnvironment baseEnvironment,BaseTrackers.JavaVariablePath variablePath, boolean includeFinalVariableInPath, SimpleVector<IRecordVariable> pathStore) {
    
  	IRecordVariable variable =baseResolve(baseEnvironment,variablePath, includeFinalVariableInPath,pathStore);
    if(variable!=null) {
  	  try {
       return variable.getAsFoundataionNodeReferenceVariable(variablePath).getAsDataBlockEnvironment();
  	  } catch (ParsingException e) {
        e.updateError(this.elements_.lastElement().getError());
        return null;
      }
     
    } else {
      return null;
    }
  }
  /**
   * Call during final pass to resolve references to variables.
   * @return true if succssful in resolving all variables in chain, false otherwise
   */
  public SLayer resolveContainerQuiet(SLayer base) {
    
  	final int numberOfElements = this.elements_.size();
  	SLayer result = base;
    for(int i = 0 ; i < numberOfElements ; i++) {
    	if(result==null) {
    		return null;
    	}
      result = this.elements_.get(i).getChildContainerQuiet(result);
    }
    return result;
  }
  /**
   * Call during final pass to resolve references to variables.
   * @return true if succssful in resolving all variables in chain, false otherwise
   */
//  public PrimitiveResolution resolvePrimitiveQuiet(BuilderVariableAccess baseEnvironment) {
//    BaseTrackers.JavaVariablePath variablePath = new BaseTrackers.JavaVariablePath();
//    IRecordVariable variable =baseResolve(baseEnvironment,variablePath,null);
//    if(variable!=null) {
//      return new PrimitiveResolution( variable,variablePath,this.elements_.lastElement().getError());
//    } else {
//      return null;
//    }
//  }
  
  /**
   * Handles error stuff - just need to check if returns null
   */
  public IRecordVariable resolveRecordVariableQuiet(SLayer baseEnvironment,BaseTrackers.JavaVariablePath variablePath, boolean includeFinalVariableInPath,SimpleVector<IRecordVariable> pathStore ) {
  	return baseResolve(baseEnvironment.getAsDataBlockEnvironment(),variablePath, includeFinalVariableInPath,pathStore);
  }
  /**
   * Handles error stuff - just need to check if returns null
   */
  public IRecordVariable resolveRecordVariableQuiet(SLayer baseEnvironment,BaseTrackers.JavaVariablePath variablePath, boolean includeFinalVariableInPath) {
  	return baseResolve(baseEnvironment.getAsDataBlockEnvironment(),variablePath, includeFinalVariableInPath,null);
  }
  public SpecificCommonErrorOutput getEndError() {
  	return this.elements_.lastElement().getError();
  }
  // ------------------------------
  // PathStore maybe null
  private final IRecordVariable baseResolve(BuilderVariableAccess baseEnvironment,BaseTrackers.JavaVariablePath variablePath, boolean includeFinalVariableInPath, SimpleVector<IRecordVariable> pathStore ) {
   // Debug.println("BASE RESULT:",this.elements_);
  	final int numberOfElements = this.elements_.size();
  	//Element currentElement = this.elements_.firstElement();
   // IRecordVariable variable = currentElement.resolveDownstreamQuiet(baseEnvironment,variablePath);
   // if(pathStore!=null) { pathStore.addElement(variable); }
  	Element currentElement = null;
  	IRecordVariable variable = null;
  	try {
    	final int lastElement = numberOfElements-1;
    	BuilderVariableAccess currentEnvironment = baseEnvironment;
	    for(int i = 0 ; (i < numberOfElements) ; i++) {
	    	currentElement = this.elements_.get(i);
	    	variable = currentElement.resolveDownstreamQuiet(currentEnvironment,variablePath);
	      if(variable==null) {
	      	break;
	      }
	      if((i!=lastElement||includeFinalVariableInPath)&&(pathStore!=null)) {
	      	pathStore.addElement(variable); 
	      }
	    	SLayer targetNode;
	    	if(i!=lastElement) {
	      	targetNode = variable.getAsFoundataionNodeReferenceVariable(variablePath);
	      
	    		currentEnvironment= targetNode.getAsDataBlockEnvironment();
	    	}
	    	// variable = currentElement.resolveDownstreamQuiet(targetNode.getAsDataBlockEnvironment(),variablePath);
	     
	    }
    } catch(ParsingException e) {
    	//CurrentElement must be not null
    	e.updateError(currentElement.getError());
    }
    return variable;
  }
  
//  public static final class PrimitiveResolution {
//    private final JavaVariablePath variablePath_;
//    private final SpecificCommonErrorOutput endError_;
//    private final IRecordVariable resolvedVariable_;
//
//    private PrimitiveResolution(IRecordVariable resolvedVariable, BaseTrackers.JavaVariablePath variablePath, SpecificCommonErrorOutput endError) {
//      this.resolvedVariable_ = resolvedVariable;
//      this.variablePath_ = variablePath;
//      this.endError_ = endError;
//    }
//    public void appendToTrackerChain(final BaseTrackers.JavaVariablePath variablePath ) {
//      this.variablePath_.transferTo(variablePath);
//    }
//    public void updateNameBlock(BaseBuilder.NameBuilderCode.ComplexDataBlock target)  {
//      try {
//        this.resolvedVariable_.updateNameBlock(target, variablePath_.getCopy(), null);
//      } catch(ParsingException e) {
//        e.updateError(this.endError_);
//      }
//    }
//  }
//  
//  
//  /**
//   * Get the end environment (the environment of the value of the builder variable - if it has one)
//   * @return the environment, or null if variable is not of right type (eg, is an integer)
//   * @note will flag an error if of wrong type, valid only after a call to resolve()
//   */
//  public final DataBlockEnvironment getEndEnvironmentQuiet()  {
//    return this.elements_.lastElement().getValueEnvironment();
//  }
//  
//  /**
//   * Get the end environment (the environment of the value of the builder variable - if it has one)
//   * @return the environment, or null if variable is not of right type (eg, is an integer)
//   * @note will flag an error if of wrong type, valid only after a call to resolve()
//   */
//  public final RecordAccess getEndRecordQuiet()  {
//    return this.elements_.lastElement().getValueRecordAccessQuiet();
//  }

  /**
   * Get a target type in the space of the last variable (if last variable refers to a builder record
   * @param name
   * @param fromThisSpaceToTypeDeclarationPath the path from the environment used to resolve()
   * @return
   * @throws ParsingException if last variable is not a reference to a builder record, or the target requested does not exist in record
   * @note  valid only after a call to resolve()
   */
//  public final TargetType getEndTargetType(String name,JavaVariablePath fromThisSpaceToTypeDeclarationPath) throws ParsingException {
//    appendToTrackerChain(fromThisSpaceToTypeDeclarationPath);
//    return this.elements_.lastElement().getTargetType(name,fromThisSpaceToTypeDeclarationPath);
//  }
//  public final BaseTrackers.JavaVariablePath createTrackerChain() {
//
//    return variablePath_.getCopy();
//  }
 
  
  // ==========================================================================
  private static final class Element {
    private final String value_;
    private final SpecificCommonErrorOutput error_;
    public Element(String value, SpecificCommonErrorOutput error) {
      this.value_ = value;
      this.error_ = error;
    }
    public String toString() { return value_+"("+error_.getLineNumber()+")"; }
    
    // ------------------------------------------------------------------------
    // Mirrors IRecordVariable stuff (with errors handled)

//    public TargetType getTargetType(String name,JavaVariablePath fromThisSpaceToTypeDeclarationPath) throws ParsingException {
//      return this.resolvedVariable_.getValueEnvironment().getTargetType(name,fromThisSpaceToTypeDeclarationPath);
//    }
    public SpecificCommonErrorOutput getError() { return this.error_; }

    
    
    // ------------------------------------------------------------------------
    public SLayer getChildContainerQuiet(SLayer base) {
    	try {
				return base.getVariableChild(this.value_,this.error_.getLineNumber());
			} catch (ParsingException e) {
				e.updateError(this.error_);
				return null;
			}
    }
    public IRecordVariable resolveDownstreamQuiet(BuilderVariableAccess base,BaseTrackers.JavaVariablePath variablePath) {
    	//THIS IS THE RESTRICTIVE BIT!
    	IRecordVariable result = base.getBuilderVariableQuiet(this.value_,variablePath);
      if(result==null ) {
        this.error_.objectNotFoundError("builder variable", this.value_);
      }
//	      if(value_.equals("generalResultType")) {
//	      	Debug.println("--------------------------------found iVariable:",value_,"   path:",variablePath);
//	      	Debug.println("FOUND:",result);
//	      	return new DebugIRecordVariable(result,this.value_);
//	      	
//	      } else {
      	return result;
//	      }
	   
      
    }
  }
}
