package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.austenx.peg.base.JavaClassInterfaceElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaClassPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaInterfacePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.NamePeer.BasicPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.SetsListPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeListPatternPeer;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.ClassImplementation.ClassAccessSource;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BClassAccessFormHolder.BClassAccessFormBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.language.GeneralCommonErrorOutput;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class ClassConstruction implements  JavaClassPatternPeer,JavaInterfacePatternPeer {
  private final String className_;

  private final JavaClassElements elements_;
  private NameBlock nameBlock_ = null;
  private final SpecificCommonErrorOutput baseError_;
  
  // Built during basicBuild()
  private ClassImplementation classImplementation_;

	private final boolean isClass_;
	private boolean isSubClass_ = false;
	
	private TargetTypeListBlock extendsList_ = null;
	private TargetTypeListBlock implementsList_ = null;
  
  private RecordClassAccess relatedRecordClassAccess_;
	private ClassSkeletons classSkeletons_;

	private boolean isFinal_ = false;
	private boolean isStatic_ = false;

	private SetsHandler setsTail_;
	
	private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock classSkeleton_;

	private final BaseBuilder rootBuilder_;

	
	/**
	 * 
	 * @param className
	 * @param isClass
	 * @param environment
	 * @param error
	 */
  public ClassConstruction(String className, BaseBuilder rootBuilder, boolean isClass,FileEnvironment environment, SpecificCommonErrorOutput error) {
//    if(className.equals("Decoder")) {
//      Debug.println("Class:",className,"  ",error.getLineNumber());
//      Thread.dumpStack();
//    }
  	this.className_ = className;
		this.rootBuilder_ = rootBuilder;
		this.isClass_ = isClass;

    this.baseError_ = error;
    this.elements_ = new JavaClassElements(className,rootBuilder, environment,error);
  }
  
	public void addIsFinal(int lineNumber, int characterNumber) {
		if(this.isClass_) {
			this.isFinal_ = true;
		} else {
			this.baseError_.createAdjusted(lineNumber, characterNumber).generalSyntaxError("interfaces can't be final");
		}
		Debug.finishMeMarker();

	}
	public void addIsStatic(int lineNumber, int characterNumber) {
		this.isStatic_ = true;
		Debug.finishMeMarker();
	}

	
	public SetsListPatternPeer addSetsListForClassSet() {
		//TODO
		return new SetsListPatternPeer() {
			public void end() { }
			public void addClassName(String value, int lineNumber, int characterNumber) {
				SpecificCommonErrorOutput error = baseError_.createAdjusted(lineNumber, characterNumber);
				ClassConstruction.this.setsTail_ = new SetsHandler(value,rootBuilder_, error,ClassConstruction.this.setsTail_);
			}
		};
	}

  public void end() { }
	
	
  private final static class SetsHandler {
  	private final SetsHandler	next_;
		private final SpecificCommonErrorOutput	error_;
		private final String className_;
		private GlobalClassReference	globalClassReference_;
		private final BaseTrackers.JavaVariablePath pathToGlobalRefRecord_ = new BaseTrackers.JavaVariablePath();
		private final BaseBuilder rootBuilder_;
	
		public SetsHandler(String className, BaseBuilder rootBuilder,  SpecificCommonErrorOutput error, SetsHandler next) {
			this.className_ = className;
			this.rootBuilder_ = rootBuilder;
			this.error_ = error;
			this.next_ = next;
  	}
		private int getFoundClassCount() {
			final int myCount = ((this.globalClassReference_!=null) ? 1 : 0);

			if(next_!=null) {
				return next_.getFoundClassCount() + myCount;
			} else {
				return myCount;
			}
		}
		private final void fillInGlobalReferences(GlobalClassReference[] result, int myIndex) {
			if(this.globalClassReference_!=null) {
				result[myIndex]=this.globalClassReference_;
				myIndex--;
			} 
			if(next_!=null) {
				next_.fillInGlobalReferences(result, myIndex);
			}
		}
		/**
		 * Get an array of all found Global references (or null if no references found)
		 * Array in order of declaration
		 * @return
		 */
		public GlobalClassReference[] getAllClassReferences() {
			int size = getFoundClassCount();
			if(size==0) {
				return null;
			}
			GlobalClassReference[] result = new GlobalClassReference[size];
			fillInGlobalReferences(result, size-1);
			return result;
		}
	  private void basicBuild(ClassSkeletons classSkeletons, SLayer parentLayer)  {
	  	if(this.next_!=null) {
				this.next_.basicBuild(classSkeletons, parentLayer);
			}
	  	this.globalClassReference_ = parentLayer.getGlobalClassReferenceQuiet(this.className_,pathToGlobalRefRecord_,true);
			
	  	if(this.globalClassReference_==null) {
				this.error_.objectNotFoundError("global class reference", className_);
			}
	  }
	  public void finalBuild( JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock classSkeleton) {
	  	if(this.next_!=null) {
				this.next_.finalBuild(classSkeleton);
			}
	  	this.globalClassReference_.updateSetGlobalFinal(classSkeleton.getGlobalVariableStoreParameter(),this.pathToGlobalRefRecord_);
	  }
  }

  
  private static interface ClassSkeletons{
		public void basicBuild(SLayer environment, JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock classSkeleton);
		
  	public void preBuildCodeSetType(TypeSetterDetailsFormBlock target,BaseTrackers.JavaVariablePath pathToClassChainTracker);

  }

  //====================================
  private void doBasicBuildGeneral(ClassSkeletons classSkeletons, SLayer parentLayer)  {
  	this.classSkeletons_ = classSkeletons;
		this.classSkeleton_ = parentLayer.addItemSkeleton().addMakeClass(this.className_, this.isClass_,this.rootBuilder_.createBClassAccessChild(),this.rootBuilder_.createGlobalClassSourceChild(),this.rootBuilder_.createNameDetailsChild());

		this.classImplementation_ =  new ClassImplementation(this.className_, new ClassAccessSource() {
			public BClassAccessFormBlock getClassAccess() { return classSkeleton_.getVariableStoreParameter(); }
		},parentLayer,this.extendsList_,this.implementsList_);
		try {
			parentLayer.addType(classImplementation_, this.className_);
		} catch(ItemAlreadyExistsException e) {
			this.baseError_.repeatedObjectError("target type", this.className_);
		}
		final GlobalClassReference[] allGlobalReferences;
		classSkeletons.basicBuild(parentLayer, this.classSkeleton_);
  	if(this.setsTail_!=null) {
    	this.setsTail_.basicBuild(classSkeletons,parentLayer);
    	allGlobalReferences = this.setsTail_.getAllClassReferences();
    } else {
    	allGlobalReferences = null;
    }
  	RecordClassAccess recordClassAccess = this.relatedRecordClassAccess_ = new RecordClassAccess(parentLayer,this.classImplementation_,allGlobalReferences);
  	
  	elements_.basicBuild(recordClassAccess);   
	  if(this.implementsList_!=null) {
	   	this.implementsList_.basicBuild(parentLayer); 	
	  }
    if(this.extendsList_!=null) {
    	this.extendsList_.basicBuild(parentLayer);
    }
    
  }
  /**
   * Do the build as though the type is a root type
   * @param parentRecord
	 * @param packageSubName may be null to represent no sub package adjustment
   */
  public void basicBuildRoot(SLayer parentLayer, String packageSubName) {
  	this.isSubClass_ = false;
    doBasicBuildGeneral(new RootClassSkeletons(packageSubName), parentLayer);
  }
  public void basicBuildInContainer(RecordContainerAccess parentRecord) {
  	this.isSubClass_ = false;
    doBasicBuildGeneral(new InContainerSkeletons(parentRecord), parentRecord.getBaseLayer());
  }
  /**
   * Do the build for types that are sub types of other types 
   * @param parentRecord
   */
  public final void basicBuildSubClass(RecordClassAccess parentRecord) {
  	this.basicBuildSubClass(parentRecord.getBaseLayer(),parentRecord.getContainingClass());
  }
  

  public final void basicBuildSubClass(SLayer parentRecord, ClassImplementation parentClass) {
  	this.isSubClass_ = true;
    doBasicBuildGeneral(new SubClassSkeletons(this,parentClass), parentRecord);
  }
  public final void basicBuildGeneralContainer(SLayer parentRecord, ClassImplementation parentClass) {
  	this.isSubClass_ = true;
    doBasicBuildGeneral(new SubClassSkeletons(this,parentClass), parentRecord);
  }

  
  public final void finalBuild() {
  	BaseBuilder rootBuilder = classSkeleton_.getRootBuilder();
  	if(this.implementsList_!=null) {
  		this.implementsList_.finalBuild(this.classSkeleton_.getClassDetailsMacro().addImplements(rootBuilder.createTypeListDetailsChild()).getTypeListDetailsParameter());
  	}
  	if(this.extendsList_!=null) {
  		this.extendsList_.finalBuild(this.classSkeleton_.getClassDetailsMacro().addExtends(rootBuilder.createTypeListDetailsChild()).getTypeListDetailsParameter());
  	}
  	if(this.nameBlock_==null) {
  		this.classSkeleton_.getNameDetailsParameter().addSimple(this.className_);
    } else {
      this.nameBlock_.updateTypeNameSkeleton(this.className_,this.classSkeleton_.getNameDetailsParameter(),this.relatedRecordClassAccess_.getBaseLayer());
    }
    elements_.finalBuild();
    if(this.setsTail_!=null) {
    	this.setsTail_.finalBuild(this.classSkeleton_);
    }
  }
  
	public TargetTypeListPatternPeer addTargetTypeListForExtensionList() {
		return this.extendsList_ = new TargetTypeListBlock(this.baseError_);
	}
	public TargetTypeListPatternPeer addTargetTypeListForImplementsList() {
		return this.implementsList_ = new TargetTypeListBlock(this.baseError_);
	}
  
  

  
  public void buildCodeSetType(TypeSetterDetailsFormBlock target,BaseTrackers.JavaVariablePath pathToClassChainTracker) {
  	this.classSkeletons_.preBuildCodeSetType(target, pathToClassChainTracker);
  	target.addFromClass(this.classSkeleton_.getVariableStoreParameter(), pathToClassChainTracker);
    
  }

  

  // ===================================================================
  
  public TargetMethod getTargetMethodQuiet(String name, BaseTrackers.JavaVariablePath pathToDeclaration)  {
    return this.relatedRecordClassAccess_.getTargetMethodQuiet(name, pathToDeclaration);
  }
  public TargetVariable getTargetVariableQuiet(boolean allowLocalVariables,String name, BaseTrackers.JavaVariablePath pathToDeclaration)  {
    return this.relatedRecordClassAccess_.getTargetVariableQuiet(allowLocalVariables, name, pathToDeclaration);
  }
  
  //public ImplementationTargetType getContainingClassTargetType() { return this.classImplementation_; }
  
  public String toString() { 
  	return 
  	 "("+(this.isSubClass_ ? "SUB ":"ROOT")+(this.isClass_ ? "CLASS" : "INTERFACE")+":"+className_+")"; 
  }
 
  public void addIsPrivate() {
  	Debug.finishMeMarker();
    baseError_.generalSyntaxError("Private classes not supported for root level classes");
  }
  public void addIsPublic() { }

  // ------------------------------------------------------------------------

//  public JavaClassElements createLinkedClassElements(SpecificCommonErrorOutput error) {
//    final JavaClassElements result = new JavaClassElements(error,this);
//    //Bit of an intrusive hack (classSkeleton_ may not be available at the call time of this method - keep in mind if you want to tidy things up)
//    linkedRecord.addRecordItem(new RecordItem() {
//      public void basicBuild(JavaImplementationForm.BlockDataBlock.ElementSetDataBlock relatedDataBlock) {
//        result.basicBuild(relatedDataBlock, classSkeleton_);
//      }
//      public void finalBuild() {
//         result.finalBuild();
//      }
//    });
//    return result;
//  }

 
  //  ------------------------------------------------------------------------

	public BasicPatternPeer addOptionBasicOfNameForNameOutput() { return this.nameBlock_ = new NameBlock(this.baseError_); }

	public void addOptionImplementsOfNameForNameOutput(final int keywordLineNumber, final int keywordCharacterNumber, final String classNameParameter, final String methodNameParameter, final int methodNameLineNumber, final int methodNameCharacterNumber) {
		this.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber).generalSyntaxError("implementing name not applicable to class declaration");
	}
 
  public JavaClassInterfaceElementPeer.Indirect getJavaClassInterfaceElementForElements() {
    return elements_;
  }

  // ==========================================================================
	//public void checkSkeleton() {
	//if(this.classSkeleton_==null) {
	//	JavaVariablePath chainToParentClassDef = new JavaVariablePath();
	//	RecordAccess parentRecord = parent_.relatedRecordClassAccess_.getBaseRecord();
	//	parentRecord.accessType(parentClass_, chainToParentClassDef);
	//	this.classSkeleton_ = parentRecord.addItemSkeleton().addSubTypeBlock(parent_.className_,parent_.isClass_,parentClass_.getClassAccess(),chainToParentClassDef );  
	//}
	//}
	private static final class SubClassSkeletons implements ClassSkeletons{
		private final ClassImplementation parentClass_;
		private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock.SubTypeDataBlock classSkeleton_;
		private final ClassConstruction parent_;
		private final JavaVariablePath chainToParentClassDef_  = new JavaVariablePath();
		public SubClassSkeletons( ClassConstruction parent,ClassImplementation parentClass) {
			this.parent_ = parent;
			this.parentClass_ = parentClass;
			
		}
		public void basicBuild(SLayer environment, JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock classSkeleton) {
			//parentClass_.
			this.parentClass_.getPathToClass(environment,this.chainToParentClassDef_);
			//The problem with the following is that the dummy macro class implementation does not appear to be added to the SLayer 
			//environment.buildPathToType(parentClass_, this.chainToParentClassDef_,true);

			this.classSkeleton_ = classSkeleton.addSubType(parentClass_.getClassAccess(),chainToParentClassDef_ );  		
		}
		
		public void preBuildCodeSetType(TypeSetterDetailsFormBlock target,BaseTrackers.JavaVariablePath pathToClassChainTracker) {
	    //TODO: check if this acctually does anything useful
			JavaVariablePath chainToParentClassDef = new JavaVariablePath();
	    pathToClassChainTracker.transferTo(chainToParentClassDef);
	  	parent_.relatedRecordClassAccess_.accessType(this.parentClass_, chainToParentClassDef );
		}
	}
	private static final class InContainerSkeletons implements ClassSkeletons{
		private final RecordContainerAccess parentContainer_;
		private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock.GeneralTypeDataBlock classSkeleton_;
		public InContainerSkeletons( RecordContainerAccess parentContainer) {

			this.parentContainer_ = parentContainer;
			
		}
		public void basicBuild(SLayer environment, JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock classSkeleton) {
			this.classSkeleton_ = parentContainer_.buildGeneralTypeSkeleton(environment, classSkeleton);  		
		}
		
		public void preBuildCodeSetType(TypeSetterDetailsFormBlock target,BaseTrackers.JavaVariablePath pathToClassChainTracker) {
//	    JavaVariablePath chainToParentClassDef = new JavaVariablePath();
//	    pathToClassChainTracker.transferTo(chainToParentClassDef);
//	  	parent_.relatedRecordClassAccess_.getBaseRecord().accessType(this.parentClass_, chainToParentClassDef );
		}
	}
	private static final class RootClassSkeletons implements ClassSkeletons{
		private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock.RootTypeDataBlock classSkeleton_;
	//	private final ClassConstruction parent_;
		private final String	packageSubName_;
		public RootClassSkeletons( String packageSubName) {
			this.packageSubName_ = packageSubName;
		}
		public void basicBuild(SLayer environment, JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MakeClassDataBlock classSkeleton) {
			this.classSkeleton_ = classSkeleton.addRootType(); 
			//Check if the a package adjustment is required specifies that the created class needs
			if(this.packageSubName_!=null) {
				this.classSkeleton_.addAdjustPackage(this.packageSubName_);
			}
		
		}
		public void preBuildCodeSetType(TypeSetterDetailsFormBlock target,BaseTrackers.JavaVariablePath pathToClassChainTracker) { }
	}
	
	




  
}
