package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.austenx.peg.base.BuilderVariableChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.DesignTypePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.FindDefinitionPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.FindTargetPeer.BackThreadBasedPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.FindTargetPeer.RootBasedPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.FindTargetPeer.ThreadBasedPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.FindTargetPeer.VariableBasedPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.SearchQueryPeer;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.NodeLayerChild;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.GeneralCommonErrorOutput;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class FindVariable extends AbstractIRecordVariable implements FindDefinitionPatternPeer,SearchQueryPeer.Indirect, IRecordVariable{ 
	private final String	variableName_;
	private final SpecificCommonErrorOutput	error_;
	private final SimpleVector<SearchQuery> queries_ = new SimpleVector<SearchQuery>();
	private FindType type_;
	private JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock	searchBlockBaseSkeleton_;
	public FindVariable(String variableName,  SpecificCommonErrorOutput error) {
		this.variableName_ = variableName;
		this.error_ = error;
	}
  @Override public boolean isMatchesVariableName(String queryVariableName) { return this.variableName_.equals(queryVariableName); }
	@Override public String getRoughVariableName() { return this.variableName_; }
	
	
	// ==========================================================================
	@Override public void addAsMacroRecordParameter(
			JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock macroCallBlock,
			JavaVariablePath variablePath, IVariableType type)
			throws ParsingException {
		//throw ParsingException.createGeneralSyntax("not a record moo");
		if(type==IVariableType.RecordReference) {
			macroCallBlock.addCallParameter().addFindVariableBlockVariable(this.searchBlockBaseSkeleton_, variablePath);
			//return true;
		} else {
			throw ParsingException.createGeneralSyntax("cannot provided for "+type.getName());
		}
	}
	
	
	// IRecordVariable methods
	public SLayer getAsFoundataionNodeReferenceVariable( JavaVariablePath variablePath) throws ParsingException {
		SLayer resultType = this.type_.getResultType();
		if(resultType!=null) {
			if(variablePath!=null) {
				variablePath.addFoundVariable(this.searchBlockBaseSkeleton_);
			}
			return resultType;
		} else {
			throw ParsingException.createGeneralSemantic("not a record");
		}
	}

	public void getAsReference(IRecordTypeTarget target) {
		target.setNotReference();
	}
	// ==========================================================================

	private static interface FindType {
		public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock doBasicBuild(FoundationNode rootRecord,SLayer containingLayer);
		/**
		 * Do finish
		 * @param searchBlockBase
		 * @param rootRecord
		 * @param containingLayer
		 * @return the layer of the target search
		 */
		public SLayer doFinishBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase, FoundationNode rootRecord,SLayer containingLayer);
		//public SLayer getAsFoundataionNodeReferenceVariable( JavaImplementationForm.BlockDataBlock.SearchRecordDataBlock searchBlockBase,JavaVariablePath variablePath) throws ParsingException;
		public SLayer getResultType();

	}
	private final static class RootFindType implements FindType,RootBasedPatternPeer {
		private final FindVariable	parent_;
		private final TypeBlock designType_;
		private FoundationNode resultType_;
		public RootFindType(FindVariable parent) {
			this.parent_ = parent;
			this.designType_ = new TypeBlock( parent.error_);

		}
		public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock doBasicBuild(FoundationNode rootRecord,SLayer containingLayer) {
			this.resultType_ = this.designType_.getEndNodeQuiet(rootRecord);
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase = null;
			if(this.resultType_!=null) {
				searchBlockBase = containingLayer.addRootTypeSearchBlock(this.resultType_,parent_.error_.getLineNumber()).getParent();
			}
		
			return searchBlockBase;
			
		}
		public SLayer doFinishBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase,FoundationNode rootRecord,SLayer containingLayer) {
			return this.resultType_.getRelatedLayerForLayerComponents();
		}
		public SLayer getResultType() {
			if(resultType_!=null) {
				return this.resultType_.getRelatedLayerForLayerComponents();
			} else {
				return null;
			}
		}
//		public SLayer getAsFoundataionNodeReferenceVariable( JavaImplementationForm.BlockDataBlock.SearchRecordDataBlock searchBlockBase,JavaVariablePath variablePath) throws ParsingException {
//			if(this.resultType_!=null) {
//				variablePath.addFoundVariable(searchBlockBase);
//				return this.resultType_.getRelatedLayerForLayerComponents();
//			}
//			throw ParsingException.createGeneralSemantic("not a record");
//
//		}
		public void end() { 	}
		public DesignTypePatternPeer addDesignTypeForResultType() { return designType_; 	}
	}
	// --------------------------------------------------------------------------
	private final static class LocalFindType implements FindType {
		private final FindVariable	parent_;
//	private final BuilderVariableChainBlock variableChain_;
		private SLayer resultType_;
		private final String	childName_;
		private final SpecificCommonErrorOutput	childError_;
		public LocalFindType(FindVariable parent, String childName, SpecificCommonErrorOutput childError) {
			this.parent_ = parent;
			this.childName_ = childName;
			this.childError_ = childError;
		}
		public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock doBasicBuild(FoundationNode rootRecord,SLayer containingLayer) {
			BaseTrackers.JavaVariablePath pathToVariables = new BaseTrackers.JavaVariablePath();
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase = null;
			final SLayer baseLayer = containingLayer;
			try {
		
				NodeLayerChild nodeLayerChild = baseLayer.getNodeLayerChild(childName_,this.childError_.getLineNumber());
				this.resultType_ = nodeLayerChild.getBase();
				searchBlockBase = containingLayer.addSearchBlock(this.resultType_,childError_.getLineNumber());
				nodeLayerChild.doAddAncestorTypeSearch(searchBlockBase, pathToVariables);
			} catch (ParsingException e) {
				e.updateError(this.childError_);
				return searchBlockBase;
			}
			return searchBlockBase;
		}
		public SLayer doFinishBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase, FoundationNode rootRecord,SLayer containingLayer) {
			return this.resultType_;
		}
		public SLayer getResultType() { return resultType_; }
	}
	// --------------------------------------------------------------------------
	private final static class SimpleThreadFindType implements FindType {
		private final FindVariable	parent_;
//		private final BuilderVariableChainBlock variableChain_;
		private final String	childName_;
		private final SpecificCommonErrorOutput	childError_;
		private final String	nodeLayerType_;
		private final SpecificCommonErrorOutput	nodeLayerError_;
		private SLayer resultType_;
		public SimpleThreadFindType(FindVariable parent,String nodeLayerType,  String childName, SpecificCommonErrorOutput nodeLayerError, SpecificCommonErrorOutput childError) {
			this.parent_ = parent;
			this.nodeLayerType_ = nodeLayerType;
			this.childName_ = childName;
			this.nodeLayerError_ = nodeLayerError;
			this.childError_ = childError;
			 //this.variableChain_ = new BuilderVariableChainBlock(parent.error_);
		}

		public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock doBasicBuild(FoundationNode rootRecord,SLayer containingLayer) {
			BaseTrackers.JavaVariablePath pathToVariables = new BaseTrackers.JavaVariablePath();
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase = null;
			try {
				final SLayer baseLayer  = containingLayer.getAncestorNodeSLayer(this.nodeLayerType_, pathToVariables);
				NodeLayerChild nodeLayerChild = baseLayer.getNodeLayerChild(childName_,this.nodeLayerError_.getLineNumber());
				this.resultType_ = nodeLayerChild.getBase();
				searchBlockBase = containingLayer.addSearchBlock(this.resultType_,childError_.getLineNumber());
				nodeLayerChild.doAddAncestorTypeSearch(searchBlockBase, pathToVariables);
			} catch (ParsingException e) {
				e.updateError(this.nodeLayerError_);
				return searchBlockBase;
			}
			return searchBlockBase;
		}
		public SLayer doFinishBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase, FoundationNode rootRecord,SLayer containingLayer) {
			return this.resultType_;
		}
		public SLayer getResultType() { return resultType_; }
	} // end of class SimpleThreadFindType 
	private final static class VariableFindType implements FindType,VariableBasedPatternPeer {
		private final FindVariable	parent_;
		private final String	childName_;
		private final SpecificCommonErrorOutput	childError_;
		private final SimpleVector<String>	variableNameList_ = new SimpleVector<String>();
		private SLayer resultType_;
		public VariableFindType(FindVariable parent, String childName, SpecificCommonErrorOutput childError) {
			this.parent_ = parent;
			this.childName_ = childName;
			this.childError_ = childError;
		}
		
		// ------------------------------------------------------------------------
		public void end() { }

		public void addVariableName(String value, int lineNumber, int characterNumber) {
			this.variableNameList_.addElement(value);
		}
		// ------------------------------------------------------------------------

		public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock doBasicBuild(FoundationNode rootRecord,SLayer containingLayer) {
			BaseTrackers.JavaVariablePath pathToVariables = new BaseTrackers.JavaVariablePath();
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase = null;
			try {
				//Find the layer the variable is on
				SLayer variableLayer  = containingLayer.getAncestorVariableSLayer(this.variableNameList_.firstElement(), pathToVariables);

				//Get the variable
				IRecordVariable variable = variableLayer.getBuilderVariable(this.variableNameList_.firstElement(), pathToVariables, true);
				
				//Get what the layer is pointing to
				variableLayer = variable.getAsFoundataionNodeReferenceVariable(pathToVariables);
				
				for(int i = 1 ; (variableLayer!=null)&&(i< this.variableNameList_.size()); i++) {
					variableLayer = variableLayer.getVariableChild(this.variableNameList_.elementAt(i),this.childError_.getLineNumber());
				}
				if(variableLayer!=null) {
					NodeLayerChild nodeLayerChild = variableLayer.getNodeLayerChild(childName_,this.childError_.getLineNumber());
					this.resultType_ = nodeLayerChild.getBase();
					searchBlockBase = containingLayer.addSearchBlock(this.resultType_,childError_.getLineNumber());
					nodeLayerChild.doAddAncestorTypeSearch(searchBlockBase, pathToVariables);
				}
			} catch (ParsingException e) {
				e.updateError(this.childError_);
				return searchBlockBase;
			}
			return searchBlockBase;		
		}
		public SLayer doFinishBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase, FoundationNode rootRecord,SLayer containingLayer) {
			return this.resultType_;
		}
		public SLayer getResultType() { return resultType_; }


	}
	//--------------------------------------------------------------------------

	// BackThreadBasedPatternPeer
	private static final class BackThreadFindType implements FindType, BackThreadBasedPatternPeer {
		private final FindVariable	parent_;
		private final String	nodeLayerType_;
		private final SpecificCommonErrorOutput	nodeLayerError_;

		private SLayer resultType_;
		private final String	childName_;
		private final BuilderVariableChainBlock preVariables_;
		public BackThreadFindType(FindVariable parent,String nodeLayerType,  String childName, SpecificCommonErrorOutput nodeLayerError) {
			this.parent_ = parent;
			this.nodeLayerType_ = nodeLayerType;
			this.childName_ = childName;
			this.nodeLayerError_ = nodeLayerError;
			this.preVariables_ = new BuilderVariableChainBlock(nodeLayerError);
		}
		public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock doBasicBuild(FoundationNode rootRecord,SLayer containingLayer) {
			if(true) { throw new RuntimeException("this doesn't work!"); }
			BaseTrackers.JavaVariablePath pathToVariables = new BaseTrackers.JavaVariablePath();
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase = null;
			//containingLayer.getBuilderVariable(name, pathToVariables, true);
			IRecordVariable lastVariable = this.preVariables_.resolveRecordVariableQuiet(containingLayer,pathToVariables,true);
			if(lastVariable==null) {
				return searchBlockBase;		
			}
			IRecordTypeTarget typeTarget = new IRecordTypeTarget();
			
			
			try {
				containingLayer = lastVariable.getAsFoundataionNodeReferenceVariable(pathToVariables);
				final SLayer baseLayer  = containingLayer.getNodeChild(this.nodeLayerType_,this.nodeLayerError_.getLineNumber());
				
				if(baseLayer!=null) {

					NodeLayerChild nodeLayerChild = baseLayer.getNodeLayerChild(childName_,this.nodeLayerError_.getLineNumber());
					
					this.resultType_ = nodeLayerChild.getBase();
					searchBlockBase = containingLayer.addSearchBlock(this.resultType_,nodeLayerError_.getLineNumber());
					nodeLayerChild.doAddAncestorTypeSearch(searchBlockBase, pathToVariables);
				}
			} catch (ParsingException e) {
				e.updateError(this.nodeLayerError_);
				return searchBlockBase;
			}
			return searchBlockBase;		
		}
		public SLayer doFinishBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase, FoundationNode rootRecord,SLayer containingLayer) {
			return this.resultType_;
		}
		public SLayer getResultType() { return resultType_; }
		public void end() { }

		public BuilderVariableChainPatternPeer addBuilderVariableChainForVariableChain() { return this.preVariables_; }
	}

//--------------------------------------------------------------------------
	private static final class ComplexThreadFindType implements FindType, ThreadBasedPatternPeer {
		private final FindVariable	parent_;
		private final String	nodeLayerType_;
		private final SpecificCommonErrorOutput	nodeLayerError_;

		private SLayer resultType_;
		private final SimpleVector<String> variableElements_ = new SimpleVector<String>();
		private final String	childName_;
		public ComplexThreadFindType(FindVariable parent,String nodeLayerType,  String childName, SpecificCommonErrorOutput nodeLayerError) {
			this.parent_ = parent;
			this.nodeLayerType_ = nodeLayerType;
			this.childName_ = childName;
			this.nodeLayerError_ = nodeLayerError;
		}
		public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock doBasicBuild(FoundationNode rootRecord,SLayer containingLayer) {
			BaseTrackers.JavaVariablePath pathToVariables = new BaseTrackers.JavaVariablePath();
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase = null;
			try {
				final SLayer baseLayer  = containingLayer.getAncestorNodeSLayer(this.nodeLayerType_, pathToVariables);
				
				SLayer variableLayer = baseLayer;
				for(String variable : this.variableElements_) {
					variableLayer = baseLayer.getVariableChild(variable,this.nodeLayerError_.getLineNumber());
					if(variableLayer==null) { break; }
				}
				if(variableLayer!=null) {

					NodeLayerChild nodeLayerChild = variableLayer.getNodeLayerChild(childName_,this.nodeLayerError_.getLineNumber());
					
					this.resultType_ = nodeLayerChild.getBase();
					searchBlockBase = containingLayer.addSearchBlock(this.resultType_,nodeLayerError_.getLineNumber());
					nodeLayerChild.doAddAncestorTypeSearch(searchBlockBase, pathToVariables);
					
				}
			} catch (ParsingException e) {
				e.updateError(this.nodeLayerError_);
				return searchBlockBase;
			}
			return searchBlockBase;		
		}
		public SLayer doFinishBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase, FoundationNode rootRecord,SLayer containingLayer) {
			return this.resultType_;
		}
		public SLayer getResultType() { return resultType_; }
		public void end() { }
		public void addVariableName(String value, int lineNumber, int characterNumber) {
			this.variableElements_.addElement(value);
		}
	}

//--------------------------------------------------------------------------
	private final static class ThreadFindType implements FindType {
		private final FindVariable	parent_;
		private final String	childName_;
		private final SpecificCommonErrorOutput	childError_;
		private final String	nodeLayerType_;
		private final SpecificCommonErrorOutput	nodeLayerError_;
		private final String	variableName_;
		private SLayer resultType_;
		public ThreadFindType(FindVariable parent,String nodeLayerType, String variableName, String childName, SpecificCommonErrorOutput nodeLayerError, SpecificCommonErrorOutput childError) {
			this.parent_ = parent;
			this.nodeLayerType_ = nodeLayerType;
			this.variableName_ = variableName;
			this.childName_ = childName;
			this.nodeLayerError_ = nodeLayerError;
			this.childError_ = childError;
		}

		public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock doBasicBuild(FoundationNode rootRecord,SLayer containingLayer) {
			BaseTrackers.JavaVariablePath pathToVariables = new BaseTrackers.JavaVariablePath();
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase = null;
			try {
				final SLayer baseLayer  = containingLayer.getAncestorNodeSLayer(this.nodeLayerType_, pathToVariables);
				SLayer variableLayer = baseLayer.getVariableChild(this.variableName_,this.nodeLayerError_.getLineNumber());
				if(variableLayer!=null) {
					NodeLayerChild nodeLayerChild = variableLayer.getNodeLayerChild(childName_,this.nodeLayerError_.getLineNumber());
					this.resultType_ = nodeLayerChild.getBase();
					searchBlockBase = containingLayer.addSearchBlock(this.resultType_,childError_.getLineNumber());
					nodeLayerChild.doAddAncestorTypeSearch(searchBlockBase, pathToVariables);
				}
			} catch (ParsingException e) {
				e.updateError(this.nodeLayerError_);
				return searchBlockBase;
			}
			return searchBlockBase;		
		}
		public SLayer doFinishBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock searchBlockBase, FoundationNode rootRecord,SLayer containingLayer) {
			return this.resultType_;
		}
		public SLayer getResultType() { return resultType_; }
	} // end of class ThreadFindType 
	// --------------------------------------------------------------------------
	public RootBasedPatternPeer addOptionRootBasedOfFindTargetForTarget() {
		RootFindType result = new RootFindType(this);
		this.type_ = result;
		return result;
	}

	public VariableBasedPatternPeer addOptionVariableBasedOfFindTargetForTarget(String childNameValue, int childNameLine, int childNameCharacter) {
		SpecificCommonErrorOutput childError = this.error_.createAdjusted(childNameLine, childNameCharacter);
		VariableFindType result = new VariableFindType(this,childNameValue,childError);
		this.type_ = result;
		return result;
	}
	public void addOptionLocalBasedOfFindTargetForTarget(String childNameValue, int childNameLine, int childNameCharacter) {
		SpecificCommonErrorOutput childError = this.error_.createAdjusted(childNameLine, childNameCharacter);
		LocalFindType result = new LocalFindType(this,childNameValue,childError);
		this.type_ = result;
		
	}
	public BackThreadBasedPatternPeer addOptionBackThreadBasedOfFindTargetForTarget(String nodeLayerNameValue, int nodeLayerNameLine, int nodeLayerNameCharacter, String childNameValue, int childNameLine, int childNameCharacter) {
		SpecificCommonErrorOutput nodeLayerError = this.error_.createAdjusted(nodeLayerNameLine, nodeLayerNameCharacter);
		BackThreadFindType result = new BackThreadFindType(this,nodeLayerNameValue,childNameValue,nodeLayerError);
		this.type_ = result;
		return result;
	}
	public void addOptionSimpleThreadBasedOfFindTargetForTarget(
			String nodeLayerNameValue, int nodeLayerNameLine,
			int nodeLayerNameCharacter, String childNameValue, int childNameLine,
			int childNameCharacter) {
		SpecificCommonErrorOutput childError = this.error_.createAdjusted(childNameLine, childNameCharacter);
		SpecificCommonErrorOutput nodeLayerError = this.error_.createAdjusted(nodeLayerNameLine, nodeLayerNameCharacter);
		SimpleThreadFindType result = new SimpleThreadFindType(this,nodeLayerNameValue,childNameValue,nodeLayerError,childError);
		this.type_ = result;
	}
	public void addOptionThreadBasedOfFindTargetForTarget(
			String nodeLayerNameValue, int nodeLayerNameLine,
			int nodeLayerNameCharacter, String variableNameValue,
			int variableNameLine, int variableNameCharacter, String childNameValue,
			int childNameLine, int childNameCharacter) {
		SpecificCommonErrorOutput childError = this.error_.createAdjusted(childNameLine, childNameCharacter);
		SpecificCommonErrorOutput nodeLayerError = this.error_.createAdjusted(nodeLayerNameLine, nodeLayerNameCharacter);
		ThreadFindType result = new ThreadFindType(this,nodeLayerNameValue,variableNameValue,childNameValue,nodeLayerError,childError);
		this.type_ = result;
	}
	public ThreadBasedPatternPeer addOptionThreadBasedOfFindTargetForTarget(String nodeLayerNameValue, int nodeLayerNameLine, int nodeLayerNameCharacter, String childNameValue, int childNameLine, int childNameCharacter) {
		SpecificCommonErrorOutput nodeLayerError = this.error_.createAdjusted(nodeLayerNameLine, nodeLayerNameCharacter);
		ComplexThreadFindType result = new ComplexThreadFindType(this,nodeLayerNameValue,childNameValue,nodeLayerError);
		this.type_ = result;
		return result;
	}
	int callCount_ = 0;
	public void basicBuild(FoundationNode rootRecord,SLayer containingLayer) {
		try {
			containingLayer.addBuilderVariable(this.variableName_, this);
		} catch (ItemAlreadyExistsException e) {
			e.updateError(this.error_);
		}
		this.searchBlockBaseSkeleton_ = this.type_.doBasicBuild(rootRecord, containingLayer);

		if(this.searchBlockBaseSkeleton_ !=null) {
			for(SearchQuery query : this.queries_) {
//				This needs a path thingy?
				query.basicBuild(this.searchBlockBaseSkeleton_,rootRecord);
			}
		}
//		if(this.variableChain_!=null) {
//			
//			
//			
//		} else {
//			this.resultType_ = this.designType_.getEndNodeQuiet(rootRecord);
//			if(this.resultType_!=null) {
//				this.searchBlockSkeleton_ = containingLayer.addSearchBlock(this.resultType_);
//			}
//			
//			if(this.resultType_ !=null) {
//				for(SearchQuery query : this.queries_) {
////					This needs a path thingy?
//					query.basicBuild(this.searchBlockSkeleton_,rootRecord);
//				}
//			}
//		}
	}
	public void finishBuild(FoundationNode rootRecord,SLayer containingLayer) {
//		this.type_.finalBuild();
//		for(GlobalsBlock child : this.children_) {
//			child.finishBuild(rootRecord);
//		}
		SLayer resultType = this.type_.doFinishBuild(this.searchBlockBaseSkeleton_, rootRecord, containingLayer);
		for(SearchQuery query : this.queries_) {
			query.finalBuild(this.searchBlockBaseSkeleton_,resultType,containingLayer);
		}
	}
	// ------- SetPatternPeer stuff -----------

	public void end() { }


	public SearchQueryPeer.Indirect getSearchQueryForQuery() { return this; }
	
	public SearchQueryPeer.MatchVariablePatternPeer createMatchVariable(String resultFieldParameter, int resultFieldLineNumber, int resultFieldCharacterNumber) {
		SpecificCommonErrorOutput queryError = this.error_.createAdjusted(resultFieldLineNumber, resultFieldCharacterNumber);
		SearchQueryVariable query = new SearchQueryVariable(resultFieldParameter, queryError);
		this.queries_.addElement(query);
		return query;
	}
	public void createMatchThis(String resultFieldValue, int resultFieldLine, int resultFieldCharacter) {
		SpecificCommonErrorOutput queryError = this.error_.createAdjusted(resultFieldLine, resultFieldCharacter);
		SearchQueryThis query = new SearchQueryThis(resultFieldValue, queryError);
		this.queries_.addElement(query);
	}
	// ==========================================================================

	private final static class TypeBlock implements DesignTypePatternPeer {
		private final SpecificCommonErrorOutput	error_;
		private TypeElement typeElements_= null;
		
		public TypeBlock(SpecificCommonErrorOutput error) {
			this.error_ = error;
		}
		// -------------------------------------------
		public void end() { }

		public void addTypeElement(String value, int lineNumber, int characterNumber) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(lineNumber, characterNumber);
			if(this.typeElements_==null) {
				this.typeElements_ = new TypeElement(value, error);
			} else {
				this.typeElements_.addElement(value, error);
			}
		}
		public FoundationNode getEndNodeQuiet(FoundationNode rootRecord) {
			return this.typeElements_.getEndNodeQuiet(rootRecord);
			
		}
//		public SLayer getEndLayerQuiet(SLayer baseLayer) {
//			return this.typeElements_.getEndLayerQuiet(baseLayer);
//			
//		}
	}
	// ==========================================================================

	private final static class TypeElement {
		private final SpecificCommonErrorOutput	error_;
		private final String	value_;
		private TypeElement next_ = null;
		public TypeElement(String value, SpecificCommonErrorOutput error) {
			this.value_ = value;
			this.error_ = error;
		}
		public FoundationNode getEndNodeQuiet(FoundationNode parent) {
			FoundationNode child = parent.getChildQuiet(this.value_);
			if(child==null) {
				error_.objectNotFoundError("node child", value_);
			} else if(this.next_!=null) {
				//Continue chain.
				child = this.next_.getEndNodeQuiet(child);
			}
			return child;
			
		}
		public SLayer getEndLayerQuiet(SLayer parent) {
			SLayer child;
			try {
				child = parent.getNodeChild(value_,this.error_.getLineNumber());
				if(this.next_!=null) { child = this.next_.getEndLayerQuiet(child); }
				return child;
			} catch (ParsingException e) {
				e.updateError(this.error_);
				return null;
			}
			
			
		}
		
		public void addElement(String value, SpecificCommonErrorOutput error) {
			if(this.next_!=null) {
				this.next_.addElement(value,error);
			} else {
				this.next_ = new TypeElement(value,error);
			}
		}
	}
	private static interface SearchQuery {
		public void basicBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock	searchBlockSkeleton, FoundationNode rootRecord);
		public void finalBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock	searchBlockSkeleton, SLayer resultType,SLayer containingLayer);
		
	}
	// ==========================================================================
	private final static class SearchQueryVariable implements SearchQuery, SearchQueryPeer.MatchVariablePatternPeer{
		private final String	resultFieldName_;
		private final SpecificCommonErrorOutput	error_;
		private final BuilderVariableChainBlock variableChain_;
		public SearchQueryVariable(String resultFieldName, SpecificCommonErrorOutput error) {
			this.resultFieldName_ = resultFieldName;
			this.error_ = error;
			this.variableChain_ = new BuilderVariableChainBlock(error);
		}
		// A----------------------------
		public void basicBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock	searchBlockSkeleton, FoundationNode rootRecord) {

		}
		public void finalBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock	searchBlockSkeleton, SLayer resultType,SLayer containingLayer) {
			BaseTrackers.JavaVariablePath pathToTestParameter = new BaseTrackers.JavaVariablePath();
			BaseTrackers.JavaVariablePath pathToQueryValueVariable = new BaseTrackers.JavaVariablePath();

			IRecordVariable queryValue =this.variableChain_.resolveRecordVariableQuiet(containingLayer, pathToQueryValueVariable,true);
			if(queryValue!=null) {
				try {
					
					IRecordVariable testField = resultType.getBuilderVariable(this.resultFieldName_,pathToTestParameter,false);
					IRecordTypeTarget.generateTarget(testField).doVariableQueryMatch(searchBlockSkeleton,pathToTestParameter, queryValue,pathToQueryValueVariable);					
				} catch(ParsingException e) {
					e.updateError(this.error_);
				}
			}
		}
		
		// ----------------------------
		// Austen Bits
		public void end() { }
		public BuilderVariableChainPatternPeer addBuilderVariableChainForVariable() {
			return variableChain_;
		}
	} // end of class SearchQueryVariable
	
//==========================================================================
	private final static class SearchQueryThis implements SearchQuery{
		private final String	resultFieldName_;
		private final SpecificCommonErrorOutput	error_;
		public SearchQueryThis(String resultFieldName, SpecificCommonErrorOutput error) {
			this.resultFieldName_ = resultFieldName;
			this.error_ = error;
		}
		// A----------------------------
		public void basicBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock	searchBlockSkeleton, FoundationNode rootRecord) {

		}
		public void finalBuild(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock	searchBlockSkeleton, SLayer resultType,SLayer containingLayer) {
			BaseTrackers.JavaVariablePath pathToFoundationLayer = new BaseTrackers.JavaVariablePath();
			FoundationNode myType = containingLayer.getRelatedFoundationNode(pathToFoundationLayer);
			
			BaseTrackers.JavaVariablePath pathToParameter = new BaseTrackers.JavaVariablePath();
			try {
				IRecordVariable testField = resultType.getBuilderVariable(this.resultFieldName_,pathToParameter,false);
				IRecordTypeTarget.generateTarget(testField).doRecordThisMatchAsQueryParameter(
						searchBlockSkeleton,pathToParameter,myType, pathToFoundationLayer);
			} catch(ParsingException e) {
				e.updateError(this.error_);
			}
			
		}
	} //end of class SearchQueryThis




	
}
