package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.ChainHookListPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.DeferredGlobalElement;
import org.ffd2.skeletonx.austenx.peg.base.GlobalElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.GlobalMethodDefinitionPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.LinkChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferencePatternPeer;
import org.ffd2.skeletonx.compile.java.ChainHookListBlock.BlockUser;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;

import org.ffd2.skeletonx.skeleton.ArgumentSetDetailsFormHolder.ArgumentSetDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.GlobalClassSourceFormHolder.GlobalClassSourceFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock.ImplementsNameDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.ChainDataBlock.LinkRefDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;

public  class GlobalsBlock implements GlobalElementPeer.Indirect{
	private final SpecificCommonErrorOutput	baseError_;
//
//	
	
	//private final SimpleVector<MethodRequirement> methods_ = new SimpleVector<GlobalsBlock.MethodRequirement>();
	
	private SLayer targetLayer_;
	private final SimpleVector<Element> elements_ = new SimpleVector<GlobalsBlock.Element>();
//	//The Root constructor
	public GlobalsBlock( SpecificCommonErrorOutput error) {
		this.baseError_ = error;
//		this.adder_ = new Adder(this);
	}
	
	public GlobalElementPeer.LinkPatternPeer createLink(int keywordLineNumber, int keywordCharacterNumber) {
		SpecificCommonErrorOutput error = this.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		LinkElement le = new LinkElement(error);
		this.elements_.addElement(le);
		return le;
	}
	public GlobalElementPeer.MethodPatternPeer createMethod() {
		return new GlobalElementPeer.MethodPatternPeer() {

			public void end() { }

			public GlobalMethodDefinitionPatternPeer addGlobalMethodDefinitionForDefinition( String nameParameter, int nameLineNumber, int nameCharacterNumber) {
				
				SpecificCommonErrorOutput error = baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
				MethodElement me = new MethodElement(nameParameter,error);
				elements_.addElement(me);
				return me;
			}

		};
	}
	public boolean basicBuild(SLayer currentRecord, GlobalsClass globalClass) {
		this.targetLayer_ = currentRecord;
		boolean success = true;
		for(Element element : this.elements_) {
			success &= element.basicBuild(currentRecord,globalClass);
		}
		return success;
		
	}
	public void finishBuild( GlobalsClass globalClass) {
		assert this.targetLayer_ !=null: "Should be non null if we get to the finish pass";
		for(Element element : this.elements_) {
			element.finishBuild(this.targetLayer_,globalClass);
		}
		
	}
//==========================================================================

	
//	public static final GlobalsBlock createRoot(String className, SpecificCommonErrorOutput error) {
//		return new GlobalsBlock(new ClassRequirement(className,error),error);
//	}
	// ==========================================================================
	// Austen
//	private final static class Adder implements GlobalElementPeer.Indirect,skeletonx.austenx.peg.base.GlobalElementPeerBase.MethodPatternPeer {
//		private final GlobalsBlock	parent_;
//
//		public Adder(GlobalsBlock parent) {
//			this.parent_ = parent;
//			
//		}
//		public LinkPatternPeer createLink(int keywordLineNumber, int keywordCharacterNumber) {
//			SpecificCommonErrorOutput error = this.parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
//			LinkElement le = new LinkElement(new GlobalsBlock(this.parent_.details_,false, error),error);
//			this.parent_.elements_.addElement(le);
//			return le;
//		}
//
//		public skeletonx.austenx.peg.base.GlobalElementPeerBase.MethodPatternPeer createMethod() {
//			return this;
//		}
//		public void end() { /* Don't use */}
//		public GlobalMethodDefinitionPatternPeer addGlobalMethodDefinitionForDefinition( String nameParameter, int nameLineNumber, int nameCharacterNumber) {
//			SpecificCommonErrorOutput error = this.parent_.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
//			MethodElement me = new MethodElement(nameParameter,error);
//			this.parent_.elements_.addElement(me);
//			return me;
//		}
//		
//		
//	};
//	private final Adder adder_;
//	public GlobalElementPeer.Indirect getGlobalElementForElements() {
//		return adder_;
//	}
	public void end() {}
	
	// ==========================================================================
	private static interface Element {
		//Return true if success
		public boolean basicBuild(SLayer currentRecord, GlobalsClass globalClass);
		public void finishBuild(SLayer currentRecord, GlobalsClass globalClass);

	}
//--------------------------------------------------------------------------BlockUser
	private static final class MethodElement implements Element,GlobalMethodDefinitionPatternPeer, GlobalMethodReference,JavaConcreteChainLink {
		private final String	methodName_;
		private final SpecificCommonErrorOutput	error_;
		private TargetTypeBlock returnType_;
		private JavaImplementationFormBlock.BlockDataBlock.GlobalMethodVariableDataBlock	globalMethodStoreSkeleton_;
		private ChainHookListBlock possibleChainHookList_ = null;
		private boolean isStatic_ = false;
		public MethodElement(String methodName, SpecificCommonErrorOutput error) {
			this.methodName_ = methodName;
			this.error_ = error;
		}

		public boolean basicBuild(final SLayer currentRecord, GlobalsClass globalClass) {

			try {
				this.globalMethodStoreSkeleton_ = currentRecord.addGlobalMethodReference( this.methodName_, globalClass,this);
			} catch (ItemAlreadyExistsException e) {
				error_.repeatedObjectError("global method", this.methodName_);
			}
			this.returnType_.basicBuild(currentRecord);
			if(this.possibleChainHookList_!=null) {
      	this.possibleChainHookList_.requestUpdate(new BlockUser() {

					public void newLink(String chainName, String linkName, SpecificCommonErrorOutput error) {
						try {
							currentRecord.addConcreteChainLink(chainName, linkName, MethodElement.this, error);
						} catch(ParsingException e) {
							e.updateError(error);
							e.updateError(error_);
						}
					}
      		
      	});
      }
			return true;
		}

		public void finishBuild(SLayer currentRecord, GlobalsClass globalClass) { } 
		// ====== Austen stuff ===========

		public void end() { 	}

		public void addIsStatic() { this.isStatic_ = true; Debug.finishMeMarker(); }

		public TargetTypeReferencePatternPeer addTargetTypeReferenceForReturnType( String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
			SpecificCommonErrorOutput  error = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
			return this.returnType_ = new TargetTypeBlock(typeNameParameter, error);
		}
		// TargetMethod stuff
		// ==================

		public ArgumentSetDetailsFormBlock createExpressionFinal(ExpressionDetailsFormBlock target, JavaVariablePath basePathDisposable) {
			BaseBuilder rootBuilder = target.getRootBuilder();
			BasicCallChainFormBlock callChainBase = rootBuilder.createBasicCallChainChild();
			
			ArgumentSetDetailsFormBlock argumentSetSkeleton = rootBuilder.createArgumentSetDetailsChild();
			target.addCallChain(callChainBase).getBasicChainParameter().addElement().addMethod(this.globalMethodStoreSkeleton_.getVariableStoreParameter(), basePathDisposable,argumentSetSkeleton);
			return argumentSetSkeleton;
		}
		public ArgumentSetDetailsFormBlock createCallExpressionFinal(BasicCallChainFormBlock target, JavaVariablePath basePathDisposable) {
			BaseBuilder rootBuilder = target.getRootBuilder();

			return target.addElement().addMethod(this.globalMethodStoreSkeleton_.getVariableStoreParameter(), basePathDisposable,rootBuilder.createArgumentSetDetailsChild()).getMethodArgumentDetailsParameter();
		}
		public TargetTypeReference getReturnTargetTypeReferenceQuietFinal() {
			return returnType_.getResolvedTypeReferenceQuietFinal();
		}

		public GlobalMethodReference getAsGlobalQuiet() { return this; }

		public boolean isStatic() { return this.isStatic_; }
	// =====
//	// Global stuff
		public void updateSetGlobalFinal(MethodDataBlock source, JavaVariablePath pathToTarget) {
			source.addSetGlobal(this.globalMethodStoreSkeleton_, pathToTarget);
		}

		public ImplementsNameDataBlock addMethodImplementsNameBlock( MethodDataBlock target, JavaVariablePath methodDeclarationPathChainTracker) throws ParsingException {
			return target.addImplementsName(this.globalMethodStoreSkeleton_.getVariableStoreParameter(), methodDeclarationPathChainTracker);
		}

		public ChainHookListPatternPeer addChainHookListForChainHookList() {
			return this.possibleChainHookList_ = new ChainHookListBlock(this.error_);
		}

		public void updateChainLink(LinkRefDataBlock targetSkeleton) {
    	targetSkeleton.addGlobalMethod(this.globalMethodStoreSkeleton_);			
		}




	}
	// --------------------------------------------------------------------------
	private final static class LinkedBuild {
		private final LinkedBuild	previous_;
		private final SLayer	linkedLayer_;
		private final GlobalsBlock	relatedBlock_;
	

		public LinkedBuild(SLayer linkedLayer,GlobalsBlock	relatedBlock,LinkedBuild previous) {
			this.linkedLayer_ = linkedLayer;
			this.relatedBlock_ = relatedBlock;
			this.previous_ = previous;
			
		}
		public void finishBuild(SLayer currentRecord, GlobalsClass globalClass) {
			if(this.previous_!=null) {
				this.previous_.finishBuild(currentRecord, globalClass);
			}
			this.relatedBlock_.finishBuild(globalClass);			
		}
	}
	private static final class LinkElement implements Element,GlobalElementPeer.LinkPatternPeer {

		private final SpecificCommonErrorOutput	error_;
		private LinkChainBlock lastLinkChain_;
		private LinkedBuild lastLinkedBuild_ = null;
		private final SimpleVector<DeferredGlobalElement> deferredElements_ = new SimpleVector<DeferredGlobalElement>();
		public LinkElement(SpecificCommonErrorOutput error) {
			this.error_ = error;
			
			
		}
		//=========================================================================
		// --- Austen Stuff ---
		public void end() {}


		// ========================================================================
		private final GlobalsBlock createSubGlobalBlock() {
			GlobalsBlock result = new GlobalsBlock(this.error_);
			for(DeferredGlobalElement de : this.deferredElements_) {
				de.buildGlobalElement(result);
			}
			return result;
		}
		public boolean basicBuild(SLayer currentRecord, GlobalsClass globalClass) {
			LinkChainBlock currentLinkChain = lastLinkChain_;
			boolean okay = true;
			while(currentLinkChain!=null) {
				SLayer linkedLayer = currentLinkChain.getLinkChainEndContainerQuiet(currentRecord);
				GlobalsBlock relatedBlock = createSubGlobalBlock();
				if(linkedLayer!=null) {
					okay&=relatedBlock.basicBuild(linkedLayer,globalClass);
					this.lastLinkedBuild_ = new LinkedBuild(linkedLayer,relatedBlock,this.lastLinkedBuild_);

				} else {
					okay = false;
				}
				currentLinkChain = currentLinkChain.getPrevious();
			}
			return okay;
			
		}

		public void finishBuild(SLayer currentRecord, GlobalsClass globalClass) {
			if(this.lastLinkedBuild_!=null) {
				this.lastLinkedBuild_.finishBuild(currentRecord, globalClass);
			}
		}
		public LinkChainPatternPeer addLinkChainForSingleChain() {
			return this.lastLinkChain_ = new LinkChainBlock(this.error_,this.lastLinkChain_);
		}
		public void setGlobalElementForElement(DeferredGlobalElement value) {
			deferredElements_.addElement(value);
		}
	}
	
	
	// ======================================================
//private final SpecificCommonErrorOutput	error_;
//private final ClassRequirement_	classReq;
//private final ChildAccess childAccess_; //If null means we don't move to a child block
//private final SimpleVector<MethodRequirement> methods_ = new SimpleVector<GlobalsBlock.MethodRequirement>();
//private SLayer targetRecord_;
//public InsideClassType(ChildAccess childAccess, ClassRequirement classReq, SpecificCommonErrorOutput error) {
//	this.childAccess_ = childAccess;
//	this.classReq_ = classReq;
//	this.error_ = error;
//}
//public InsideClassType(ClassRequirement classReq, SpecificCommonErrorOutput error) {
//	this(null,classReq,error);
//}
//public MethodPatternPeer addMethod(String methodName, SpecificCommonErrorOutput error) throws ParsingException {
//	MethodRequirement req = new MethodRequirement(methodName, error);
//	this.methods_.addElement(req);
//	return req;
//}
//public BlockType createChildType(String childName,SpecificCommonErrorOutput error) {
//	return new InsideClassType(childName,this.classReq_,error);
//}
//public final void assignToParent(SLayer parentRecord, SimpleVector<GlobalsBlock> children) {
//	SLayer targetRecord = childAccess_.getChild(parentRecord);
////	if(this.blockChildName_==null ) {
////		this.classReq_.addToRecord(parentRecord);
////		targetRecord = parentRecord; //We don't change record
////	} else {
////		try {
////			targetRecord = parentRecord.getChild(this.blockChildName_);
////		} catch (ItemNotFoundException e) {
////			e.updateError(error_);
////		}
////	}
//	if(targetRecord!=null) {
//		this.targetRecord_ = targetRecord;
//		for(MethodRequirement req : this.methods_) {
//			req.assignToRecord(targetRecord,this.classReq_);
//		}
//		for(GlobalsBlock child : children) {
//			child.assignRecords(targetRecord);
//		}
//	}
//}
////public final void assignToParent(ImplementationRecord parentRecord, SimpleVector<SpecificsBlock> children) {
////  ImplementationRecord targetRecord = null;
////	if(this.blockChildName_==null ) {
////		this.classReq_.addToRecord(parentRecord);
////		targetRecord = parentRecord; //We don't change record
////	} else {
////		try {
////			targetRecord = parentRecord.getChild(this.blockChildName_);
////		} catch (ItemNotFoundException e) {
////			e.updateError(error_);
////		}
////	}
////	if(targetRecord!=null) {
////		this.targetRecord_ = targetRecord;
////		for(MethodRequirement req : this.methods_) {
////			req.assignToRecord(targetRecord,this.classReq_);
////		}
////		for(SpecificsBlock child : children) {
////			child.assignRecords(targetRecord);
////		}
////	}
////	
////}
//public BlockType getWithAssignedClass(ClassRequirement classReq, SpecificCommonErrorOutput error) throws ParsingException {
//	throw ParsingException.createGeneralSyntax("Class within class not allowed in details");
//}
//
//public void basicBuild() {
//	this.childAccess_.basicBuild(this.classReq_,this.targetRecord_);
//	
//	for(MethodRequirement req : this.methods_) {
//		req.basicBuild(this.targetRecord_,this.classReq_);
//	}
//}
//public void finalBuild() {
//	if(this.blockChildName_==null) {
//		this.classReq_.finalBuild();
//	}
//	for(MethodRequirement req : this.methods_) {
//		req.finalBuild(this.targetRecord_,this.classReq_);
//	}
//}
//
	
//	public SpecificsBlockElementPeer.Indirect getSpecificsBlockElementForElements() {
//		return this;
//	}
//	
//	public void assignRecords(SLayer rootRecord) {
//		this.type_.assignToParent(rootRecord, this.children_);
//	}
//	
//	public void end() { }
//
////	public SpecificsBlock getChild(String childName,SpecificCommonErrorOutput error)  {
////		if(this.childMappings_==null) {
////			this.childMappings_ = new HashMap<String, SpecificsBlock>();
////			SpecificsBlock child = new SpecificsBlock(new OutsideClassType(childName),this.environment_,error);
////			this.childMappings_.put(childName,child);
////			return child;
////		} else {
////			SpecificsBlock child = this.childMappings_.get(childName);
////			if(child==null) {
////				child = new SpecificsBlock(childName, this.environment_,error);
////				this.childMappings_.put(childName,child);
////			} else {
////				child.accessErrors_.addError(error);
////			}
////			return child;
////		} 
////	}
//	private final GlobalsBlock addChild(BlockType type, SpecificCommonErrorOutput error ) {
//		GlobalsBlock child = new GlobalsBlock(type,this.environment_,error);
//		this.children_.add(child);
//		return child;
//	}
//	public GlobalsBlock addTopLevelChild(String childName, SpecificCommonErrorOutput error) {
//		return addChild(new OutsideClassType(childName,error),error);
//	}
//	
//	public MethodPatternPeer createMethod(String nameParameter,
//			int nameLineNumber, int nameCharacterNumber) {
//		SpecificCommonErrorOutput error = this.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
//		try {
//			return this.type_.addMethod(nameParameter,error);
//		} catch(ParsingException e) {
//			e.updateError(error);
//			return null;
//		}
//	}
//
//	public SubPatternPeer createSub() {
//		return new SubPatternPeer() {
//			public void end() { }
//			public JavaImplementationSpecificsBlockPatternPeer 
//				addJavaImplementationSpecificsBlockForSubBlock(
//					String blockNameParameter, int blockNameLineNumber, int blockNameCharacterNumber
//				) {
//				SpecificCommonErrorOutput error = baseError_.createAdjusted(blockNameLineNumber, blockNameCharacterNumber);
//				return addChild(type_.createChildType(blockNameParameter,error),error);
//			}
//			
//		};
//	}
//	public ClassReferencePatternPeer createClassReference( String classNameParameter, int classNameLineNumber, int classNameCharacterNumber) {
//		SpecificCommonErrorOutput error = this.baseError_.createAdjusted(classNameLineNumber, classNameCharacterNumber);
//		ClassRequirement requirement = new ClassRequirement(classNameParameter, error);
//		try {
//			final GlobalsBlock child = addChild(this.type_.getWithAssignedClass(requirement,error), error);
//			return new ClassReferencePatternPeer() {
//				public void end() {			}
//				public SpecificsBlockElementPeer.Indirect getSpecificsBlockElementForElements() { return child; }
//			};
//		} catch(ParsingException e) {
//			e.updateError(error);
//			return null;
//		}
//		
//	}
//
//
////	// ==========================================================================
////	private final static class SetVariable implements SetPatternPeer,SearchQueryPeer.Indirect{ 
////		private final String	variableName_;
////		private final SpecificCommonErrorOutput	error_;
////		private final TypeBlock designType_;
////		private final SimpleVector<SearchQuery> queries_ = new SimpleVector<SearchQuery>();
////		public SetVariable(String variableName, GlobalEnvironment environment, SpecificCommonErrorOutput error) {
////			this.variableName_ = variableName;
////			this.error_ = error;
////			this.designType_ = new TypeBlock( error);
////		}
////		public void basicBuild(FoundationNode rootRecord) {
////			this.designType_.basicBuild(rootRecord);
////			for(SearchQuery query : this.queries_) {
////				query.basicBuild(rootRecord);
////			}
////		}
////		public void finishBuild(FoundationNode rootRecord) {
//////			this.type_.finalBuild();
//////			for(GlobalsBlock child : this.children_) {
//////				child.finishBuild(rootRecord);
//////			}
////			for(SearchQuery query : this.queries_) {
////				query.finalBuild(rootRecord);
////			}
////		}
////		// ------- SetPatternPeer stuff -----------
////
////		public void end() { }
////
////		public DesignTypePatternPeer addDesignTypeForResultType() { return this.designType_; }
////
////		public SearchQueryPeer.Indirect getSearchQueryForQuery() { return this; }
////		
////		public MatchVariablePatternPeer createMatchVariable(String resultFieldParameter, int resultFieldLineNumber, int resultFieldCharacterNumber) {
////			SpecificCommonErrorOutput queryError = this.error_.createAdjusted(resultFieldLineNumber, resultFieldCharacterNumber);
////			SearchQuery query = new SearchQuery(resultFieldParameter, queryError);
////			this.queries_.addElement(query);
////			return query;
////		}
////		
////	}
////	private final static class SearchQuery implements MatchVariablePatternPeer{
////		private final String	resultFieldName_;
////		private final SpecificCommonErrorOutput	error_;
////		private final BuilderVariableChainBlock variableChain_;
////		public SearchQuery(String resultFieldName, SpecificCommonErrorOutput error) {
////			this.resultFieldName_ = resultFieldName;
////			this.error_ = error;
////			this.variableChain_ = new BuilderVariableChainBlock(error);
////		}
////		// ----------------------------
////		public void basicBuild(FoundationNode rootRecord) {
////		}
////		public void finalBuild(FoundationNode rootRecord) {
////		}
////		
////		// ----------------------------
////		public void end() { }
////		public BuilderVariableChainPatternPeer addBuilderVariableChainForVariable() {
////			return variableChain_;
////		}
////		
////		
////	}
////	
////	
	// ==========================================================================
	/**
	 * Class holding informatino of Target Class requirement variables in the implementation 
	 * specifications. 
	 */
	private final static class ClassRequirement implements TargetType,GlobalClassReference{
		private final SpecificCommonErrorOutput	error_;
		private final String	className_;
		private JavaImplementationFormBlock.BlockDataBlock.GlobalClassVariableDataBlock	classVariableSkeleton_;

		public ClassRequirement(String className, SpecificCommonErrorOutput error ) {
			this.className_ = className;
			this.error_ = error;
		}
		public void updateSetGlobalFinal(GlobalClassSourceFormBlock source, JavaVariablePath pathToTarget) {
			 source.addSetGlobal(this.classVariableSkeleton_, pathToTarget);
			
		}
		public boolean isSubTypeOf(TargetType parentType) {
			//TODO:
			return parentType==this;
		}
		
		public void addToRecord(SLayer rootRecord) {
			try {
				this.classVariableSkeleton_ = rootRecord.addGlobalClassReference(className_, this);
			} catch(ItemAlreadyExistsException e) {
				error_.repeatedObjectError("class global reference", className_);
			}
			
		}
		public void basicBuild(SLayer rootRecord) {
		}
		public void finalBuild() { }
		
		// ========================================================================
		// TARGET TYPE STUFF

	  public void buildCodeSetType(TypeSetterDetailsFormBlock target,JavaVariablePath disposableTargetPath) {
	  	target.addFromClass(this.classVariableSkeleton_.getVariableStoreParameter(), disposableTargetPath);
	  }
	  public String getOutputName() { return className_; }
		
		public DirectTargetElementAccess getDirectElementAccessQuiet() {
			//No direct access - internal class
			return null;
		}
		public TargetType specifyGenericTypes(TargetTypeWithPath[] types) throws ParsingException {
  		throw ParsingException.createGeneralSyntax("Internal types cannot take generic types");
		}
		public TargetType getArrayed(int dimensions) throws ParsingException {
			return new ArrayedTargetType(this,dimensions);
		}

	
		

	}
//
//	// ==========================================================================
	private static interface BlockType {
		public BlockType createChildType(String childName,SpecificCommonErrorOutput error);
		//public  void assignToParent(ImplementationRecord parentRecord, SimpleVector<SpecificsBlock> children);
		public  void assignToParent(SLayer parentRecord, SimpleVector<GlobalsBlock> children);

		public BlockType getWithAssignedClass(ClassRequirement classReq, SpecificCommonErrorOutput error) throws ParsingException;
		public GlobalElementPeer.MethodPatternPeer addMethod(String methodName, SpecificCommonErrorOutput error) throws ParsingException;
		
		public void basicBuild();
		public void finalBuild();
	}
	// ----------------------------------
//	private static final class RootType implements BlockType {
//		public BlockType createChildType(String childName,SpecificCommonErrorOutput error) {
//			return new OutsideClassType(childName,error);
//		}
//		
//		public final void assignToParent(SLayer parentRecord, SimpleVector<GlobalsBlock> children) {
//			for(GlobalsBlock child : children) {
//				child.assignRecords(parentRecord);
//			}
//
//		}
//		public MethodPatternPeer addMethod(String methodName, SpecificCommonErrorOutput error) throws ParsingException {
//			throw ParsingException.createGeneralSyntax("Illegal method outside of class");
//		}
//
//		public BlockType getWithAssignedClass(ClassRequirement classReq, SpecificCommonErrorOutput error) {
//			return new InsideClassType( classReq, error);
//		}
//		public void basicBuild() { }
//		public void finalBuild() { }
//	}
//
//	private static final class OutsideClassType implements BlockType {
//		private SLayer myRecord_;
//		private final SpecificCommonErrorOutput	error_;
//		private final ChildAccess	childAccess_;
//
//		public OutsideClassType(ChildAccess childAccess,SpecificCommonErrorOutput error) {
//			this.childAccess_ = childAccess;
//			this.error_ = error;
//			
//		}
//		public MethodPatternPeer addMethod(String methodName, SpecificCommonErrorOutput error) throws ParsingException {
//			throw ParsingException.createGeneralSyntax("Illegal method outside of class");
//		}
//		public BlockType createChildType(String childName,SpecificCommonErrorOutput error) {
//			return new OutsideClassType(new SubRecordAccess(childName),error);
//		}
////		public final void assignToParent(ImplementationRecord parentRecord, SimpleVector<SpecificsBlock> children) {
////			try {
////				myRecord_ = parentRecord.getChild(this.blockName_);
////				for(SpecificsBlock child : children) {
////					child.assignRecords(myRecord_);
////				}
////
////			} catch(ItemNotFoundException e) {
////				error_.objectNotFoundError("design block", this.blockName_);
////			}
////		}
//		public final void assignToParent(SLayer parentRecord, SimpleVector<GlobalsBlock> children) {
//			try {
//				myRecord_ = this.childAccess_.getChild(parentRecord);
//				//.getChild(this.blockName_);
//				for(GlobalsBlock child : children) {
//					child.assignRecords(myRecord_);
//				}
//
//			} catch(ItemNotFoundException e) {
//				error_.objectNotFoundError("design block", this.blockName_);
//			}
//		}
//		public BlockType getWithAssignedClass(ClassRequirement classReq, SpecificCommonErrorOutput error) {
//			return new InsideClassType( classReq, error);
//		}
//		public void basicBuild() { }
//		public void finalBuild() { }
//		
//	}
//	private static interface ChildAccess {
//		public SLayer getChild(SLayer parentLayer, ClassRequirement classReq) throws ParsingException;
//		public void basicBuild(ClassRequirement req, SLayer childLayer);
//		public String getDescriptor();
//	}
//	
//	private static final class SubRecordAccess implements ChildAccess {
//		private final String	recordName_;
//
//		public SubRecordAccess(String recordName) {
//			this.recordName_ = recordName;
//			
//		}
//		public String getDescriptor() { return "Record:"+recordName_; }
//		
//		public SLayer getChild(SLayer parentLayer, ClassRequirement classRequirement) throws ParsingException {
//			return parentLayer.getNodeChild(this.recordName_);
//		}
//		public void basicBuild(ClassRequirement classReq) {
//			
//		}
//	}
//	private static final class VariableAccess implements ChildAccess {
//		private final String	variableName_;
//
//		public VariableAccess(String variableName) {
//			this.variableName_ = variableName;
//			
//		}
//		public String getDescriptor() { return "Variable:"+variableName_; }
//
//	}
//	private final static class NoMoveChildAccess implements ChildAccess {
//
//		public String getDescriptor() { return "No child"; }
//
//		public SLayer getChild(SLayer parentLayer, ClassRequirement classReq) throws ParsingException {
//			classReq.addToRecord(parentLayer);
//			return parentLayer; //We don't change record
//		}
//
//		public void basicBuild(ClassRequirement classReq, SLayer childlayer) {
//			classReq.basicBuild(childlayer);
//		}
//	}
//	// ===================
//	private static final class InsideClassType implements BlockType {
//		private final SpecificCommonErrorOutput	error_;
//		private final ClassRequirement	classReq_;
//		private final ChildAccess childAccess_; //If null means we don't move to a child block
//		private final SimpleVector<MethodRequirement> methods_ = new SimpleVector<GlobalsBlock.MethodRequirement>();
//		private SLayer targetRecord_;
//		public InsideClassType(ChildAccess childAccess, ClassRequirement classReq, SpecificCommonErrorOutput error) {
//			this.childAccess_ = childAccess;
//			this.classReq_ = classReq;
//			this.error_ = error;
//		}
//		public InsideClassType(ClassRequirement classReq, SpecificCommonErrorOutput error) {
//			this(null,classReq,error);
//		}
//		public MethodPatternPeer addMethod(String methodName, SpecificCommonErrorOutput error) throws ParsingException {
//			MethodRequirement req = new MethodRequirement(methodName, error);
//			this.methods_.addElement(req);
//			return req;
//		}
//		public BlockType createChildType(String childName,SpecificCommonErrorOutput error) {
//			return new InsideClassType(childName,this.classReq_,error);
//		}
//		public final void assignToParent(SLayer parentRecord, SimpleVector<GlobalsBlock> children) {
//			SLayer targetRecord = childAccess_.getChild(parentRecord);
////			if(this.blockChildName_==null ) {
////				this.classReq_.addToRecord(parentRecord);
////				targetRecord = parentRecord; //We don't change record
////			} else {
////				try {
////					targetRecord = parentRecord.getChild(this.blockChildName_);
////				} catch (ItemNotFoundException e) {
////					e.updateError(error_);
////				}
////			}
//			if(targetRecord!=null) {
//				this.targetRecord_ = targetRecord;
//				for(MethodRequirement req : this.methods_) {
//					req.assignToRecord(targetRecord,this.classReq_);
//				}
//				for(GlobalsBlock child : children) {
//					child.assignRecords(targetRecord);
//				}
//			}
//		}
////		public final void assignToParent(ImplementationRecord parentRecord, SimpleVector<SpecificsBlock> children) {
////		  ImplementationRecord targetRecord = null;
////			if(this.blockChildName_==null ) {
////				this.classReq_.addToRecord(parentRecord);
////				targetRecord = parentRecord; //We don't change record
////			} else {
////				try {
////					targetRecord = parentRecord.getChild(this.blockChildName_);
////				} catch (ItemNotFoundException e) {
////					e.updateError(error_);
////				}
////			}
////			if(targetRecord!=null) {
////				this.targetRecord_ = targetRecord;
////				for(MethodRequirement req : this.methods_) {
////					req.assignToRecord(targetRecord,this.classReq_);
////				}
////				for(SpecificsBlock child : children) {
////					child.assignRecords(targetRecord);
////				}
////			}
////			
////		}
//		public BlockType getWithAssignedClass(ClassRequirement classReq, SpecificCommonErrorOutput error) throws ParsingException {
//			throw ParsingException.createGeneralSyntax("Class within class not allowed in details");
//		}
//		
//		public void basicBuild() {
//			this.childAccess_.basicBuild(this.classReq_,this.targetRecord_);
//			
//			for(MethodRequirement req : this.methods_) {
//				req.basicBuild(this.targetRecord_,this.classReq_);
//			}
//		}
//		public void finalBuild() {
//			if(this.blockChildName_==null) {
//				this.classReq_.finalBuild();
//			}
//			for(MethodRequirement req : this.methods_) {
//				req.finalBuild(this.targetRecord_,this.classReq_);
//			}
//		}
//		
//	}
//	private final static class MethodRequirement implements MethodPatternPeer, TargetMethod, GlobalMethodReference {
//		private final SpecificCommonErrorOutput	error_;
//		private final String	methodName_;
//		private  TargetTypeBlock returnType_;
//		private GlobalMethodVariableDataBlock	globalMethodStoreSkeleton_;
//		private boolean isStatic_ = false;
//		public MethodRequirement(String methodName,SpecificCommonErrorOutput error) {
//			this.methodName_ = methodName;
//
//			this.error_ = error;
//			
//		}
//		public final void assignToRecord(SLayer record, ClassRequirement containingClass) {
//			try {
//				this.globalMethodStoreSkeleton_ = record.addGlobalMethodReference(this.methodName_,containingClass,  this);
//			} catch(ItemAlreadyExistsException e) {
//				error_.repeatedObjectError("method", this.methodName_);
//			}
//		}
//		public void basicBuild(SLayer record, ClassRequirement containingClass) {
//			this.returnType_.basicBuild(record);
////			this.globalMethodStoreSkeleton_ = record.getDesignLinkedSkeleton().getParent().addGlobalMethodVariableBlock(this.methodName_);
//			
//		}
//		public void finalBuild(SLayer record, ClassRequirement containingClass) {
//		}
//		// ====
//		// MethodPatternPeer stuff
//
//		public void end() { }
//
//		public TargetTypeReferencePatternPeer addTargetTypeReferenceForReturnType(
//				String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
//			SpecificCommonErrorOutput error = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
//			return this.returnType_ = new TargetTypeBlock(typeNameParameter, error);
//		}
//		// ==========
//		// Target method stuff
//		public TargetTypeReference getReturnTargetTypeReferenceQuietFinal() {
//			return this.returnType_.getResolvedTypeReferenceQuietFinal();
//		}
//		public ArgumentSetCodeCode createExpressionFinal(ExpressionCode target, JavaVariablePath basePathDisposable) {
//			return target.addCallChainBlock().addElementBlock().addMethodBlock(this.globalMethodStoreSkeleton_, basePathDisposable).getCodeFixtureSetArguments();
//		}
//		public ArgumentSetCodeCode createCallExpressionFinal( BasicCallChainTemplate target, JavaVariablePath basePathDisposable) {
//			return target.addElementBlock().addMethodBlock(this.globalMethodStoreSkeleton_, basePathDisposable).getSetArgumentsTargetFixture();
//		}
//		public GlobalMethodReference getAsGlobalQuiet() { return this; }
//		// =====
//		// Global stuff
//		public void updateSetGlobalFinal(MethodDataBlock source, JavaVariablePath pathToTarget) {
//			source.addSetGlobalBlock(this.globalMethodStoreSkeleton_, pathToTarget);
//		}
//		public boolean isStatic() { return this.isStatic_; }
//		public void addIsStatic() { this.isStatic_ = true; }
//		public ImplementsNameDataBlock addMethodImplementsNameBlock( MethodDataBlock target, JavaVariablePath methodDeclarationPathChainTracker) throws ParsingException {
//			return target.addImplementsNameBlock(this.globalMethodStoreSkeleton_, methodDeclarationPathChainTracker);
//		}
//	}
	
}
