package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.skeleton.BExpressionAccessFormHolder;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder;
import org.ffd2.skeletonx.skeleton.BaseBuilder.CodeBlockMacroBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock;

import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;

public final class IRecordTypeTarget {
	public static final String CODE_REFERENCE_NAME = "code reference";
	public static final String FOUNDATION_RECORD_REFERENCE_NAME = "foundation record reference";
	public static final String EXPRESSION_SETTER_NAME = "expression setter";

	public static final String STRING_REFERENCE_NAME = "String target variable reference";

	public static final String ARG_TYPES_NAME =  "arg types";
	public static final String ARG_VALUES_NAME = "arg values";
	public static final String MAPPING_REFERENCE_NAME = "mapping reference";
	public static final String KEY_REFERENCE_NAME = "key reference";
	private Type currentType_ = NULL_TYPE;
	
	private final static Type NULL_TYPE = new AbstractType("non-reference") {
	};
	
	public IRecordTypeTarget() {
	
	}
	//public void setVariableQueryMatch(JavaImplementationForm.BlockDataBlock.SearchRecordDataBlock targetSkeleton, IRecordVariable queryMatchValue, BaseTrackers.JavaVariablePath variablePathToQueryMatch) throws ParsingException;
	public void setNotReference() {
		this.currentType_ = NULL_TYPE;
	}
	public void setStringReference(final JavaParameterSettingsFormBlock.ParameterDataBlock.StringDataBlock parameterSkeleton) {
		this.currentType_ = new AbstractType(STRING_REFERENCE_NAME) {

			@Override public void doVariableQueryMatch(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, BaseTrackers.JavaVariablePath  variablePathToTestValueFromSearchLayer, IRecordVariable queryMatchValue, BaseTrackers.JavaVariablePath variablePathToQueryMatch) throws ParsingException {
				IRecordTypeTarget.generateTarget(queryMatchValue).doVariableQueryMatchAsQueryValue(parameterSkeleton, variablePathToTestValueFromSearchLayer,targetSkeleton, variablePathToQueryMatch);
			  
			//	value.doVariableQueryMatchAsQueryParameter(targetSkeleton,  queryMatchValue, variablePathToQueryMatch);
			}
			@Override		public void doVariableQueryMatchAsQueryValue( 
					JavaParameterSettingsFormBlock.ParameterDataBlock.StringDataBlock queryParameter, 
					BaseTrackers.JavaVariablePath variablePathToTestValueFromSearchLayer, 
					JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
					BaseTrackers.JavaVariablePath variablePathToQueryMatch) throws ParsingException {
				targetSkeleton.addStringMatch(queryParameter,variablePathToTestValueFromSearchLayer, parameterSkeleton, variablePathToQueryMatch);
			}
		};
	}
	
	public void setAsExpressionSetter(final BExpressionAccessFormHolder.BExpressionAccessFormBlock expressionStore) {
		this.currentType_ = new AbstractType(EXPRESSION_SETTER_NAME) {
			@Override public void doSetExpression(
					CodeBlockMacroBlock baseCodeSkeleton,
					BaseTrackers.JavaVariablePath  variablePathVariable, 
					ExpressionDetailsFormHolder.ExpressionDetailsFormBlock expressionSkeleton		
				)throws ParsingException {
				baseCodeSkeleton.addStatement().addSetExpression(expressionStore, variablePathVariable, expressionSkeleton);
			}
		};
	}
	public void setAsFoundationRecordReference(final SLayerRecordReference value) {
		this.currentType_ = new AbstractType(FOUNDATION_RECORD_REFERENCE_NAME) {
			@Override public SLayerRecordReference getAsFoundationRecordReference() throws ParsingException { return value; }

			@Override public void doVariableQueryMatch(
					JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
					BaseTrackers.JavaVariablePath  variablePathToTestValueFromSearchLayer, 
					IRecordVariable queryMatchValue, 
					BaseTrackers.JavaVariablePath variablePathToQueryMatch
				) throws ParsingException {
				value.doVariableQueryMatchAsQueryParameter(
							targetSkeleton, variablePathToTestValueFromSearchLayer, queryMatchValue, variablePathToQueryMatch
					);
			}
			@Override 	public void addAsPathElement(BaseTrackers.JavaVariablePath path) throws ParsingException {
				value.addAsPathElement(path);
			}
			@Override  public void doRecordThisMatchAsQueryParameter(
					JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
					BaseTrackers.JavaVariablePath  variablePathToParameterFromSearchLayer,
					FoundationNode recordType,
					BaseTrackers.JavaVariablePath  variablePathToRecordQueryValueFromSearchLayer
				) throws ParsingException {
				value.doRecordThisMatchAsQueryParameter(targetSkeleton, variablePathToParameterFromSearchLayer, recordType, variablePathToRecordQueryValueFromSearchLayer);
			}
		};
	}
	public void setAsCodeReference(final CodeReference value){
		this.currentType_ = new AbstractType(CODE_REFERENCE_NAME) {
			@Override public CodeReference getAsCodeReference() throws ParsingException { return value; }
		};
	}
	public void setAsArgTypesMarkReference(final ArgTypesReference value){
		this.currentType_ = new AbstractType(ARG_TYPES_NAME) {
			@Override public ArgTypesReference getAsArgTypesMarkReference()throws ParsingException { return value; }
		};
	}
	public void setAsArgValuesMarkReference(final ArgValuesReference value){
		this.currentType_ = new AbstractType(ARG_VALUES_NAME) {
			@Override public ArgValuesReference getAsArgValuesMarkReference() throws ParsingException { return value; }
		};
	}
	public void setAsMappingReference(final MappingReference value) {
		this.currentType_ = new AbstractType(MAPPING_REFERENCE_NAME) {
			@Override public  MappingReference getAsMappingReference() throws ParsingException { return value; }
		};
	}
	public void setAsKeyReference(final KeyMarkerReference value){
		this.currentType_ = new AbstractType(KEY_REFERENCE_NAME) {
			@Override public KeyMarkerReference getAsKeyReference()throws ParsingException { return value; }
		};
	}
	/**
	 * This record is the test field (of the things being searched).
	 * @param targetSkeleton where to put stuff
	 * @param variablePathToTestValueFromSearchLayer the path from the search layer to the specific field
	 * @param queryMatchValue what we are comparing with
	 * @param variablePathToQueryMatch the path to that from current position
	 * @throws ParsingException
	 */
	public void doVariableQueryMatch(
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
			BaseTrackers.JavaVariablePath  variablePathToTestValueFromSearchLayer, 
			IRecordVariable queryMatchValue, 
			BaseTrackers.JavaVariablePath variablePathToQueryMatch
		) throws ParsingException {
		this.currentType_.doVariableQueryMatch(targetSkeleton, variablePathToTestValueFromSearchLayer,queryMatchValue, variablePathToQueryMatch);
	}
	public void doCreateSetExpression(
			BaseBuilder.CodeBlockMacroBlock targetSkeleton,
			BaseTrackers.JavaVariablePath  variablePathVariable, 
			ExpressionDetailsFormHolder.ExpressionDetailsFormBlock expressionSkeleton		
		) throws ParsingException {
		this.currentType_.doSetExpression(targetSkeleton,variablePathVariable,  expressionSkeleton );
		
	}
	
	public void doRecordThisMatchAsQueryParameter(
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
			BaseTrackers.JavaVariablePath  variablePathToParameterFromSearchLayer,
			FoundationNode recordType,
			BaseTrackers.JavaVariablePath  variablePathToRecordQueryValueFromSearchLayer
		) throws ParsingException {
		this.currentType_.doRecordThisMatchAsQueryParameter(targetSkeleton, variablePathToParameterFromSearchLayer, recordType, variablePathToRecordQueryValueFromSearchLayer);

	}
	public void doVariableQueryMatchAsQueryValue( 
			JavaParameterSettingsFormBlock.ParameterDataBlock.StringDataBlock queryParameter, 
			BaseTrackers.JavaVariablePath variablePathToTestValueFromSearchLayer,
			JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
			BaseTrackers.JavaVariablePath variablePathToQueryMatch)  throws ParsingException{
		this.currentType_.doVariableQueryMatchAsQueryValue( 
				queryParameter, 
				variablePathToTestValueFromSearchLayer,
				targetSkeleton, 
				variablePathToQueryMatch);
	}
	// --------------------------------------------------------------------------
	public CodeReference getAsCodeReference() throws ParsingException { return this.currentType_.getAsCodeReference(); }
	public ArgTypesReference getAsArgTypesMarkReference() throws ParsingException{ return this.currentType_.getAsArgTypesMarkReference(); }
	public ArgValuesReference getAsArgValuesMarkReference() throws ParsingException{ return this.currentType_.getAsArgValuesMarkReference(); }
	public MappingReference getAsMappingReference() throws ParsingException{ return this.currentType_.getAsMappingReference(); }
	public KeyMarkerReference getAsKeyReference() throws ParsingException{ return this.currentType_.getAsKeyReference(); }
	public SLayerRecordReference getAsFoundationRecordReference() throws ParsingException{ return this.currentType_.getAsFoundationRecordReference(); }
	public void addAsPathElement(BaseTrackers.JavaVariablePath path) throws ParsingException {
		this.currentType_.addAsPathElement(path); 

	}
	//---------------------------------------------------------------------------
	
	public static final IRecordTypeTarget generateTarget(IRecordVariable source) {
		IRecordTypeTarget target =  new IRecordTypeTarget();
		source.getAsReference(target);
		return target;
	}
	
	// ==========================================================================
	private static interface Type {
		public CodeReference getAsCodeReference() throws ParsingException;
		public ArgTypesReference getAsArgTypesMarkReference() throws ParsingException;
		public ArgValuesReference getAsArgValuesMarkReference() throws ParsingException;
		public MappingReference getAsMappingReference() throws ParsingException;
		public KeyMarkerReference getAsKeyReference() throws ParsingException;
		public SLayerRecordReference getAsFoundationRecordReference() throws ParsingException;
		public void addAsPathElement(BaseTrackers.JavaVariablePath path) throws ParsingException;

		public void doVariableQueryMatch(
					JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton,  
					BaseTrackers.JavaVariablePath  variablePathToTestValueFromSearchLayer,
					IRecordVariable queryMatchValue, BaseTrackers.JavaVariablePath variablePathToQueryMatch) throws ParsingException;
		public void doSetExpression(
				CodeBlockMacroBlock targetSkeleton,
				BaseTrackers.JavaVariablePath  variablePathVariable, 
				ExpressionDetailsFormHolder.ExpressionDetailsFormBlock expressionSkeleton		
			) throws ParsingException;
		public void doRecordThisMatchAsQueryParameter(
				JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
				BaseTrackers.JavaVariablePath  variablePathToParameterFromSearchLayer,
				FoundationNode recordType,
				BaseTrackers.JavaVariablePath  variablePathToRecordQueryValueFromSearchLayer
			) throws ParsingException;
		public void doVariableQueryMatchAsQueryValue( 
				JavaParameterSettingsFormBlock.ParameterDataBlock.StringDataBlock queryParameter, 
				BaseTrackers.JavaVariablePath variablePathToTestValueFromSearchLayer,
				JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
				BaseTrackers.JavaVariablePath variablePathToQueryMatch) throws ParsingException;
	}
	private static abstract class AbstractType implements Type{
		private final String	typeName_;
		public AbstractType(String typeName) {
			this.typeName_ = typeName;
			
		}
		@Override public void doSetExpression(
				CodeBlockMacroBlock baseCodeSkeleton,
				BaseTrackers.JavaVariablePath  variablePathVariable, 
				ExpressionDetailsFormHolder.ExpressionDetailsFormBlock expressionSkeleton		
			)throws ParsingException {
			throw ParsingException.createGeneralSemantic(this.typeName_+" cannot be set to an expression");
		}
		public ParsingException createError(String requestedType) { return ParsingException.createInvalidType(typeName_, requestedType); }
		@Override public void doRecordThisMatchAsQueryParameter(
				JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
				BaseTrackers.JavaVariablePath  variablePathToParameterFromSearchLayer,
				FoundationNode recordType,
				BaseTrackers.JavaVariablePath  variablePathToRecordQueryValueFromSearchLayer
			) throws ParsingException {
			throw ParsingException.createGeneralSemantic(this.typeName_+" cannot do specified matching");

		}
		@Override 	public void addAsPathElement(BaseTrackers.JavaVariablePath path) throws ParsingException {
			//TODO Fix
			Debug.finishMeMarker();
			throw ParsingException.createGeneralSemantic("can't added to chaing");
		}
		@Override public CodeReference getAsCodeReference() throws ParsingException { throw createError(CODE_REFERENCE_NAME); }
		@Override public ArgTypesReference getAsArgTypesMarkReference() throws ParsingException { throw createError(ARG_TYPES_NAME); }
		@Override public ArgValuesReference getAsArgValuesMarkReference()throws ParsingException { throw createError(ARG_VALUES_NAME); }
		@Override public MappingReference getAsMappingReference()throws ParsingException { throw createError(MAPPING_REFERENCE_NAME); }
		@Override public KeyMarkerReference getAsKeyReference()throws ParsingException { throw createError(KEY_REFERENCE_NAME); }
		@Override public void doVariableQueryMatch(
				JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, BaseTrackers.JavaVariablePath  variablePathToTestValueFromSearchLayer,  IRecordVariable queryMatchValue, BaseTrackers.JavaVariablePath variablePathToQueryMatch) throws ParsingException {
			throw ParsingException.createGeneralSemantic("parameter cannot do matching");
		}
		@Override public void doVariableQueryMatchAsQueryValue( 
				JavaParameterSettingsFormBlock.ParameterDataBlock.StringDataBlock queryParameter, 
				BaseTrackers.JavaVariablePath variablePathToTestValueFromSearchLayer,

				JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, 
				BaseTrackers.JavaVariablePath variablePathToQueryMatch) throws ParsingException{
			throw ParsingException.createGeneralSemantic("parameter cannot do String matching");

		}
		@Override public SLayerRecordReference getAsFoundationRecordReference() throws ParsingException{ throw createError(FOUNDATION_RECORD_REFERENCE_NAME); }

	}
}
