package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.ChainHookListPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.DeferredJavaClassInterfaceElement;
import org.ffd2.skeletonx.austenx.peg.base.FindDefinitionPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.GeneralMacroCallPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.GlobalClassDefinitionPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaClassInterfaceElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaClassPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaCodeBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaConstructorPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaExpressionPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaInstanceVariablePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaInterfacePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaMacroBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaMethodPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaPrototypePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.LinkChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.NamePeer;
import org.ffd2.skeletonx.austenx.peg.base.SetsListPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetParameterPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeListPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferencePatternPeer;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.ChainHookListBlock.BlockUser;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;

import org.ffd2.skeletonx.skeleton.ArgumentSetDetailsFormHolder.ArgumentSetDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.ChainDataBlock.LinkRefDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.NameDetailsFormHolder.NameDetailsFormBlock;

public class JavaClassElements implements JavaClassInterfaceElementPeer.Indirect{
  private final SpecificCommonErrorOutput baseError_;

  private final SimpleVector<Element> elements_ = new SimpleVector<Element>();
  private final ElementAdderHack adder_;

	private final FileEnvironment environment_;

	private final String	className_;
 
	// ========================================================================
	private static interface BuildEnvironment {
  	public void doFinalBuild(Element e);
  }
	private BuildEnvironment buildEnvironment_;

	private final BaseBuilder rootBuilder_;
	// ==========================================================================
  /**
   * 
   * @param classDefinitionRecord the record link for the record under which the class containing these elements is defined
   * @param currentRecord the current link level record that we are working at
   * @param baseError
   */
  public JavaClassElements(String className,BaseBuilder rootBuilder,  FileEnvironment environment, SpecificCommonErrorOutput baseError) {

    this.className_ = className;
		this.rootBuilder_ = rootBuilder;
		this.environment_ = environment;
		this.baseError_ = baseError;
    this.adder_ = new ElementAdderHack(this);
    
 
  }
  public JavaClassInterfaceElementPeer.MethodPatternPeer createMethod() { return adder_; }
	public JavaClassInterfaceElementPeer.MacroDefPatternPeer createMacroDef() { return adder_; }
	public JavaClassInterfaceElementPeer.GlobalClassPatternPeer createGlobalClass() { return adder_; }
  public JavaClassInterfaceElementPeer.FindPatternPeer createFind() {   	return adder_; }
  public JavaClassInterfaceElementPeer.PrototypePatternPeer createPrototype() { return adder_; }
  public JavaClassInterfaceElementPeer.ClassPatternPeer createClass() { return adder_; }
  public JavaClassInterfaceElementPeer.InterfacePatternPeer createInterface() { return adder_; }
  public JavaClassInterfaceElementPeer.InstanceVariablePatternPeer createInstanceVariable() { return adder_; }
	public JavaClassInterfaceElementPeer.ConstructorPatternPeer createConstructor() { return adder_; }
	public JavaClassInterfaceElementPeer.MacroCallPatternPeer createMacroCall() { return adder_; }
	/**
	 * @param classRecordAccess
	 * @param allGlobalReferences this may be null if no global references made for the containg class
	 */
  public void basicBuild(final RecordClassAccess classRecordAccess) {
  	this.buildEnvironment_ = new BuildEnvironment() {
    	public void doFinalBuild(Element e) {
        e.finalBuild(classRecordAccess);
    	}
  	};
  	for(Element e : this.elements_) {
      e.basicBuild(classRecordAccess);
    }
  }
  public void finalBuild() { //Need defining class stuff
    final BuildEnvironment buildEnvironmentLocal = this.buildEnvironment_;
    for(Element e : this.elements_) {
    	buildEnvironmentLocal.doFinalBuild(e);
    }
//    currentRecord.addPathVariablesToRecordData(this.variablePathChainFromDefiningRecordToCurrent_);
    
  }
 
  
  public void basicBuild(final RecordContainerAccess containerRecordAccess) {
  	this.buildEnvironment_ = new BuildEnvironment() {
    	public void doFinalBuild(Element e) {
        e.finalBuild(containerRecordAccess);
    	}
  	};
  	for(Element e : this.elements_) {
      e.basicBuild(containerRecordAccess);
    }
  }
  // ==========================================================================
  private final void addElement(Element e) {
    this.elements_.addElement(e);
  }
  public JavaClassInterfaceElementPeer.LinkPatternPeer createLink(int keywordLineNumber, int keywordCharacterNumber) {
    SpecificCommonErrorOutput error = baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
    LinkBlock lb = new LinkBlock(this.className_, this.rootBuilder_, this.environment_, error);
    addElement(lb);
    return lb;
  }
  //==========================================================================
  private static interface Element {
    public void basicBuild(RecordClassAccess classRecordAccess);
    public void finalBuild(RecordClassAccess classRecordAccess);
    
    public void basicBuild(RecordContainerAccess containerRecordAccess);
    public void finalBuild(RecordContainerAccess containerRecordAccess);

  }
  //==========================================================================
  private final static class ElementAdderHack implements JavaClassInterfaceElementPeer.MacroDefPatternPeer,JavaClassInterfaceElementPeer.GlobalClassPatternPeer,JavaClassInterfaceElementPeer.FindPatternPeer,JavaClassInterfaceElementPeer.MacroCallPatternPeer, JavaClassInterfaceElementPeer.ConstructorPatternPeer, JavaClassInterfaceElementPeer.ClassPatternPeer, JavaClassInterfaceElementPeer.InterfacePatternPeer, JavaClassInterfaceElementPeer.MethodPatternPeer,JavaClassInterfaceElementPeer.PrototypePatternPeer,JavaClassInterfaceElementPeer.InstanceVariablePatternPeer {
    private final JavaClassElements parent_;
    public ElementAdderHack(JavaClassElements parent) {
      this.parent_ = parent;
    }
    public ClassConstruction addType(final String className, boolean isClass,final SpecificCommonErrorOutput error) {
  
    	final ClassConstruction subClass = new ClassConstruction(className,parent_.rootBuilder_,isClass,parent_.environment_,error);
      parent_.addElement(new Element() {
      	public void basicBuild(RecordClassAccess classRecordAccess) {
					subClass.basicBuildSubClass(classRecordAccess);
				}
				public void finalBuild(RecordClassAccess classRecordAccess) {
					subClass.finalBuild();					
				}
			  public void basicBuild(RecordContainerAccess containerRecordAccess) {
			  	subClass.basicBuildInContainer(containerRecordAccess);
						
			  }
		    public void finalBuild(RecordContainerAccess containerRecordAccess) {
		    	subClass.finalBuild();						
		    }
      });
      return subClass;
    }
    public JavaClassPatternPeer addJavaClassForValue(final int keywordLineNumber, final int keywordCharacterNumber, final String nameParameter) {
    	final SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
      return addType(nameParameter,true,error);
    }
    
    public JavaInterfacePatternPeer addJavaInterfaceForValue(final int keywordLineNumber, final int keywordCharacterNumber, final String nameParameter) {
      final SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
      return addType(nameParameter,false,error);
    }
    
    public JavaMethodPatternPeer addJavaMethodForValue(String nameParameter, int nameLineNumber, int nameCharacterNumber) {
      SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
      MethodElement me =  new MethodElement( nameParameter, new FullMethodType(parent_.rootBuilder_, error),parent_.environment_,error);
      parent_.addElement(me);
      
      return me;
    }
    public JavaPrototypePatternPeer addJavaPrototypeForValue(final int keywordLineNumber, final int keywordCharacterNumber, final String nameParameter) {
      SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
      
      MethodElement pe =  new MethodElement(nameParameter,PROTOTYPE_METHOD_TYPE,parent_.environment_,error);
      
      parent_.addElement(pe);
      return pe;
      
    }
    
    public JavaInstanceVariablePatternPeer addJavaInstanceVariableForVariable(final String nameParameter, final int nameLineNumber, final int nameCharacterNumber) {
      SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
      VariableElement ve = new VariableElement(nameParameter,this.parent_.environment_,error);
      parent_.addElement(ve);
     
      
      return ve;
    }
    public JavaConstructorPatternPeer addJavaConstructorForValue(int keywordLineNumber,
				int keywordCharacterNumber) {
      SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
      ConstructorElement ce = new ConstructorElement(parent_.rootBuilder_, error);
      parent_.addElement(ce);
      return ce;
		} 
    // Multi used - don't touch
    public void end() { }
		public FindDefinitionPatternPeer addFindDefinitionForDefinition(final String varibleNameParameter, final int varibleNameLineNumber, final int varibleNameCharacterNumber) {
			SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(varibleNameLineNumber, varibleNameCharacterNumber);
			final FindVariable variable = new FindVariable(varibleNameParameter,error);
      parent_.addElement(new Element() {
      	SLayer containingLayerStore;
      	FoundationNode rootNodeStore;
      	
				public void basicBuild(RecordClassAccess classRecordAccess) {
					this.containingLayerStore = classRecordAccess.getBaseLayer(); this.rootNodeStore = this.containingLayerStore.getRootFoundationNodeBasic();
					variable.basicBuild(this.rootNodeStore,this.containingLayerStore);
				}

				public void finalBuild(RecordClassAccess classRecordAccess) {
					variable.finishBuild(this.rootNodeStore,this.containingLayerStore);
				}

				public void basicBuild(RecordContainerAccess containerRecordAccess) {
					this.containingLayerStore = containerRecordAccess.getBaseLayer(); this.rootNodeStore = this.containingLayerStore.getRootFoundationNodeBasic();
					variable.basicBuild(this.rootNodeStore,this.containingLayerStore);					
				}

				public void finalBuild(RecordContainerAccess containerRecordAccess) {
					variable.finishBuild(this.rootNodeStore,this.containingLayerStore);
				}
      	
      });
  		return variable;
		}
    public GeneralMacroCallPatternPeer addGeneralMacroCallForCall( int keywordLineNumber, int keywordCharacterNumber, String macroNameParameter) {
    	 SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
       GeneralMacroCall baseCall = new GeneralMacroCall(macroNameParameter, error);
    	 MacroCallElement ce = new MacroCallElement(baseCall, this.parent_.environment_,error);
       parent_.addElement(ce);
       return baseCall;
		}
		public GlobalClassDefinitionPatternPeer addGlobalClassDefinitionForDefinition( String classNameParameter, int classNameLineNumber, int classNameCharacterNumber) {
			 SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(classNameLineNumber, classNameCharacterNumber);
       
			 final GlobalsClass baseCall = new GlobalsClass(classNameParameter, error);
       parent_.addElement(new  Element() {

				public void basicBuild(RecordClassAccess classRecordAccess) { baseCall.basicBuild(classRecordAccess.getBaseLayer()); }
				public void basicBuild(RecordContainerAccess containerRecordAccess) { baseCall.basicBuild(containerRecordAccess.getBaseLayer()); }

				public void finalBuild(RecordContainerAccess containerRecordAccess) { baseCall.finishBuild(); }

				public void finalBuild(RecordClassAccess classRecordAccess) { baseCall.finishBuild(); }
      	 
       });
       return baseCall;
		}
		public JavaMacroBlockPatternPeer addJavaMacroBlockForMacroDef(int keywordLine, int keywordCharacter, String macronameValue) {
			SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLine, keywordCharacter);
      
			final MacroConstruction macroDef = new MacroConstruction(macronameValue,this.parent_.rootBuilder_,this.parent_.environment_, error);
			//TODO: include stuff with encomapassing class
			// 
			Debug.finishMeMarker();
      parent_.addElement(new  Element() {

				public void basicBuild(RecordClassAccess classRecordAccess) { macroDef.basicBuild(classRecordAccess); }
				public void basicBuild(RecordContainerAccess containerRecordAccess) { macroDef.basicBuild(containerRecordAccess.getBaseLayer()); }

				public void finalBuild(RecordContainerAccess containerRecordAccess) { macroDef.finishBuild(); }

				public void finalBuild(RecordClassAccess classRecordAccess) { macroDef.finishBuild(); }
     	 
      });
      return macroDef;
		}
		

		
  }
  //==========================================================================
  //==========================================================================
 
  private final static class LinkBlock implements JavaClassInterfaceElementPeer.LinkPatternPeer,Element {
    private final SpecificCommonErrorOutput error_;
    private LinkChainBlock lastLinkChain_;
   // private final JavaClassElements elements_;
		//private final BaseTrackers.JavaVariablePath linkPathToRecord_ = new BaseTrackers.JavaVariablePath();
    private final SimpleVector<DeferredJavaClassInterfaceElement> deferredElements_ = new SimpleVector<DeferredJavaClassInterfaceElement>();
    private final SimpleVector<JavaClassElements> createdElements_ = new SimpleVector<JavaClassElements>();
		private final String	className_;
		private final FileEnvironment	environment_;
		private final BaseBuilder rootBuilder_;
    public LinkBlock(String className, BaseBuilder rootBuilder,  FileEnvironment environment, SpecificCommonErrorOutput error ){

      this.className_ = className;
			this.rootBuilder_ = rootBuilder;
			this.environment_ = environment;
			this.error_ = error; 
     
   
    }
//    public LinkChainPatternPeer addLinkChainForChain() {
//      return this.linkChain_;
//    }

    public void end() { }

//    public JavaClassInterfaceElementPeer.Indirect getJavaClassInterfaceElementForElement() {
//      //By the time we get to parse this we have already formed the link chain...
//      return this.elements_;
//    }
    private JavaClassElements createClassElements() {
    	JavaClassElements result = new JavaClassElements(this.className_,this.rootBuilder_,this. environment_, error_);
    	for(DeferredJavaClassInterfaceElement deferred : this.deferredElements_) {
    		deferred.buildJavaClassInterfaceElement(result);
    	}
    	return result;
    }
    public void basicBuild(RecordClassAccess classRecordAccess) {
      LinkChainBlock currentLinkChain = this.lastLinkChain_;
      while(currentLinkChain!=null) {
      	final RecordClassAccess linkedClassRecordAccess = classRecordAccess.getLinkedQuiet(currentLinkChain);
      	if(linkedClassRecordAccess!=null) {
      		JavaClassElements classElements = createClassElements();
      		classElements.basicBuild(linkedClassRecordAccess);
      		this.createdElements_.addElement(classElements); 
  	    }
      	currentLinkChain = currentLinkChain.getPrevious();
      }
    	//final RecordClassAccess linkedClassRecordAccess = classRecordAccess.getLinkedQuiet(linkChain_);
	    
    }
  	public void basicBuild(RecordContainerAccess containerRecordAccess) {	    
  		LinkChainBlock currentLinkChain = this.lastLinkChain_;
      while(currentLinkChain!=null) {
      	final RecordContainerAccess linkedContainerRecordAccess = containerRecordAccess.getLinkedQuiet(currentLinkChain);
      	if(linkedContainerRecordAccess!=null) {
      		JavaClassElements classElements = createClassElements();
      		classElements.basicBuild(linkedContainerRecordAccess);
      		this.createdElements_.addElement(classElements); 
  	    }
      	currentLinkChain = currentLinkChain.getPrevious();
      }
		}
	
    public void finalBuild() {
    	for(JavaClassElements classElements : this.createdElements_) {
    		classElements.finalBuild();
    	}

    }
    public void finalBuild(RecordClassAccess classRecordAccess) { finalBuild(); }
    public void finalBuild(RecordContainerAccess containerRecordAccess) {finalBuild(); }
		public LinkChainPatternPeer addLinkChainForSingleChain() {
			return this.lastLinkChain_ = new LinkChainBlock(this.error_,this.lastLinkChain_);
		}
		public void setJavaClassInterfaceElementForElement(DeferredJavaClassInterfaceElement value) {
			this.deferredElements_.addElement(value);
		}
  }
  //--------------------------------------------------------------------------
  private final static class VariableElement implements  TargetVariable, Element,JavaInstanceVariablePatternPeer,JavaConcreteChainLink {
    private final SpecificCommonErrorOutput error_;
    private final String name_;
    private boolean isFinal_ = false;
    private boolean isStatic_= false;
    private boolean isPrivate_ = false;
    private boolean isPublic_ = false;
    private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock variableSkeleton_;
    private TargetTypeBlock typeBlock_;
    private ClassImplementation hostingType_;
    private NameBlock nameBlock_;
    private RecordClassAccess definitionClassRecordAccess_;
    
    private ChainHookListBlock chainHookList_ = null;
		private final FileEnvironment environment_;
		private ExpressionBlock initialValue_ = null;
		private CodeBlockEnvironment initialEnvironment_ = null;

    public VariableElement( String name, FileEnvironment environment, SpecificCommonErrorOutput error) {

      this.name_ = name;
			this.environment_ = environment;

      this.error_ = error;
    }
		public void basicBuild(RecordContainerAccess containerRecordAccess) {
			this.error_.generalSyntaxError("wrong context for instance variables");
		}
    public void basicBuild(RecordClassAccess classRecordAccess) {
      this.definitionClassRecordAccess_ = classRecordAccess;
      this.hostingType_ = classRecordAccess.getContainingClass();
      this.typeBlock_.basicBuild(classRecordAccess.getBaseLayer());
      try {
        classRecordAccess.addClassVariableBasic( name_,this);
       
      } catch(ItemAlreadyExistsException e) {
        this.error_.repeatedObjectError("target variable", name_);
      }
      if(this.initialValue_!=null) {
      	this.initialValue_.basicBuild(this.initialEnvironment_ = classRecordAccess.createRootCodeBlock());
      }
      checkChainHooks(); 
    }
    //For JavaConcreteChainLink
    public void updateChainLink(JavaImplementationFormBlock.ChainDataBlock.LinkRefDataBlock targetSkeleton) {
			targetSkeleton.addInstanceVariable(this.variableSkeleton_);
		}
    private final void checkChainHooks() {
    	if(this.chainHookList_!=null) {
    		this.chainHookList_.requestUpdate(new ChainHookListBlock.BlockUser() {
	      	public void newLink(final String chainName, final String linkName, SpecificCommonErrorOutput error) {
	    			try {
	    				definitionClassRecordAccess_.getBaseLayer().addConcreteChainLink(chainName, linkName, VariableElement.this, error);
	    			} catch(ParsingException e) {
	    				e.updateError(error);
	    			}
	    		}
	      });
    	}
    }
    
    /**
     * Only valid after basic pass
     */
    public TargetTypeReference getTargetTypeReferenceQuietFinal() {
    	return this.typeBlock_.getResolvedTypeReferenceQuietFinal();

    }
    private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock
        getVariableSkeletonFinal() {
      if(this.variableSkeleton_==null) {
        this.variableSkeleton_ = this.definitionClassRecordAccess_.createClassVariableFinal(name_,this);
      }
      return this.variableSkeleton_;
    }
    public void finalBuild(RecordClassAccess record) {
      InstanceVariableDataBlock variableSkeleton = getVariableSkeletonFinal(); //Ensure exists
      if(this.isFinal_) {
      	variableSkeleton.addIsFinal();
      }
      if(this.isPrivate_) {
      	variableSkeleton.addIsPrivate();
      }
      if(this.isPublic_) {
      	variableSkeleton.addIsPublic();
      }
      this.typeBlock_.buildCodeSetTypeFinal(variableSkeleton.getVariableTypeDetailsParameter());
      if(this.nameBlock_==null) {
      	variableSkeleton.getNameDetailsParameter().addSimple(this.name_);
      } else {
      	this.nameBlock_.updateInstanceNameSkeleton(this.name_,variableSkeleton.getNameDetailsParameter(), record.getBaseLayer());
      }
      if(this.initialValue_!=null) {
      	this.initialValue_.finish(this.initialEnvironment_, variableSkeleton.addInitialValue(variableSkeleton.getRootBuilder().createExpressionDetailsChild()).getValueDetailsParameter());
      }
    }
    // ------------------------------------------------------------------------
    public void addIsFinal() { this.isFinal_ = true;   }
    public void addIsStatic() { this.isStatic_ = true;   }
   
    public void addPrivate() { this.isPrivate_ = true;   }
    public void addPublic() { this.isPublic_ = true; }
   
    public JavaExpressionPeer.Indirect getJavaExpressionForInitialValue() {
      return this.initialValue_ = new ExpressionBlock(this.error_);
      
    }
   
    public void end() { }
    
    public void createCallExpressionFinal(BasicCallChainFormBlock target, BaseTrackers.JavaVariablePath basePathDisposable) {
    	target.addElement().addVariable(getVariableSkeletonFinal().getVariableStoreParameter(), basePathDisposable);
    }

    /**
     * Not valid until Intermediate pass
     */
//    public TargetType getTargetType(BaseTrackers.JavaVariablePath fromThisSpaceToTypeDeclarationPath) {
//      return this.typeBlock_.getResolvedType(fromThisSpaceToTypeDeclarationPath);
//    }
    
    public TargetTypeReferencePatternPeer addTargetTypeReferenceForType(String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
      SpecificCommonErrorOutput error = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
      this.typeBlock_ = new TargetTypeBlock(typeNameParameter,error);
      return typeBlock_;
    }
		public void createMacroCallParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath pathToVariable) {
			macroCallBlock.addCallParameter().addTargetVariable().addSimple(this.variableSkeleton_.getVariableStoreParameter(), pathToVariable);
		}
		public ChainHookListPatternPeer addChainHookListForChainHookList() {
			return this.chainHookList_ = new ChainHookListBlock( this.error_);
		}

		public void finalBuild(RecordContainerAccess containerRecordAccess) { }
		
		public NamePeer.BasicPatternPeer addOptionBasicOfNameForNameOutput() { return this.nameBlock_ = new NameBlock(this.error_); }

		public void addOptionImplementsOfNameForNameOutput(final int keywordLineNumber, final int keywordCharacterNumber, final String classNameParameter, final String methodNameParameter, final int methodNameLineNumber, final int methodNameCharacterNumber){
			this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber).generalSyntaxError("implementing name not applicable to variable declaration");
		}
		
    
  }
  // --------------------------------------------------------------------------
  
  // --------------------------------------------------------------------------
  // METHOD
  private static interface MethodType {
    public void basicBuild(CodeBlockEnvironment environment);
    public void finishBuild( JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock  methodSkeleton) ;
    public JavaCodeBlockPatternPeer addJavaCodeBlockForCodeMaybeNull();
  }
  private static final MethodType PROTOTYPE_METHOD_TYPE = new MethodType()  {
  	public void basicBuild(CodeBlockEnvironment environment) { }
    public void finishBuild( JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock  methodSkeleton) {
    	methodSkeleton.addPrototype();
    }
    public JavaCodeBlockPatternPeer addJavaCodeBlockForCodeMaybeNull() {return null; }
  };
  
  private static final class FullMethodType implements MethodType {
    private final JavaCodeBlock codeBlock_;
    public FullMethodType(BaseBuilder root, SpecificCommonErrorOutput error ){ 
      this.codeBlock_ = new JavaCodeBlock(root,  error );
    }
    public void basicBuild(CodeBlockEnvironment environment) {
      codeBlock_.basicBuild(environment);
    }
    public void finishBuild( JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock  methodSkeleton) {
      codeBlock_.finishBuild(methodSkeleton.addFullMethod(methodSkeleton.getRootBuilder().createBCodeBlockAccessChild()).getBaseCodeBlockMacro());
    }
    public JavaCodeBlockPatternPeer addJavaCodeBlockForCodeMaybeNull() {
    	return this.codeBlock_.createJavaCodeBlockPeer();
    }
  }
  
  private final static NameHandler DEFAULT_NAME_HANDLER = new NameHandler() {
		public void updateInstanceNameSkeleton(String defaultName, JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock methodSkeleton, SLayer baseEnvironment) {
    	NameDetailsFormBlock nameSkeleton = methodSkeleton.getRootBuilder().createNameDetailsChild();
			methodSkeleton.addNormalName(nameSkeleton);
			nameSkeleton.addSimple(defaultName);
		}
  };
  private final static class MethodElement implements Element,JavaMethodPatternPeer, JavaPrototypePatternPeer,TargetMethod, BlockUser, JavaConcreteChainLink {
    private final SpecificCommonErrorOutput error_;
    private final String nameBase_;
    private NameHandler name_;
    private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock methodSkeleton_;
    private TargetTypeBlock returnTypeBlock_;
    
    private RecordClassAccess definitionRecordClassAccess_;
    private final TargetParametersBlock parametersBlock_;
    private TargetTypeListBlock throwsList_ = null;
    private ChainHookListBlock chainHookList_ = null;
		private final MethodType methodType_;
		private boolean isPublic_ = true;
		private boolean isStatic_ = false;
    private MethodSetsBlock globalSets_ = null;
    
    public MethodElement(String name, MethodType methodType, FileEnvironment fileEnvironment,  SpecificCommonErrorOutput error) {
      this.nameBase_ = name;
			this.methodType_ = methodType;
			this.error_ = error;
			this.parametersBlock_ =  new TargetParametersBlock( error );
			this.name_ = DEFAULT_NAME_HANDLER;
    }
    public String toString() { return "JC Method Element:"+nameBase_; }
    public boolean isStatic() { 
			return this.isStatic_;
		}
    // ---------------------------------------------------------------------
    // BlockUser stuff
    public void newLink(String chainName, String linkName, SpecificCommonErrorOutput error) {
			try {
    		definitionRecordClassAccess_.getBaseLayer().addConcreteChainLink(chainName, linkName, this, error);
			} catch(ParsingException e) {
				e.updateError(error);
			}
		}
    //JavaConcreteChainLink stuff
    
    public void updateChainLink(LinkRefDataBlock targetSkeleton) {
    	targetSkeleton.addInstanceMethod(this.methodSkeleton_);
		}
  	public void basicBuild(RecordContainerAccess containerRecordAccess) {
			this.error_.generalSyntaxError("wrong context for instance methods");
		}
		public void finalBuild(RecordContainerAccess containerRecordAccess) { }
		
    // =====================================================================
    // Building stuff
    
    public void basicBuild(RecordClassAccess recordClassAccess) {
      this.definitionRecordClassAccess_ = recordClassAccess;
      CodeBlockEnvironment environment = recordClassAccess.createRootCodeBlock();
      
      this.returnTypeBlock_.basicBuild( recordClassAccess.getBaseLayer());
      
      try {
        recordClassAccess.addClassMethodBasic(nameBase_,this);
      } catch(ItemAlreadyExistsException e) {
        this.error_.repeatedObjectError("target method", nameBase_);
      }
      if(this.parametersBlock_!=null) {
      	this.parametersBlock_.basicBuildMethod(environment);
      }
      if(this.throwsList_!=null) {
      	this.throwsList_.basicBuild(environment.getBaseLayer());
      }
      this.methodType_.basicBuild(environment);
      if(this.globalSets_!=null) {
      	this.globalSets_.basicBuild(recordClassAccess);
      }
    } //end of Method basicBuild()

    public void finalBuild(RecordClassAccess recordClassAccess) {
    	final  JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock  methodSkeleton = getMethodSkeleton();
      final BaseBuilder rootBuilder = methodSkeleton.getRootBuilder();
      if(this.parametersBlock_!=null) {
				parametersBlock_.finishBuildMethod(methodSkeleton.getParameterSetMacro());
			}
      methodType_.finishBuild(methodSkeleton);

    	this.name_.updateInstanceNameSkeleton(this.nameBase_, methodSkeleton, recordClassAccess.getBaseLayer());
			
    	this.returnTypeBlock_.buildCodeSetTypeFinal(methodSkeleton.getReturnTypeDetailsParameter());
      if(this.chainHookList_!=null) {
      	this.chainHookList_.requestUpdate(this);
      }
      if(this.throwsList_!=null) {
      	this.throwsList_.finalBuild(methodSkeleton.addThrows(rootBuilder.createTypeListDetailsChild()).getTypeListDetailsParameter());
      }
      if(isPublic_) {
      	methodSkeleton.addIsPublic();
      } else {
       	methodSkeleton.addIsPrivate(); 	
      }
      if(isStatic_) {
      	methodSkeleton.addIsStatic();
      }
      if(this.globalSets_!=null) {
      	this.globalSets_.finalBuild(methodSkeleton);
      }
    } //End of method finalBuild()
    
    // -------------------------------------------------------------------------------------------------
    public  JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock getMethodSkeleton() {
      if(methodSkeleton_==null) {
        methodSkeleton_ = definitionRecordClassAccess_.createClassMethodFinal(this.nameBase_);
      }
      return this.methodSkeleton_;
    }
    // ------------------------------------------------------------------------
    
    

    public void end() { }
    public JavaCodeBlockPatternPeer addJavaCodeBlockForCode() {
      return this.methodType_.addJavaCodeBlockForCodeMaybeNull();
    }

		public NamePeer.BasicPatternPeer addOptionBasicOfNameForNameOutput() { 
			final NameBlock nameBlock = new NameBlock(this.error_);
			this.name_ = new NameHandler() {

		    public void updateInstanceNameSkeleton(String defaultName,   JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock methodSkeleton, SLayer baseEnvironment){
		    	NameDetailsFormBlock nameSkeleton = methodSkeleton.getRootBuilder().createNameDetailsChild();

		    	methodSkeleton.addNormalName(nameSkeleton);
		    	nameBlock.updateInstanceNameSkeleton(defaultName, nameSkeleton, baseEnvironment);

				}
				
			};
			return nameBlock; 
		}

		public void addOptionImplementsOfNameForNameOutput(final int keywordLineNumber, final int keywordCharacterNumber, final String classNameParameter, final String methodNameParameter, final int methodNameLineNumber, final int methodNameCharacterNumber){
			ImplementsNameHandler nameHandler = new ImplementsNameHandler(classNameParameter, methodNameParameter,this.error_);
			this.name_ = nameHandler;
		}
    public TargetParameterPeer.Indirect getTargetParameterForParameter() {
    	return this.parametersBlock_;
    }
    public TargetTypeReferencePatternPeer addTargetTypeReferenceForReturnType(String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
      SpecificCommonErrorOutput error = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
      this.returnTypeBlock_ = new TargetTypeBlock(typeNameParameter,error);
      return returnTypeBlock_;
    }
    public ChainHookListPatternPeer addChainHookListForChainHookList() {
			return this.chainHookList_ = new ChainHookListBlock(this.error_);
		}
		public TargetTypeListPatternPeer addTargetTypeListForThrowsList() {
			return this.throwsList_ = new TargetTypeListBlock(this.error_);
		}
		public SetsListPatternPeer addSetsListForMethodSets() {
			return this.globalSets_ = new MethodSetsBlock(this);
		}  
    // ------------------------------------------------------------------------
    // TargetMethod stuff

    public TargetTypeReference getReturnTargetTypeReferenceQuietFinal() { return this.returnTypeBlock_.getResolvedTypeReferenceQuietFinal();   }
   
    public ArgumentSetDetailsFormBlock createExpressionFinal( ExpressionDetailsFormBlock target, JavaVariablePath basePathDisposable) {
    	BaseBuilder rootBuilder = target.getRootBuilder();
    	ArgumentSetDetailsFormBlock argumentSetSkeleton = rootBuilder.createArgumentSetDetailsChild();

    	BasicCallChainFormBlock callChainSkeleton = rootBuilder.createBasicCallChainChild();
			target.addCallChain(callChainSkeleton);
			callChainSkeleton.addElement().addMethod(getMethodSkeleton().getVariableStoreParameter(), basePathDisposable,argumentSetSkeleton);
      return argumentSetSkeleton;
    }
    public ArgumentSetDetailsFormBlock createCallExpressionFinal(BasicCallChainFormBlock target, JavaVariablePath basePathDisposable) {
    	ArgumentSetDetailsFormBlock argumentSetSkeleton = target.getRootBuilder().createArgumentSetDetailsChild();
			BasicCallChainFormBlock.ElementDataBlock.MethodDataBlock methodBlock =target.addElement().addMethod(getMethodSkeleton().getVariableStoreParameter(), basePathDisposable,argumentSetSkeleton);
      return argumentSetSkeleton;
  	}
    public void addIsFinal(int lineNumber, int characterNumber) {
			Debug.finishMeMarker();
			// TODO Auto-generated method stub	
		}

		public void addIsStatic(int lineNumber, int characterNumber) {
			Debug.println("ADD IS STATIC!");
			this.isStatic_ = true;
		}

		public void addIsPrivate(int lineNumber, int characterNumber) { 	this.isPublic_ = false; 	}
		public void addIsPublic(int lineNumber, int characterNumber) { this.isPublic_ = true; }
		public GlobalMethodReference getAsGlobalQuiet() { return null; }
		public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock.ImplementsNameDataBlock addMethodImplementsNameBlock( JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock target, JavaVariablePath methodDeclarationPathChainTracker) throws ParsingException {
			return target.addImplementsName(getMethodSkeleton().getVariableStoreParameter(), methodDeclarationPathChainTracker);
		}
		
  } //End of MethodElement
  // ==========================================================================
  private static interface NameHandler {
    public void updateInstanceNameSkeleton(String defaultName,   JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock  methodSkeleton,SLayer baseEnvironment);

  }
  private final static class ImplementsNameHandler implements NameHandler {
  	private final SpecificCommonErrorOutput	error_;
		private final String	methodName_;
		private final String	className_;
		public ImplementsNameHandler(String className, String methodName,SpecificCommonErrorOutput error) {
			this.className_ = className;
			this.methodName_ = methodName;
			this.error_ = error;
  		
  	}

		public void updateInstanceNameSkeleton(String defaultName, JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock methodSkeleton, SLayer baseEnvironment) {
			
			BaseTrackers.JavaVariablePath chainToVariable = new BaseTrackers.JavaVariablePath();
			TargetType foundType = baseEnvironment.getTargetTypeQuiet(className_, chainToVariable , true);
			if(foundType==null) {
				this.error_.objectNotFoundError("interface", className_);
			} else {
				//We don't need the chain to the type declaration
				chainToVariable.clear();
				TargetMethod foundMethod = baseEnvironment.getTargetMethodQuiet(foundType, methodName_, chainToVariable, true);
				if(foundMethod==null) {
					this.error_.objectNotFoundError("method in type:"+className_, methodName_);
				} else {
					try {
						foundMethod.addMethodImplementsNameBlock(methodSkeleton, chainToVariable);
					
						//methodSkeleton.addImplementsNameBlock(baseMethodParameter, methodDeclarationPathChainTracker)
					} catch (ParsingException e) {
						e.updateError(this.error_);
					}
				}
			}
		}
		
  }

  // ==========================================================================

  private final static class MethodSetsBlock implements SetsListPatternPeer {
  	private final MethodElement	parent_;
  	private ToSet toSetTail_;
		public MethodSetsBlock(MethodElement parent) {
			this.parent_ = parent;
  		
  	}
		public void basicBuild(RecordClassAccess recordClassAccess) {
			if(toSetTail_!=null) {
				toSetTail_.basicBuild(recordClassAccess);
			}
		}
		public void finalBuild(JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock methodSkeleton) {
			if(toSetTail_!=null) {
				toSetTail_.finalBuild(methodSkeleton);
			}
		}
		//------------------------------------------------------------------------
		public void end() { }

		public void addClassName(String value, int lineNumber, int characterNumber) {
			SpecificCommonErrorOutput error = this.parent_.error_.createAdjusted(lineNumber, characterNumber);
			this.toSetTail_ = new ToSet(value,error, this.toSetTail_);
		}
		private static final class ToSet {
			private final String	methodName_;
			private final ToSet	next_;
			private final SpecificCommonErrorOutput	error_;
			private final BaseTrackers.JavaVariablePath variablePath_ = new BaseTrackers.JavaVariablePath();
			private GlobalMethodReference foundMethod_;
			private GlobalClassReference foundClass_;
			public ToSet(String methodName,SpecificCommonErrorOutput error, ToSet next) {
				this.methodName_ = methodName;
				this.error_ = error;
				this.next_ = next;
			}
			public final void basicBuild(RecordClassAccess environment) {
				if(next_!=null) {
					next_.basicBuild(environment);
				}
				final GlobalClassReference[] allGlobalClasses = environment.getAllGlobalClassReferences();
				if(allGlobalClasses==null) {
					error_.generalSyntaxError("No global classes set in context");
					//error_.objectNotFoundError("global method ", methodName_);//Might want to change to say that no global classes set, if that were the case (as opposed to no global classes specified were found)
				} else {
					for(int i = 0 ; i < allGlobalClasses.length ; i++) {
						final GlobalClassReference current = allGlobalClasses[i];
						this.variablePath_.clear();
						GlobalMethodReference found = environment.getBaseLayer().getGlobalMethodReferenceQuiet(current,this.methodName_, this.variablePath_,true);
						if(found!=null) {
							this.foundMethod_ = found;
							this.foundClass_ = current;
							break;
						} 
					}
				}
				if(this.foundMethod_==null) {
					this.error_.objectNotFoundError("global method", methodName_);
				}
			}	//end of method basicBuild();
			public void finalBuild(JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock methodSkeleton) {
				if(next_!=null) {
					next_.finalBuild(methodSkeleton);
				}
				this.foundMethod_.updateSetGlobalFinal(methodSkeleton, this.variablePath_);
				
			} //End of method finalBuild()
		}//eND OF CLASS ToSet
  } //End of Class MethodSetsBlock
  
  
  // --------------------------
  
  private final static class MacroCallElement implements Element {
    private final FileEnvironment	fileEnvironment_;
		private final SpecificCommonErrorOutput	error_;
		private final GeneralMacroCall baseCall_;
		public MacroCallElement(GeneralMacroCall baseCall,FileEnvironment fileEnvironment,  SpecificCommonErrorOutput error) {
			this.baseCall_ = baseCall;
			this.fileEnvironment_ = fileEnvironment;
			this.error_ = error;

    }

		public void basicBuild(RecordClassAccess classRecordAccess) { }

		public void finalBuild(RecordClassAccess classRecordAccess) {
			this.baseCall_.finishBuildInClass(classRecordAccess);			
		}
		public void basicBuild(RecordContainerAccess containerRecordAccess) {
			error_.systemError("unimplemented feature", "can not provide general macros within macros");
			Debug.finishMeMarker();
		}
		public void finalBuild(RecordContainerAccess containerRecordAccess) { 
			//TODO: will need to pass package subname as macro parameter, or have sub name set within macro
			// I suspect this works fine once a class has been define in the macro...
			
			//this.baseCall_.finishBuildInGeneralContainer(containerRecordAccess );			
		}
		

  }
  // --------------------------
  private final static class ConstructorElement implements Element,JavaConstructorPatternPeer, BlockUser, JavaConcreteChainLink {
    private final SpecificCommonErrorOutput error_;
    private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ConstructorDataBlock constructorSkeleton_;
    private RecordClassAccess definitionRecordClassAccess_;
    private final TargetParametersBlock parametersBlock_ ;
    private TargetTypeListBlock throwsList_ = null;
    private final JavaCodeBlock codeBlock_;
    public ConstructorElement( BaseBuilder rootBuilder, SpecificCommonErrorOutput error) {
	    this.error_ = error;
	    this.codeBlock_ = new JavaCodeBlock(rootBuilder,error);
			this.parametersBlock_ = new TargetParametersBlock(error);
		}
  	public void basicBuild(RecordContainerAccess containerRecordAccess) {
			this.error_.generalSyntaxError("wrong context for constructors");
		}
		public void finalBuild(RecordContainerAccess containerRecordAccess) { }
		
    public String toString() { return "JC Constructor Element"; }
    // ---------------------------------------------------------------------
    // BlockUser stuff
    public void newLink(String chainName, String linkName, SpecificCommonErrorOutput error) {
			try {
    		definitionRecordClassAccess_.getBaseLayer().addConcreteChainLink(chainName, linkName, this, error);
			} catch(ParsingException e) {
				e.updateError(error);
			}
		}
    //JavaConcreteChainLink stuff
		public void updateChainLink(LinkRefDataBlock targetSkeleton) {
			Debug.finishMeMarker();
    	//I don't think this ever happens in the current version
    	//targetSkeleton.addInstanceMethodBlock(this.constructorSkeleton_);
		}
    // =====================================================================
    // Building stuff
    
    public void basicBuild(RecordClassAccess recordClassAccess) {
      this.definitionRecordClassAccess_ = recordClassAccess;
      CodeBlockEnvironment environment = recordClassAccess.createRootCodeBlock();
      
      this.codeBlock_.basicBuild(environment);
      if(this.parametersBlock_!=null) {
      	this.parametersBlock_.basicBuildMethod(environment);
      }
      if(this.throwsList_!=null) {
      	this.throwsList_.basicBuild(environment.getBaseLayer());
      }
    } //end of Method basicBuild()

    public void finalBuild(RecordClassAccess recordClassAccess) {
    	JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ConstructorDataBlock constructorSkeleton = getConstructorSkeleton();
      BaseBuilder rootBuilder = constructorSkeleton.getRootBuilder();
    	this.codeBlock_.finishBuild(constructorSkeleton.getBaseCodeBlockMacro());
      if(this.parametersBlock_!=null) {
				parametersBlock_.finishBuildMethod(constructorSkeleton.getParameterSetMacro());
			}
      
      if(this.throwsList_!=null) {
      	this.throwsList_.finalBuild(constructorSkeleton.addThrows(rootBuilder.createTypeListDetailsChild()).getTypeListDetailsParameter());
      }
    } //End of method finalBuild()
    
    // -------------------------------------------------------------------------------------------------
    public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ConstructorDataBlock getConstructorSkeleton() {
      if(constructorSkeleton_==null) {
      	constructorSkeleton_ = definitionRecordClassAccess_.createClassConstructorFinal();
      }
      return this.constructorSkeleton_;
    }
    // ------------------------------------------------------------------------
    
    

    public void end() { }
    public JavaCodeBlockPatternPeer addJavaCodeBlockForCode() {
      return this.codeBlock_.createJavaCodeBlockPeer();
    }
 
    public TargetParameterPeer.Indirect getTargetParameterForParameter() {
    	return this.parametersBlock_;
    }
   
		public TargetTypeListPatternPeer addTargetTypeListForThrowsList() {
			return this.throwsList_ = new TargetTypeListBlock(this.error_);
		}
    // ------------------------------------------------------------------------
    // TargetMethod stuff



//   
//    public ArgumentSetCodeCode createExpressionFinal(ExpressionCode target, JavaVariablePath basePathDisposable) {
//      ExpressionCode.CallChainDataBlock.ElementDataBlock.MethodDataBlock methodBlock =target.addCallChainBlock().addElementBlock().addConstructorBlock(getConstructorSkeleton(), basePathDisposable);
//      return methodBlock.getCodeFixtureSetArguments();
//    }
//    public ArgumentSetCodeCode createCallExpressionFinal(BasicCallChainTemplate target, JavaVariablePath basePathDisposable) {
//    	BasicCallChainTemplate.ElementTemplate.MethodTemplate methodBlock =target.addElementBlock().addMethodBlock(getMethodSkeleton(), basePathDisposable);
//      return methodBlock.getSetArgumentsTargetFixture();
//  	}   
  }


	
	


}
