package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.austenx.peg.base.DeferredJavaStatement;
import org.ffd2.skeletonx.austenx.peg.base.FindDefinitionPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.ForInitialStatementPeer;
import org.ffd2.skeletonx.austenx.peg.base.GeneralMacroCallPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaCodeBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaExpressionPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaExpressionPeer.Indirect;
import org.ffd2.skeletonx.austenx.peg.base.JavaStatementPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaStatementPeer.SetPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaStatementPeer.SyncPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.LinkChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.NamePeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferencePatternPeer;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder;
import org.ffd2.skeletonx.skeleton.BaseBuilder.CodeBlockMacroBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder.CodeBlockMacroBlock.StatementDataBlock.LinkedCodeDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.GeneralExpressionDefinitionDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.LinkedCodeBlockRefDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedForBlockStoreDataBlock;
import org.ffd2.skeletonx.skeleton.NameDetailsFormHolder.NameDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleLinkedListX;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;


public class JavaCodeBlock implements JavaStatementPeer.Indirect{
  private final SpecificCommonErrorOutput error_;

  private final SimpleVector<Statement> statements_ = new SimpleVector<Statement>();
  private CodeBlockEnvironment usedEnvironment_;
  
  private static int localVariableIndexCount_ = 0;

	private final BaseBuilder builderRoot_;
  public JavaCodeBlock(BaseBuilder baseRoot, SpecificCommonErrorOutput error) {
    this.builderRoot_ = baseRoot;
		this.error_ = error;
  }

  
  public void basicBuild(CodeBlockEnvironment environment ) {
    this.usedEnvironment_ = environment;
    for(Statement s : this.statements_) {
      s.basicBuild(environment);
    }
  }
  public void finishBuild(CodeBlockMacroBlock baseCodeSkeleton) {
    for(Statement s : this.statements_) {
      s.finishBuild(this.usedEnvironment_, baseCodeSkeleton);
    }
  }
	public SyncPatternPeer createSync(int keywordLine, int keywordCharacter) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLine, keywordCharacter);
		SyncStatement handler = new SyncStatement(this.builderRoot_, error);
		addStatement(handler);
		return handler;
	}
	public SetPatternPeer createSet(String variableName, int variableNameLine, int variableNameCharacter) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(variableNameLine, variableNameCharacter);
		SetStatement handler = new SetStatement(variableName,this.builderRoot_, error);
		addStatement(handler);
		return handler;
	}
  
  public void createNote(final String noteValue) {
		addStatement(new Statement() {

			public void basicBuild(CodeBlockEnvironment environment) { }

			public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
				baseCodeSkeleton.addStatement().addNoteStatement(noteValue);
			}
			
		});
	}
	public JavaStatementPeer.ExpressionVarPatternPeer createExpressionVar( String variableNameParameter, int variableNameLineNumber, 	int variableNameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(variableNameLineNumber, variableNameCharacterNumber);
		ExpressionVarHandler handler = new ExpressionVarHandler(variableNameParameter, error);
		addStatement(handler);

		return handler;
	}
	public JavaStatementPeer.StatementVarPatternPeer createStatementVar( String variableNameParameter, int variableNameLineNumber, 	int variableNameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(variableNameLineNumber, variableNameCharacterNumber);
		Debug.finishMeMarker();
		error.systemError("To be completed", "statement variables not implemented yet");
		// TODO Auto-generated method stub
		return null;
	}

  public void end() { }
  
	public JavaStatementPeer.ForPatternPeer createFor(int keywordLineNumber, int keywordCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		ForHandler handler = new ForHandler(this.builderRoot_,error);
		addStatement(handler);
		return handler;
	}
	
	public JavaStatementPeer.IfPatternPeer createIf(int keywordLineNumber, int keywordCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		IfStatement s = new IfStatement(this.builderRoot_,error);
		addStatement(s);
		return s;
	}
	public JavaStatementPeer.ReturnPatternPeer createReturn(int keywordLineNumber, int keywordCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		ReturnThrowNormal result = new ReturnThrowNormal(true,error);
		addStatement(result);
		return result;
	}

	public void createMark(final String markNameParameter, final int markNameLineNumber, final int markNameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(markNameLineNumber, markNameCharacterNumber);
		MarkStatement result = new MarkStatement(markNameParameter,error);
		addStatement(result);
	}
	
	public void createReturnSimple(int keywordLineNumber, int keywordCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		addStatement(new Statement() {
			public void basicBuild(CodeBlockEnvironment environment) { }
			public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
				baseCodeSkeleton.addStatement().addReturnSimple();
			}
		});
	}

	public JavaStatementPeer.ThrowPatternPeer createThrow(int keywordLineNumber, int keywordCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		ReturnThrowNormal result = new ReturnThrowNormal(false,error);
		addStatement(result);
		return result;
	}

	public JavaStatementPeer.WhilePatternPeer createWhile(int keywordLineNumber, int keywordCharacterNumber) {
		final SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		WhileStatement s = new WhileStatement(this.builderRoot_,error);
		addStatement(s);
		return s;
	}
	
  
  public JavaStatementPeer.Indirect getJavaStatementForStatement() { return this;  }
  
  public JavaStatementPeer.BlockPatternPeer createBlock() {
    final JavaCodeBlock sub = new JavaCodeBlock(this.builderRoot_,this.error_);
    this.addStatement(new Statement() {
      public void basicBuild(CodeBlockEnvironment environment) {
        sub.basicBuild(environment.createSubBlockEnvironment());
      }
      public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) { sub.finishBuild(baseCodeSkeleton); }
    });
    return new JavaStatementPeer.BlockPatternPeer() {
      public JavaCodeBlockPatternPeer addJavaCodeBlockForValue() { return sub.createJavaCodeBlockPeer(); }
      public void end() { }
    };   
  }
  public JavaCodeBlockPatternPeer createJavaCodeBlockPeer() {
  	return new JavaCodeBlockPatternPeer() {
			public void end() { }
			public JavaStatementPeer.Indirect getJavaStatementForStatement() {
				return JavaCodeBlock.this;
			}
  	}; 
  }
  private final void addStatement(Statement s) {
    this.statements_.addElement(s);
  }
  public GeneralMacroCallPatternPeer createMacroCall(int keywordLineNumber,
			int keywordCharacterNumber, String macroNameParameter) {
		SpecificCommonErrorOutput macroError = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		GeneralMacroCall baseCall = new GeneralMacroCall(macroNameParameter,macroError);
		MacroCall mc = new MacroCall(baseCall,macroError);
		addStatement(mc);
		return baseCall;
	}
	public JavaStatementPeer.MacroCallPatternPeer createMacroCall() {
		return new JavaStatementPeer.MacroCallPatternPeer() {
			public void end() { }
			public GeneralMacroCallPatternPeer addGeneralMacroCallForCall( int keywordLineNumber, int keywordCharacterNumber, String macroNameParameter) {
				return createMacroCall(keywordLineNumber, keywordCharacterNumber,  macroNameParameter);
			}
		};
	}
  public JavaStatementPeer.ExpressionPatternPeer createExpression() {
    ExpressionStatement es = new ExpressionStatement(this);
    addStatement(es);
    return es;
  }
  public JavaStatementPeer.DeclarationPatternPeer createDeclaration(final String nameParameter, final int nameLineNumber, final int nameCharacterNumber) {
  	SpecificCommonErrorOutput error = this.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
  	DeclarationStatement ds = new DeclarationStatement(nameParameter, this.builderRoot_,error);
  	addStatement(ds);
  	return ds;
  }
  
	public JavaStatementPeer.LinkBlockPatternPeer createLinkBlock() {
		LinkBlockStatement lb = new LinkBlockStatement(this.builderRoot_, this.error_ );
  	addStatement(lb);
  	return lb;
	}
  // ==========================================================================
  private static interface Statement {
    public void basicBuild(CodeBlockEnvironment environment);
    public void finishBuild(CodeBlockEnvironment environment,CodeBlockMacroBlock baseCodeSkeleton);
    
  }
  //===========================================================================
  private static class LinkedBuild {
  	private CodeBlockEnvironment	linkedEnvironment_;
		private LinkedBuild	previous_;
		private final JavaCodeBlock	linkedCodeBlock_;

		public LinkedBuild(CodeBlockEnvironment linkedEnvironment,JavaCodeBlock linkedCodeBlock, LinkedBuild previous) {
			this.linkedEnvironment_ = linkedEnvironment;
			this.linkedCodeBlock_ = linkedCodeBlock;
			this.previous_ = previous;
  		
  	}
		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			if(previous_!=null) {
				this.previous_.finishBuild(environment, baseCodeSkeleton);
			}
			JavaImplementationFormBlock.BlockDataBlock.LinkedCodeBlockStoreDataBlock  linkedCodeSkeleton = 
					environment.allocateLinkCodeBlock();
			baseCodeSkeleton.addStatement().addLinkedCode(linkedCodeSkeleton.getVariableStoreParameter(), new BaseTrackers.JavaVariablePath());
			
			BaseTrackers.JavaVariablePath pathToBackBaseEnvironment = new BaseTrackers.JavaVariablePath();
			this.linkedEnvironment_.getBaseLayer().buildPathToUpperLayer(environment.getBaseLayer(), pathToBackBaseEnvironment);
			
			LinkedCodeBlockRefDataBlock linkedCodeBlockRef = this.linkedEnvironment_.getBaseLayer().addItemSkeleton().addLinkedCodeBlockRef(linkedCodeSkeleton.getVariableStoreParameter(),pathToBackBaseEnvironment);
			this.linkedCodeBlock_.finishBuild(linkedCodeBlockRef.getLinkCodeBlockMacro());
		}
  }
  private static final class LinkBlockStatement implements Statement,JavaStatementPeer.LinkBlockPatternPeer {
  	private final SpecificCommonErrorOutput error_;
  	private LinkChainBlock lastLinkBlock_;
  	//private CodeBlockEnvironment linkedEnvironment_;
  	private final SimpleVector<DeferredJavaStatement> deferredStatements_ = new SimpleVector<DeferredJavaStatement>();
  	private LinkedBuild lastLinkedBuild_ = null;
		private final BaseBuilder builderRoot_;
		public LinkBlockStatement(BaseBuilder builderRoot, SpecificCommonErrorOutput error) {
			this.builderRoot_ = builderRoot;
			this.error_ = error;

  	}
		public LinkChainPatternPeer addLinkChainForSingleChain() {
			return this.lastLinkBlock_ = new LinkChainBlock(this.error_,this.lastLinkBlock_);
		}
		public void setJavaStatementForStatement(DeferredJavaStatement value) {
			this.deferredStatements_.addElement(value);
		}
		public void addIsWithSubBlock() {
			// TODO: ignore for now...
		}
		private JavaCodeBlock buildLinkCodeBlock() {
			JavaCodeBlock result =  new JavaCodeBlock(this.builderRoot_,this.error_);
			for(DeferredJavaStatement statement : this.deferredStatements_) {
				statement.buildJavaStatement(result);
			}
			return result;
		}
		public void basicBuild(CodeBlockEnvironment environment) {
			LinkChainBlock currentLinkBlock = this.lastLinkBlock_;
			while(currentLinkBlock!=null) {
				CodeBlockEnvironment linkedEnvironment = environment.getLinkEndQuiet(currentLinkBlock);
				if(linkedEnvironment!=null) {
					JavaCodeBlock linkedCodeBlock =  buildLinkCodeBlock();

					linkedCodeBlock.basicBuild(linkedEnvironment);
					this.lastLinkedBuild_ = new LinkedBuild(linkedEnvironment,linkedCodeBlock,this.lastLinkedBuild_);
				}
				currentLinkBlock = currentLinkBlock.getPrevious();
			}
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			if(this.lastLinkedBuild_!=null) {
				this.lastLinkedBuild_.finishBuild(environment, baseCodeSkeleton);
			}
		}

//		public LinkChainPatternPeer addLinkChainForChain() {
//			return this.linkBlock_;
//		}
//		public JavaStatementPeer.Indirect getJavaStatementForStatement() {
//			return this.linkedCodeBlock_;
//		}
		public void end() {	}
  }
  //===========================================================================
  
  private static final class DeclarationStatement implements TargetVariable, Statement, JavaStatementPeer.DeclarationPatternPeer {
  	private final SpecificCommonErrorOutput error_;
		private final String variableName_;
		private ExpressionBlock value_;
		private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.LocalVariableDataBlock  variableSkeleton_;
		private NameBlock nameBlock_;
		private TargetTypeBlock typeBlock_;
		private boolean isFinal_ = false;
		private final BaseBuilder builderRoot_;
		private final NameDetailsFormBlock nameDetailsSkeleton_;
		private final TypeSetterDetailsFormBlock typeDetailsSkeleton_;
		public DeclarationStatement(String variableName, BaseBuilder builderRoot, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.builderRoot_ = builderRoot;
			this.error_ = error;
			this.nameDetailsSkeleton_ = builderRoot.createNameDetailsChild();
			this.typeDetailsSkeleton_ = builderRoot.createTypeSetterDetailsChild();
		}  	

		public void basicBuild(CodeBlockEnvironment environment) {
			if(this.value_!=null) {
				this.value_.basicBuild(environment);
			}
			this.variableSkeleton_ = environment.getBaseLayer().addItemSkeleton().addLocalVariable(this.variableName_,localVariableIndexCount_++,this.builderRoot_.createCodeVariableChild(),this.nameDetailsSkeleton_,this.typeDetailsSkeleton_);
			try {
				environment.addLocalVariable(this.variableName_,this);
			} catch(ItemAlreadyExistsException e) {
				this.error_.repeatedObjectError("local variable", this.variableName_);
			}
			this.typeBlock_.basicBuild(environment.getBaseLayer());
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			CodeBlockMacroBlock.StatementDataBlock.LocalVariableDataBlock codeSkeleton = baseCodeSkeleton.addStatement().addLocalVariable(this.variableSkeleton_);
			if(this.value_!=null) {
				this.value_.finish(environment,codeSkeleton.addInitialValue(this.builderRoot_.createExpressionDetailsChild()).getExpressionDetailsParameter());
			}
			if(this.isFinal_) {
				codeSkeleton.addIsFinal();
			}
			if(this.nameBlock_==null ) {
				this.nameDetailsSkeleton_.addSimple(this.variableName_);
			} else {
				this.nameBlock_.updateInstanceNameSkeleton(this.variableName_, this.nameDetailsSkeleton_, environment.getBaseLayer());
			}
			this.typeBlock_.buildCodeSetTypeFinal(this.typeDetailsSkeleton_);
		}
		// ===================== Peer stuff ===============

		public void addIsFinal() { this.isFinal_ = true; }


			
			
		public NamePeer.BasicPatternPeer addOptionBasicOfNameForNameOutput() { return this.nameBlock_ = new NameBlock(this.error_); }

		public void addOptionImplementsOfNameForNameOutput(final int keywordLineNumber, final int keywordCharacterNumber, final String classNameParameter, final String methodNameParameter, final int methodNameLineNumber, final int methodNameCharacterNumber) {
			this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber).generalSyntaxError("implementing name not applicable to declaration");
		}
		public TargetTypeReferencePatternPeer addTargetTypeReferenceForType(String typeNameParameter,
				int typeNameLineNumber, int typeNameCharacterNumber) {
			SpecificCommonErrorOutput typeError = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
			return this.typeBlock_ = new TargetTypeBlock(typeNameParameter, this.error_);
		}
		
		public void end() {
			
		}

		public JavaExpressionPeer.Indirect getJavaExpressionForInitialValue() {
			return this.value_ = new ExpressionBlock(this.error_);
		}
		
		// ----------------------------------------------------------------------------------------
		// TargetVariable stuff

		public void createCallExpressionFinal(BasicCallChainFormBlock target,
				JavaVariablePath basePathDisposable) {
			target.addElement().addVariable(this.variableSkeleton_.getVariableStoreParameter(), basePathDisposable);
		}

		public void createExpressionFinal(ExpressionDetailsFormBlock target, JavaVariablePath basePathDisposable) {
			target.addCallChain(target.getRootBuilder().createBasicCallChainChild()).getBasicChainParameter().addElement().addVariable(this.variableSkeleton_.getVariableStoreParameter(), basePathDisposable);
			
		}

		public void createMacroCallParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath pathToVariable) {
			macroCallBlock.addCallParameter().addTargetVariable().addSimple(this.variableSkeleton_.getVariableStoreParameter(), pathToVariable);
		}

		public TargetTypeReference getTargetTypeReferenceQuietFinal() {
			return this.typeBlock_.getResolvedTypeReferenceQuietFinal();
		}
  }
  //==========================================================================
  private static final class MacroCall implements Statement {
		private final SpecificCommonErrorOutput error_;
	  private final GeneralMacroCall baseCall_;
		public MacroCall(GeneralMacroCall baseCall,  SpecificCommonErrorOutput error) {
			this.baseCall_ = baseCall;
			this.error_ = error;
  	}
		public void basicBuild(CodeBlockEnvironment environment) { 			
		}
		public void finishBuild(final CodeBlockEnvironment environment, final CodeBlockMacroBlock baseCodeSkeleton) {
			baseCall_.finishBuild(environment, baseCodeSkeleton);
		}
  }
  // --------------------------------------------------------------------------
//  private static interface CallParameterValue {
//  	public void doAddTargetVarRequirement(TargetType varType,MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment);
//  	public void doAddRecordVarRequirement(FoundationNode recordType,MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment);
//  	public void doAddCodeReferenceRequirement(MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment);
//  	public void doAddArgValuesMarkRequirement(MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment);
//
//  }

  // =================================================================================================
//  private final static class TargetVariableCallParameter implements MacroCallParameterPeer.TargetVariablePatternPeer, CallParameterValue {
//  	private final SpecificCommonErrorOutput error_;
////  	private BuilderVariableChainBlock chain_ = null; //Null by default
////		private final String targetVariableName_;
//		private final XCodeCallChainBlock chainBlock_;
//		public TargetVariableCallParameter( SpecificCommonErrorOutput error) {
//
//			this.error_ = error;
//			this.chainBlock_ = new XCodeCallChainBlock(error);
//		}
//		public JavaCallChainPatternPeer addJavaCallChainForCallChain() {
//			return this.chainBlock_;
//		}
//
////		public BuilderVariableChainPatternPeer addBuilderVariableChainForBuilderChain() {
////			return this.chain_ = new BuilderVariableChainBlock(error_); //Move chain from being null
////		}
//
//		public void end() { }
//		//The varType is for type checking.
//		public void doAddTargetVarRequirement(TargetType varType,
//				MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {
//
//			this.chainBlock_.doAddTargetVarRequirement(varType, macroCallBlock, environment);
//			
//
////			JavaVariablePath pathToVariableChainTracker = new BaseTrackers.JavaVariablePath();
////			final VariableAccess foundVariable;
////			if(this.chain_!=null) {
////				RedirectingTargetRecordAccess resolvedRecord = chain_.resolveRecordQuiet(environment.getBaseRecord(), pathToVariableChainTracker);
////				if(resolvedRecord!=null) {
////					foundVariable = resolvedRecord.getTargetVariableQuiet(environment.getEnclosingTargetType(), this.targetVariableName_, pathToVariableChainTracker);
////				} else {
////					foundVariable = null;
////				}
////			} else {
////				foundVariable = environment.getTargetVariableQuiet(this.targetVariableName_, pathToVariableChainTracker);
////			}
////			if(foundVariable!=null) {
////				foundVariable.getVariable().createMacroCallParameter(macroCallBlock, pathToVariableChainTracker);
////			}
//		}
//
//		public void doAddRecordVarRequirement(FoundationNode requiredRecordType, MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {
//			this.chainBlock_.doAddRecordVarRequirement(requiredRecordType, macroCallBlock, environment.getBaseLayer());
//		}
//		public void doAddCodeReferenceRequirement(MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {
//			this.chainBlock_.doAddRequirement( new RecordVariableActions.GetCodeReferenceAction(macroCallBlock), "code reference", environment.getBaseLayer());
//		}
//		public void doAddArgValuesMarkRequirement(MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {
//			this.chainBlock_.doAddRequirement( new RecordVariableActions.GetArgValuesMarkAction(macroCallBlock), "argument values mark", environment.getBaseLayer());
//		}
//		
//		
//  }
// 
  // ==========================================================================
  
  // ==========================================================================
  private static final class ExpressionStatement implements Statement,JavaStatementPeer.ExpressionPatternPeer {
    private final JavaCodeBlock parent_;
    private final ExpressionBlock expression_;
    public ExpressionStatement(JavaCodeBlock parent) {
      this.parent_ = parent;
      this.expression_ = new ExpressionBlock(parent_.error_);
    }
    public void basicBuild(CodeBlockEnvironment environment) {
      this.expression_.basicBuild(environment);
    }
    public void finishBuild(CodeBlockEnvironment environment,CodeBlockMacroBlock baseCodeSkeleton) {
      CodeBlockMacroBlock.StatementDataBlock.ExpressionStatementDataBlock statementSkeleton = baseCodeSkeleton.addStatement().addExpressionStatement(baseCodeSkeleton.getRootBuilder().createExpressionDetailsChild());
      ExpressionDetailsFormBlock expressionSkeleton = statementSkeleton.getExpressionDetailsParameter();
     this.expression_.finish(environment,expressionSkeleton);  
    }
    public void end() {     }

    public JavaExpressionPeer.Indirect getJavaExpressionForValue() {
      return this.expression_;
    }
    
  }
  // ========================================================
  private final static class ForHandler implements JavaStatementPeer.ForPatternPeer, Statement,ForInitialStatementPeer.Indirect {
		private final ExpressionBlock conditionalExpression_;
		private final ExpressionBlock roundExpression_;
		private final JavaCodeBlock loopCode_;
		private CodeBlockEnvironment loopEnvironment_;
		private SimpleLinkedListX<InitialHandler> initials_ = null;
		private final SpecificCommonErrorOutput error_;
		private LinkedForBlockStoreDataBlock	forBlockSkeleton_;
		private final ExpressionDetailsFormBlock loopExpressionDetailsSkeleton_;
		private final ExpressionDetailsFormBlock terminateExpressionDetailsSkeleton_;
		private final BaseBuilder builderRoot_;

		public ForHandler(BaseBuilder builderRoot, SpecificCommonErrorOutput error) {
			this.builderRoot_ = builderRoot;
			this.error_ = error;
			this.terminateExpressionDetailsSkeleton_ = builderRoot.createExpressionDetailsChild();
			this.loopExpressionDetailsSkeleton_ = builderRoot.createExpressionDetailsChild();
			
			this.conditionalExpression_ = new ExpressionBlock(error);
			this.roundExpression_ = new ExpressionBlock(error);
			this.loopCode_ = new JavaCodeBlock(builderRoot,error);
		}

		public void end() { }

		public ForInitialStatementPeer.Indirect getForInitialStatementForInitialStatement() { return this; }
		public JavaExpressionPeer.Indirect getJavaExpressionForConditionalExpression() { return conditionalExpression_; }

		public JavaExpressionPeer.Indirect getJavaExpressionForRoundExpression() { return roundExpression_; }

		public JavaStatementPeer.Indirect getJavaStatementForLoopStatement() {
			return loopCode_;
		}
		public void basicBuild(CodeBlockEnvironment environment) {
			this.loopEnvironment_ = environment.createSubBlockEnvironment();
			this.conditionalExpression_.basicBuild(this.loopEnvironment_);
			this.roundExpression_.basicBuild(this.loopEnvironment_);
			this.loopCode_.basicBuild(this.loopEnvironment_);
			this.forBlockSkeleton_ = environment.getBaseLayer().allocateForBlock();
			SimpleLinkedListX<InitialHandler> current = this.initials_;
			while(current!=null) {
				current.getLinkRelatedData().basicBuild(this.loopEnvironment_);
				current = current.getPrevious();
			}
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			final CodeBlockMacroBlock.StatementDataBlock.ForBlockDataBlock forBlockSkeleton = baseCodeSkeleton.addStatement().addForBlock(
						this.forBlockSkeleton_.getVariableStoreParameter(),new BaseTrackers.JavaVariablePath(),this.loopExpressionDetailsSkeleton_,this.terminateExpressionDetailsSkeleton_
						
					);
			if(this.initials_!=null) {
				this.initials_.visitInOrder(new SimpleLinkedListX.Visitor<InitialHandler>() {
					public void visit(InitialHandler relatedData) {
						
						relatedData.finishBuild(loopEnvironment_, forBlockSkeleton);
					}
				});
			}
			this.conditionalExpression_.finish(this.loopEnvironment_, this.terminateExpressionDetailsSkeleton_);
			this.roundExpression_.finish(this.loopEnvironment_, this.loopExpressionDetailsSkeleton_);
			
			this.loopCode_.finishBuild(forBlockSkeleton.getMainCodeMacro());
			
		}

		public ForInitialStatementPeer.DeclarationPatternPeer createDeclaration(String nameParameter, int nameLineNumber, int nameCharacterNumber) {
			SpecificCommonErrorOutput declError = this.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
			DeclarationInitial di = new DeclarationInitial(nameParameter,this.builderRoot_,declError);
			addInitial(di);
			return di;
		}

		public ForInitialStatementPeer.ExpressionPatternPeer createExpression() {
			ExpressionInitial ie = new ExpressionInitial(this.error_);
			addInitial(ie);
			return ie;
		}
		private final void addInitial(InitialHandler handler) {
			this.initials_ = new SimpleLinkedListX<InitialHandler>(handler,this.initials_);
		}
		
	}
	// ------------------------------------------------
	private static interface InitialHandler {
		public void basicBuild(CodeBlockEnvironment loopEnvironment);
		public void finishBuild(CodeBlockEnvironment loopEnvironment, CodeBlockMacroBlock.StatementDataBlock.ForBlockDataBlock baseCodeSkeleton);
	}
	private static final class DeclarationInitial implements InitialHandler, ForInitialStatementPeer.DeclarationPatternPeer, TargetVariable {
		private final SpecificCommonErrorOutput error_;
		private final String variableName_;
		private final NameBlock nameBlock_;
		private final ExpressionBlock valueExpression_;
		private TargetTypeBlock typeBlock_;
		private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.LocalVariableDataBlock	variableSkeleton_;
		private final NameDetailsFormBlock nameDetailsSkeleton_;
		private final TypeSetterDetailsFormBlock typeDetailsSkeleton_;

		private final BaseBuilder builderRoot_;

		public DeclarationInitial(String variableName,BaseBuilder builderRoot, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.builderRoot_ = builderRoot;
			this.nameDetailsSkeleton_ = builderRoot.createNameDetailsChild();
			this.typeDetailsSkeleton_ = builderRoot.createTypeSetterDetailsChild();

			this.error_ = error;
			this.nameBlock_ = new NameBlock(error);
			this.valueExpression_ = new ExpressionBlock(error);
		}

		public void basicBuild(CodeBlockEnvironment loopEnvironment) {
			this.valueExpression_.basicBuild(loopEnvironment);
			this.variableSkeleton_ = loopEnvironment.getBaseLayer().addItemSkeleton().addLocalVariable(this.variableName_,localVariableIndexCount_++,this.builderRoot_.createCodeVariableChild(),nameDetailsSkeleton_,this.typeDetailsSkeleton_);

			try {
				loopEnvironment.addLocalVariable(this.variableName_, this);
			} catch(ItemAlreadyExistsException e) {
				this.error_.repeatedObjectError("target variable", this.variableName_);
			}
			this.typeBlock_.basicBuild(loopEnvironment.getBaseLayer());
		}

		public void finishBuild(CodeBlockEnvironment loopEnvironment, CodeBlockMacroBlock.StatementDataBlock.ForBlockDataBlock baseCodeSkeleton) {
			CodeBlockMacroBlock.StatementDataBlock.ForBlockDataBlock.InitialVariableDataBlock declarationSkeleton = baseCodeSkeleton.addInitialVariable(this.variableSkeleton_,builderRoot_.createExpressionDetailsChild());
			this.valueExpression_.finish(loopEnvironment, declarationSkeleton.getInitialValueExpressionDetailsParameter());
			this.nameBlock_.updateInstanceNameSkeleton(this.variableName_, this.nameDetailsSkeleton_, loopEnvironment.getBaseLayer());
			this.typeBlock_.buildCodeSetTypeFinal(this.typeDetailsSkeleton_);
		}

		public NamePeer.BasicPatternPeer addOptionBasicOfNameForNameOutput() { return this.nameBlock_; }

		public void addOptionImplementsOfNameForNameOutput(final int keywordLineNumber, final int keywordCharacterNumber, final String classNameParameter, final String methodNameParameter, final int methodNameLineNumber, final int methodNameCharacterNumber) {
			this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber).generalSyntaxError("implementing name not applicable to declaration");
		}
		public TargetTypeReferencePatternPeer addTargetTypeReferenceForType(String typeNameParameter,
				int typeNameLineNumber, int typeNameCharacterNumber) {
			final SpecificCommonErrorOutput typeError = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
			return this.typeBlock_ = new TargetTypeBlock(typeNameParameter,typeError);
		}

		public void end() { }

		public JavaExpressionPeer.Indirect getJavaExpressionForInitialValue() {
			return valueExpression_;
		}

		public void createCallExpressionFinal(BasicCallChainFormBlock target, JavaVariablePath basePathDisposable) {
			Debug.finishMeMarker();
			//TODO FIX - need to change the base bones stuff
			target.addElement().addVariable(this.variableSkeleton_.getVariableStoreParameter(), basePathDisposable);
		}

		public void createMacroCallParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath pathToVariable) {
			this.error_.systemError("Not implemented", "For variables not available for macro parameters yet");
			throw Debug.finishMeMarker();
		}
		public TargetTypeReference getTargetTypeReferenceQuietFinal() {		return this.typeBlock_.getResolvedTypeReferenceQuietFinal(); 		}

		
	}
	// ---------
	private static final class ExpressionInitial implements InitialHandler,ForInitialStatementPeer.ExpressionPatternPeer {
		private final ExpressionBlock expression_;
		public ExpressionInitial(SpecificCommonErrorOutput error) {
			this.expression_ = new ExpressionBlock(error);
		}
		public void end() { }
		public JavaExpressionPeer.Indirect getJavaExpressionForValue() { return this.expression_; }
		public void basicBuild(CodeBlockEnvironment loopEnvironment) {
			expression_.basicBuild(loopEnvironment);
		}
		public void finishBuild(CodeBlockEnvironment loopEnvironment, CodeBlockMacroBlock.StatementDataBlock.ForBlockDataBlock baseCodeSkeleton) {
			expression_.finish(loopEnvironment, baseCodeSkeleton.addInitialExpression(baseCodeSkeleton.getRootBuilder().createExpressionDetailsChild()).getExpressionDetailsParameter());
		}
	}
	// ================
	
	private final static class ReturnThrowNormal implements Statement,JavaStatementPeer.ReturnPatternPeer,JavaStatementPeer.ThrowPatternPeer{
		private final SpecificCommonErrorOutput error_;
		private final ExpressionBlock resultExpression_;
		private final boolean isReturn_; //If true then return, else throw
		public ReturnThrowNormal(boolean isReturn, SpecificCommonErrorOutput error) {
			this.error_ = error;
			this.isReturn_ = isReturn;
			this.resultExpression_ = new ExpressionBlock(error);
		}
		public void basicBuild(CodeBlockEnvironment environment) {
			this.resultExpression_.basicBuild(environment);
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			ExpressionDetailsFormBlock expressionTarget; 
			if(this.isReturn_) {
				expressionTarget = baseCodeSkeleton.addStatement().addReturnNormal(baseCodeSkeleton.getRootBuilder().createExpressionDetailsChild()).getExpressionDetailsParameter();
			} else {
				expressionTarget = baseCodeSkeleton.addStatement().addThrowNormal(baseCodeSkeleton.getRootBuilder().createExpressionDetailsChild()).getExpressionDetailsParameter();
			}
			this.resultExpression_.finish(environment, expressionTarget);
		}
		public void end() { 	}

		public JavaExpressionPeer.Indirect getJavaExpressionForResult() { return this.resultExpression_;		}
	} // End of class ReturnThrowNormal
	// ----------------------------------------------------------------------------


	// ----------------------------------------------------------------------------
	private final static class MarkStatement extends AbstractIRecordVariable implements Statement, IRecordVariable {
		private final SpecificCommonErrorOutput error_;
		private final String	variableName_;
		private JavaImplementationFormBlock.BlockDataBlock.LinkedCodeBlockStoreDataBlock	linkedCodeBlock_;
		public MarkStatement(String variableName, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.error_ = error;
		}
		@Override public boolean isMatchesVariableName(String queryVariableName) { return this.variableName_.equals(queryVariableName); }
		@Override public String getRoughVariableName() { return this.variableName_; }
		public void basicBuild(CodeBlockEnvironment environment) {
			this.linkedCodeBlock_ = environment.allocateLinkCodeBlock();
			try {
				environment.addIRecordVariable(this.variableName_, this);
			} catch (ItemAlreadyExistsException e) {
				this.error_.repeatedObjectError("record variable", this.variableName_);
			}
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			LinkedCodeDataBlock addLinkedCodeBlock = baseCodeSkeleton.addStatement().addLinkedCode(this.linkedCodeBlock_.getVariableStoreParameter(), new BaseTrackers.JavaVariablePath());
		}

		// -----------------------
		// IRecordStuff

		public SLayer getAsFoundataionNodeReferenceVariable(BaseTrackers.JavaVariablePath pathToExtend) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Code marks have no records");
	  }
		public void updateNameBlock(NameDetailsFormBlock.ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Code marks are not for names");
		}
		public void updateExpressionFinal(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Code marks are not expressions");
		}
		public void addAsMacroRecordKeyParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath variablePath, KeyDefinitionBlock type)
				throws ParsingException {
			throw ParsingException.createGeneralSyntax("Code marks cannot provide key information");		

		}
		
		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath,IVariableType type) throws ParsingException {
			if(type==IVariableType.CodeMarkReference) {
				macroCallBlock.addInCode(linkedCodeBlock_.getVariableStoreParameter(), variablePath);
			} else {
				throw ParsingException.createInvalidType("code reference", type.getName());
			}
		}
		public void updateCallChainFinal(BasicCallChainFormBlock target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Code marks are not call links");
		}
		
		public void getAsReference(IRecordTypeTarget target) {
	  	//Not a reference - yet?
	  	target.setNotReference();
	  }
		public void addAsMacroRecordMappingParameter( MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Code marks can not supply mappings");
		}
		
//		public CodeBlockCode  getInCodeBlock() {
//			return this.linkedCodeBlock_.;
//		}

	} // End of class ReturnThrowNormal
	// ----------------------------------------------------------------------------
	private static final class IfStatement implements JavaStatementPeer.IfPatternPeer, Statement {
		private final ExpressionBlock conditionalExpression_;
		private final JavaCodeBlock trueCode_;
		private JavaCodeBlock falseCode_ = null;
		private final SpecificCommonErrorOutput error_;
		private final BaseBuilder builderRoot_;
		public IfStatement(BaseBuilder builderRoot, SpecificCommonErrorOutput error) {
			this.builderRoot_ = builderRoot;
			this.error_ = error;
			this.conditionalExpression_ = new ExpressionBlock(error);
			this.trueCode_ = new JavaCodeBlock(builderRoot,error);
		}
		public void end() { }

		public JavaExpressionPeer.Indirect getJavaExpressionForConditional() { return this.conditionalExpression_; }

		public JavaStatementPeer.Indirect getJavaStatementForFalseStatement() { return this.falseCode_ = new JavaCodeBlock(this.builderRoot_, this.error_); }

		public JavaStatementPeer.Indirect getJavaStatementForTrueStatement() { return this.trueCode_; }

		public void basicBuild(CodeBlockEnvironment environment) {
			this.conditionalExpression_.basicBuild(environment);
			this.trueCode_.basicBuild(environment);
			if(this.falseCode_!=null) {
				this.falseCode_.basicBuild(environment);
			}
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			CodeBlockMacroBlock.StatementDataBlock.IfBlockDataBlock ifSkeleton = baseCodeSkeleton.addStatement().addIfBlock(baseCodeSkeleton.getRootBuilder().createExpressionDetailsChild());
			this.conditionalExpression_.finish(environment, ifSkeleton.getConditionalDetailsParameter());
			this.trueCode_.finishBuild(ifSkeleton.getTrueCodeMacro());
			if(this.falseCode_!=null) {
				this.falseCode_.finishBuild(ifSkeleton.addHasFalse().getFalseCodeMacro());
			}
		}
	} //end of IfStatement
	// -------------------------------------------------------------------------------------------

	// -------------------------------------------------------------------------------------------
	private final static class ExpressionVarHandler implements Statement,JavaStatementPeer.ExpressionVarPatternPeer,IRecordVariable {
		private final String	variableName_;
		private final SpecificCommonErrorOutput	error_;
		private final ExpressionBlock value_;
		private GeneralExpressionDefinitionDataBlock	expressionDefinitionSkeleton_;
		public ExpressionVarHandler(String variableName, SpecificCommonErrorOutput error ) {
			this.variableName_ = variableName;
			this.error_ = error;
			this.value_ = new ExpressionBlock(error);
		}
    @Override public boolean isMatchesVariableName(String queryVariableName) { return this.variableName_.equals(queryVariableName); }
		@Override public String getRoughVariableName() { return this.variableName_; }
		public void end() {}

		public JavaExpressionPeer.Indirect getJavaExpressionForValue() { return value_; }
		// ------------------------------------------------------------------
		// Expression Object stuff
		public void basicBuild(CodeBlockEnvironment environment) {
			this.expressionDefinitionSkeleton_ = environment.getBaseLayer().allocateExpressionDefinition();
			try {
				environment.addIRecordVariable(this.variableName_, this);
			} catch(ItemAlreadyExistsException e) {
				this.error_.repeatedObjectError("builder variable", this.variableName_);
			}
			value_.basicBuild(environment);
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			value_.finish(environment, this.expressionDefinitionSkeleton_.getExpressionDetailsParameter());
		}
		// ------------------------------------------------------------------
		public SLayer getAsFoundataionNodeReferenceVariable( JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression variable cannot provide foundation record information");		
		}
		public void addAsMacroRecordMappingParameter( MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression variable can not supply mappings");
		}
		public void updateNameBlock(NameDetailsFormBlock.ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression variable cannot provide name block information");		
		}
		public void getAsReference(IRecordTypeTarget target) {
	  	//Not a reference - yet?
	  	target.setNotReference();
	  }
		public void updateExpressionFinal(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath variablePath) throws ParsingException {
			expressionSkeleton.addViaExpressionDefinition(this.expressionDefinitionSkeleton_, variablePath);
		}
		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) 	throws ParsingException {
			//TODO NEED TO CHECK IF PARAMTER REQUIRED IS AN EXPRESSION
			//Hard to see how to do this - pass in the SLayer of the Expression def, but variablePath points to that layer, not to
			// anything in the layer (can return the build method)
			
			throw ParsingException.createGeneralSyntax("expression variable cannot provide macro parameter for "+type.getName());
		}
		public void updateCallChainFinal(BasicCallChainFormBlock target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression variable cannot provide chain information");
		}

		public void addAsMacroRecordKeyParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath variablePath, KeyDefinitionBlock type)
				throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression variable  can not supply mappings");
			
		}
		public void addAsGeneralExpressionTargetRequirement(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression variable  can not expression targets");
			
		}
		
	}	
	//--------------------------------------------------------------------------------------------
	private final static class SetStatement implements Statement, JavaStatementPeer.SetPatternPeer {
		private final String variableName_;
		private final SpecificCommonErrorOutput error_;
		private final ExpressionBlock value_;
		private final BaseTrackers.JavaVariablePath pathToVariable_;
		private IRecordVariable foundVariable_;
		public SetStatement(String variableName, BaseBuilder builderRoot, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.error_ = error;
			this.value_ = new ExpressionBlock(error);
			this.pathToVariable_ = new BaseTrackers.JavaVariablePath();
		}
		// --- Austen X bits -------
		public void end() { }

		public Indirect getJavaExpressionForValue() { return this.value_; }
		// ---- Handler bits ----
		public void basicBuild(CodeBlockEnvironment environment) {
			// TODO Auto-generated method stub
			this.value_.basicBuild(environment);
			try {
				this.foundVariable_ = environment.getBaseLayer().getBuilderVariable(this.variableName_, this.pathToVariable_,true);
			} catch(ParsingException e) {
				e.updateError(this.error_);
			}
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			// TODO Auto-generated method stub
			ExpressionDetailsFormHolder.ExpressionDetailsFormBlock expressionSkeleton = baseCodeSkeleton.getRootBuilder().createExpressionDetailsChild();
			this.value_.finish(environment,expressionSkeleton);
			IRecordTypeTarget target = new IRecordTypeTarget();
			this.foundVariable_.getAsReference(target);
			try {
				target.doCreateSetExpression(baseCodeSkeleton,this.pathToVariable_, expressionSkeleton);
			} catch(ParsingException e) {
				e.updateError(this.error_);
			}
		}
	}// end of class SetStatement
	//--------------------------------------------------------------------------------------------
	private final static class SyncStatement implements Statement,JavaStatementPeer.SyncPatternPeer {
		private final ExpressionBlock syncObjectExpression_;
		private final JavaCodeBlock mainCode_;
		public SyncStatement(BaseBuilder builderRoot, SpecificCommonErrorOutput error) {
			this.syncObjectExpression_ = new ExpressionBlock(error);
			this.mainCode_ = new JavaCodeBlock(builderRoot,error);
		}
		public void basicBuild(CodeBlockEnvironment environment) {
			this.syncObjectExpression_.basicBuild(environment);
			this.mainCode_.basicBuild(environment);
		}
		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			CodeBlockMacroBlock.StatementDataBlock.SyncBlockDataBlock syncSkeleton = baseCodeSkeleton.addStatement().addSyncBlock(baseCodeSkeleton.getRootBuilder().createExpressionDetailsChild());
			this.syncObjectExpression_.finish(environment, syncSkeleton.getSyncDetailsParameter());
			this.mainCode_.finishBuild(syncSkeleton.getMainCodeMacro());
		}
		public void end() { }
		public JavaExpressionPeer.Indirect getJavaExpressionForSyncObject() {	return this.syncObjectExpression_;	}
		public JavaStatementPeer.Indirect getJavaStatementForStatement() { return this.mainCode_; }
	} // End of WhileStatement class
	//--------------------------------------------------------------------------------------------
	private final static class WhileStatement implements Statement,JavaStatementPeer.WhilePatternPeer {
		private final ExpressionBlock conditionalExpression_;
		private final JavaCodeBlock mainCode_;
		public WhileStatement(BaseBuilder builderRoot, SpecificCommonErrorOutput error) {
			this.conditionalExpression_ = new ExpressionBlock(error);
			this.mainCode_ = new JavaCodeBlock(builderRoot, error);
		}

		public void basicBuild(CodeBlockEnvironment environment) {
			this.conditionalExpression_.basicBuild(environment);
			this.mainCode_.basicBuild(environment);
		}

		public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
			CodeBlockMacroBlock.StatementDataBlock.WhileBlockDataBlock whileSkeleton = baseCodeSkeleton.addStatement().addWhileBlock(baseCodeSkeleton.getRootBuilder().createExpressionDetailsChild());
			this.conditionalExpression_.finish(environment, whileSkeleton.getConditionalDetailsParameter());
			this.mainCode_.finishBuild(whileSkeleton.getMainCodeMacro());
		}

		public void end() { }

		public JavaExpressionPeer.Indirect getJavaExpressionForConditional() { return this.conditionalExpression_; }

		public JavaStatementPeer.Indirect getJavaStatementForLoopStatement() { return this.mainCode_; }
	} // End of WhileStatement class
	// --------------------------------------------------------------------------

	public JavaStatementPeer.FindPatternPeer createFind() {
		return new JavaStatementPeer.FindPatternPeer() {
			
			public void end() { 	}
			
			public FindDefinitionPatternPeer addFindDefinitionForDefinition( String varibleNameParameter, int varibleNameLineNumber, int varibleNameCharacterNumber) {
				SpecificCommonErrorOutput error = error_.createAdjusted(varibleNameLineNumber, varibleNameCharacterNumber);
				return doAddFindDefinitionForDefinition(varibleNameParameter,error); 
			}
		};
	}
	
	public FindDefinitionPatternPeer doAddFindDefinitionForDefinition(final String varibleNameParameter, final SpecificCommonErrorOutput error) {
		final FindVariable variable = new FindVariable(varibleNameParameter,error);
		Debug.println("Code find:",varibleNameParameter, " at ",error.getLineNumber());

    addStatement(new Statement() {
    	SLayer containingLayerStore;
    	FoundationNode rootNodeStore;
			public void basicBuild(CodeBlockEnvironment environment) {
				this.containingLayerStore = environment.getBaseLayer(); this.rootNodeStore = this.containingLayerStore.getRootFoundationNodeBasic();
				variable.basicBuild(this.rootNodeStore,this.containingLayerStore);					
			}

			public void finishBuild(CodeBlockEnvironment environment, CodeBlockMacroBlock baseCodeSkeleton) {
				variable.finishBuild(this.rootNodeStore,this.containingLayerStore);
			}
    });
		return variable;
	}











	
}
