package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.austenx.peg.base.GeneralTypePeer;
import org.ffd2.skeletonx.austenx.peg.base.GenericsPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaExternalBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaExternalElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.LabelingSubsPeer;
import org.ffd2.skeletonx.austenx.peg.base.PackagePathElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.SimpleParameterPatternPeer;
import org.ffd2.skeletonx.compile.base.GeneralTypeBlock;
import org.ffd2.skeletonx.compile.base.PathNode;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.skeleton.ArgumentSetDetailsFormHolder.ArgumentSetDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock.ImplementsNameDataBlock;
import org.ffd2.skeletonx.skeleton.TypeListDetailsFormHolder.TypeListDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock.WithGenericsDataBlock;
import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class JavaExternalBlock implements  JavaExternalBlockPatternPeer {
	
	private static final ExternalTypeEnvironment VOID_GENERICS_ENVIRONMENT = new ExternalTypeEnvironment() {
		public TargetTypeReference getInstantiatedGenericTypeReference(int genericIndex) {
			return new TargetTypeReference(StandardTargetTypes.VOID_TARGET_TYPE);
		}
		
	};
  private final SpecificCommonErrorOutput baseError_;
  private final AccessStoreX<ExternalTypeHandler> types_;
  private final SimpleVector<GeneralTypeBlock> extendsBlocksTypes_ = new SimpleVector<GeneralTypeBlock>();
	private final FileEnvironment environment_;
	private JavaExternalBlock[] extendsBlocks_;
  public JavaExternalBlock(FileEnvironment environment, SpecificCommonErrorOutput error) {
    this.environment_ = environment;
    this.baseError_ = error;
    this.types_ = environment.createEntityStore("type");
  }

  public void end() {}

  public JavaExternalElementPeer.NormalPatternPeer addOptionNormalOfJavaExternalElementForElement(String nameParameter, int nameLineNumber, int nameCharacterNumber) {
    SpecificCommonErrorOutput error = this.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
    NormalType normal = new NormalType(nameParameter, this.environment_.getImplementationGlobalEnvironment(),error);
    try {
      this.types_.put(nameParameter, normal);
    } catch(ItemAlreadyExistsException e) {
      error.repeatedObjectError("external type",nameParameter);
    }
    
    return normal;
  }
	public GeneralTypePeer.Indirect getGeneralTypeForExternalType() {
		GeneralTypeBlock type = new GeneralTypeBlock(this.baseError_);
		this.extendsBlocksTypes_.add(type);
		return type;
	}
  public void resolutionPass() {
  	
  	final int numberOfExtendsBlocks = this.extendsBlocksTypes_.size();
		final JavaExternalBlock[] extendsBlocks = new JavaExternalBlock[numberOfExtendsBlocks];
  	boolean parsedOkay = true;
		for(int i = 0 ; i < numberOfExtendsBlocks ; i++) {
			GeneralTypeBlock type = this.extendsBlocksTypes_.get(i);
  		try {
  			JavaExternalBlock resolved = type.getJavaExternalsBlock(this.environment_);
  			extendsBlocks[i] = resolved;
			} catch (ParsingException e) {
				e.updateError(type.getTypeErrorAccess());
				parsedOkay = false;
			}
  	}
  	this.extendsBlocks_ = extendsBlocks;
    if(parsedOkay) {
    	final ExternalTypeAccess baseEnvironment = new ExternalTypeAccess() {
  			public ExternalTypeBlueprint getTypeBlueprint(String name) throws ParsingException {
  				return getExternalType(name);
  			}
    	};

    	for(ExternalTypeHandler type: this.types_) {
        type.resolveTypes(baseEnvironment);
      }
    }
  }
  
  private ExternalTypeBlueprint getExternalType(String name) throws ItemNotFoundException {
    
    TargetType base =  this.types_.getQuiet(name);
    if(base!=null) {
      return new SimpleBlueprint(new TargetTypeReference(base));
    } else {
    	final JavaExternalBlock[] extendsBlocks = this.extendsBlocks_;
    	final int numberOfExtendsBlocks = extendsBlocks.length;
    	for(int i = 0 ; i < numberOfExtendsBlocks ; i++) {
    		ExternalTypeBlueprint externalType = extendsBlocks[i].getExternalLocalTypeQuiet(name);
    		if(externalType!=null) {
    			return externalType;
    		}
    	}
      return new SimpleBlueprint(new TargetTypeReference(StandardTargetTypes.getStandardType(name)));
    }
  }
  private ExternalTypeBlueprint getExternalLocalTypeQuiet(String name)  {
    
    TargetType base =  this.types_.getQuiet(name);
    if(base!=null) {
      return new SimpleBlueprint(new TargetTypeReference(base));
    } else {
    	return null;
    }
  }
  private static final class SimpleBlueprint implements ExternalTypeBlueprint {
  	private final TargetTypeReference base_;

		public SimpleBlueprint(TargetTypeReference base) { this.base_ = base; }
		public TargetTypeReference getConstructedTargetTypeReference(ExternalTypeEnvironment environment) { return base_; }
  }
  // ==========================================================================
  public TargetTypeReference getTypeReferenceQuiet(String name) {
    return new TargetTypeReference(this.types_.getQuiet(name));
  }
  public TargetType getTypeQuiet(String name) {
    return this.types_.getQuiet(name);
  }
  
  // ==========================================================================

  private interface ExternalTypeHandler extends TargetType {
    public void resolveTypes(ExternalTypeAccess baseEnvironment);
  }
  //--------------------------------------------------------------------------
  
  private static final class NormalWithGenericsType implements TargetType,DirectTargetElementAccess, ExternalTypeEnvironment {
  	private final NormalType parent_;
		private final TargetTypeWithPath[] genericTypes_;
		public NormalWithGenericsType(TargetTypeWithPath[] genericTypes,NormalType parent) {
			this.parent_ = parent;
			this.genericTypes_ = genericTypes;
  		
  	}
		
		
		public boolean isSubTypeOf(TargetType parentType) {
			return parent_.isSubTypeOf(parentType);
		}
		
		public TargetTypeReference getInstantiatedGenericTypeReference(int genericIndex) {
			return this.genericTypes_[genericIndex].getTypeReference();
		}
		// -----

		public DirectTargetElementAccess getDirectElementAccessQuiet() {
			return this;
		}
		public TargetType getArrayed(int dimensions) throws ParsingException {
			//Probably shouldn't allow this as you can't go the otherway (eg, array, then generics).
			return new ArrayedTargetType(this,dimensions);
		}
		public void buildCodeSetType(TypeSetterDetailsFormBlock target, JavaVariablePath disposableTargetPath) {
			parent_.buildCodeSetType(target, disposableTargetPath);
			//Add in the 
			BaseBuilder rootBuilder = target.getRootBuilder();
			TypeListDetailsFormBlock typeListSkeleton = rootBuilder.createTypeListDetailsChild();
			WithGenericsDataBlock withGenericsSkeleton = target.addWithGenerics(typeListSkeleton);
			for(TargetTypeWithPath generic : this.genericTypes_) {
				generic.buildCodeSetType(typeListSkeleton.addSimple(rootBuilder.createTypeSetterDetailsChild()).getTypeDetailsParameter());
			}
		}

		public String getOutputName() {
			return parent_.getOutputName();
		}

		public TargetType specifyGenericTypes(TargetTypeWithPath[] types) throws ParsingException {
			//I don't think this will ever get called...
			throw ParsingException.createGeneralSyntax("Already have generics!");
		}
		public TargetMethod getTargetMethodQuiet(String name, JavaVariablePath pathToDeclaration) {
			MethodElement baseMethod = parent_.getMethodElementQuiet(name);
			if(baseMethod!=null) {
				return new ConcreteMethod(baseMethod,this);
			} else {
				return null;
			}
		}
		public TargetVariable getTargetVariableQuiet(String name, JavaVariablePath pathToDeclaration) {
			VariableElement baseVariable = parent_.getVariableElementQuiet(name);
			if(baseVariable!=null) {
				return new ConcreteVariable(baseVariable,this);
			} else {
				return null;
			}
		}


	
  }
  // --------------------------------------------------------------------------
  private final static class ExtensionValue {
  	private final String	name_;
		private final SpecificCommonErrorOutput	error_;
		private final ExtensionValue	previous_;
		private TargetTypeReference type_;
		public ExtensionValue(String name, SpecificCommonErrorOutput error, ExtensionValue previous) {
			this.name_ = name;
			this.error_ = error;
			this.previous_ = previous;
  	}
    public void resolveTypes(ExternalTypeAccess baseEnvironment) {
    	try {
				this.type_ = baseEnvironment.getTypeBlueprint(this.name_).getConstructedTargetTypeReference(VOID_GENERICS_ENVIRONMENT);
			} catch (ParsingException e) {
				e.updateError(this.error_);
			}
    	
    	if(previous_!=null) {
    		previous_.resolveTypes(baseEnvironment);
    	}
    }
    public boolean isSubTypeOf(TargetType parentType) {
			if(this.type_.getBaseType().isSubTypeOf(parentType)) {
				return true;
			} else if(this.previous_!=null) {
				return this.previous_.isSubTypeOf(parentType);
			} else {
				return false;
			}
		}
  }
   // --------------------------------------------------------------------------
  private static final class NormalType implements ExternalTypeHandler,JavaExternalElementPeer.NormalPatternPeer,DirectTargetElementAccess {
    private final SpecificCommonErrorOutput error_;
    private final String name_;
    private final PathNode path_;
    private AccessStoreX<VariableElement> possibleVariables_;
    private AccessStoreX<MethodElement> possibleMethods_;
    private final GlobalEnvironment environment_;
    private String[] pathComponents_;
    private GenericsHandler generics_ = null;
    private  ExtensionValue extensionsEnd_ = null;
    public NormalType(String name, GlobalEnvironment environment, SpecificCommonErrorOutput error) {
      this.name_ = name;
      this.environment_ = environment;
      this.error_ = error;
      this.path_ = new PathNode();

    }
		public boolean isSubTypeOf(TargetType parentType) {
			if(parentType==this) {
				return true;
			} else if(this.extensionsEnd_!=null) {
				return this.extensionsEnd_.isSubTypeOf(parentType);
			} else {
				return false;
			}
		}
  	public DirectTargetElementAccess getDirectElementAccessQuiet() {
  		return this;
  	}
    public TargetType specifyGenericTypes(TargetTypeWithPath[] types) throws ParsingException {
    	if(this.generics_==null) {
    		throw ParsingException.createGeneralSyntax("No generics for type");
    	}
    	this.generics_.checkTypeGenericTypes(types);
    	
    	return new NormalWithGenericsType(types, this);
    	
    }
    public TargetType getArrayed(int dimensions) throws ParsingException {
  		return new ArrayedTargetType(this,dimensions);
  	}
    public String getOutputName() { 
    	return "External:"+name_;
    }
    
    public void resolveTypes(ExternalTypeAccess baseEnvironment) {
    	if(this.generics_!=null) {
    		baseEnvironment = this.generics_.createAccessWithGenericTypes(baseEnvironment);
    	}
    	if(this.extensionsEnd_!=null) {
    		this.extensionsEnd_.resolveTypes(baseEnvironment);
    	}
    	if(possibleMethods_!=null) {
        for(MethodElement me : this.possibleMethods_) {
          me.resolveTypes(baseEnvironment);
        }
      }
      if(this.possibleVariables_!=null) {
        for(VariableElement ve : this.possibleVariables_) {
          ve.resolveTypes(baseEnvironment);
        }
      }
    }
    
    public GenericsPatternPeer addGenericsForGenerics(int keywordLineNumber, int keywordCharacterNumber) {
      SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
    	return this.generics_ = new GenericsHandler(error);
      
    }
    public PackagePathElementPeer.NormalPatternPeer addOptionNormalOfPackagePathElementForPathInfo() {
      return path_;
    }
    public LabelingSubsPeer.MethodSubPatternPeer addOptionMethodSubOfLabelingSubsForSubs(String typeParameter, String nameParameter, int nameLineNumber, int nameCharacterNumber) {

      SpecificCommonErrorOutput newError = this.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
      MethodElement me = new MethodElement(typeParameter, nameParameter,newError);
      try {
        this.getMethods().put(nameParameter, me);
      } catch(ItemAlreadyExistsException e) {
        newError.repeatedObjectError("method", nameParameter);
      }
      return me;
    }
    private AccessStoreX<MethodElement> getMethods() {
      if(this.possibleMethods_==null) {
        this.possibleMethods_ = this.environment_.createEntityStore("target method");
      }
      return this.possibleMethods_;
    }
    private AccessStoreX<VariableElement> getVariables() {
      if(this.possibleVariables_==null) {
        this.possibleVariables_ = this.environment_.createEntityStore("target variable");
      }
      return this.possibleVariables_;
    }
    

    public LabelingSubsPeer.VariableSubPatternPeer addOptionVariableSubOfLabelingSubsForSubs(String typeParameter, String nameParameter, int nameLineNumber, int nameCharacterNumber) {

      SpecificCommonErrorOutput newError = this.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
      VariableElement ve = new VariableElement(typeParameter, nameParameter,newError);
      try {
        this.getVariables().put(nameParameter, ve);
      } catch(ItemAlreadyExistsException e) {
        newError.repeatedObjectError("variable", nameParameter);
      }
      return ve;
    }

    public void end() { 
      this.pathComponents_ = this.path_.getPathComponents();
      
    }
    
    // -------- TARGET TYPE STUFF ---------------------
    
		public void buildCodeSetType(TypeSetterDetailsFormBlock target, JavaVariablePath disposableTargetPath) {
			TypeSetterDetailsFormBlock.FromExternalDataBlock block = target.addFromExternal();
      for(int i = 0 ; i < this.pathComponents_.length ; i++) {
        block.addComponent(this.pathComponents_[i]);
      }
    }
   
    public MethodElement getMethodElementQuiet(String name)  {
      return getMethods().getQuiet(name);
    }
    public VariableElement getVariableElementQuiet(String name)  {
      return getVariables().getQuiet(name);
    }
    public TargetMethod getTargetMethodQuiet(String name, JavaVariablePath pathToDeclaration) {
    	return getMethods().getQuiet(name);
		}
		public TargetVariable getTargetVariableQuiet(String name, JavaVariablePath pathToDeclaration) {
			return getVariables().getQuiet(name);
		}

		public void addExtension(String value, int lineNumber, int characterNumber) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(lineNumber, characterNumber);
			this.extensionsEnd_ = new ExtensionValue(value, error, this.extensionsEnd_);
		}

		public void addImplementation(String value, int lineNumber, int characterNumber) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(lineNumber, characterNumber);
			this.extensionsEnd_ = new ExtensionValue(value, error, this.extensionsEnd_);
			
		}

  }
  // --------------------------------------------------------------------------
  private static final class GenericType implements ExternalTypeBlueprint {
  	private final String name_;
		private final int typeIndex_;

		public GenericType(String name, int typeIndex) {
			this.name_ = name;
			this.typeIndex_ = typeIndex;
  	}
		public boolean isMatchingName(String queryName) {
			return this.name_.equals(queryName);
		}
		public TargetTypeReference getConstructedTargetTypeReference(ExternalTypeEnvironment environment) {
			return environment.getInstantiatedGenericTypeReference(this.typeIndex_);
		}
  }
 // ddd - need to instantiate new type when generics given, create an external type environment
  private static final class GenericsHandler implements GenericsPatternPeer {
  	private final SimpleVector<GenericType> types_ = new SimpleVector<GenericType>();
		private final SpecificCommonErrorOutput error_;
  	
  	public GenericsHandler(SpecificCommonErrorOutput error) {
			this.error_ = error; 
		}
  	public ExternalTypeAccess createAccessWithGenericTypes(final ExternalTypeAccess baseEnvironment) {
  		return new ExternalTypeAccess() {

				public ExternalTypeBlueprint getTypeBlueprint(String name) throws ParsingException {
					ExternalTypeBlueprint genericLocal = GenericsHandler.this.getTypeQuiet(name);
					if(genericLocal!=null) {
						return genericLocal;
					} else {
						return baseEnvironment.getTypeBlueprint(name);
					}
				}
  			
  		};
  	}
		public void addName(String value) {
			this.types_.addElement(new GenericType(value,this.types_.size()));
		}
		public ExternalTypeBlueprint getTypeQuiet(String typeName) {
			for(GenericType type : this.types_) {
				if(type.isMatchingName(typeName)) {
					return type;
				}
			}
			return null;
		}
		public void checkTypeGenericTypes(TargetTypeWithPath[] givenGenericTypes) throws ParsingException {
			if(givenGenericTypes.length!=this.types_.size()) {
				throw ParsingException.createGeneralSyntax("Incorrect number of generic type parameters; found "+givenGenericTypes.length+", expecting "+this.types_.size());
			}
		}
		public void end() { }
  }
  	
  // --------------------------------------------------------------------------
  private static final class MethodElement implements LabelingSubsPeer.MethodSubPatternPeer, TargetMethod {
    private final SpecificCommonErrorOutput error_;
    private String actualName_;
    private int arrayDimensions_;
    private final String returnTypeName_;
    private ExternalTypeBlueprint returnType_;
    private boolean isStatic_ = false;

    public MethodElement(String typeName, String name,  SpecificCommonErrorOutput error) {
      this.returnTypeName_ = typeName;
      this.error_ = error;
      this.actualName_ = name;
    }
    public void resolveTypes(ExternalTypeAccess environment) {
      try {
        returnType_ = environment.getTypeBlueprint(this.returnTypeName_);
      } catch(ParsingException e) {
      	e.updateError(this.error_);
      }
    }
    public void addActual(String value) { this.actualName_ = value; }

    public void addArrayDimension() { this.arrayDimensions_++;  }

    public SimpleParameterPatternPeer addSimpleParameterForParameter(String typeParameter, int typeLineNumber, int typeCharacterNumber, String nameParameter) {
      //Just ignore for now...
      return null;
    }

    public void end() { }
    // ---------------------- TARGET METHOD STUFF ----------------------------
		public ArgumentSetDetailsFormBlock createExpressionFinal(ExpressionDetailsFormBlock target, JavaVariablePath basePathDisposable) {
      BasicCallChainFormBlock basicChain = target.getRootBuilder().createBasicCallChainChild();
			target.addCallChain(basicChain );
			ArgumentSetDetailsFormBlock methodArgumentDetails = target.getRootBuilder().createArgumentSetDetailsChild();
			basicChain.addElement().addExternalMethod(this.actualName_, methodArgumentDetails);
			return methodArgumentDetails;

    }
		public ArgumentSetDetailsFormBlock createCallExpressionFinal(BasicCallChainFormBlock target, JavaVariablePath basePathDisposable) {
			ArgumentSetDetailsFormBlock methodArgumentDetails = target.getRootBuilder().createArgumentSetDetailsChild();
			target.addElement().addExternalMethod(this.actualName_,methodArgumentDetails);
      return methodArgumentDetails;
    }
		
    public TargetTypeReference getReturnTargetTypeReferenceFinal(ExternalTypeEnvironment environment) {
       return this.returnType_.getConstructedTargetTypeReference(environment);
    }
    public TargetTypeReference getReturnTargetTypeReferenceQuietFinal() {
      return this.returnType_.getConstructedTargetTypeReference(VOID_GENERICS_ENVIRONMENT);
    }
		public GlobalMethodReference getAsGlobalQuiet() { return null; }
		public boolean isStatic() { 
			return isStatic_;
		}
		public void addIsStatic() {
			this.isStatic_ = true;
		}
		public ImplementsNameDataBlock addMethodImplementsNameBlock(MethodDataBlock target, JavaVariablePath methodDeclarationPathChainTracker) throws ParsingException {
			throw ParsingException.createGeneralSyntax("External methods cannot provide source for implementation");
		}



  }
  /// ------------------------------------------------------
  private final static class ConcreteMethod implements TargetMethod {
  	private final MethodElement base_;
		private final ExternalTypeEnvironment environment_;

		public ConcreteMethod(MethodElement base, ExternalTypeEnvironment environment) {
			this.base_ = base;
			this.environment_ = environment;
  	}
		public boolean isStatic() { 
			return false;
		}
		public ImplementsNameDataBlock addMethodImplementsNameBlock( MethodDataBlock target, JavaVariablePath methodDeclarationPathChainTracker) throws ParsingException {
			throw ParsingException.createGeneralSyntax("External methods cannot provide source for implementation");
		}
		public ArgumentSetDetailsFormBlock createCallExpressionFinal(BasicCallChainFormBlock target, JavaVariablePath basePathDisposable) {
			return base_.createCallExpressionFinal(target, basePathDisposable);
		}

		public ArgumentSetDetailsFormBlock createExpressionFinal(ExpressionDetailsFormBlock target, JavaVariablePath basePathDisposable) {
			return base_.createExpressionFinal(target, basePathDisposable);
		}

		public TargetTypeReference getReturnTargetTypeReferenceQuietFinal() {
			return base_.getReturnTargetTypeReferenceFinal(this.environment_);
		}

		public GlobalMethodReference getAsGlobalQuiet() { return null; }
  }
  // ============================================================================================
  
  private static final class VariableElement implements LabelingSubsPeer.VariableSubPatternPeer,TargetVariable{
    private final SpecificCommonErrorOutput error_;
    private String actualName_;
    private int arrayDimensions_;
    private final String typeName_;
    private ExternalTypeBlueprint valueType_;
    private boolean isStatic_ = false;
    public VariableElement(String typeName, String name, SpecificCommonErrorOutput error) {
      this.error_ = error;
      this.actualName_ = name;
      this.typeName_ = typeName;
    }
    public boolean isStatic() { 
    	//TODO: have static stuff for variables...
			return isStatic_;
		}
		public void addIsStatic() {
			this.isStatic_ = true;
		}
    public void resolveTypes(ExternalTypeAccess environment) {
      try {
        valueType_ = environment.getTypeBlueprint(this.typeName_);
      } catch(ParsingException e) {
      	e.updateError(error_);
      }
    }
    public void addActual(String value) { this.actualName_ = value; }
    public void addArrayDimension() { this.arrayDimensions_++; }

    public void end() { }
    // TargetVariableStuff
    public void createCallExpressionFinal(BasicCallChainFormBlock target, BaseTrackers.JavaVariablePath basePathDisposable) {
    	target.addElement().addExternalVariable(this.actualName_);
    }
   
    public TargetTypeReference getTargetTypeReferenceQuietFinal() {
      return getTargetTypeReference(VOID_GENERICS_ENVIRONMENT);
    }
    public TargetTypeReference getTargetTypeReference(ExternalTypeEnvironment environment) {
      return valueType_.getConstructedTargetTypeReference(environment);
    }
    public void createMacroCallParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath pathToVariable) throws ParsingException{
    	throw ParsingException.createGeneralSyntax("External variables cannot supply macros");
    }
  }
  // ----------------------------------------------
  private static final class ConcreteVariable implements TargetVariable {
  	private final VariableElement base_;
		private final ExternalTypeEnvironment environment_;

		public ConcreteVariable(VariableElement base, ExternalTypeEnvironment environment ) {
			this.base_ = base;
			this.environment_ = environment;
  	}

		public void createCallExpressionFinal(BasicCallChainFormBlock target, JavaVariablePath basePathDisposable) {
			base_.createCallExpressionFinal(target, basePathDisposable);
		}

		public void createMacroCallParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath pathToVariable) throws ParsingException {
			this.base_.createMacroCallParameter(macroCallBlock, pathToVariable);
		}

		public TargetTypeReference getTargetTypeReferenceQuietFinal() {
			return this.base_.getTargetTypeReference(this.environment_);
		}
  }
  // ================================================================================
  
  private static interface ExternalTypeBlueprint {
  	public TargetTypeReference getConstructedTargetTypeReference(ExternalTypeEnvironment environment);
  }
  private static interface ExternalTypeAccess {
  	public ExternalTypeBlueprint getTypeBlueprint(String name) throws ParsingException;
  }
  private static interface ExternalTypeEnvironment {
  	public TargetTypeReference getInstantiatedGenericTypeReference(int genericIndex);
  }

 
}
