package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.austenx.peg.base.GeneralTypePeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaImplementationElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaImplementatonBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.ParameterBlockPatternPeer;
import org.ffd2.skeletonx.compile.base.GeneralTypeBlock;
import org.ffd2.skeletonx.compile.impl.BlockImplementationTarget;
import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.ChainImplementationTarget;
import org.ffd2.skeletonx.compile.impl.DesignBaseImplementationTarget;
import org.ffd2.skeletonx.compile.impl.DesignBlock;
import org.ffd2.skeletonx.compile.impl.DesignBridgeComponents.DesignRecordReference;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder.TargetBaseFormBlock;
import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;


public class JavaImplementationBlock implements DataBlockEnvironment, JavaImplementatonBlockPatternPeer {
  private final SpecificCommonErrorOutput baseError_;
  private final GeneralTypeBlock designType_;
  private final BaseBuilder baseBuilder_;
  private final String name_;
  private final FileEnvironment environment_;
  private DesignBlock relatedDesignBlock_;
  private JavaImplementationFormBlock implementationSkeleton_;
  private DesignImplementationConnection implementationConnection_;

  private final RootLevelElementsBlock elements_;
  private final SimpleVector<GeneralTypeBlock> externalsBase_ = new SimpleVector<GeneralTypeBlock>();
  private JavaExternalBlock[] resolvedExternals_ = null;
	private LinkupTracker linkupTracker_;
//  private final GlobalsBlock rootGlobals_;
  
  private FoundationNode.RootFoundationNodeAccess designRoot_;
 
  public JavaImplementationBlock(BaseBuilder baseBuilder, String name, FileEnvironment environment, SpecificCommonErrorOutput baseError) {
    this.baseError_ = baseError;
    this.environment_ = environment;
    this.name_ = name;
    this.baseBuilder_= baseBuilder;
    this.designType_ = new GeneralTypeBlock(baseError);
    this.elements_ = new RootLevelElementsBlock(this.environment_,baseBuilder,null,baseError);
    
    final GlobalEnvironment implementationGlobalEnvironment = environment.getImplementationGlobalEnvironment();
  }
//	public JavaImplementationSpecificsBlockPatternPeer 
//			addJavaImplementationSpecificsBlockForSpecifics( 
//				String blockNameParameter, int blockNameLineNumber, int blockNameCharacterNumber
//			) {
//		SpecificCommonErrorOutput error = this.baseError_.createAdjusted(blockNameLineNumber, blockNameCharacterNumber);
//		return this.rootGlobals_.addTopLevelChild(blockNameParameter, error);
//	}
	
	
  public String getRecordName() { return "<Implementation base>"; }
  
  public GeneralTypePeer.Indirect getGeneralTypeForExternalType() {
    GeneralTypeBlock base = new GeneralTypeBlock(this.baseError_);
    this.externalsBase_.addElement(base);
    return base;
  }
  private JavaExternalBlock[] getResolvedExternals() {
    if(this.resolvedExternals_==null) {
      JavaExternalBlock[] workspace = new JavaExternalBlock[this.externalsBase_.size()];
      int index = 0;
      for(GeneralTypeBlock type : this.externalsBase_) {
        JavaExternalBlock resolved;
        try {
          resolved = type.getJavaExternalsBlock(this.environment_);
          workspace[index++] = resolved;
        } catch (ParsingException e) {
          e.updateError(type.getTypeErrorAccess());
        } 
      }
      if(index==workspace.length) {
        this.resolvedExternals_ = workspace;
      } else {
        // Not all resolved
        this.resolvedExternals_ = new JavaExternalBlock[index];
        System.arraycopy(workspace,0,this.resolvedExternals_,0,index);
      }
    }
    return this.resolvedExternals_;
  }
 /*
  *  public JavaImplementationForm.BlockDataBlock getBlockSkeleton() {
        return this.relatedDataBlock_;
      }
      public void setupRelatedReference(  LinkupTracker relatedTracker, RecordReference reference) {
        this.relatedDataBlock_ = relatedTracker.instantiateRecord(reference, this.actualLink_);
        this.actualLink_.setRelatedReference(relatedTracker, relatedDataBlock_, reference);
      }
      
  */
  // ==========================================================================
  // Parsing pass stuff
  public void resolutionPassImplementation() {
    getResolvedExternals(); // Just to check up.
  }
  public void intermediatePassImplementation() {
//    LinkupBuilder builder = new LinkupBuilder(this.relatedDesignBlock_.getTotalNumberOfRecords());
//    this.relatedDesignBlock_.buildJavaImplementations(this.implementationSkeleton_,builder);
//    builder.buildChildren(); 

   // this.designRoot_.buildRemainderIncludingParameters();
    SLayer rootLayer = this.designRoot_.getRootLayer();
    this.elements_.basicBuild(rootLayer);
//    this.rootGlobals_.basicBuild(this.designRoot_.getBuiltNode());
  }
  public void finalPassImplementation() {
		implementationConnection_.finalPass();
		this.elements_.finishBuild();

    //Get the design to build the tree 

//    this.rootGlobals_.finishBuild(this.designRoot_.getBuiltNode()); 
   }
  //Declaration
  public final void finalPassDesign() {
    try {
      this.relatedDesignBlock_ = this.designType_.getDesignBlock(this.environment_);
      this.implementationSkeleton_ = baseBuilder_.createJavaImplementationChild(name_, relatedDesignBlock_.getTargetBaseSkeleton());
      
    } catch(ParsingException e) {
      e.updateError(this.designType_.getTypeErrorAccess());
      return;
    }
    
    linkupTracker_ = new LinkupTracker(
    											this.implementationSkeleton_, 
    											this.relatedDesignBlock_.getTotalNumberOfRecords(),
    											this.relatedDesignBlock_.getTotalNumberOfChains()
    										);
    DesignRecordReference reference = this.relatedDesignBlock_.getRootRecord();
    
    //Create the root node (without the tree)
    this.designRoot_ = FoundationNode.createInvisibleRoot(reference, this.linkupTracker_,this.environment_,this,this.baseError_);
    
//    //Get the design to build the tree 
    this.implementationConnection_ = new DesignImplementationConnection(this,this.designRoot_.getBuiltNode());
		this.relatedDesignBlock_.buildImplementations(implementationConnection_);
		
//    Debug.println("Tree built");
    
    //Fill in the left over bits (eg, parameters)
    this.designRoot_.buildRemainderIncludingParameters();
    
//    Debug.println("Remainder foundation built");
    
  //  this.relatedDataBlock_ = linkupTracker_.instantiateRecord(reference, this.rootLevelRecord_);
    
    //Hack stuff - maybe the linkup tracker bit should do the work?
    // Actually, maybe DesignRecordReference should...
    //************* SHIFTED TO DESIGN ROOT
//    this.relatedDataBlock_.getParent().addIsRootBlock();
//    this.relatedDataBlock_.addIsTopLevelBlock();
    
    
     

//    this.rootGlobals_.assignRecords(this.designRoot_.getBuiltNode());
//    Debug.println("Globals assigned");
    
  }
  /*Stuff from constructor
   * 
//    this.rootGlobals_ = new GlobalsBlock(environment.getImplementationGlobalEnvironment(),baseError);
//    this.rootLevelRecord_ = 
//      new ImplementationRecord(
//         environment,new RecordImplementationType() {
//          public String getStringDescription() { 
//             return "Chain root";
//          }
//          public void buildPathToRecordBackwardsQuiet(SLayer record,BaseTrackers.JavaVariablePath chainToVariable) {
//          	throw new RuntimeException("Assertion error : calling with an unknown record!");
//          }
//          public ChainDefinitionBlock getChainDefinition(String name) throws ItemNotFoundException {
//          	return relatedDesignBlock_.getChainDefinition(name);
//          }
//          
//          public VariableAccess getTargetVariableAccessQuietBackwards(boolean allowLocalVariables,TargetType outputClass,RecordVariableTrace variableTrace, String variableName, BaseTrackers.JavaVariablePath chainToVariable) {
//          	TargetType type = getTargetTypeQuiet(variableName,chainToVariable);
//          	if(type==null) {
//          		return null;
//          	} else {
//          		return new VariableAccess(new StaticTypeVariable(new TargetTypeReference(type)),JavaImplementationBlock.this);
//          	}
//          }
//          public MethodAccess getTargetMethodAccessQuietBackwards(TargetType outputClass,RecordVariableTrace variableTrace,  String name, JavaVariablePath chainToVariable) {
//            return null;
//          }		
//          public SLayer getRootRecordQuiet(String childName) {
//          	//Slight hack perhaps - messy referencing...
//        		return rootLevelRecord_.getDirectChildQuiet(childName);
//        	}
//
//          public void accessInternalTypeBackwards(TargetType outputClass,  JavaVariablePath chainToVariable) {
//            Debug.println("Where did this type come from?:  ",outputClass!=null);
//            Debug.println("Output class:",outputClass.getOutputName());
//            throw new RuntimeException("Assertion error : this should not happen!");
//          }
//          public IRecordVariable getResolvedVariableQuietBackwards(RecordVariableTrace variableTrace,String variableName, JavaVariablePath variablePath){
//            return null;
//          }
//          public TargetType getTargetTypeQuietBackwards(RecordVariableTrace variableTrace, String typeName, JavaVariablePath chainToVariable) {
//            return JavaImplementationBlock.this.getTargetTypeQuiet(typeName, chainToVariable);
//          }
//          public boolean queryAccessTypeInternalBackwards(TargetType outputClass, JavaVariablePath chainToVariable) {
//            //No can find eh!
//            return false;
//          }
//					public MacroConstruction getMacroConstructionBasicBackwards( String name) throws ItemNotFoundException {
//					  throw new ItemNotFoundException(name,"macro",name);
//		         
//					}
//					public TargetVariable getTargetLocalVariableQuietBackwards(ScopeContainer relatedScope,RecordVariableTrace variableTrace, 
//							String variableName, JavaVariablePath chainToVariable) {
//						return null;
//					}
//					public boolean isLocalVariableQuietBackwards(ScopeContainer relatedScope,RecordVariableTrace variableTrace,  String variableName) {
//						return false;
//					}
//
//
//
//         },
//         "<ROOT>"
//         
//      );
   */
  
  // ==========================================================================
  private static final class DesignImplementationConnection implements DesignBaseImplementationTarget {
  	private final JavaImplementationBlock parent_;
		private final BlockImplementationTarget rootTarget_;
		private final SimpleVector<ChainImplementationConnection> connections_ = new SimpleVector<ChainImplementationConnection>();
		public DesignImplementationConnection(JavaImplementationBlock parent, BlockImplementationTarget rootTarget) {
			this.parent_ = parent;
			this.rootTarget_ = rootTarget;
  	}
		public void finalPass() {
			for(ChainImplementationConnection connection : this.connections_) {
				connection.finalPass();
			}
		}
		public BlockImplementationTarget getRootBlockTarget() {
			return rootTarget_;
		}


		public ChainImplementationTarget newChain(int chainID, String chainName, ChainDefinitionBlock reference, TargetBaseFormBlock.ChainDataBlock targetPeerSkeleton) {
			ChainImplementationConnection impl = new ChainImplementationConnection(chainID, chainName, this.parent_.linkupTracker_,parent_.implementationSkeleton_.addChain(chainID, targetPeerSkeleton),parent_.getFileEnvironment(),this.parent_.baseError_);
			this.connections_.addElement(impl);
			return impl;
		}
  } //End of class DesignImplementationConnection
  // ==========================================================================
  // IRecordEnvironment stuff
  //Remove this


  // ==========================================================================
  // Austen Peer stuff
  public ParameterBlockPatternPeer addParameterBlockForParameters() {
    // TODO Auto-generated method stub
  	Debug.finishMeMarker();
    return null;
  }

  public void end() { }

  public GeneralTypePeer.Indirect getGeneralTypeForDesignType() { return designType_; }

  public JavaImplementationElementPeer.Indirect getJavaImplementationElementForElement() {
    return elements_;
  }
  public FileEnvironment getFileEnvironment() {
    return this.environment_;
  }
  
  
  public TargetType getTargetTypeQuiet(String typeName, BaseTrackers.JavaVariablePath path) {
   
//  	Debug.println("Looking for type:",typeName);
    TargetType standard = StandardTargetTypes.getStandardTypeQuiet(typeName);
    if(standard!=null) {
 //   	Debug.println("Found:",typeName);
      return standard;
    } else {
      JavaExternalBlock[] externals = getResolvedExternals();
      for(JavaExternalBlock block : externals) {
        TargetType type = block.getTypeQuiet(typeName);
        if(type!=null) {
          return type;
        }
      }
      return null;
    }
  }
  
 
  public TargetMethod getTargetMethodQuiet(TargetType targetType, String name, JavaVariablePath pathToDeclaration) {
    return null;
  }
  public TargetVariable getTargetVariableQuiet(boolean allowLocal,TargetType targetType, String name, JavaVariablePath pathToDeclaration) {
    return null;
  }
	
  public IRecordVariable getBuilderVariableQuiet(String name, JavaVariablePath pathToVariable) {
		return null;
	}




  
  // ==========================================================================
 
}
