package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.ArgTypesBlockBasePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.ArgValsBlockBasePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaClassInterfaceElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaMacroBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaStatementPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroElementPeer.ArgTypesBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroElementPeer.ArgValsBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroElementPeer.CodeBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroElementPeer.ContainerBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroElementPeer.ContextBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroParameterBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetCallArgumentsPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetParameterPeer;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.ClassImplementation.ClassAccessSource;
import org.ffd2.skeletonx.compile.java.layer.ImplementationBlockSkeletonManager;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BClassAccessFormHolder.BClassAccessFormBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseBuilder.CodeBlockMacroBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.InClassDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;

public class MacroConstruction implements JavaMacroBlockPatternPeer, RecordItem {
	private final SpecificCommonErrorOutput error_;
	private final SimpleVector<Block> blocks_ = new SimpleVector<Block>();
	private final MacroParameterBlock parameterBlock_;
	private final String name_;
	private final FileEnvironment environment_;
	private JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock macroBlock_;

	// =====================================================

	private ClassDetails formedClassDetails_;
	private GeneralDetails formedGeneralDetails_;

	private CodeDetails formedCodeDetails_;
	private final SimpleVector<Element> elements_ = new SimpleVector<Element>();
	private final VariableStore parameterVariables_;
	private SLayer definingLayer_;
	private SLayer peerLayer_;
	private ClassImplementation surroundingClassImplementation_;
	private final BaseBuilder rootBuilder_;

	// ===================
	// public MacroRootRecord(SLayer rootRecord, LinkupTracker linkupTracker,
	// MacroParameterBlock parameters, String macroName,IsMacroDataBlock
	// baseBuilder) {
	// this.rootRecord_ = rootRecord;
	// this.macroName_ = macroName;
	// this.parameters_ = parameters;
	// this.baseBuilder_ = baseBuilder;
	//
	// ImplementationParameterTarget setMacroReference =
	// this.basicContainer_.setMacroReference(linkupTracker,baseBuilder_);
	// this.parameters_.build(baseBuilder,this, setMacroReference);
	// }

	public MacroConstruction(String name, BaseBuilder rootBuilder, FileEnvironment environment, SpecificCommonErrorOutput error) {
		this.name_ = name;
		this.rootBuilder_ = rootBuilder;

		this.error_ = error;
		this.environment_ = environment;
		this.parameterVariables_ = new VariableStore(environment);
		this.parameterBlock_ = new MacroParameterBlock(rootBuilder,error);
	}

	// ---------------------------------------------------------------------------
	public static interface ParentContainerLink {

	}

	public static final ParentContainerLink NO_PARENT = new ParentContainerLink() {

	};

	public String getName() {
		return this.name_;
	}

	public MacroCallInstance createMacroCall(String variableName,
			SLayer targetLayer, MacroCallRequirementsAccess requirementsAccess) {
		return targetLayer.createMacroCallFinal(variableName, this,
				requirementsAccess);
	}

	public MacroParameterBlockPatternPeer addMacroParameterBlockForParameters() {
		return this.parameterBlock_;
	}

	public MacroCallDataBlock createCallFinal(String variableName,
			JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock target,
			MacroCallRequirementsAccess requirementsAccess) {
		MacroCallDataBlock macroCallBlock = target.addMacroCall(variableName,
				this.macroBlock_,
				requirementsAccess.buildPathToSLayer(this.definingLayer_));
		// this.createCallInFinish(macroCallBlock,requirementsAccess);
		for (Element e : this.elements_) {
			e.createCallInFinish(macroCallBlock, requirementsAccess);
		}
		this.parameterBlock_.requestParametersInCall(macroCallBlock,
				requirementsAccess);

		return macroCallBlock;
	}

	public ArgValsBlockPatternPeer addOptionArgValsBlockOfMacroElementForElement() {
		return new ArgValsBlockPatternPeer() {
			public void end() {
			}

			public ArgValsBlockBasePatternPeer addArgValsBlockBaseForBlock(
					String givenCodeVariableNameParameter,
					int givenCodeVariableNameLineNumber,
					int givenCodeVariableNameCharacterNumber) {
				SpecificCommonErrorOutput error = MacroConstruction.this.error_
						.createAdjusted(givenCodeVariableNameLineNumber,
								givenCodeVariableNameCharacterNumber);
				ArgValuesBlock block = new ArgValuesBlock(givenCodeVariableNameParameter, error);
				MacroConstruction.this.blocks_.addElement(block);
				return block;
			}
		};

	}

	public ArgTypesBlockPatternPeer addOptionArgTypesBlockOfMacroElementForElement() {
		return new ArgTypesBlockPatternPeer() {
			public void end() {
			}

			public ArgTypesBlockBasePatternPeer addArgTypesBlockBaseForBlock(
					String givenCodeVariableNameParameter,
					int givenCodeVariableNameLineNumber,
					int givenCodeVariableNameCharacterNumber) {
				SpecificCommonErrorOutput error = MacroConstruction.this.error_
						.createAdjusted(givenCodeVariableNameLineNumber,
								givenCodeVariableNameCharacterNumber);
				ArgTypesBlock block = new ArgTypesBlock(givenCodeVariableNameParameter,
						error);
				MacroConstruction.this.blocks_.addElement(block);
				return block;
			}
		};
	}

	public CodeBlockPatternPeer addOptionCodeBlockOfMacroElementForElement(
			int keywordLineNumber, int keywordCharacterNumber) {
		// New section of code.
		SpecificCommonErrorOutput error = this.error_.createAdjusted(
				keywordLineNumber, keywordCharacterNumber);
		CodeBlock code = new CodeBlock(this.rootBuilder_, this, error);
		this.blocks_.addElement(code);
		return code;
	}

	public ContextBlockPatternPeer addOptionContextBlockOfMacroElementForElement(
			int keywordLineNumber, int keywordCharacterNumber) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(
				keywordLineNumber, keywordCharacterNumber);
		// TODO: new code block for each typekeywordCharacterNumber
		ContextBlock context = new ContextBlock(this, error);
		this.blocks_.addElement(context);

		return context;
	}

	public ContainerBlockPatternPeer addOptionContainerBlockOfMacroElementForElement(
			int keywordLineNumber, int keywordCharacterNumber) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(
				keywordLineNumber, keywordCharacterNumber);
		// TODO: new code block for each typekeywordCharacterNumber
		ContainerBlock container = new ContainerBlock(this, error);
		this.blocks_.addElement(container);

		return container;
	}

	/**
	 * public void setAsGeneral( JavaImplementationForm.BlockDataBlock
	 * elementSetBlock) { this.relatedDataBlock_ = elementSetBlock.getParent();
	 * this.primaryItemDataBlock_ = elementSetBlock; }
	 * 
	 * public ImplementationParameterTarget setMacroReference(LinkupTracker
	 * linkupTracker,
	 * JavaImplementationForm.BlockDataBlock.IsMacroDataBlock
	 * relatedDataBlock) { this.relatedDataBlock_ = relatedDataBlock.getParent();
	 * this.primaryItemDataBlock_ = relatedDataBlock.getElementSetParameter();
	 * return new
	 * ParameterBuilder(relatedDataBlock.getParent(),linkupTracker,this); }
	 */
	private boolean basicBuildBasics(SLayer parentRecord) {
		this.definingLayer_ = parentRecord;
		LinkupTracker linkupTracker = parentRecord.getRelatedTracker();
		JavaImplementationFormBlock.BlockDataBlock baseBlock = linkupTracker
				.allocateSpareNode(this.name_, parentRecord.getFullName() + ":[MACRO]" + this.name_);
		this.macroBlock_ = baseBlock
				.addIsMacro(parentRecord.getSkeletonBase());

		blockSkeletonManager_ = new ImplementationBlockSkeletonManager(baseBlock,	linkupTracker, this.environment_);
		boolean successful = false;

		try {
			this.peerLayer_ = parentRecord.addMacroLayer(this.name_, this,
					this.parameterVariables_, blockSkeletonManager_, this.macroBlock_,
					linkupTracker);
			successful = true;
		} catch (ParsingException e) {
			e.updateError(this.error_);
		}
		if (successful) {
			// The following builds the variables and places them in the parameter
			// variable store.
			this.parameterBlock_.build(this.macroBlock_, this.peerLayer_,
					this.blockSkeletonManager_, this.parameterVariables_);
		}
		return successful;
	}

	public boolean basicBuild(RecordClassAccess classAccess) {
		boolean successful = basicBuildBasics(classAccess.getBaseLayer());
		RecordClassAccess subClassAccess = classAccess.getAdjusted(this.peerLayer_);
		this.surroundingClassImplementation_ = classAccess.getContainingClass();
		if (successful) {
			for (Block b : this.blocks_) {
				successful &= b.basicBuild(this, subClassAccess);
			}
		}
		return successful;
	}

	public boolean basicBuild(SLayer parentRecord) {
		boolean successful = basicBuildBasics(parentRecord);

		// peerLayer_.basicBuild(new MacroRootType(peerLayer_, this,
		// this.parameterBlock_), blockSkeleton, linkupTracker);
		// this.macroRecord_ = new MacroRootRecord(parentRecord,linkupTracker,
		// this.parameterBlock_,this.name_,macroBlock_);
		if (successful) {
			// The following builds the variables and places them in the parameter
			// variable store.
			for (Block b : this.blocks_) {
				successful &= b.basicBuild(this, this.peerLayer_);
			}
		}
		return successful;
	}

	public void finishBuild() {
		for (Block b : this.blocks_) {
			b.finishBuild();
		}
	}

	public void end() {
	}

	private static interface Block {
		/**
		 * @param parentRecord
		 * @return true if successful
		 */
		public boolean basicBuild(MacroConstruction parentRecord, SLayer peerLayer);

		public boolean basicBuild(MacroConstruction parentRecord, RecordClassAccess peerClassAccess);

		public void finishBuild();
		// public void createCallInFinish(MacroCallDataBlock
		// macroCallBlock,MacroCallRequirementsAccess requirementsAccess);

	}

	// ==========================================================================
	// Macro Record stuff

	public ClassDetails formClassDetails() {
		if (this.formedClassDetails_ == null) {
			this.formedClassDetails_ = new ClassDetails(
					this,
					null, // this was this.surroundingClassImplementation_, but that
								// caused problems with the class used being the one the macro
								// was defined in, and not the one the macro was called from
								// (which might be useful for a different mode).
					this.macroBlock_);
			this.elements_.addElement(new Element() {
				public void createCallInFinish(MacroCallDataBlock macroCallBlock,
						MacroCallRequirementsAccess requirementsAccess) {
					requirementsAccess.accessContextReference(macroCallBlock);
				}

			});
		}
		return this.formedClassDetails_;
	}

	// ==========================================================================

	private final static class ContextBlock implements ContextBlockPatternPeer,
			Block {
		private final MacroConstruction parent_;
		private final JavaClassElements elements_;
		private MacroConstruction.ClassDetails context_;

		public ContextBlock(MacroConstruction parent,
				SpecificCommonErrorOutput baseError) {

			this.parent_ = parent;
			this.elements_ = new JavaClassElements("macro.context:" + parent.name_,parent.rootBuilder_, parent.environment_, baseError);
		}

		public boolean basicBuild(MacroConstruction parentRecord, RecordClassAccess peerClassAccess) {
			return basicBuild(parentRecord, peerClassAccess.getBaseLayer());
		}

		public boolean basicBuild(MacroConstruction macroRecord, SLayer peerLayer) {
			this.context_ = macroRecord.formClassDetails();
			elements_.basicBuild(this.context_.getClassRecordAccess());
			return true;
		}

		public void finishBuild() {
			elements_.finalBuild();
		}

		public JavaClassInterfaceElementPeer.Indirect getJavaClassInterfaceElementForElements() {
			return this.elements_;
		}

		public void addExternalTypeName(String value) {
			// TODO Auto-generated method stub
			Debug.finishMeMarker(); // For external types.
		}

		public void end() {
		}

	}

	// ==========================================================================

	private final static class ContainerBlock implements
			ContainerBlockPatternPeer, Block {
		private final MacroConstruction parent_;
		private final JavaClassElements elements_;
		private MacroConstruction.GeneralDetails context_;

		public ContainerBlock(MacroConstruction parent,
				SpecificCommonErrorOutput baseError) {

			this.parent_ = parent;
			this.elements_ = new JavaClassElements("macro.container:" + parent.name_,parent_.rootBuilder_, parent.environment_, baseError);
		}

		public boolean basicBuild(MacroConstruction parentRecord, RecordClassAccess peerClassAccess) {
			return basicBuild(parentRecord, peerClassAccess.getBaseLayer());
		}

		public boolean basicBuild(MacroConstruction macroRecord, SLayer peerLayer) {
			this.context_ = macroRecord.formGeneralDetails();
			elements_.basicBuild(this.context_.getContainerRecordAccess());
			return true;
		}

		public void finishBuild() {
			elements_.finalBuild();
		}

		public JavaClassInterfaceElementPeer.Indirect getJavaClassInterfaceElementForElements() {
			return this.elements_;
		}

		public void end() {
		}

	}

	// ==========================================================================
	private static interface CodeType {
		public CodeDetails formCodeDetailsQuiet(MacroConstruction environment);
	}

	private static final CodeType INLINE_CODE_TYPE = new CodeType() {
		public CodeDetails formCodeDetailsQuiet(MacroConstruction environment) {
			return environment.formContainingCodeBlock();

		}

	};

	private ImplementationBlockSkeletonManager blockSkeletonManager_;

	private final static class ArgValuesBlock implements
			ArgValsBlockBasePatternPeer, Block {
		private final SpecificCommonErrorOutput baseError_;
		private final ArgumentBucket arguments_;
		private final String argsVariableName_;
		private CodeBlockEnvironment codeEnvironment_;
		private ArgValuesReference argValuesParameter_;

		public ArgValuesBlock(String argsVariableName,
				SpecificCommonErrorOutput baseError) {
			this.baseError_ = baseError;
			this.argsVariableName_ = argsVariableName;
			this.arguments_ = new ArgumentBucket(baseError);

		}

		public boolean basicBuild(MacroConstruction parentRecord, RecordClassAccess peerClassAccess) {
			return basicBuild(parentRecord, peerClassAccess.getBaseLayer());
		}

		public boolean basicBuild(MacroConstruction parentRecord, SLayer peerLayer) {
			this.codeEnvironment_ = parentRecord.formContainingCodeBlock().getRecordLinkup();
			this.arguments_.basicBuild(codeEnvironment_);
			try {
				argValuesParameter_ = IRecordTypeTarget.generateTarget(
						parentRecord.parameterVariables_
								.getBuilderVariable(this.argsVariableName_))
						.getAsArgValuesMarkReference();
				return true;
			} catch (ParsingException e) {
				e.updateError(this.baseError_);
				return false;
			}
		}

		public boolean basicBuildWithEnvironment(CodeBlockEnvironment codeEnvironment, MacroConstruction parentRecord) {
			this.codeEnvironment_ = codeEnvironment;
			this.arguments_.basicBuild(codeEnvironment_);
			try {
				argValuesParameter_ = IRecordTypeTarget.generateTarget(
						parentRecord.parameterVariables_
								.getBuilderVariable(this.argsVariableName_))
						.getAsArgValuesMarkReference();
				return true;
			} catch (ParsingException e) {
				e.updateError(this.baseError_);
				return false;
			}
		}

		public void finishBuild() {
			this.arguments_.finish(codeEnvironment_, argValuesParameter_.getSetCodeTemplate());
		}

		public void end() {
		}

		public TargetCallArgumentsPatternPeer addTargetCallArgumentsForArguments() {
			return this.arguments_;
		}
	}

	// ==========================================================================
	private final static class ArgTypesBlock implements
			ArgTypesBlockBasePatternPeer, Block {
		private final SpecificCommonErrorOutput baseError_;
		private final TargetParametersBlock arguments_;
		private final String argsVariableName_;
		private CodeBlockEnvironment codeEnvironment_;
		private ArgTypesReference argTypesParameter_;

		public ArgTypesBlock(String argsVariableName,
				SpecificCommonErrorOutput baseError) {
			this.baseError_ = baseError;
			this.argsVariableName_ = argsVariableName;
			this.arguments_ = new TargetParametersBlock(baseError);

		}

		public boolean basicBuild(MacroConstruction parentRecord, RecordClassAccess peerClassAccess) {
			return basicBuild(parentRecord, peerClassAccess.getBaseLayer());
		}

		public boolean basicBuild(MacroConstruction parentRecord, SLayer peerLayer) {
			this.codeEnvironment_ = parentRecord.formContainingCodeBlock()
					.getRecordLinkup();
			this.arguments_.basicBuildMethod(this.codeEnvironment_);
			try {
				argTypesParameter_ = IRecordTypeTarget.generateTarget(
						parentRecord.parameterVariables_
								.getBuilderVariable(this.argsVariableName_))
						.getAsArgTypesMarkReference();
				return true;
			} catch (ParsingException e) {
				e.updateError(this.baseError_);
				return false;
			}
		}

		// public boolean basicBuildWithEnvironment(CodeBlockEnvironment
		// codeEnvironment, MacroRootRecord parentRecord) {
		// this.codeEnvironment_= codeEnvironment;
		// this.arguments_.basicBuildMethod(codeEnvironment_);
		// try {
		// argTypesParameter_ =
		// parentRecord.getArgTypesMarkReference(this.argsVariableName_);
		// return true;
		// } catch(ParsingException e) {
		// e.updateError(this.baseError_);
		// return false;
		// }
		// }
		public void finishBuild() {
			this.arguments_.finishBuildMethod(argTypesParameter_ .getTypeSetCodeTemplate());
		}

		public void end() {
		}

		public TargetParameterPeer.Indirect getTargetParameterForParameter() {
			return this.arguments_;
		}
	}

	// ==========================================================================
	private final static class CodeBlock implements CodeBlockPatternPeer, Block {
		private final MacroConstruction parent_;
		private final JavaCodeBlock actualCodeBlock_;
		private CodeDetails formCodeDetails_;
		private CodeType codeType_ = INLINE_CODE_TYPE;
		private final SpecificCommonErrorOutput baseError_;
		private SimpleVector<ArgValuesBlock> argValuesBlocks_ = null;
		private SimpleVector<ArgTypesBlock> argTypesBlocks_ = null;

		public CodeBlock(BaseBuilder baseRoot, MacroConstruction parent,
				SpecificCommonErrorOutput baseError) {
			this.parent_ = parent;
			this.baseError_ = baseError;
			this.actualCodeBlock_ = new JavaCodeBlock(baseRoot,baseError);
		}

		public void addGivenCodeVariableName(final String variableName, 	final int lineNumber, final int characterNumber) {
			final SpecificCommonErrorOutput variableError = this.baseError_ .createAdjusted(lineNumber, characterNumber);
			this.codeType_ = new CodeType() {
				public CodeDetails formCodeDetailsQuiet(MacroConstruction environment) {
					try {
						return environment.formVariableBasedCodeBlock(variableName);

					} catch (ParsingException e) {
						e.updateError(variableError);
						return null;
					}
				}

			};

		}

		// Ignore as called multiple times
		public void end() {
		}

		public JavaStatementPeer.Indirect getJavaStatementForStatement() {
			return this.actualCodeBlock_.getJavaStatementForStatement();
		}

		public boolean basicBuild(MacroConstruction environment, RecordClassAccess peerClassAccess) {
			this.formCodeDetails_ = this.codeType_.formCodeDetailsQuiet(environment);
			if (this.formCodeDetails_ != null) {
				CodeBlockEnvironment codeEnvironment = this.formCodeDetails_.getRecordLinkup();

				return basicBuildImpl(environment, codeEnvironment);
			} else {
				return false;
			}
		}

		public boolean basicBuild(MacroConstruction environment, SLayer peerLayer) {
			this.formCodeDetails_ = this.codeType_.formCodeDetailsQuiet(environment);
			if (this.formCodeDetails_ != null) {
				this.formCodeDetails_ = this.codeType_.formCodeDetailsQuiet(environment);
				CodeBlockEnvironment codeEnvironment = this.formCodeDetails_.getRecordLinkup();
				return basicBuildImpl(environment, codeEnvironment);
			} else {
				return false;
			}
		}

		public boolean basicBuildImpl(MacroConstruction environment, CodeBlockEnvironment codeEnvironment) {

			if (codeEnvironment != null) {
				// if(this.argValuesBlocks_!=null) {
				// for(ArgValuesBlock block : this.argValuesBlocks_) {
				// block.basicBuildWithEnvironment(codeEnvironment, environment);
				// }
				// }
				actualCodeBlock_.basicBuild(codeEnvironment);
				if (this.argValuesBlocks_ != null) {
					for (ArgValuesBlock block : this.argValuesBlocks_) {
						block.basicBuildWithEnvironment(codeEnvironment, environment);
					}
				}
				return true;
			} else {
				return false;
			}

		}

		public void finishBuild() {
			if (this.argTypesBlocks_ != null) {
				for (ArgTypesBlock block : this.argTypesBlocks_) {
					block.finishBuild();
				}
			}
			this.actualCodeBlock_.finishBuild(this.formCodeDetails_ .getCodeBlockSkeleton());
			if (this.argValuesBlocks_ != null) {
				for (ArgValuesBlock block : this.argValuesBlocks_) {
					block.finishBuild();
				}
			}
		}

		public ArgValsBlockBasePatternPeer addArgValsBlockBaseForBlock(
				String givenCodeVariableNameParameter,
				int givenCodeVariableNameLineNumber,
				int givenCodeVariableNameCharacterNumber) {
			SpecificCommonErrorOutput error = this.baseError_
					.createAdjusted(givenCodeVariableNameLineNumber,
							givenCodeVariableNameCharacterNumber);
			ArgValuesBlock block = new ArgValuesBlock(givenCodeVariableNameParameter,
					error);
			if (this.argValuesBlocks_ == null) {
				this.argValuesBlocks_ = new SimpleVector<MacroConstruction.ArgValuesBlock>();
			}

			this.argValuesBlocks_.addElement(block);
			return block;
		}

		public ArgTypesBlockBasePatternPeer addArgTypesBlockBaseForParametersBlock(
				String givenCodeVariableNameParameter,
				int givenCodeVariableNameLineNumber,
				int givenCodeVariableNameCharacterNumber) {
			SpecificCommonErrorOutput error = this.baseError_
					.createAdjusted(givenCodeVariableNameLineNumber,
							givenCodeVariableNameCharacterNumber);
			ArgTypesBlock block = new ArgTypesBlock(givenCodeVariableNameParameter,
					error);
			if (this.argTypesBlocks_ == null) {
				this.argTypesBlocks_ = new SimpleVector<MacroConstruction.ArgTypesBlock>();
			}

			this.argTypesBlocks_.addElement(block);
			return block;
		}
	}

	// ==========================================================================

	public CodeDetails formContainingCodeBlock() {
		if (this.formedCodeDetails_ == null) {
			CodeBlockEnvironment recordLinkup =
					CodeBlockEnvironment.createRootEnvironment(formClassDetails().getClassRecordAccess());
			this.formedCodeDetails_ = new CodeDetails(this, recordLinkup,
					this.macroBlock_.addInCode("_root",this.macroBlock_.getRootBuilder().createBCodeBlockAccessChild()).getBaseCodeBlockMacro(), "_root");
			this.elements_.addElement(new Element() {
				public void createCallInFinish(MacroCallDataBlock macroCallBlock,
						MacroCallRequirementsAccess requirementsAccess) {
					requirementsAccess.accessCodeReference(macroCallBlock);
				}

			});
		}
		return this.formedCodeDetails_;
	}

	public CodeDetails formVariableBasedCodeBlock(final String variableName)
			throws ParsingException {

		CodeBlockEnvironment recordLinkup = CodeBlockEnvironment
				.createRootEnvironment(formClassDetails().getClassRecordAccess());
		final CodeReference codeReference = IRecordTypeTarget.generateTarget(
				this.parameterVariables_.getBuilderVariable(variableName))
				.getAsCodeReference();
		CodeDetails resultDetails = new CodeDetails(this, recordLinkup,
				codeReference.getInCodeBlock(), variableName);
		this.elements_.addElement(new Element() {
			public void createCallInFinish(MacroCallDataBlock macroCallBlock,
					MacroCallRequirementsAccess requirementsAccess) {
				// requirementsAccess.addCodeReferenceRequirement(variableName,
				// macroCallBlock);

			}

		});
		return resultDetails;
	}

	// public MacroParameterBlock.ArgValuesReferenceParameter
	// getArgValuesMarkReference(String variableName) throws ParsingException {
	// return parameterBlock_.getArgValuesMarkReference(variableName);
	// }
	// public MacroParameterBlock.ArgTypesReferenceParameter
	// getArgTypesMarkReference(String variableName) throws ParsingException {
	// return parameterBlock_.getArgTypesMarkReference(variableName);
	// }

	public GeneralDetails formGeneralDetails() {
		if (this.formedGeneralDetails_ == null) {
			this.formedGeneralDetails_ = new GeneralDetails(this.peerLayer_, this.macroBlock_);
			this.elements_.addElement(new Element() {
				public void createCallInFinish(MacroCallDataBlock macroCallBlock,
						MacroCallRequirementsAccess requirementsAccess) {
					requirementsAccess.accessGeneralReference(macroCallBlock);
				}

			});
		}
		return this.formedGeneralDetails_;
	}

	// =============================================================================
	public static final class ClassDetails {
		private final RecordClassAccess classRecordAccess_;
		private final MacroConstruction parent_;
		private final InClassDataBlock classBlockSkeleton_;
		private final ClassImplementation classImplemenation_;

		private ClassDetails(
				MacroConstruction parent, ClassImplementation baseClassImplementationMaybeNull, IsMacroDataBlock baseBuilder) {
			this.parent_ = parent;
			this.classBlockSkeleton_ = baseBuilder.addInClass(baseBuilder.getRootBuilder().createBClassAccessChild());
			if (baseClassImplementationMaybeNull != null) {
				this.classImplemenation_ = baseClassImplementationMaybeNull;
			} else {
				this.classImplemenation_ = new ClassImplementation("[MACRO CLASS:"
						+ parent.name_ + "]", new ClassAccessSource() {
					public BClassAccessFormBlock getClassAccess() {
						return classBlockSkeleton_.getVariableStoreParameter();
					}
				}, parent.peerLayer_, null, null);
			}

			this.classRecordAccess_ = new RecordClassAccess(parent.peerLayer_,
					this.classImplemenation_, null);
		}

		public ClassImplementation getClassImplementation() {
			return this.classImplemenation_;
		}

		/**
		 * @return the classRecordAccess
		 */
		public RecordClassAccess getClassRecordAccess() {
			return classRecordAccess_;
		}
	}

	// --------------------------------------------------------------------------
	public static final class GeneralDetails {
		private final RecordContainerAccess containerRecordAccess_;
		private final JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.InGeneralDataBlock generalBlockSkeleton_;

		// private final ClassImplementation classImplemenation_;
		private GeneralDetails(SLayer peerLayer, IsMacroDataBlock baseBuilder) {
			this.generalBlockSkeleton_ = baseBuilder.addInGeneral(baseBuilder.getRootBuilder().createBClassContainerAccessChild());
			// this.classImplemenation_ = new
			// ClassImplementation("[MACRO CLASS:"+parent.macroName_+"]",new
			// ClassAccessSource() {
			// public BClassAccessTemplate getClassAccess() { return
			// classBlockSkeleton_; }
			// },parent);
			this.containerRecordAccess_ = new RecordContainerAccess(peerLayer, peerLayer, this.generalBlockSkeleton_.getVariableStoreParameter(), null);
		}

		// public ClassImplementation getClassImplementation() {
		// return this.classImplemenation_;
		// }
		// /**
		// * @return the classRecordAccess
		// */
		public RecordContainerAccess getContainerRecordAccess() {
			return containerRecordAccess_;
		}
	}

	// -------------------------------------------------------------------------------------------
	public static final class CodeDetails {
		private final CodeBlockEnvironment recordLinkup_;
		private final MacroConstruction parent_;
		private final CodeBlockMacroBlock macroInCodeSkeleton_;

		private CodeDetails(MacroConstruction parent,
				CodeBlockEnvironment recordLinkup,
				CodeBlockMacroBlock macroInCodeSkeleton, String name) {
			this.parent_ = parent;
			this.recordLinkup_ = recordLinkup;
			this.macroInCodeSkeleton_ = macroInCodeSkeleton;
		}

		/**
		 * @return the recordLinkup
		 */
		public CodeBlockEnvironment getRecordLinkup() {
			return recordLinkup_;
		}

		public CodeBlockMacroBlock getCodeBlockSkeleton() {
			return this.macroInCodeSkeleton_;
		}
	}

	// -------------
	private static interface Element {
		public void createCallInFinish(MacroCallDataBlock macroCallBlock,
				MacroCallRequirementsAccess requirementsAccess);
	}

}
