/**
 * 
 */
package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.MacroParameterBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroParameterPeer.RecordPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroParameterPeer.TargetPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroParameterPeer.VarPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferencePatternPeer;
import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.ImplementationBlockSkeletonManager;
import org.ffd2.skeletonx.compile.java.layer.PrimitiveVariableTypes;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.compile.java.layer.TypicalBuilderIVariable;
import org.ffd2.skeletonx.skeleton.BExpressionAccessFormHolder;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder;

import org.ffd2.skeletonx.skeleton.ArgumentSetDetailsFormHolder.ArgumentSetDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BCodeBlockAccessFormHolder.BCodeBlockAccessFormBlock;
import org.ffd2.skeletonx.skeleton.BParametersAccessFormHolder.BParametersAccessFormBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder.CodeBlockMacroBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder.ParameterSetterMacroBlock;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.CallChainStoreFormHolder.CallChainStoreFormBlock;
import org.ffd2.skeletonx.skeleton.CodeVariableFormHolder.CodeVariableFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.IntBuilderVariableAccessFormHolder.IntBuilderVariableAccessFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.ArgValuesMarkDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.ChainVariableDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.InCodeDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.IntValueDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.StringValueDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.NameDetailsFormHolder.NameDetailsFormBlock.ComplexDataBlock;
import org.ffd2.skeletonx.skeleton.StringBuilderVariableAccessFormHolder.StringBuilderVariableAccessFormBlock;

public final class MacroParameterBlock implements MacroParameterBlockPatternPeer {

  private final SimpleVector<Parameter> orderedParameters_ = new SimpleVector<Parameter>();
  private final SpecificCommonErrorOutput baseError_;
	private final BaseBuilder rootBuilder_;

	public MacroParameterBlock( BaseBuilder rootBuilder, SpecificCommonErrorOutput baseError) { 
		this.rootBuilder_ = rootBuilder;
		this.baseError_ = baseError;
	}
	public void requestParametersInCall(MacroCallDataBlock macroCallBlock,
			MacroCallRequirementsAccess requirementsAccess) {
		for(Parameter p : this.orderedParameters_) {
			p.requestParametersInCallFinal(requirementsAccess,macroCallBlock);
		}
	}
	public void build(IsMacroDataBlock baseBuilder,SLayer rootLayer,ImplementationBlockSkeletonManager builderParameters,VariableStore store ) {
		for(Parameter p : this.orderedParameters_) {
			p.build(baseBuilder,rootLayer, builderParameters,store);
		}
	}
	
	private final void  addParameter(String name, Parameter p) {
		this.orderedParameters_.addElement(p);
	}
	
	public void end() {  }
	
	public void addOptionArgTypesReferenceOfMacroParameterForParameter(
			String variableNameParameter, int variableNameLineNumber,
			int variableNameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(variableNameLineNumber, variableNameCharacterNumber);
		final ArgTypesReferenceParameter p = new ArgTypesReferenceParameter(variableNameParameter, error);
		addParameter(variableNameParameter,p);
		
	}

	
	public void addOptionCodeReferenceOfMacroParameterForParameter(
			String variableNameParameter, int variableNameLineNumber,
			int variableNameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(variableNameLineNumber, variableNameCharacterNumber);
		final CodeReferenceParameter p = new CodeReferenceParameter(variableNameParameter,this.rootBuilder_, error);
		addParameter(variableNameParameter, p);
	}
	public void addOptionMappingOfMacroParameterForParameter( int keywordLineNumber, int keywordCharacterNumber, 	String mappingNameParameter, String variableNameParameter) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		final MappingParameter p = new MappingParameter(variableNameParameter,mappingNameParameter, error);
		addParameter(variableNameParameter, p);
	}
	public void addOptionKeyOfMacroParameterForParameter(int keywordLineNumber,
			int keywordCharacterNumber, String keyNameParameter,
			String variableNameParameter) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		final KeyParameter p = new KeyParameter(variableNameParameter,keyNameParameter, error);
		addParameter(variableNameParameter, p);
		
	}
	
	public void addOptionArgValuesReferenceOfMacroParameterForParameter(
			String variableNameParameter, int variableNameLineNumber,
			int variableNameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(variableNameLineNumber, variableNameCharacterNumber);
		final ArgValuesReferenceParameter p = new ArgValuesReferenceParameter(variableNameParameter, error);
		addParameter(variableNameParameter, p);
		
	}
//	public CodeReferenceParameter getCodeReference(String variableName) throws ParsingException {
//		return getParameter(variableName).getAsCodeReference();
//	}
//	public ArgValuesReferenceParameter getArgValuesMarkReference(String variableName) throws ParsingException {
//		return getParameter(variableName).getAsArgValuesMarkReference();
//	}
//	public ArgTypesReferenceParameter getArgTypesMarkReference(String variableName) throws ParsingException {
//		return getParameter(variableName).getAsArgTypesMarkReference();
//	}
	/**
	 * Need to add "class variable" stuff to macro skeleton code
	 */
	public void addOptionClassReferenceOfMacroParameterForParameter(
			String nameParameter, int nameLineNumber, int nameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
		Debug.finishMeMarker();
		addParameter(nameParameter, new AbstractGeneralParameter() {
			public TargetVariable getAsTargetVariableQuiet() { return null; }
			public void build(IsMacroDataBlock baseBuilder, SLayer environment,ImplementationBlockSkeletonManager builderParameters,VariableStore store  ) {
				//baseBuilder.ad
				//TODO
				Debug.finishMeMarker();
			}
			public void requestParametersInCallFinal(
					MacroCallRequirementsAccess requirementsAccess,
					MacroCallDataBlock macroCallBlock) {
				// TODO Auto-generated method stub
				
			}			
			
		});
	}
	public void addOptionChainOfMacroParameterForParameter(int keywordLineNumber,
			int keywordCharacterNumber, String chainNameParameter, String variableNameParameter) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		
		ChainParameter rp = new ChainParameter(variableNameParameter,chainNameParameter,error);
		addParameter(variableNameParameter, rp);
	}
	public RecordPatternPeer addOptionRecordOfMacroParameterForParameter(
			String nameParameter, int nameLineNumber, int nameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
		RecordParameter rp = new RecordParameter(nameParameter,error);
		addParameter(nameParameter, rp);
		return rp;
	}
//	private final Parameter getParameter(String name) throws ItemNotFoundException {
//		Parameter p = this.parameterLookup_.get(name);
//		if(p==null) {
//			throw new ItemNotFoundException(name);
//		} else {
//			return p;
//		}
//	}
//	private final Parameter getParameterQuiet(String name) {
//		return this.parameterLookup_.get(name);
//	}
//	
//	public TargetVariable getTargetVariableQuiet(String name)  {
//		Parameter p = getParameterQuiet(name);
//		if(p!=null) {
//			return p.getAsTargetVariableQuiet();
//		} else {
//			return null;
//		}
//	}
//	public boolean isTargetVariableQuiet(String name)  {
//		Parameter p = getParameterQuiet(name);
//		return p!=null;
//	}


	public VarPatternPeer addOptionVarOfMacroParameterForParameter( String nameParameter, int nameLineNumber, int nameCharacterNumber) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
		TargetVariableParameter p = new TargetVariableParameter( nameParameter,this.rootBuilder_, error);
		addParameter(nameParameter,p);		
		return p;
	}
	
	public TargetPatternPeer addOptionTargetOfMacroParameterForParameter(String name, int nameLine, int nameCharacter) {
		final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(nameLine, nameCharacter);
		GeneralTargetParameter p = new GeneralTargetParameter( name,this.rootBuilder_, error);
		addParameter(name,p);		
		return p;
	}
	
	// =========================================================================
	private static interface Parameter {
		public void build(IsMacroDataBlock baseBuilder, SLayer environment,ImplementationBlockSkeletonManager builderParameters,VariableStore store );
		public void requestParametersInCallFinal(MacroCallRequirementsAccess requirementsAccess,MacroCallDataBlock macroCallBlock);
	}
	private abstract static class AbstractGeneralParameter implements Parameter {
		public CodeReference getAsCodeReference() throws ParsingException {
			throw ParsingException.createGeneralSyntax("variable not code reference");
		}
		public ArgValuesReference getAsArgValuesMarkReference() throws ParsingException {
			throw ParsingException.createGeneralSyntax("variable not arg values mark");
		}
		public ArgTypesReference getAsArgTypesMarkReference() throws ParsingException {
			throw ParsingException.createGeneralSyntax("variable not arg types mark");
		}

	}
	// ==========================================================================
	private final static class GeneralTargetParameter extends AbstractGeneralParameter implements Parameter, TargetPatternPeer, IRecordVariable {
		private final String variableName_;
		private final SpecificCommonErrorOutput error_;
		private TargetTypeBlock typeBlock_;
		private final BExpressionAccessFormHolder.BExpressionAccessFormBlock variableStore_;
		private JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.GeneralTargetDataBlock macroArgumentStore_;
		private TargetTypeReference resolvedType_;
		public GeneralTargetParameter( String variableName, BaseBuilder rootBuilder, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.variableStore_ = rootBuilder.createBExpressionAccessChild();
			this.error_ = error;
		}
		// ---
		// ---- AustenX bits

		public void end() { }
		public TargetTypeReferencePatternPeer addTargetTypeReferenceForType(String typeName, int typeNameLine, int typeNameCharacter) {
			return this.typeBlock_ = new TargetTypeBlock(typeName, this.error_.createAdjusted(typeNameLine, typeNameCharacter));
		}
		// =====

		public void build(JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock baseBuilder, SLayer environment, ImplementationBlockSkeletonManager builderParameters, VariableStore store) {
			this.typeBlock_.basicBuild(environment.getAsDataBlockEnvironment());
			this.macroArgumentStore_ = 
				baseBuilder.addGeneralTarget(this.variableName_,this.variableStore_);
			this.resolvedType_ = this.typeBlock_.getResolvedTypeReferenceQuietFinal();
			try {
				store.addBuilderVariable(this.variableName_, this);
			} catch(ParsingException e) {
				e.updateError(this.error_);
			}
		}

		public void requestParametersInCallFinal(MacroCallRequirementsAccess requirementsAccess, MacroCallDataBlock macroCallBlock) {
			Debug.println("Requesing general expression for:",this.variableName_);
			requirementsAccess.addGeneralExpressionTargetRequirement(this.variableName_,this.resolvedType_.getBaseType(),  macroCallBlock);
		}

		// ========================================================================

		public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
			throw ParsingException.createGeneralSemantic("general targets cannot provide name components");
		}

		public boolean isMatchesVariableName(String queryVariableName) { return this.variableName_.equals(queryVariableName); }

		public String getRoughVariableName() { return this.variableName_;	}

		public void updateExpressionFinal(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSemantic("general targets cannot provide general expressions");
		}

		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) throws ParsingException {
			throw ParsingException.createGeneralSemantic("general targets cannot provide record parameters");			
		}

		public void addAsMacroRecordMappingParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSemantic("general targets cannot provide mapping parameters");

		}

		public void addAsMacroRecordKeyParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, KeyDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSemantic("general targets cannot provide key parameters");
			
		}

		public void updateCallChainFinal(BasicCallChainFormBlock target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
			throw ParsingException.createGeneralSemantic("general targets cannot provide call chains");
		}

		public void getAsReference(IRecordTypeTarget target) {
			target.setAsExpressionSetter(this.variableStore_);
		}

		public SLayer getAsFoundataionNodeReferenceVariable(JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSemantic("general targets cannot record references");

		}

		public void addAsGeneralExpressionTargetRequirement(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath) throws ParsingException {
			macroCallBlock.addCallParameter().addExpressionSource(this.variableStore_, variablePath);
		}
	} // end of class GeneralTargetParameter
	// ==========================================================================

	private final static class TargetVariableParameter extends AbstractGeneralParameter implements Parameter, VarPatternPeer, TargetVariable {
		private final SpecificCommonErrorOutput error_;
		private final String variableName_;
		private JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.TargetVariableDataBlock macroTargetVariableBlock_;
    private TargetTypeBlock typeBlock_;
		private final CodeVariableFormBlock macroCodeVariableSkeleton_;
		public TargetVariableParameter( String variableName, BaseBuilder rootBuilder, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.error_ = error;
			this.macroCodeVariableSkeleton_ = rootBuilder.createCodeVariableChild();
		}

		public void requestParametersInCallFinal(MacroCallRequirementsAccess requirementsAccess,MacroCallDataBlock macroCallBlock) {
			TargetTypeReference resolvedType = this.typeBlock_.getResolvedTypeReferenceQuietFinal();
			if(resolvedType!=null) {
				//Hmmm... isn't this a bit tricky
				Debug.futureFeatureMarker("I'm not sure how macros are supposed to handle complex variable referenced types...");
				requirementsAccess.addTargetVarRequirement(this.variableName_, resolvedType.getBaseType(), macroCallBlock);
			}
		}
		// ================= Target Variable stuff =======================
	
		public void createCallExpressionFinal(BasicCallChainFormBlock target, BaseTrackers.JavaVariablePath basePathDisposable) {
    	target.addElement().addVariable(this.macroCodeVariableSkeleton_, basePathDisposable);
    }
		public TargetTypeReference getTargetTypeReferenceQuietFinal() {
			return this.typeBlock_.getResolvedTypeReferenceQuietFinal();
		}
		public void build(IsMacroDataBlock baseBuilder, SLayer environment,ImplementationBlockSkeletonManager builderParameters,VariableStore store  ) {
			this.typeBlock_.basicBuild(environment.getAsDataBlockEnvironment());
			this.macroTargetVariableBlock_ = 
				baseBuilder.addTargetVariable(this.variableName_,this.macroCodeVariableSkeleton_);
			try {
				store.addTargetVariable(this.variableName_, this);
			} catch(ParsingException e) {
				e.updateError(this.error_);
			}
		}
		public void createMacroCallParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath pathToVariable) {
			macroCallBlock.addCallParameter().addTargetVariable().addSimple(this.macroCodeVariableSkeleton_, pathToVariable);
		}
		// ============== VarPattern stuff ==================================
		
		public TargetTypeReferencePatternPeer addTargetTypeReferenceForType(String typeNameParameter, int typeNameLineNumber,	int typeNameCharacterNumber) {
			return this.typeBlock_ = new TargetTypeBlock(typeNameParameter, this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber));
		}
		public void end() {}
	}
	private static final class KeyParameter implements Parameter {
		private final String	variableName_;
		private final SpecificCommonErrorOutput	error_;
		private final String	keyName_;
//		private JavaImplementationForm.BlockDataBlock.IsMacroDataBlock.MappingVariableDataBlock	mappingParameterSkeleton_;
		private TypicalBuilderIVariable	blockParameter_;
		private KeyDefinitionBlock definition_;
		public KeyParameter(String variableName, String keyName, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.keyName_ = keyName;
			this.error_ = error;
		
		}
		public void build(IsMacroDataBlock baseBuilder, SLayer environment, ImplementationBlockSkeletonManager builderParameters, VariableStore store) {
			
			try {
				this.definition_ = environment.getKeyDefinition(this.keyName_);
				this.blockParameter_ = builderParameters.addKeyParameter(this.variableName_, this.definition_);

				store.addBuilderVariable(this.variableName_, this.blockParameter_);
			} catch(ParsingException e) {
				e.updateError(this.error_);
			}
			
			
			
		}

		public void requestParametersInCallFinal( MacroCallRequirementsAccess requirementsAccess, MacroCallDataBlock macroCallBlock) {
			requirementsAccess.addKeyRequirement(this.variableName_, this.definition_, macroCallBlock);
		}
		
	}
	public static final class MappingParameter implements Parameter {
		private final String	variableName_;
		private final SpecificCommonErrorOutput	error_;
		private final String	mappingName_;
//		private JavaImplementationForm.BlockDataBlock.IsMacroDataBlock.MappingVariableDataBlock	mappingParameterSkeleton_;
		private TypicalBuilderIVariable	blockParameter_;
		private MappingDefinitionBlock definition_;
		public MappingParameter(String variableName, String mappingName, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.mappingName_ = mappingName;
			this.error_ = error;
		
		}
		public void build(IsMacroDataBlock baseBuilder, SLayer environment, ImplementationBlockSkeletonManager builderParameters, VariableStore store) {
			
			try {
				this.definition_ = environment.getMappingDefinition(this.mappingName_);
				this.blockParameter_ = builderParameters.addMappingParameter(this.variableName_, this.definition_);
//				this.mappingParameterSkeleton_ = definition.addMappingMacro(variableName_,baseBuilder);
//			}
//
//		
//			public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock,
//					JavaVariablePath variablePath, IVariableType type)
//					throws ParsingException {
//				
//			
//				this.argTypesMarkSkeleton_ = baseBuilder.addArgTypesMarkBlock(variableName_);
//
//				store.addBuilderVariable(this.variableName_,new SpecificPurposeBuilderVariable(new SpecificPurposeBuilderVariable.Details() {
//			
//					public CodeReference getAsCodeReference() { return null; }
//					public ArgValuesReference getAsArgValuesMarkReference() { return null; }
//					public ArgTypesReference getAsArgTypesMarkReference() { return ArgTypesReferenceParameter.this; 	}
//					
//					public boolean addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) {
//						if(type==IVariableType.ArgumentTypesReference) {
//							macroCallBlock.addArgumentTypesReferenceBlock(argTypesMarkSkeleton_, variablePath);
//							return true;
//						} else {
//							return false;
//						}
//					}
//				}),this.error_);
				store.addBuilderVariable(this.variableName_, this.blockParameter_);
				
			//	this.mappingVariableParameterSkeleton_ = definition.addMappingMacro(base)
			//	baseBuilder.addMappingVariableBlock(this.variableName_);
			} catch(ParsingException e) {
				e.updateError(this.error_);
			}
			
			
			
		}

		public void requestParametersInCallFinal( MacroCallRequirementsAccess requirementsAccess, MacroCallDataBlock macroCallBlock) {
			requirementsAccess.addMappingRequirement(this.variableName_, this.definition_, macroCallBlock);
		}
		
	}
	// --------------------------------------------------------------------------
	public static final class CodeReferenceParameter extends AbstractGeneralParameter implements CodeReference, Parameter {
		private final String	variableName_;
		private final SpecificCommonErrorOutput	error_;
		private InCodeDataBlock	inCodeBlock_;

		public CodeReferenceParameter(String variableName, BaseBuilder rootBuilder,  SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.error_ = error;
		}
		public CodeBlockMacroBlock getInCodeBlock()  {
			return this.inCodeBlock_.getBaseCodeBlockMacro();
		}
		public void build(IsMacroDataBlock baseBuilder, SLayer environment, ImplementationBlockSkeletonManager builderParameters, VariableStore store) {
			BaseBuilder root = baseBuilder.getRootBuilder();

			final BCodeBlockAccessFormBlock valueStoreSkeleton = root.createBCodeBlockAccessChild();
			this.inCodeBlock_ = baseBuilder.addInCode(variableName_, valueStoreSkeleton);
			try {
				store.addBuilderVariable(this.variableName_, new SpecificPurposeBuilderVariable(new SpecificPurposeBuilderVariable.Details() {
					public void getAsReference(IRecordTypeTarget target) { target.setAsCodeReference(CodeReferenceParameter.this); }
					public boolean addAsMacroRecordParameter(JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) {
						if(type==IVariableType.CodeMarkReference) {
							macroCallBlock.addInCode(valueStoreSkeleton, variablePath);
							return true;
						} else {
							return false;
						}
					}
				}));
			} catch(ItemAlreadyExistsException e) {
				e.updateError(this.error_);
			}
		}

		public void requestParametersInCallFinal( MacroCallRequirementsAccess requirementsAccess, MacroCallDataBlock macroCallBlock) {
			requirementsAccess.addCodeReferenceRequirement(this.variableName_, macroCallBlock);
		}
	
	}
	//--------------------------------------------------------------------------

	public final static class ArgValuesReferenceParameter extends AbstractGeneralParameter implements ArgValuesReference, Parameter {
		private final String	variableName_;
		private final SpecificCommonErrorOutput	error_;
		private ArgValuesMarkDataBlock	argValuesMarkSkeleton_;

		public ArgValuesReferenceParameter(String variableName, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.error_ = error;
		}

		public ArgumentSetDetailsFormBlock getSetCodeTemplate() {
			return this.argValuesMarkSkeleton_.getArgumentsDetailsParameter();
		}
		public void build(IsMacroDataBlock baseBuilder, SLayer environment, ImplementationBlockSkeletonManager builderParameters, VariableStore store) {
			BaseBuilder root = baseBuilder.getRootBuilder();
			this.argValuesMarkSkeleton_ = baseBuilder.addArgValuesMark(variableName_,root.createArgumentSetStoreChild(),root.createArgumentSetDetailsChild());
			try {
				store.addBuilderVariable(this.variableName_, new SpecificPurposeBuilderVariable(new SpecificPurposeBuilderVariable.Details() {
					public void getAsReference(IRecordTypeTarget target) { target.setAsArgValuesMarkReference(ArgValuesReferenceParameter.this); }
	
					public boolean addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) { return false; }
				}));
			} catch(ItemAlreadyExistsException e) {
				e.updateError(this.error_);
			}
		}

		public void requestParametersInCallFinal(
				MacroCallRequirementsAccess requirementsAccess,
				MacroCallDataBlock macroCallBlock) {
			requirementsAccess.addArgValuesMarkRequirement(this.variableName_, macroCallBlock);
		}
		
	}
//--------------------------------------------------------------------------

	public final static class ArgTypesReferenceParameter extends AbstractGeneralParameter implements ArgTypesReference,Parameter {
		private final String	variableName_;
		private final SpecificCommonErrorOutput	error_;
		private JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.ArgTypesMarkDataBlock	argTypesMarkSkeleton_;

		public ArgTypesReferenceParameter(String variableName, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
		
			this.error_ = error;
		
		}
		public ArgTypesReference getAsArgTypesMarkReference() throws ParsingException {
			return this;
		}
		public TargetVariable getAsTargetVariableQuiet() {	return null; }
		public JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock.ArgTypesMarkDataBlock getArgTypesMark()  {
			return this.argTypesMarkSkeleton_;
		}
		public ParameterSetterMacroBlock getTypeSetCodeTemplate() {
			return this.argTypesMarkSkeleton_.getSetArgumentsMacro();
		}

		public void build(IsMacroDataBlock baseBuilder, SLayer environment, ImplementationBlockSkeletonManager builderParameters, VariableStore store) {
			BaseBuilder root = baseBuilder.getRootBuilder();
			final BParametersAccessFormBlock valueStoreSkeleton = root.createBParametersAccessChild();
			this.argTypesMarkSkeleton_ = baseBuilder.addArgTypesMark(variableName_,valueStoreSkeleton);
			
			store.addBuilderVariable(this.variableName_,new SpecificPurposeBuilderVariable(new SpecificPurposeBuilderVariable.Details() {
				public void getAsReference(IRecordTypeTarget target) { target.setAsArgTypesMarkReference(ArgTypesReferenceParameter.this); }
				public boolean addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) {
					if(type==IVariableType.ArgumentTypesReference) {
						macroCallBlock.addArgumentTypesReference(valueStoreSkeleton, variablePath);
						return true;
					} else {
						return false;
					}
				}
			}),this.error_);
		
//			builderParameters.newCodeReferenceparameter(this.variableName_);
		}

		public void requestParametersInCallFinal(
				MacroCallRequirementsAccess requirementsAccess,
				MacroCallDataBlock macroCallBlock) {
			requirementsAccess.addArgTypesMarkRequirement(this.variableName_, macroCallBlock);
		}
		
	}
	//--------------------------------------------------------------------------
	
	private final static class ChainParameter extends AbstractGeneralParameter  implements Parameter {
		private final SpecificCommonErrorOutput error_;
		private final String variableName_;
		//private TargetVariableDataBlock macroTargetVariableBlock_;
		private final String chainName_;
		private ChainVariableDataBlock chainVariableParameterSkeleton_;
		public ChainParameter( String variableName, String chainName, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.chainName_ = chainName;
			this.error_ = error;
			
		}
	
		public void requestParametersInCallFinal(MacroCallRequirementsAccess requirementsAccess,MacroCallDataBlock macroCallBlock) {
			Debug.finishMeMarker();
			//TODO
			//requirementsAccess.addTargetVarRequirement(this.variableName_, this.typeBlock_.getResolvedType(), macroCallBlock);
		}
		// ================= Target Variable stuff =======================
		//?

		
		public void build(IsMacroDataBlock baseBuilder, SLayer environment,ImplementationBlockSkeletonManager builderParameters, VariableStore store) {
			BaseBuilder root = baseBuilder.getRootBuilder();
			final CallChainStoreFormBlock callChainSkeleton = root.createCallChainStoreChild();
			this.chainVariableParameterSkeleton_ = baseBuilder.addChainVariable(this.variableName_,callChainSkeleton);
			try {
				ChainDefinitionBlock definition = environment.getChainDefinition(this.chainName_);
				builderParameters.addChainParameter(this.variableName_, definition);
			} catch(ParsingException e) {
				e.updateError(this.error_);
			}
			store.addBuilderVariable(this.variableName_, new SpecificPurposeBuilderVariable(new SpecificPurposeBuilderVariable.Details() {
				public void getAsReference(IRecordTypeTarget target) { target.setNotReference(); }

				
				public boolean addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) {
					if(type==IVariableType.ChainVariable) {
						macroCallBlock.addCallParameter().addGeneralChainVariable(callChainSkeleton, variablePath); 
						return true;
					} else {
						return false;
					}
				}
			}),error_);
		}
			
	
	}
	
	// --------------
	private final static class RecordLink {
		private final String name_;
		private final SpecificCommonErrorOutput error_;
		private FoundationNode found_;
		public RecordLink(String name, SpecificCommonErrorOutput error) {
			this.name_ = name;
			this.error_ = error;
		}
		public FoundationNode getNextQuiet(FoundationNode base) {
			FoundationNode result = base.getChildQuiet(this.name_);
			if(result==null) {
				this.error_.objectNotFoundError("Root record", this.name_);
				return null;
			} else {
				
				return this.found_ = result;
			}
		}
	}
	private static interface RecordType {
		public void doRequestParameter(String variableName,MacroCallRequirementsAccess requirementsAccess, MacroCallDataBlock macroCallBlock);
	}
	private final static RecordType STRING_PRIMITIVE_RECORD_TYPE = new RecordType() {
		public void doRequestParameter(String variableName,
				MacroCallRequirementsAccess requirementsAccess,
				MacroCallDataBlock macroCallBlock) {
			requirementsAccess.addStringRequirement(variableName, macroCallBlock);
		}
	};
	private final static RecordType INT_PRIMITIVE_RECORD_TYPE = new RecordType() {
		public void doRequestParameter(String variableName,
				MacroCallRequirementsAccess requirementsAccess,
				MacroCallDataBlock macroCallBlock) {
			requirementsAccess.addIntRequirement(variableName, macroCallBlock);
		}
	};
	private final static class RecordParameter extends AbstractGeneralParameter  implements Parameter, RecordPatternPeer {
		private final SpecificCommonErrorOutput error_;
		private final SimpleVector<RecordLink> links_ = new SimpleVector<RecordLink>();
		private final String variableName_;
		private RecordType recordType_;
		
		public RecordParameter(String variableName, SpecificCommonErrorOutput error) {
			this.variableName_ = variableName;
			this.error_ = error;			
		}

		public void addTypeElement(String value, int lineNumber, int characterNumber) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(lineNumber, characterNumber);
			this.links_.addElement(new RecordLink(value,error));
		}

		public void end() {			}
		private boolean checkPrimitive(String type, IsMacroDataBlock baseBuilder, VariableStore store ) {
			if(type.equals("String")) {
				StringBuilderVariableAccessFormBlock stringAccessSkeleton = baseBuilder.getRootBuilder().createStringBuilderVariableAccessChild();
				StringValueDataBlock stringValueSkeleton = baseBuilder.addStringValue(this.variableName_,stringAccessSkeleton);
				store.addBuilderVariable(
						this.variableName_, 
						new TypicalBuilderIVariable(this.variableName_, PrimitiveVariableTypes.createStringVariableType( stringAccessSkeleton,null)),
						this.error_);
				this.recordType_ = STRING_PRIMITIVE_RECORD_TYPE;
				return true;
			} 
			if(type.equals("int")) {
				IntBuilderVariableAccessFormBlock intAccessSkeleton = baseBuilder.getRootBuilder().createIntBuilderVariableAccessChild();
				IntValueDataBlock valueSkeleton = baseBuilder.addIntValue(this.variableName_,intAccessSkeleton);
				store.addBuilderVariable(
						this.variableName_, 
						new TypicalBuilderIVariable(this.variableName_, PrimitiveVariableTypes.createIntVariableType( intAccessSkeleton)),
						this.error_);
				this.recordType_ = INT_PRIMITIVE_RECORD_TYPE;

				return true;
			} 
			return false;
		}
		public void build(IsMacroDataBlock baseBuilder, SLayer environment,ImplementationBlockSkeletonManager builderParameters, VariableStore store ) {
			int numberOfLinks = links_.size();
			RecordLink initialLink = links_.firstElement();
			if(numberOfLinks!=1||!checkPrimitive(initialLink.name_,baseBuilder,store)) {
				
				FoundationNode rootNode = environment.getRootFoundationNodeBasic();
				
				FoundationNode firstRecord = initialLink.getNextQuiet(rootNode);
				FoundationNode endRecord = firstRecord;
				for(int i = 1 ; endRecord!=null && i < numberOfLinks ; i++) {
					RecordLink nextLink = links_.get(i);
					endRecord = nextLink.getNextQuiet(endRecord);
				}
				if(endRecord!=null) {
					store.addBuilderVariable(this.variableName_, builderParameters.addGeneralRecordParameter(this.variableName_, endRecord),this.error_);
					final FoundationNode finalEndRecord = endRecord;
					this.recordType_ = new RecordType() {
						public void doRequestParameter(String variableName,MacroCallRequirementsAccess requirementsAccess, MacroCallDataBlock macroCallBlock) {
							requirementsAccess.addRecordVarRequirement(variableName, finalEndRecord, macroCallBlock);
							;
						}
					};
				}
			}
		}
		public void requestParametersInCallFinal(
				MacroCallRequirementsAccess requirementsAccess,
				MacroCallDataBlock macroCallBlock) {
			this.recordType_.doRequestParameter(this.variableName_, requirementsAccess, macroCallBlock);
//			requirementsAccess.addRecordVarRequirement(this.variableName_, this.recordType_, macroCallBlock);
		}
	}

	
	



	// ========================================
	
}