package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;

public class RecordClassAccess {

  private final ClassImplementation containingClass_;
  private final SLayer currentLayer_;
  private BaseTrackers.JavaVariablePath builtPathToClassDef_;
	private final GlobalClassReference[]	allGlobalReferences_;
	/**
	 * 
	 * @param currentRecord
	 * @param containingClass
	 * @param allGlobalReferences the global references of the containing class - may be null if no such references made
	 */
  public RecordClassAccess(SLayer currentLayer, ClassImplementation containingClass,final GlobalClassReference[] allGlobalReferences) {
    this.currentLayer_ = currentLayer;
    this.containingClass_ = containingClass;
		this.allGlobalReferences_ = allGlobalReferences;
  }
  public BaseTrackers.JavaVariablePath getBuiltPathToClassDef() {
  	return builtPathToClassDef_;
  }
  public RecordClassAccess getAdjusted(SLayer decendentLayer) {
  	return new RecordClassAccess(decendentLayer, this.containingClass_, this.allGlobalReferences_);
  }
  public void addContextRef(MacroCallDataBlock macroCallBlock) {
  	macroCallBlock.addInClass(containingClass_.getClassAccess(), this.getPathToClassDefinition());
  }
  public void addGeneralRef(MacroCallDataBlock macroCallBlock) {
   	macroCallBlock.addInClass(containingClass_.getClassAccess(), this.getPathToClassDefinition()); 	
  }
  public MacroCallInstance createMacroCallFinal(
			String variableName,
			MacroConstruction macro,
			MacroCallRequirementsAccess requirementsAccess
		) {
  	return this.currentLayer_.createMacroCallFinal(variableName, macro,requirementsAccess);
  }
  //Repeated with Basic version
  //  public MacroConstruction getMacroConstructionFinal(String macroName) throws ItemNotFoundException {
//  	return this.currentRecord_.getMacroConstructionBasic(macroName);
//  }
  
  public MacroCallInstance createMacroCallFinal(
			MacroConstruction macro,
			MacroCallRequirementsAccess requirementsAccess
		) {
		return this.currentLayer_.createMacroCallFinal(macro.getName(), macro, requirementsAccess);
	}
  public GlobalClassReference[] getAllGlobalClassReferences() { return this.allGlobalReferences_; }
  public RecordClassAccess getLinkedQuiet(LinkChainBlock chainBlock)  {
  	//This is wrong as it does not handle variable path stuff?
  	//Answer: No, as the linkage will be rebuilt later when requested..
  	//UPDATE: HAVE SHIFTED STUFF BACK TO NORMAL - NEED TO CHECK THE UTILS. method
    SLayer linkedLayer = chainBlock.getLinkChainEndContainerQuiet(this.currentLayer_);
    if(linkedLayer==null) {
    	return null;
    }
//    Debug.println("LINK PATH:",linkPathToRecord);
    return new RecordClassAccess(linkedLayer, this.containingClass_,this.allGlobalReferences_);
 
  }
  public void accessType(TargetType typeName, BaseTrackers.JavaVariablePath pathToVarible) {
  	this.currentLayer_.buildPathToType(typeName, pathToVarible,true);
  }
  
	public MacroConstruction getMacroConstructionFinal(String name, BaseTrackers.JavaVariablePath pathToContainingLayer) throws ItemNotFoundException {
		return this.currentLayer_.getMacroConstructionFinal(name, pathToContainingLayer);
	}
  public CodeBlockEnvironment createRootCodeBlock() {
    return CodeBlockEnvironment.createRootEnvironment(this);
  }
  public ClassImplementation getContainingClass() {
    return this.containingClass_;
  }
  public SLayer getBaseLayer() { return this.currentLayer_; }
  
  public boolean isMatchingLocation(TargetType locationType) {
    return locationType==this.containingClass_;
  }
  public RecordClassAccess getChild(String childName, int lineNumber) throws ParsingException{
    SLayer linkedRecord = this.currentLayer_.getNodeChild(childName,lineNumber);
    return new RecordClassAccess(linkedRecord,this.containingClass_,this.allGlobalReferences_);
  }
  public TargetMethod getTargetMethodQuiet(String name, JavaVariablePath pathToDeclaration) {
    return this.currentLayer_.getTargetMethodQuiet(this.containingClass_, name,pathToDeclaration,true);
  }

  public TargetVariable getTargetVariableQuiet( boolean allowLocalVariables,String name, JavaVariablePath pathToDeclaration) {
    return this.currentLayer_.getTargetVariableQuiet(allowLocalVariables, this.containingClass_, name, pathToDeclaration,true);
  }
  private BaseTrackers.JavaVariablePath getPathToClassDefinition() {
    if(builtPathToClassDef_==null) {
    	this.builtPathToClassDef_ = this.containingClass_.getPathToClass(this.currentLayer_);
    }
    return builtPathToClassDef_;
  }
  public void addClassVariableBasic(String name, TargetVariable variable) throws ItemAlreadyExistsException {
    this.currentLayer_.addClassVariable(this.containingClass_, name, variable);
  }
  public void addClassMethodBasic(String name, TargetMethod method)  throws ItemAlreadyExistsException {
    this.currentLayer_.addClassMethod(this.containingClass_, name, method);
  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock 
          createClassVariableFinal(String name, TargetVariable variable)
        {
    return this.containingClass_.createInstanceVariable(this.currentLayer_, name, variable,getPathToClassDefinition());
  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock 
      createClassMethodFinal(String name) {

      return this.containingClass_.createInstanceMethodFinal(this.currentLayer_, name, getPathToClassDefinition());
  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ConstructorDataBlock 
  	createClassConstructorFinal() {

  	return this.containingClass_.createConstructorFinal(this.currentLayer_,getPathToClassDefinition());
  }
}
