package org.ffd2.skeletonx.compile.java;

import java.io.IOException;

import org.ffd2.skeletonx.austenx.peg.base.DeferredJavaImplementationElement;
import org.ffd2.skeletonx.austenx.peg.base.FindDefinitionPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.GeneralMacroCallPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.GlobalClassDefinitionPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaClassPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaImplementationElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaInterfacePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaMacroBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.LinkChainPatternPeer;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;

import org.ffd2.solar.general.Debug;
import  org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.GeneralCommonErrorOutput;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class RootLevelElementsBlock implements JavaImplementationElementPeer.Indirect{

  private final SpecificCommonErrorOutput baseError_;

  private final PassthroughHandler passthroughHandler_;
  private final GeneralRecordItemStore itemStore_;

	private final FileEnvironment fileEnvironment_;
	private String currentPackageSubName_ = null;
	private final SimpleVector<FindVariable> finds_ = new SimpleVector<FindVariable>();
	private SLayer containingLayer_;

	private final BaseBuilder rootBuilder_;
	
  public RootLevelElementsBlock( FileEnvironment fileEnvironment, BaseBuilder rootBuilder, String currentPackageSubName, SpecificCommonErrorOutput baseError) {
    this.rootBuilder_ = rootBuilder;
		this.itemStore_ = new GeneralRecordItemStore();
    this.currentPackageSubName_ = currentPackageSubName;
		this.fileEnvironment_ = fileEnvironment;

    this.baseError_ = baseError;
    this.passthroughHandler_ = new PassthroughHandler(this);
  }
  private RootLevelElementsBlock(RootLevelElementsBlock parent, BaseBuilder rootBuilder, FileEnvironment newEnvironment, SpecificCommonErrorOutput newBaseError ) {
  	this.rootBuilder_ = rootBuilder;
		this.itemStore_ = parent.itemStore_;
    this.currentPackageSubName_ = parent.currentPackageSubName_;
 		this.fileEnvironment_ = newEnvironment;

     this.baseError_ = newBaseError;
     this.passthroughHandler_ = new PassthroughHandler(this);
  }
  
  public boolean basicBuild(SLayer containingLayer) {
  	this.containingLayer_ = containingLayer;
  	if(!this.finds_.isEmpty()) {
			FoundationNode rootRecord= containingLayer.getRootFoundationNodeBasic();

	  	for(FindVariable variable : this.finds_ ) {
	  		variable.basicBuild(rootRecord,containingLayer);
	  	}
  	}
  	return this.itemStore_.basicBuild(containingLayer);
  	
  }
  public void finishBuild() {
  	if(!this.finds_.isEmpty()) {
			FoundationNode rootRecord= this.containingLayer_.getRootFoundationNodeBasic();

	  	for(FindVariable variable : this.finds_ ) {
	  		variable.finishBuild(rootRecord,this.containingLayer_);
	  	}
  	} 	
  	this.itemStore_.finishBuild();
  }
  
	public void createInclude(String fileName, int fileNameLineNumber, int fileNameCharacterNumber) {
		SpecificCommonErrorOutput error = this.baseError_.createAdjusted(fileNameLineNumber, fileNameCharacterNumber);
		try {
			
			String fileText = this.fileEnvironment_.readFile(fileName);
			GeneralCommonErrorOutput adjustedBase = error.createAdjustedBase(fileName);

			SpecificCommonErrorOutput newBaseError = adjustedBase.createNewOriginSpecificCommonErrorOutput();
			//TODO: Need to fix file environment to be slightly adjusted so that further readTexts and so on get from correct file context (eg, if this file is a library one, further access should be from the library only)
			
			JavaIncludeImplementationElementTool includeTool = new JavaIncludeImplementationElementTool(new RootLevelElementsBlock(this,this.rootBuilder_, this.fileEnvironment_,newBaseError));
			includeTool.doInclude(fileText, adjustedBase);
		} catch(IOException e) {
			error.generalIOError("File not accesible:"+fileName+"("+e.getMessage()+")");
		}
		
	}
  public JavaImplementationElementPeer.ClassPatternPeer createClass() { return passthroughHandler_; }

  public JavaImplementationElementPeer.InterfacePatternPeer createInterface() { return passthroughHandler_; }
  
  public JavaImplementationElementPeer.MacroDefPatternPeer createMacroDef()  { return passthroughHandler_; }
  public JavaImplementationElementPeer.FindPatternPeer createFind() { return passthroughHandler_; }
  
	public JavaImplementationElementPeer.MacroCallPatternPeer createMacroCall() { return passthroughHandler_; }
  
  public JavaImplementationElementPeer.LinkPatternPeer createLink() { 
    LinkHandler linkHandler = new LinkHandler(this.fileEnvironment_,this.rootBuilder_,currentPackageSubName_, this.baseError_); 
    this.itemStore_.addElement(linkHandler);
    return linkHandler;
  }
	public JavaImplementationElementPeer.IfPatternPeer createIf() { Debug.finishMeMarker();
		// TODO Auto-generated method stub
		return null;
	}

	// ==========================================================================
  
	private final void changeCurrentPackage(int keywordLineNumber,int keywordCharacterNumber, String nameParameter) {
		this.currentPackageSubName_ = nameParameter;
	}
	public JavaImplementationElementPeer.PackageSetPatternPeer createPackageSet(final int keywordLineNumber, final int keywordCharacterNumber) {
		return new JavaImplementationElementPeer.PackageSetPatternPeer() {
			StringBuffer store = new StringBuffer();
			public void end() { 
				changeCurrentPackage(keywordLineNumber, keywordCharacterNumber,store.toString());
			}
			
			public void addNameElement(String value) {
				if(store.length()!=0) {
					store.append('.');
				}	
				store.append(value);
				
			}
		};
	}
  


  private static final class PassthroughHandler implements 
  		JavaImplementationElementPeer.FindPatternPeer, JavaImplementationElementPeer.MacroDefPatternPeer,
  		JavaImplementationElementPeer.ClassPatternPeer,JavaImplementationElementPeer.InterfacePatternPeer,JavaImplementationElementPeer.MacroCallPatternPeer {
    private final RootLevelElementsBlock parent_;
    public PassthroughHandler(RootLevelElementsBlock parent) {
      this.parent_ = parent;
    }
  	
    public JavaClassPatternPeer addJavaClassForJavaClass(int keywordLineNumber, int keywordCharacterNumber, final String nameParameter) {
      final SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
      return addType(nameParameter,true,error);
    }
    private ClassConstruction addType( final String nameParameter, boolean isClass,  final SpecificCommonErrorOutput error) {
      final ClassConstruction classConstruction = new ClassConstruction(nameParameter,this.parent_.rootBuilder_, isClass,this.parent_.fileEnvironment_,error);
      final String packageSubName = parent_.currentPackageSubName_;
      parent_.itemStore_.addElement(new RecordItem() {
      	public String toString() { return "Class("+nameParameter+"):"+error.getLineNumber(); }
				public boolean basicBuild(SLayer parentRecord) {
					classConstruction.basicBuildRoot(parentRecord,packageSubName);
					return true;
				}
				public void finishBuild() {
					classConstruction.finalBuild();
				}
      });
      return classConstruction;
    }

    public void end() {

    }
    public FindDefinitionPatternPeer addFindDefinitionForDefinition(final String varibleNameParameter, final int varibleNameLineNumber, final int varibleNameCharacterNumber) {
  		SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(varibleNameLineNumber, varibleNameCharacterNumber);

  		FindVariable variable = new FindVariable(varibleNameParameter,error);
  		parent_.finds_.addElement(variable);
  		
  		return variable;
		}
    public JavaInterfacePatternPeer addJavaInterfaceForJavaInterface(int keywordLineNumber, int keywordCharacterNumber, String nameParameter) {
      final SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
      return addType(nameParameter,false,error);
    }
    public JavaMacroBlockPatternPeer addJavaMacroBlockForMacroDef(int keywordLineNumber, int keywordCharacterNumber, String macronameParameter) {
      final SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
      final MacroConstruction construction = new MacroConstruction(macronameParameter, this.parent_.rootBuilder_, this.parent_.fileEnvironment_,error);
      parent_.itemStore_.addElement(construction);
      return construction;
    }
		public GeneralMacroCallPatternPeer addGeneralMacroCallForCall( int keywordLineNumber, int keywordCharacterNumber, final String macroNameParameter) {
      final SpecificCommonErrorOutput error = parent_.baseError_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
      final String packageSubName = parent_.currentPackageSubName_;
      if(packageSubName==null) {
      	error.generalSyntaxError("No packaged defined (and needed for macro calls)");
      }
			final GeneralMacroCall macroCall = new GeneralMacroCall(macroNameParameter,error);
			parent_.itemStore_.addElement(new RecordItem() {
				private SLayer parentRecord_;
      	public String toString() { return "MacroCall("+macroNameParameter+"):"+error.getLineNumber(); }

				public boolean basicBuild(SLayer parentRecord) {
					this.parentRecord_ = parentRecord;
					return true;
				}

				public void finishBuild() {
					macroCall.finishBuildOutsideClass(parentRecord_,packageSubName);
				}
				
			});
			return macroCall;
		}
    
  }

  // ==========================================================================
  
  // --------------------------------------------------------------------------
  private static final class LinkHandler implements  JavaImplementationElementPeer.LinkPatternPeer,RecordItem {
  
    private final SpecificCommonErrorOutput baseError_;
    private LinkChainBlock linkChainEnd_;
   // private SLayer linkedRecord_;
		private final FileEnvironment fileEnvironment_;
		private final String	packageSubName_;
		private final SimpleVector<DeferredJavaImplementationElement> elements_ = new SimpleVector<DeferredJavaImplementationElement>();
		private final SimpleVector<RootLevelElementsBlock> builtElements_ = new SimpleVector<RootLevelElementsBlock>();
		private final BaseBuilder rootBuilder_;
    public LinkHandler(FileEnvironment fileEnvironment,BaseBuilder rootBuilder, String packageSubName, SpecificCommonErrorOutput baseError) {
      this.fileEnvironment_ = fileEnvironment;
			this.rootBuilder_ = rootBuilder;
			this.packageSubName_ = packageSubName;
			this.baseError_ = baseError;
     // this. = new LinkChainBlock(baseError);


    }
    private RootLevelElementsBlock buildLinkElements() {
    	RootLevelElementsBlock result = new RootLevelElementsBlock(this.fileEnvironment_,this.rootBuilder_,packageSubName_, baseError_);
    	for(DeferredJavaImplementationElement element : this.elements_) {
    		element.buildJavaImplementationElement(result);
    	}
    	return result;
    }

    public boolean basicBuild(SLayer rootAccess) {
    	LinkChainBlock currentLinkChain = this.linkChainEnd_;
    	boolean okay = true;
    	while(currentLinkChain!=null) {
        SLayer linkedRecord = currentLinkChain.getLinkChainEndContainerQuiet(rootAccess);
    	
        if(linkedRecord == null) {
        	okay = false;;
        } else {
      		RootLevelElementsBlock elements = buildLinkElements();

        	okay &= elements.basicBuild(linkedRecord); 
        	this.builtElements_.addElement(elements);
        }
        currentLinkChain = currentLinkChain.getPrevious();
    	}
    	return okay;
      
    }
    public void finishBuild() {
    	for(RootLevelElementsBlock element : this.builtElements_) {
    	  element.finishBuild();
    	}
    }  
    public void end() {} 
//    public LinkChainPatternPeer addLinkChainForChain() {
//      return this.linkChain_;
//    }

//    public JavaImplementationElementPeer.Indirect getJavaImplementationElementForElement() {
//      return linkElements_;
//    }
		public LinkChainPatternPeer addLinkChainForSingleChain() {
			this.linkChainEnd_ = new LinkChainBlock(this.baseError_,this.linkChainEnd_);
			return this.linkChainEnd_;
		}
		public void setJavaImplementationElementForElement(DeferredJavaImplementationElement value) {
			this.elements_.addElement(value);
		}
  }

	public JavaImplementationElementPeer.GlobalClassPatternPeer createGlobalClass() {
		return new JavaImplementationElementPeer.GlobalClassPatternPeer() {

			public void end() { 	}

			public GlobalClassDefinitionPatternPeer addGlobalClassDefinitionForDefinition(
					String classNameParameter, int classNameLineNumber,
					int classNameCharacterNumber) {
				SpecificCommonErrorOutput error = baseError_.createAdjusted(classNameLineNumber, classNameCharacterNumber);
	      
				final GlobalsClass baseCall = new GlobalsClass(classNameParameter, error);
				itemStore_.addElement(baseCall);
	      return baseCall;
			}
			
		};
	}
	

//==========================================================================
	
	
	
	


	








}
