package org.ffd2.skeletonx.compile.java;

import org.ffd2.bones.base.ArgumentSet;
import org.ffd2.bones.base.BParameters;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.DeferredTargetParameter;
import org.ffd2.skeletonx.austenx.peg.base.LinkChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.NamePeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetParameterPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferencePatternPeer;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseBuilder.ParameterSetterMacroBlock;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ParameterVariableStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedParametersStoreDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;

public class TargetParametersBlock implements TargetParameterPeer.Indirect{
	private final SpecificCommonErrorOutput error_;
	private final SimpleVector<Parameter> parameters_ = new SimpleVector<Parameter>();
	public TargetParametersBlock(SpecificCommonErrorOutput error) {
		this.error_ = error;
	}
	public TargetParameterPeer.LinkPatternPeer createLink(int keywordLineNumber, int keywordCharacterNumber) {
		SpecificCommonErrorOutput adjustedError = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		LinkedParameter lp = new LinkedParameter(adjustedError);
		this.parameters_.addElement(lp);
		return lp;
	}

	public TargetParameterPeer.SimplePatternPeer createSimple(String nameParameter, final int nameLineNumber, final int nameCharacterNumber) {
		SpecificCommonErrorOutput adjustedError = this.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
		//Debug.println("NAME:",nameParameter);
		SimpleParameter p = new SimpleParameter(nameParameter, adjustedError);
		this.parameters_.addElement(p);
		return p;
	}
	
	public void createMark(String markNameParameter, int markNameLineNumber, int markNameCharacterNumber) {
		SpecificCommonErrorOutput adjustedError = this.error_.createAdjusted(markNameLineNumber, markNameCharacterNumber);
		MarkParameter p = new MarkParameter(markNameParameter, adjustedError);
		this.parameters_.addElement(p);
	}
	
	public void basicBuildMethod(CodeBlockEnvironment environment) {
		for(Parameter p : this.parameters_) {
			p.basicBuildMethod(environment);
		}
	}
	public void finishBuildMethod(ParameterSetterMacroBlock parameters) {
		for(Parameter p : this.parameters_) {
			p.finishBuildMethod(parameters);
		}
	}
	// ============================================================
	private static interface Parameter {
		//public void basicBuildPrototype(RecordClassAccess environment);
		public void basicBuildMethod(CodeBlockEnvironment environment);
		public void finishBuildMethod(ParameterSetterMacroBlock parameters);
	}
	// ------------------------------------------------------------
	private final static class LinkedBuild {
		private final LinkedBuild	previous_;
		private final CodeBlockEnvironment	linkedEnvironment_;
		private final LinkedParametersStoreDataBlock	linkedParametersSkeleton_;
		private final TargetParametersBlock	subParameters_;

		public LinkedBuild(CodeBlockEnvironment linkedEnvironment, TargetParametersBlock subParameters, LinkedParametersStoreDataBlock linkedParametersSkeleton, LinkedBuild previous) {
			this.linkedEnvironment_ = linkedEnvironment;
			this.subParameters_ = subParameters;
			this.linkedParametersSkeleton_ = linkedParametersSkeleton;
			this.previous_ = previous;
			
		}
		public void finishBuildMethod(ParameterSetterMacroBlock parameters,SLayer baseLayer ) {
			if(previous_!=null) {
				this.previous_.finishBuildMethod(parameters,baseLayer);
			}
			parameters.addParameter().addLinked(this.linkedParametersSkeleton_.getVariableStoreParameter(), new BaseTrackers.JavaVariablePath());
//			Utils.buildGeneralRecordRefParameter(this.linkedEnvironment_.getBaseRecord(), this.baseEnvironment_.getBaseRecord()));
			this.subParameters_.finishBuildMethod(
					this.linkedEnvironment_.getBaseLayer().addItemSkeleton().addLinkedParametersSet(
						this.linkedParametersSkeleton_.getVariableStoreParameter(),
						Utils.buildGeneralRecordRefParameter(this.linkedEnvironment_.getBaseLayer(), baseLayer)
					).getParameterSetMacro()
				);
		}
	}
	private final static class LinkedParameter implements Parameter,TargetParameterPeer.LinkPatternPeer {
		private LinkChainBlock lastLinkChain_;
		private CodeBlockEnvironment baseEnvironment_;
		private final SimpleVector<DeferredTargetParameter> deferredParameters_ = new SimpleVector<DeferredTargetParameter>();
		private final SpecificCommonErrorOutput	error_;
		private LinkedBuild lastLinkedBuild_ = null;
		public LinkedParameter(SpecificCommonErrorOutput error ) {
			this.error_ = error;
		}
		private TargetParametersBlock createParametersBlock() {
			TargetParametersBlock result = new TargetParametersBlock(this.error_);
			for(DeferredTargetParameter dp : this.deferredParameters_) {
				dp.buildTargetParameter(result);
			}
			return result;
		}
		public void basicBuildMethod(CodeBlockEnvironment environment) {
			this.baseEnvironment_ = environment;
			LinkChainBlock currentLinkChain = this.lastLinkChain_;
			while(currentLinkChain!=null) {
				CodeBlockEnvironment linkedEnvironment = environment.getLinkEndQuiet(currentLinkChain);
				if(linkedEnvironment!=null) {
					TargetParametersBlock subParameters = createParametersBlock();
					subParameters.basicBuildMethod(linkedEnvironment);
					 //The linked parameters reside in the original environment
					LinkedParametersStoreDataBlock linkedParametersSkeleton = environment.getBaseLayer().allocateLinkedParameters();
					this.lastLinkedBuild_ = new LinkedBuild(linkedEnvironment, subParameters, linkedParametersSkeleton,this.lastLinkedBuild_);
				}
				currentLinkChain = currentLinkChain.getPrevious();
			}
			
		}

		public void finishBuildMethod(ParameterSetterMacroBlock parameters) {
			if(this.lastLinkedBuild_!=null) {
				SLayer baseLayer = this.baseEnvironment_.getBaseLayer();
				this.lastLinkedBuild_.finishBuildMethod(parameters, baseLayer);
			}
		}

		public void end() { }


		public LinkChainPatternPeer addLinkChainForSingleChain() {
			return this.lastLinkChain_ = new LinkChainBlock(error_,this.lastLinkChain_);

		}

		public void setTargetParameterForParameters(DeferredTargetParameter value) {
			this.deferredParameters_.addElement(value);
		}
	}
	// ------------------------------------------------------------
	private final static class SimpleParameter implements TargetParameterPeer.SimplePatternPeer, Parameter, TargetVariable {
		private final SpecificCommonErrorOutput error_;
		private final NameBlock nameBlock_;
		private final String baseName_;
		private  TargetTypeBlock typeBlock_;
		private ParameterSetterMacroBlock.ParameterDataBlock.SimpleDataBlock parameterSkeleton_;
		private SLayer baseEnvironment_;
		private ParameterVariableStoreDataBlock localVariableStoreSkeleton_;
		
		public SimpleParameter( String baseName, SpecificCommonErrorOutput error ) {
			this.error_ = error;

			this.nameBlock_ = new NameBlock(error);
			if(baseName.equals("void")) {
				//This is just a hack to check if the parsing stuff mucks up - TODO: need to remove when parser upgraded
				Thread.dumpStack();
			}
			this.baseName_ = baseName;
		}
		public void basicBuildMethod(CodeBlockEnvironment environment) {
			final SLayer baseEnvironment = this.baseEnvironment_ = environment.getBaseLayer();
			this.localVariableStoreSkeleton_ = baseEnvironment.getBlockSkeleton().addParameterTargetVariableStore(this.baseName_);

			this.typeBlock_.basicBuild(baseEnvironment.getAsDataBlockEnvironment());
			try {
				environment.addLocalVariable(this.baseName_, this);
			} catch(ItemAlreadyExistsException e) {
				this.error_.repeatedObjectError("variable", this.baseName_);
			}
		}
		public void finishBuildMethod(ParameterSetterMacroBlock parameters) {
			this.parameterSkeleton_ = parameters.addParameter().addSimple(this.baseName_,this.localVariableStoreSkeleton_,parameters.getRootBuilder().createTypeSetterDetailsChild());
			this.nameBlock_.updateInstanceNameSkeleton(this.baseName_, this.localVariableStoreSkeleton_.getNameDetailsParameter(), this.baseEnvironment_);
			
			this.typeBlock_.buildCodeSetTypeFinal(this.parameterSkeleton_.getParameterTypeParameter());
//			if(this.typeBlock_.isHasGenerics()) {
//				Debug.println("Generics found for variable:",this.baseName_, " at line ",this.error_.getLineNumber());
//			}
		}

		public NamePeer.BasicPatternPeer addOptionBasicOfNameForNameOutput() { return this.nameBlock_; }

		public void addOptionImplementsOfNameForNameOutput(final int keywordLineNumber, final int keywordCharacterNumber, final String classNameParameter, final String methodNameParameter, final int methodNameLineNumber, final int methodNameCharacterNumber) {
			this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber).generalSyntaxError("implementing name not applicable to parameter declaration");
		}
	 
		public TargetTypeReferencePatternPeer addTargetTypeReferenceForType(String typeNameParameter,
				int typeNameLineNumber, int typeNameCharacterNumber) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
			return this.typeBlock_ = new TargetTypeBlock(typeNameParameter,error);
		}
		
		public void end() { }
		// =========================================================================
		//Target Variable stuff
		public void createCallExpressionFinal(BasicCallChainFormBlock target, JavaVariablePath basePathDisposable) {
			target.addElement().addVariable(this.localVariableStoreSkeleton_.getVariableStoreParameter(), basePathDisposable);
		}
		
		public void createMacroCallParameter(MacroCallDataBlock macroCallBlock,JavaVariablePath pathToVariable) {
			macroCallBlock.addCallParameter().addTargetVariable().addSimple(this.localVariableStoreSkeleton_.getVariableStoreParameter(), pathToVariable);
		}
		public TargetTypeReference getTargetTypeReferenceQuietFinal() {
			return this.typeBlock_.getResolvedTypeReferenceQuietFinal();
		}
		
	}
//------------------------------------------------------------
	private final static class MarkParameter extends AbstractMarkIRecordVariable implements Parameter, IRecordVariable {
		private final SpecificCommonErrorOutput error_;
		private final String variableName_;
		private LinkedParametersStoreDataBlock	allocatedParameters_;
		
		public MarkParameter( String variableName, SpecificCommonErrorOutput error ) {
			super("parameters mark");
			this.error_ = error;

			this.variableName_ = variableName;
		}
		@Override public boolean isMatchesVariableName(String queryVariableName) { return this.variableName_.equals(queryVariableName); }
		@Override public String getRoughVariableName() { return this.variableName_; }
		
	
		
	  public void getAsReference(IRecordTypeTarget target) {
	  	//Not a reference - yet?
	  	target.setNotReference();
	  }
		public void basicBuildMethod(CodeBlockEnvironment environment) {
			try {
				environment.addIRecordVariable(this.variableName_, this);
			} catch(ItemAlreadyExistsException e) {
				this.error_.repeatedObjectError("record variable", this.variableName_);
			}
    	this.allocatedParameters_ = environment.allocateLinkedParameters();
		}
		public void finishBuildMethod(ParameterSetterMacroBlock parameters) {
			//This is wrong
			parameters.addParameter().addLinked(this.allocatedParameters_.getVariableStoreParameter(), new JavaVariablePath());
		}
		// =========================================================================
		private final ParsingException makeGeneralException() {
			return ParsingException.createGeneralSyntax("can't be provided by parameter mark");
		}
		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) throws ParsingException {
			if(type==IVariableType.ParametersReference) {
				macroCallBlock.addArgumentTypesReference(this.allocatedParameters_.getVariableStoreParameter(), variablePath);
			} else {
				throw ParsingException.createInvalidType("parameters reference", type.getName());
			}
		}
		public void addAsMacroRecordMappingParameter( MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			throw makeGeneralException( );
		}
		public SLayer getAsFoundataionNodeReferenceVariable( JavaVariablePath variablePath) throws ParsingException {
			throw makeGeneralException( );
		}

		public void addAsMacroRecordKeyParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath variablePath, KeyDefinitionBlock type)
				throws ParsingException {
			throw makeGeneralException( );
		}
		
	}
	
}
