package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.BuilderVariableChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferencePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferenceWithoutArrayPatternPeer;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;

public class TargetTypeBlock implements TargetTypeReferencePatternPeer,TargetTypeReferenceWithoutArrayPatternPeer{
  private final SpecificCommonErrorOutput baseError_;
  private int arrayCount_ = 0;
  private BuilderVariableChainBlock builderVariables_;
  private SimpleVector<TargetTypeBlock> genericTypesMaybeNull_ = null;

  
  private final String typeName_;
  private TargetType resolvedTypeToBuild_ = null;
  private boolean isToBeBuilt_ = true;

  private final BaseTrackers.JavaVariablePath fromThisSpaceToTypeDeclarationPath_;
  private DataBlockEnvironment environment_;
  private BuilderRecordRef recordRef_;
  public TargetTypeBlock(String typeName,   SpecificCommonErrorOutput baseError) {
    this.baseError_ = baseError;
    this.typeName_ = typeName;
    this.fromThisSpaceToTypeDeclarationPath_ = new BaseTrackers.JavaVariablePath();
    
  }
  public boolean isHasGenerics() {
  	return this.genericTypesMaybeNull_!=null;
  }
  
 
  private void buildType() {
  	if(this.isToBeBuilt_) {
  		DataBlockEnvironment environment = this.environment_;
	  	TargetType baseType = null;
	    if(this.builderVariables_!=null) {
	      environment =  this.builderVariables_.resolveRecordQuiet(environment,fromThisSpaceToTypeDeclarationPath_,true);
	      if(environment!=null) {
	      	baseType = environment.getTargetTypeQuiet(this.typeName_,fromThisSpaceToTypeDeclarationPath_);    
	      }
	      this.recordRef_ = new BuilderRecordRef(environment, this.fromThisSpaceToTypeDeclarationPath_.getCopy());
	    } else {
	    	baseType = environment.getTargetTypeQuiet(this.typeName_,fromThisSpaceToTypeDeclarationPath_);
	    }
	    if(baseType==null) {
	    	this.baseError_.objectNotFoundError("target type", this.typeName_);
	    	baseType = DummyItems.DUMMY_TARGET_TYPE;
	    } else {
	    	if(this.genericTypesMaybeNull_!=null) {
	    
		    	final SimpleVector<TargetTypeBlock> localGenerics = this.genericTypesMaybeNull_;
		    	final int numberOfGenericTypes = localGenerics.size();
		    	final TargetTypeWithPath[] resolvedGenericTypes = new TargetTypeWithPath[numberOfGenericTypes];
		    	for(int i = numberOfGenericTypes-1 ; i>= 0 ; i--) {
		    		final TargetTypeBlock generic = localGenerics.elementAt(i);
		    		generic.basicBuild(environment);
		    		TargetTypeWithPath foundType = generic.getResolvedTypeWithPathQuietFinal();
		    		resolvedGenericTypes[i] = foundType;
		    	}
		    	try {
		    		baseType = baseType.specifyGenericTypes(resolvedGenericTypes);
		    	} catch(ParsingException e) {
		    		e.updateError(this.baseError_);
		    	}
		    
		    }
	  		if(this.arrayCount_>0) {
	
	  			try {
		    		baseType = baseType.getArrayed(arrayCount_);
		    	} catch(ParsingException e) {
		    		e.updateError(this.baseError_);
		    	}
	  		}
	    }
	    this.resolvedTypeToBuild_ = baseType;
  	}
  	this.isToBeBuilt_ = false;
  }
  public final void basicBuild(SLayer environment) {
  	this.basicBuild(environment.getAsDataBlockEnvironment());
  }
  public final void basicBuild(DataBlockEnvironment environment) {
    this.environment_ = environment;
  }
  public void buildCodeSetTypeFinal(TypeSetterDetailsFormBlock target) {
  	buildType();
  	if(this.resolvedTypeToBuild_!=null) {
  		this.resolvedTypeToBuild_.buildCodeSetType(target,this.fromThisSpaceToTypeDeclarationPath_.getCopy());
  	}
	}
//  /**
//   * Must call basicBuild() first
//   * @return
//   */
  public TargetTypeWithPath getResolvedTypeWithPathQuietFinal() { 
  	TargetTypeReference reference = getResolvedTypeReferenceQuietFinal();
  	if(reference!=null) {
  		return new TargetTypeWithPath(reference,this.fromThisSpaceToTypeDeclarationPath_.getCopy());
  	} else {
  		return null;
  	}
  }
  public TargetTypeReference getResolvedTypeReferenceQuietFinal() { 
  	if(this.isToBeBuilt_) {
  		buildType();
  	}
  	if(this.resolvedTypeToBuild_!=null) {
  		TargetTypeReference result = new TargetTypeReference(this.resolvedTypeToBuild_);
  		if(this.recordRef_!=null) {
  			this.recordRef_.updateReferenced(result);
  		}
  		return result;
  	} else {
  		return null;
  	}
  }
  /**
   * Must call basicBuild() first
   * @return
   */
//  public void setupSkeletonTypeFinal(BaseBuilder.TypeSetterCode target) { 
//  	TargetType resolvedType = this.getResolvedTypeQuietFinal();
//  	if(resolvedType!=null) {
//  		resolvedType.buildCodeSetType(target, this.fromThisSpaceToTypeDeclarationPath_.getCopy());
//  	}
//  }
//  public ArgumentSetCodeCode createConstructorCallFinal(ExpressionCode target) {
//  	ConstructorCallDataBlock callSkeleton = target.addConstructorCallBlock();
//  	TargetType resolvedType = this.getResolvedTypeQuietFinal();
//  	if(resolvedType!=null) {
//  		resolvedType.buildCodeSetType(callSkeleton.getCodeFixtureSetConstructorType(), this.fromThisSpaceToTypeDeclarationPath_.getCopy());
//  	}
//  	return callSkeleton.getCodeFixtureSetArguments();
//  }
  public void addArrayCount(int lineNumber, int characterNumber) {
    this.arrayCount_++;
  }

  public void end() {  }

  public TargetTypeReferencePatternPeer addTargetTypeReferenceForGenericTypes(String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
    SpecificCommonErrorOutput error = this.baseError_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
    TargetTypeBlock genericType = new TargetTypeBlock(typeNameParameter,error);
    if(this.genericTypesMaybeNull_==null) {
    	this.genericTypesMaybeNull_ = new SimpleVector<TargetTypeBlock>();
    }
    this.genericTypesMaybeNull_.addElement(genericType);
    return genericType;
  }
  
  public BuilderVariableChainPatternPeer addBuilderVariableChainForBuilderVariableChain() {
    return this.builderVariables_ = new BuilderVariableChainBlock(this.baseError_);
  }
  // ==========================================================================
  private final static class BuilderRecordRef {
  	private final DataBlockEnvironment record_;
		private final JavaVariablePath pathToRecord_;
		public BuilderRecordRef(DataBlockEnvironment record, BaseTrackers.JavaVariablePath pathToRecord) {
			this.record_ = record;
			this.pathToRecord_ = pathToRecord;
  	}
		public void updateReferenced(TargetTypeReference typeReference) {
			typeReference.addPoolUpdater(this.record_, this.pathToRecord_);
		}
  }
}
