package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.austenx.peg.base.DeferredTargetTypeElement;
import org.ffd2.skeletonx.austenx.peg.base.LinkChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeElementPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeLinkPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeListPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferencePatternPeer;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.LinkedTypeListSetDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedTypeListDataBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder.TypeListSetterMacroBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.TypeListDetailsFormHolder.TypeListDetailsFormBlock;

import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class TargetTypeListBlock implements TargetTypeListPatternPeer,TargetTypeElementPatternPeer,DeferredTargetTypeElement.DeferredTarget {
	private final SpecificCommonErrorOutput error_;
	private final SimpleVector<Element> elements_ = new SimpleVector<Element>();
	
	public TargetTypeListBlock(SpecificCommonErrorOutput error) {
		this.error_ = error;
		
	}
	public boolean isSubTypeOf(TargetType parentType) {
		for(Element e : this.elements_) {
			if(e.isSubTypeOf(parentType)) {
				return true;
			}
		}
		return false;
	}
	// ==========================================================================
	// for DeferredTargetTypeElement.DeferredTarget
	public TargetTypeElementPatternPeer createTargetTypeElement() { return this ; }
	
	// ==========================================================================

	public TargetTypeReferencePatternPeer addTargetTypeReferenceForType(String typeNameParameter,
			int typeNameLineNumber, int typeNameCharacterNumber) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
		TargetTypeBlock typeBlock = new TargetTypeBlock(typeNameParameter,error);
		this.elements_.addElement(new SingleElement(typeBlock));
		return typeBlock;
	}
	public TargetTypeLinkPatternPeer addTargetTypeLinkForSubLink() {
		LinkElement e = new LinkElement(this.error_);
		this.elements_.addElement(e);
		return e;
	}
	public void basicBuild(SLayer environment) {
		for(Element e : this.elements_) {
			e.basicBuild(environment);
		}
	}
	public void finalBuild(TypeListDetailsFormBlock target) {
		for(Element e : this.elements_) {
			e.updateListFinal(target);
		}
	}
	// =======================================================================================
	private static interface Element {
		public void basicBuild(SLayer environment);
		public void updateListFinal(TypeListDetailsFormBlock target);
		public boolean isSubTypeOf(TargetType parentType);
	}
	// -----------------------------------
	private final static class LinkedBuild {
		private final LinkedBuild	previous_;
		private final SLayer	linkedEnvironment_;
		private final TargetTypeListBlock	subList_;

		public LinkedBuild(SLayer	linkedEnvironment,TargetTypeListBlock subList, LinkedBuild previous) {
			this.linkedEnvironment_ = linkedEnvironment;
			this.subList_ = subList;
			this.previous_ = previous;
			
		}
		public void updateListFinal(TypeListDetailsFormBlock target,SLayer baseEnvironment) {
			if(this.previous_!=null) {
				this.previous_.updateListFinal(target,baseEnvironment);
			}
			LinkedTypeListDataBlock linkedInBase = baseEnvironment.allocateLinkedTypeList();
			target.addLinked(linkedInBase.getVariableStoreParameter(), new BaseTrackers.JavaVariablePath());
			ItemDataBlock addItemSkeleton = linkedEnvironment_.addItemSkeleton();
			
			TypeListDetailsFormBlock linkedListSkeleton = target.getRootBuilder().createTypeListDetailsChild();
			LinkedTypeListSetDataBlock linkedSetter = addItemSkeleton.addLinkedTypeListSet(
					linkedInBase.getVariableStoreParameter(),
					linkedEnvironment_.buildPathToUpperLayer(baseEnvironment),
					linkedListSkeleton
//					Utils.buildGeneralRecordRefParameter(this.linkedEnvironment_,this.baseEnvironment_)
				);
			this.subList_.finalBuild(linkedListSkeleton);
		}
	}
	private final static class LinkElement implements Element,TargetTypeLinkPatternPeer {
		private LinkChainBlock lastLinkChain_;
		private SLayer baseEnvironment_;
		private final SimpleVector<DeferredTargetTypeElement> deferredElements_ = new SimpleVector<DeferredTargetTypeElement>();
		private final SpecificCommonErrorOutput	error_;
		private LinkedBuild lastLinkedBuild_;
		public LinkElement(SpecificCommonErrorOutput error) {
			this.error_ = error;
			
		}
		private final TargetTypeListBlock createSubList() {
			TargetTypeListBlock result = new TargetTypeListBlock(this.error_);
			for(DeferredTargetTypeElement dt : this.deferredElements_) {
				dt.buildTargetTypeElement(result);
			}
			return result;
		}
		public boolean isSubTypeOf(TargetType parentType) {
			return false;//Does work for links?
			//TODO: correct one day so takes account of callers SLAYER level.
			//return this.subList_.isSubTypeOf(parentType);
		}
		public void basicBuild(SLayer environment) {
			LinkChainBlock currentLinkChain = this.lastLinkChain_;
			this.baseEnvironment_ = environment;

			while(currentLinkChain!=null) {
				 SLayer	linkedEnvironment = currentLinkChain.getLinkChainEndContainerQuiet(environment);
				 if(linkedEnvironment!=null) {
					 TargetTypeListBlock subList = createSubList();
					 
					 subList.basicBuild(linkedEnvironment);
					 this.lastLinkedBuild_ = new LinkedBuild(linkedEnvironment,subList,this.lastLinkedBuild_);
				}
				currentLinkChain = currentLinkChain.getPrevious();
 
			}
			
			
			
		}

		public void updateListFinal(TypeListDetailsFormBlock target) {
			if(this.lastLinkedBuild_!=null) {
				this.lastLinkedBuild_.updateListFinal(target, this.baseEnvironment_);
			}
		}

	

//		public TargetTypeListPatternPeer addTargetTypeListForSubList() {
//			return subList_;
//		}
//
//		public TargetTypeReferencePatternPeer addTargetTypeReferenceForSingleType(
//				String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
//			return subList_.addTargetTypeReferenceForType(typeNameParameter, typeNameLineNumber, typeNameCharacterNumber);
//		}

		public void end() { }
		public LinkChainPatternPeer addLinkChainForSingleChain() {
			return this.lastLinkChain_ = new LinkChainBlock(this.error_,this.lastLinkChain_);

		}
		public void setTargetTypeElementForSubList(DeferredTargetTypeElement value) {
			this.deferredElements_.addElement(value);
		}
		public void setTargetTypeElementForSingleType(DeferredTargetTypeElement value) {
			this.deferredElements_.addElement(value);
		}
	} //End of LinkElement
	// -----------------------------------
	private final static class SingleElement implements Element {
		
		private final TargetTypeBlock typeBlock_;
		
		public SingleElement(TargetTypeBlock typeBlock) {
			this.typeBlock_ = typeBlock;
		}
		public void basicBuild(SLayer environment) {
			typeBlock_.basicBuild(environment);
		}
		public boolean isSubTypeOf(TargetType parentType) {
			TargetTypeReference resolvedType = this.typeBlock_.getResolvedTypeReferenceQuietFinal();
			if(resolvedType==null) {
				return false;
			} else {
				return resolvedType.getBaseType().isSubTypeOf(parentType);
			}
		}
		public void updateListFinal(TypeListDetailsFormBlock target) {
			typeBlock_.buildCodeSetTypeFinal(target.addSimple(target.getRootBuilder().createTypeSetterDetailsChild()).getTypeDetailsParameter());
		}
	} //End of SingleElement
	// -----------------------------------------------------------------------------------------------
	public void end() { }


}
