package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class VariableStore {
	private static final String	BUILDER_VARIABLE_TYPE_NAME	= "builder variable";
	private static final String	TARGET_VARIABLE_TYPE_NAME	= "target variable";
	private static final String	ARG_VALUES_TYPE_NAME	= "arg values variable";
	private static final String	ARG_TYPESTYPE_NAME	= "arg types variable";
	private static final String	CODE_REF_VARIABLE_TYPE_NAME	= "code reference variable";

	private final FileEnvironment	environment_;
	private AccessStoreX<IRecordVariable> builderVariables_;
	private AccessStoreX<TargetVariable> targetVariables_;

	public VariableStore(FileEnvironment environment) {
		this.environment_ = environment;
	}
	// =========================================================================
	// BUILDER VARIABLES
	public void addBuilderVariable(final String name, IRecordVariable variable) throws ItemAlreadyExistsException{
		if(this.builderVariables_==null) {
			this.builderVariables_ = environment_.createEntityStore(BUILDER_VARIABLE_TYPE_NAME);
		}
		this.builderVariables_.put(name, variable);
	}
	public void forceBuilderVariable(final String name, IRecordVariable variable) {
		if(this.builderVariables_==null) {
			this.builderVariables_ = environment_.createEntityStore(BUILDER_VARIABLE_TYPE_NAME);
		}
		this.builderVariables_.force(name, variable);
	}
	public void addBuilderVariable(final String name, IRecordVariable variable, SpecificCommonErrorOutput error) {
		try {
			addBuilderVariable(name, variable);
		} catch(ItemAlreadyExistsException e) {
			e.updateError(error);
		}
	}
	public IRecordVariable getBuilderVariable(String name) throws ItemNotFoundException {
		if(this.builderVariables_==null) {
			throw new ItemNotFoundException(name, BUILDER_VARIABLE_TYPE_NAME);
		} else {
			return this.builderVariables_.get(name);
		}
	}
	public IRecordVariable getBuilderVariableQuiet(String name)  {
		if(this.builderVariables_==null) {
			return null;
		} else {
			return this.builderVariables_.getQuiet(name);
		}
	}
//=========================================================================
	// TargetVARIABLES
	public void addTargetVariable(final String name, TargetVariable variable) throws ItemAlreadyExistsException{
		if(this.targetVariables_==null) {
			this.targetVariables_ = environment_.createEntityStore(TARGET_VARIABLE_TYPE_NAME);
		}
		this.targetVariables_.put(name, variable);
	}
	public TargetVariable getTargetVariable(String name) throws ItemNotFoundException {
		if(this.targetVariables_==null) {
			throw new ItemNotFoundException(name, TARGET_VARIABLE_TYPE_NAME);
		} else {
			return this.targetVariables_.get(name);
		}
	}
	public TargetVariable getTargetVariableQuiet(String name)  {
		if(this.targetVariables_==null) {
			return null;
		} else {
			return this.targetVariables_.getQuiet(name);
		}
	}

}
