package org.ffd2.skeletonx.compile.java;

import org.ffd2.bones.base.BVariable;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.BuilderVariableChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaCallChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaCallChainSectionPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaExpressionPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaMethodCallPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetCallArgumentsPatternPeer;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.ArgumentSetDetailsFormHolder.ArgumentSetDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock.ArrayIndexDataBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock.SegmentedCallChainDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;

public class XCodeCallChainBlock implements JavaCallChainPatternPeer {
	private static interface InitialHandler {
		/**
		 * Setup the pool
		 * 
		 * @param pool
		 * @param baseElementAccess
		 *          the element access of our current context
		 * @return true if successful, or false otherwise
		 */
		public boolean initialSetup(AccessPool pool, CodeBlockEnvironment baseElementAccess);
	}

	private static final InitialHandler SIMPLE_INITIAL = new InitialHandler() {
		public boolean initialSetup(AccessPool pool, CodeBlockEnvironment baseElementAccess) {
			pool.addElementAccess(baseElementAccess, new BaseTrackers.JavaVariablePath());
			return true;
		}
	};

	private final SpecificCommonErrorOutput baseError_;

	private static int constructedTargetIndexCount_ = 0;

	private boolean allowLocalVariables_ = true;
	private final SimpleVector<Segment> segments_ = new SimpleVector<Segment>();
	private SimpleVector<ExpressionBlock> arrayIndexes_ = null;

	public XCodeCallChainBlock(SpecificCommonErrorOutput baseError) {
		this.baseError_ = baseError;
	}

	public void resolveInFinish(ExpressionDetailsFormBlock target, CodeBlockEnvironment baseEnvironment) {
		final GeneralDataTracker generalDataTracker = new GeneralDataTracker();
		if (this.segments_.size() == 1) {
			Segment s = this.segments_.firstElement();
			s.resolveForExpressionInFinish(this.allowLocalVariables_, target, generalDataTracker, baseEnvironment);
		} else {
			SegmentedCallChainDataBlock segments = target.addSegmentedCallChain();
			boolean allowLocalVariables = this.allowLocalVariables_;
			for (Segment s : this.segments_) {
				final BasicCallChainFormBlock callSkeleton = target.getRootBuilder().createBasicCallChainChild();
				segments.addSegment(callSkeleton);
				s.resolveForChainInFinish(allowLocalVariables, callSkeleton, generalDataTracker, baseEnvironment);
				allowLocalVariables = false;
			}
		}
		if (this.arrayIndexes_ != null) {
			for (ExpressionBlock indexValue : this.arrayIndexes_) {
				ExpressionDetailsFormBlock indexExpressionSkeleton = target.getRootBuilder().createExpressionDetailsChild();
				ArrayIndexDataBlock indexSkeleton = target.addArrayIndex(indexExpressionSkeleton);
				if (indexValue.basicBuild(baseEnvironment)) {
					indexValue.finish(baseEnvironment, indexExpressionSkeleton);
				}
			}
		}
	}

	public JavaExpressionPeer.Indirect getJavaExpressionForIndexValue() {
		ExpressionBlock expression = new ExpressionBlock(this.baseError_);
		if (this.arrayIndexes_ == null) {
			this.arrayIndexes_ = new SimpleVector<ExpressionBlock>();
		}
		this.arrayIndexes_.addElement(expression);
		return expression;
	}

	public void addHasThis() {
		this.allowLocalVariables_ = false;
	}

	private final void addSegment(Segment s) {
		this.segments_.addElement(s);
	}

	public void doAddRecordVarRequirement(FoundationNode requiredRecordType, MacroCallDataBlock macroCallBlock, SLayer environment) {
		if (this.segments_.size() == 1) {
			Segment only = this.segments_.firstElement();
			only.doRecordAction(new RecordVariableActions.GetRecordAction(requiredRecordType, macroCallBlock), environment);
		} else {
			this.segments_.lastElement().getEndError().invalidTypeError("multiple segments", "record variable");
		}
	}

	public void doAddRequirement(RecordVariableAction action, String requiredName, SLayer environment) {
		if (this.segments_.size() == 1) {
			Segment only = this.segments_.firstElement();
			only.doRecordAction(action, environment);
		} else {
			this.segments_.lastElement().getEndError().invalidTypeError("multiple segments", requiredName);

		}
	}

	public void doAddArgValuesMarkReferenceRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
		if (this.segments_.size() == 1) {
			Segment only = this.segments_.firstElement();
			only.doRecordAction(new RecordVariableActions.GetCodeReferenceAction(macroCallBlock), environment);
		} else {
			this.segments_.lastElement().getEndError().invalidTypeError("multiple segments", "code reference");

		}
	}

	public void doAddTargetVarRequirement(TargetType requiredVarType, MacroCallDataBlock macroCallBlock, CodeBlockEnvironment baseEnvironment) {
		final GeneralDataTracker generalDataTracker = new GeneralDataTracker();
		BVariable b;

		if (this.segments_.size() == 1) {
			Segment only = this.segments_.firstElement();
			only.doAddTargetVarRequirementDirect(this.allowLocalVariables_, requiredVarType, macroCallBlock, generalDataTracker, baseEnvironment);
		} else {
			// TODO: This probably needs to have something else done to it!
			BasicCallChainFormBlock callSkeleton = macroCallBlock.getRootBuilder().createBasicCallChainChild();
			MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock constructedSkeleton = macroCallBlock.addCallParameter().addTargetVariable().addConstructed(
					constructedTargetIndexCount_++, callSkeleton);
			boolean allowLocalVariables = this.allowLocalVariables_;
			for (Segment s : this.segments_) {
				s.doAddTargetVarRequirementCombination(allowLocalVariables, requiredVarType, constructedSkeleton, generalDataTracker, baseEnvironment);
				allowLocalVariables = false;
			}
		}

	}

	public JavaCallChainSectionPeer.LocalPatternPeer addOptionLocalOfJavaCallChainSectionForSection() {
		LocalVariableSegment s = new LocalVariableSegment(this.baseError_);
		addSegment(s);
		return s;
	}

	public JavaCallChainSectionPeer.VariablePatternPeer addOptionVariableOfJavaCallChainSectionForSection() {
		VariableSegment s = new VariableSegment(this);
		addSegment(s);
		return s;
	}

	public JavaCallChainSectionPeer.ViaPatternPeer addOptionViaOfJavaCallChainSectionForSection() {
		ViaSegment s = new ViaSegment(this);
		addSegment(s);
		return s;
	}

	public void end() {
	}

	// ------------------------------------------------------------------------------------------------------------------
	private static interface Segment {
		public boolean resolveForChainInFinish(boolean allowLocalVariables, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment);

		public boolean resolveForExpressionInFinish(boolean allowLocalVariables, ExpressionDetailsFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment);

		/**
		 * Add the target var requirements from this code call chain, when segment
		 * is the only element
		 * 
		 * @param varType
		 * @param macroCallBlock
		 * @param environment
		 */
		public void doAddTargetVarRequirementDirect(boolean allowLocalVariables, TargetType varType, MacroCallDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment environment);

		public void doAddTargetVarRequirementCombination(boolean allowLocalVariables, TargetType varType, MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment environment);

		public void doRecordAction(RecordVariableAction variableAction, SLayer environment);

		// public void doAddRecordVarRequirement(RedirectingTargetRecordAccess
		// requiredRecordType,
		// MacroCallDataBlock macroCallBlock, IVariableType type,
		// CodeBlockEnvironment environment);
		//
		// public void doAddCodeReferenceRequirement(MacroCallDataBlock
		// macroCallBlock, CodeBlockEnvironment environment);
		public SpecificCommonErrorOutput getEndError();
	}

	private final static class VariableSegment implements Segment, JavaCallChainSectionPeer.VariablePatternPeer {
		private final XCodeCallChainBlock parent_;
		private final BuilderVariableChainBlock builderChain_;

		public VariableSegment(XCodeCallChainBlock parent) {
			this.parent_ = parent;
			this.builderChain_ = new BuilderVariableChainBlock(parent.baseError_);
		}

		// public void doAddCodeReferenceRequirement(
		// MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {
		// JavaVariablePath pathToVariableChainTracker = new
		// BaseTrackers.JavaVariablePath();
		//
		// IRecordVariable resolvedRecordVariable =
		// builderChain_.resolveRecordVariableQuiet(environment.getBaseRecord(),
		// pathToVariableChainTracker);
		// if(resolvedRecordVariable!=null) {
		// try {
		// resolvedRecordVariable.addAsMacroRecordParameter(macroCallBlock,
		// pathToVariableChainTracker, IVariableType.CodeMarkReference);
		//
		// } catch(ParsingException e) {
		// e.updateError(builderChain_.getEndError());
		// }
		// }
		// }
		public void doRecordAction(RecordVariableAction variableAction, SLayer environment) {
			// Debug.println("Doing record action:",this.builderChain_);
			JavaVariablePath pathToVariableChainTracker = new BaseTrackers.JavaVariablePath();

			IRecordVariable resolvedRecordVariable = builderChain_.resolveRecordVariableQuiet(environment, pathToVariableChainTracker, false);
			// Debug.println("%%% Resolved:",resolvedRecordVariable);
			// Debug.println("%%% Path:",pathToVariableChainTracker);

			if (resolvedRecordVariable != null) {
				try {
					variableAction.doAction(resolvedRecordVariable, pathToVariableChainTracker);
				} catch (ParsingException e) {
					Debug.println("Variable action:",variableAction, "    from ", resolvedRecordVariable);
					
					e.updateError(builderChain_.getEndError());
				}
				// Need to actually add reference - this is going to require the chain
				// to build up variable access...
				// (already done - see pathToVariableChainTracker.
			}
		}

		public void doAddTargetVarRequirementDirect(boolean allowLocalVariables, TargetType varType,
				MacroCallDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment environment) {

			JavaVariablePath pathToVariableChainTracker = new BaseTrackers.JavaVariablePath();

			IRecordVariable resolvedVariable = this.builderChain_.resolveRecordVariableQuiet(environment.getBaseLayer(), pathToVariableChainTracker, true);

			if (resolvedVariable != null) {
				try {
					resolvedVariable.addAsMacroRecordParameter(macroCallBlock, pathToVariableChainTracker, IVariableType.ExpressionReference);
				} catch (ParsingException e) {
					e.updateError(this.builderChain_.getEndError());
				}
			}
		}

		public void doAddTargetVarRequirementCombination(boolean allowLocalVariables, TargetType varType,
				MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment environment) {
			JavaVariablePath pathToVariableChainTracker = new BaseTrackers.JavaVariablePath();

			IRecordVariable resolvedVariable = this.builderChain_.resolveRecordVariableQuiet(environment.getBaseLayer(), pathToVariableChainTracker, true);

			if (resolvedVariable != null) {
				try {
					resolvedVariable.updateCallChainFinal(macroCallBlock.getChainDetailsParameter(), pathToVariableChainTracker);
				} catch (ParsingException e) {
					e.updateError(this.builderChain_.getEndError());
				}
			}
		}

		public boolean resolveForChainInFinish(boolean allowLocalVariables, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			BaseTrackers.JavaVariablePath pathToRecord = new BaseTrackers.JavaVariablePath();
			IRecordVariable builderVariable = this.builderChain_.resolveRecordVariableQuiet(baseEnvironment.getBaseLayer(), pathToRecord, true);
			if (builderVariable == null) {
				return false; // FAILURE
			} else {
				try {
					builderVariable.updateCallChainFinal(target, pathToRecord);
				} catch (ParsingException e) {
					e.updateError(this.builderChain_.getEndError());
				}
				return true;
			}
		}

		public boolean resolveForExpressionInFinish(boolean allowLocalVariables, ExpressionDetailsFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			BaseTrackers.JavaVariablePath pathToRecord = new BaseTrackers.JavaVariablePath();
			IRecordVariable builderVariable = this.builderChain_.resolveRecordVariableQuiet(baseEnvironment.getBaseLayer(), pathToRecord, true);
			if (builderVariable == null) {
				return false; // FAILURE
			} else {
				try {
					builderVariable.updateExpressionFinal(baseEnvironment, target, pathToRecord);
				} catch (ParsingException e) {
					e.updateError(this.builderChain_.getEndError());
				}
				return true;
			}
		}

		public BuilderVariableChainPatternPeer addBuilderVariableChainForBuilderVariableChain() {
			// this.initial_ = new InitialHandler() {
			// public boolean initialSetup(AccessPool pool, CodeBlockEnvironment
			// baseEnvironment) {
			// BaseTrackers.JavaVariablePath pathToRecord = new
			// BaseTrackers.JavaVariablePath();
			// RecordAccess endEnvironment =
			// builderChain.resolveRecordQuiet(baseEnvironment,pathToRecord);
			// if(endEnvironment!=null) {
			// pool.addElementAccess(endEnvironment, pathToRecord);
			// return true;
			// }
			// return false;
			// }
			// };
			return this.builderChain_;
		}

		public SpecificCommonErrorOutput getEndError() {
			return this.builderChain_.getEndError();
		}

		public void end() {

		}

	}

	// ---------------
	private static final class LocalVariableSegment implements Segment, JavaCallChainSectionPeer.LocalPatternPeer {
		private final ElementStore elementStore_;
		private final SpecificCommonErrorOutput baseError_;
		private ViaVariableResolution resolution_ = DEFAULT_VIA_RESOLUTION;

		public LocalVariableSegment(SpecificCommonErrorOutput baseError) {
			this.baseError_ = baseError;
			this.elementStore_ = new ElementStore(baseError);
		}

		public SpecificCommonErrorOutput getEndError() {
			return this.elementStore_.getEndError();
		}

		public void doRecordAction(RecordVariableAction variableAction, SLayer environment) {
			Debug.finishMeMarker(); // ?
			elementStore_.getEndError().invalidTypeError("local variable");
		}

		public boolean resolveForChainInFinish(boolean allowLocalVariables, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			AccessPool pool = new AccessPool();
			pool.addElementAccess(baseEnvironment, new BaseTrackers.JavaVariablePath());
			return this.resolution_.resolveElements(allowLocalVariables, this.elementStore_, pool, target, generalDataTracker, baseEnvironment);
			// this.elementStore_.resolveDownstream(allowLocalVariables, pool, target,
			// generalDataTracker,baseEnvironment);
		}

		public boolean resolveForExpressionInFinish(boolean allowLocalVariables, ExpressionDetailsFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			BasicCallChainFormBlock callSkeleton = target.getRootBuilder().createBasicCallChainChild();
			target.addCallChain(callSkeleton);
			return this.resolveForChainInFinish(allowLocalVariables, callSkeleton, generalDataTracker, baseEnvironment);
		}

		public void doAddTargetVarRequirementDirect(boolean allowLocalVariables, TargetType requiredVariableType,
				MacroCallDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment environment) {
			AccessPool pool = new AccessPool();
			pool.addElementAccess(environment, new BaseTrackers.JavaVariablePath());
			this.elementStore_.doAddTargetVarRequirementDirect(allowLocalVariables, requiredVariableType, pool, macroCallBlock, generalDataTracker, environment);

			// if(foundVariable!=null) {
			// foundVariable.getVariable().createMacroCallParameter(macroCallBlock,
			// pathToVariableChainTracker);
			// }
		}

		public void doAddTargetVarRequirementCombination(boolean allowLocalVariables, TargetType requiredVariableType,
				MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment environment) {
			AccessPool pool = new AccessPool();
			pool.addElementAccess(environment, new BaseTrackers.JavaVariablePath());
			this.elementStore_.doAddTargetVarRequirementCombination(allowLocalVariables, requiredVariableType, pool, macroCallBlock, generalDataTracker, environment);
		}

		public JavaMethodCallPatternPeer addJavaMethodCallForMethod(
				String nameParameter,
				int nameLineNumber,
				int nameCharacterNumber) {
			return this.elementStore_.addJavaMethodCallForMethod(nameParameter, nameLineNumber, nameCharacterNumber);
		}

		public void addVariable(String value, int lineNumber, int characterNumber) {
			this.elementStore_.addVariable(value, lineNumber, characterNumber);
		}

		public void end() {
		}

		public void addSpecificClassName(final String className, int lineNumber, int characterNumber) {
			final SpecificCommonErrorOutput error = this.baseError_.createAdjusted(lineNumber, characterNumber);
			this.resolution_ = new SpecifiedClassResolution(className, error);
		}

	}

	private final static class SpecifiedClassResolution implements ViaVariableResolution {
		private final String className_;
		private final SpecificCommonErrorOutput error_;

		SpecifiedClassResolution(String className, SpecificCommonErrorOutput error) {
			this.className_ = className;
			this.error_ = error;

		}

		public boolean resolveElements(boolean allowLocalVariables, ElementStore elementStore, AccessPool pool, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			BaseTrackers.JavaVariablePath variableStuff = new BaseTrackers.JavaVariablePath();
			TargetType requiredType = pool.getTargetTypeQuiet(this.className_, variableStuff);
			if (requiredType == null) {
				this.error_.objectNotFoundError("target type", this.className_);
				return false;
			} else {
				boolean result = elementStore.resolveWithType(allowLocalVariables, requiredType, pool, target, generalDataTracker, baseEnvironment);
				return result;
			}
		}
	}

	// ---------------
	private static interface ViaVariableResolution {
		public boolean resolveElements(boolean allowLocalVariables, ElementStore elementStore, AccessPool pool, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment);
	}

	private static final ViaVariableResolution DEFAULT_VIA_RESOLUTION = new ViaVariableResolution() {
		public boolean resolveElements(boolean allowLocalVariables, ElementStore elementStore, AccessPool pool, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			return elementStore.resolveDownstream(allowLocalVariables, pool, target, generalDataTracker, baseEnvironment);
		}
	};

	// ==========================================================================
	// ==== Via Segment
	private static final class ViaSegment implements Segment, JavaCallChainSectionPeer.ViaPatternPeer {
		private final BuilderVariableChainBlock builderChain_;
		private final XCodeCallChainBlock parent_;

		private final ElementStore elementStore_;
		private ViaVariableResolution resolution_ = DEFAULT_VIA_RESOLUTION;

		public ViaSegment(XCodeCallChainBlock parent) {
			this.parent_ = parent;
			this.builderChain_ = new BuilderVariableChainBlock(parent.baseError_);
			this.elementStore_ = new ElementStore(parent.baseError_);
		}

		public SpecificCommonErrorOutput getEndError() {
			return this.elementStore_.getEndError();
		}

		public void doRecordAction(RecordVariableAction variableAction, SLayer environment) {
			Debug.finishMeMarker(); // ?
			elementStore_.getEndError().invalidTypeError("record variable");
		}

		public void doAddTargetVarRequirementDirect(
				boolean allowLocalVariables, TargetType requiredVariableType,
				MacroCallDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment environment) {

			BaseTrackers.JavaVariablePath variablePath = new BaseTrackers.JavaVariablePath();

			DataBlockEnvironment endEnvironment = this.builderChain_.resolveRecordQuiet(environment, variablePath, true);

			if (endEnvironment != null) {
				AccessPool pool = new AccessPool();
				pool.addElementAccess(endEnvironment, variablePath);
				this.elementStore_.doAddTargetVarRequirementDirect(allowLocalVariables, requiredVariableType, pool, macroCallBlock, generalDataTracker, environment);
			}
		}

		public void doAddTargetVarRequirementCombination(boolean allowLocalVariables, TargetType requiredVariableType,
				MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {

			BaseTrackers.JavaVariablePath variablePath = new BaseTrackers.JavaVariablePath();

			DataBlockEnvironment endEnvironment = this.builderChain_.resolveRecordQuiet(baseEnvironment, variablePath, true);

			if (endEnvironment != null) {
				AccessPool pool = new AccessPool();
				pool.addElementAccess(endEnvironment, variablePath);
				this.elementStore_.doAddTargetVarRequirementCombination(allowLocalVariables, requiredVariableType, pool, macroCallBlock, generalDataTracker, baseEnvironment);
			}
		}

		public BuilderVariableChainPatternPeer addBuilderVariableChainForBuilderVariableChain() {
			return this.builderChain_;
		}

		public JavaMethodCallPatternPeer addJavaMethodCallForMethod(
																																String nameParameter,
																																int nameLineNumber,
																																int nameCharacterNumber) {
			return this.elementStore_.addJavaMethodCallForMethod(nameParameter, nameLineNumber, nameCharacterNumber);
		}

		public void addVariable(String value, int lineNumber, int characterNumber) {
			this.elementStore_.addVariable(value, lineNumber, characterNumber);
		}

		public void end() {

		}

		public boolean resolveForExpressionInFinish(boolean allowLocalVariables, ExpressionDetailsFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			BasicCallChainFormBlock callSkeleton = target.getRootBuilder().createBasicCallChainChild();
			target.addCallChain(callSkeleton);
			return this.resolveForChainInFinish(allowLocalVariables, callSkeleton, generalDataTracker, baseEnvironment);
		}

		public boolean resolveForChainInFinish(boolean allowLocalVariables, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			AccessPool pool = new AccessPool();
			BaseTrackers.JavaVariablePath pathToRecord = new BaseTrackers.JavaVariablePath();
			DataBlockEnvironment endEnvironment = this.builderChain_.resolveRecordQuiet(baseEnvironment, pathToRecord, true);
			if (endEnvironment == null) {
				return false;
			}
			pool.addElementAccess(endEnvironment, pathToRecord);
			return this.resolution_.resolveElements(allowLocalVariables, this.elementStore_, pool, target, generalDataTracker, baseEnvironment);
		}

		public void addSpecificClassName(final String className, int lineNumber, int characterNumber) {
			final SpecificCommonErrorOutput error = this.parent_.baseError_.createAdjusted(lineNumber, characterNumber);
			this.resolution_ = new SpecifiedClassResolution(className, error);
		}
	}

	// =====================================================================================
	private final static class ElementStore {
		private final SimpleVector<Element> downstreamElements_ = new SimpleVector<Element>();
		private final SpecificCommonErrorOutput baseError_;

		public ElementStore(SpecificCommonErrorOutput baseError) {
			this.baseError_ = baseError;

		}

		public SpecificCommonErrorOutput getEndError() {
			return downstreamElements_.lastElement().getError();
		}

		public JavaMethodCallPatternPeer addJavaMethodCallForMethod(
				String nameParameter,
				int nameLineNumber,
				int nameCharacterNumber) {
			SpecificCommonErrorOutput error = this.baseError_.createAdjusted(nameLineNumber, nameCharacterNumber);
			MethodElement me = new MethodElement(nameParameter, error);
			this.downstreamElements_.addElement(me);
			return me;
		}

		private final void checkError(TargetType currentType, TargetType requiredVariableType) {
			if (!currentType.isSubTypeOf(requiredVariableType)) {
				// Will later need to ask if suitable type instead (to allow for
				// inheritence) - added 5/03/11!
				SpecificCommonErrorOutput error = this.downstreamElements_.lastElement().getError();
				error.invalidTypeError(currentType.getOutputName(), requiredVariableType.getOutputName());
			}
		}

		public boolean doAddTargetVarRequirementDirect(boolean allowLocalVariables, TargetType requiredVariableType, AccessPool pool,
				MacroCallDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			TargetType currentType;
			if ((currentType = baseResolveFinal(allowLocalVariables, null, pool, generalDataTracker, baseEnvironment)) == null) {
				return false;
			}
			checkError(currentType, requiredVariableType);

			if (downstreamElements_.size() == 1) {
				Element only = this.downstreamElements_.firstElement();
				only.doAddTargetVarRequirementDirect(macroCallBlock, baseEnvironment);
			} else {
				// Again - is this used?
				BasicCallChainFormBlock chainSkeleton = macroCallBlock.getRootBuilder().createBasicCallChainChild();
				MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock constructedSkeleton = macroCallBlock.addCallParameter().addTargetVariable().addConstructed(
						constructedTargetIndexCount_++, chainSkeleton);
				for (Element e : this.downstreamElements_) {
					// Probably used by this method call - need to check.
					boolean worked = e.doAddTargetVarRequirementCombination(constructedSkeleton, baseEnvironment);
					if (!worked) {
						return false;
					}
				}
			}
			return true;
		}

		public boolean doAddTargetVarRequirementCombination(boolean allowLocalVariables, TargetType requiredVariableType, AccessPool pool,
				MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			TargetType currentType;
			if ((currentType = baseResolveFinal(allowLocalVariables, null, pool, generalDataTracker, baseEnvironment)) == null) {
				return false;
			}
			checkError(currentType, requiredVariableType);
			for (Element e : this.downstreamElements_) {
				boolean worked = e.doAddTargetVarRequirementCombination(macroCallBlock, baseEnvironment);
				if (!worked) {
					return false;
				}
			}

			return true;
		}

		public void addVariable(String value, int lineNumber, int characterNumber) {
			SpecificCommonErrorOutput error = this.baseError_.createAdjusted(lineNumber, characterNumber);

			if (this.downstreamElements_.isEmpty() && Character.isUpperCase(value.charAt(0))) {
				// Treat as static call
				StaticTypeElement me = new StaticTypeElement(value, error);
				this.downstreamElements_.addElement(me);

			} else {
				VariableElement me = new VariableElement(value, error);
				this.downstreamElements_.addElement(me);
			}
		}

		public final boolean resolveDownstream(boolean allowLocalVariables, AccessPool pool, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			return resolveWithType(allowLocalVariables, null, pool, target, generalDataTracker, baseEnvironment);
		}

		/**
		 * Do basic resolve
		 * 
		 * @param currentType
		 *          use null if the local variables ought to be checked, otherwise
		 *          get the encompassing type from the baseEnvironment
		 * @param pool
		 * @param baseEnvironment
		 * @return
		 */
		private final TargetType baseResolveFinal(boolean allowLocalVariables, TargetType currentType, AccessPool pool, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			for (Element e : this.downstreamElements_) {

				if (currentType == null) {
					currentType = e.resolveFirstInFinal(allowLocalVariables, pool, generalDataTracker, baseEnvironment);
				} else {
					currentType = e.resolveFinal(currentType, pool, generalDataTracker, baseEnvironment);
				}
				if (currentType == null) {
					return null;
				}
			}
			return currentType;
		}

		// Note: may never need this form = can just call baseResolve with no type?
		// Have a mucked up here?
		// Why do we need the seperate getFirstX() stuff?
		public final boolean resolveWithType(boolean allowLocalVariables, TargetType currentType, AccessPool pool, BasicCallChainFormBlock target, GeneralDataTracker generalDataTracker, CodeBlockEnvironment baseEnvironment) {
			if ((currentType = baseResolveFinal(allowLocalVariables, currentType, pool, generalDataTracker, baseEnvironment)) == null) {
				return false;
			}

			boolean first = true;
			for (Element e : this.downstreamElements_) {
				if (first) {
					first = false;
				} else {
					target.addElement().addDivider();
				}
				e.construct(target, baseEnvironment);
			}
			return true;
		}
	}

	// ------------------------------------------------------------------------------------------------------------------

	// private static final void resolveDownstream(AccessPool pool,
	// BasicCallChainTemplate target, SimpleVector<Element>
	// downstreamElements,CodeBlockEnvironment baseEnvironment) {
	// TargetType currentType = null;
	// for(Element e : downstreamElements) {
	// if(currentType==null) {
	// currentType = e.resolveFirst( pool,baseEnvironment);
	// } else {
	// currentType = e.resolve(currentType, pool,baseEnvironment);
	// }
	// if(currentType==null) {
	// break;
	// }
	// }
	//
	// if(currentType!=null) {
	// boolean first = true;
	// for(Element e : downstreamElements) {
	// if(first) {
	// first = false;
	// } else {
	// target.addDividerBlock();
	// }
	// e.construct(target,baseEnvironment);
	// }
	// }
	// }

	// =================================================================================================================
	private static interface Element {
		public TargetType resolveFinal(TargetType currentType, AccessPool pool, GeneralDataTracker generalTracker, CodeBlockEnvironment baseEnvironment);

		public TargetType resolveFirstInFinal(boolean allowLocalVariables, AccessPool pool, GeneralDataTracker generalTracker, CodeBlockEnvironment baseEnvironment);

		public void construct(BasicCallChainFormBlock target, CodeBlockEnvironment baseEnvironment);

		public boolean doAddTargetVarRequirementDirect(MacroCallDataBlock macroCallBlock, TargetElementAccess baseEnvironment);

		public boolean doAddTargetVarRequirementCombination(MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, TargetElementAccess baseEnvironment);

		public SpecificCommonErrorOutput getError();
	} // end of interface Element

	// =================================================================================================================
	// ======= Variable Element
	// TODO: add static stuff
	private static final class VariableElement implements Element {
		private final String name_;
		private final SpecificCommonErrorOutput error_;
		private final BaseTrackers.JavaVariablePath pathToVariableDef_;
		private TargetVariable variable_;

		public VariableElement(String name, SpecificCommonErrorOutput error) {
			this.name_ = name;
			this.error_ = error;
			this.pathToVariableDef_ = new BaseTrackers.JavaVariablePath();
		}

		public SpecificCommonErrorOutput getError() {
			return error_;
		}

		public boolean doAddTargetVarRequirementDirect(MacroCallDataBlock macroCallBlock, TargetElementAccess baseEnvironment) {

			try {
				variable_.createMacroCallParameter(macroCallBlock, pathToVariableDef_);
			} catch (ParsingException e) {
				Debug.finishMeMarker(); // This may be a problem if the error is caused
																// in the last pass
				// TODO: check that an error logged at this point will affect parsing
				e.updateError(this.error_);
			}
			return true;
		}

		public boolean doAddTargetVarRequirementCombination(MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, TargetElementAccess baseEnvironment) {
			// Probably should change to take the chain details parameter (then the
			// call makes more sense)
			variable_.createCallExpressionFinal(macroCallBlock.getChainDetailsParameter(), pathToVariableDef_);
			return true;
		}

		public TargetType resolveFirstInFinal(boolean allowLocal, AccessPool pool, GeneralDataTracker generalTracker, CodeBlockEnvironment baseEnvironment) {
			generalTracker.setRequiresStatic(false);

			try {
				TargetVariable variable = pool.getFirstVariable(allowLocal, baseEnvironment, this.name_, pathToVariableDef_);
				this.variable_ = variable;
				TargetType targetType = variable.getTargetTypeReferenceQuietFinal().getBaseType();
				return targetType;

			} catch (ParsingException e) {
				e.updateError(this.error_);
				return null;
			}
		}

		// public TargetType resolve( TargetType currentType,AccessPool pool,
		// CodeBlockRecordLinkup codeEnvironment) {
		// try {
		// this.variable_ = pool.getVariable(currentType, this.name_,
		// this.pathToVariableDef_);
		// return this.variable_.getTargetType();
		// } catch (ParsingException e) {
		// e.updateError(this.error_);
		// return null;
		// }
		// }
		public TargetType resolveFinal(
				TargetType currentType, AccessPool pool, GeneralDataTracker generalTracker,
					CodeBlockEnvironment baseEnvironment) {
			generalTracker.setRequiresStatic(false);
			try {
				this.variable_ = pool.getVariable(currentType, this.name_, this.pathToVariableDef_);
				return this.variable_.getTargetTypeReferenceQuietFinal().getBaseType();
			} catch (ParsingException e) {
				e.updateError(this.error_);
				return null;
			}
		}

		public void construct(BasicCallChainFormBlock target, CodeBlockEnvironment baseEnvironment) {
			// TODO: may not need getCopy()
			variable_.createCallExpressionFinal(target, pathToVariableDef_.getCopy());
		}

	} // end of Variable Element
	// =================================================================================================================
	// ======= Variable Element

	private static final class StaticTypeElement implements Element {
		private final String name_;
		private final SpecificCommonErrorOutput error_;
		private final BaseTrackers.JavaVariablePath pathToVariableDef_;
		private TargetType type_;

		public StaticTypeElement(String name, SpecificCommonErrorOutput error) {
			this.name_ = name;
			this.error_ = error;
			this.pathToVariableDef_ = new BaseTrackers.JavaVariablePath();
		}

		public SpecificCommonErrorOutput getError() {
			return error_;
		}

		public boolean doAddTargetVarRequirementDirect(MacroCallDataBlock macroCallBlock, TargetElementAccess baseEnvironment) {

			error_.systemError("Confused", "not sure what to do here - probably isn't allowed");
			Debug.finishMeMarker();

			return true;
		}

		public boolean doAddTargetVarRequirementCombination(MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, TargetElementAccess baseEnvironment) {
			TypeSetterDetailsFormBlock typeSkeleton = macroCallBlock.getRootBuilder().createTypeSetterDetailsChild();
			macroCallBlock.getChainDetailsParameter().addElement().addStaticCall(typeSkeleton);
			this.type_.buildCodeSetType(typeSkeleton, this.pathToVariableDef_.getCopy());
			return true;
		}

		private final TargetType doBaseResolve(AccessPool pool, GeneralDataTracker generalTracker) {
			TargetType type = pool.getTargetTypeQuiet(this.name_, pathToVariableDef_);
			this.type_ = type;
			generalTracker.setRequiresStatic(true);
			if (type != null) {
				return type;
			} else {
				error_.objectNotFoundError("target type", this.name_);
				return null;
			}
		}

		public TargetType resolveFirstInFinal(boolean allowLocal, AccessPool pool, GeneralDataTracker generalTracker, CodeBlockEnvironment baseEnvironment) {
			return doBaseResolve(pool, generalTracker);
		}

		public TargetType resolveFinal(TargetType currentType, AccessPool pool, GeneralDataTracker generalTracker, CodeBlockEnvironment baseEnvironment) {
			return doBaseResolve(pool, generalTracker);
		}

		public void construct(BasicCallChainFormBlock target, CodeBlockEnvironment baseEnvironment) {
			// TODO: may not need getCopy()
			TypeSetterDetailsFormBlock typeSkeleton = target.getRootBuilder().createTypeSetterDetailsChild();
			target.addElement().addStaticCall(typeSkeleton);
			this.type_.buildCodeSetType(typeSkeleton, this.pathToVariableDef_.getCopy());
		}

	} // end of StaticType Element

	// =================================================================================================================
	// ======= Method Element
	private static final class MethodElement implements Element, JavaMethodCallPatternPeer {
		private final String name_;
		private final SpecificCommonErrorOutput error_;

		private ArgumentBucket arguments_ = null;
		private final BaseTrackers.JavaVariablePath pathToVariableDef_;
		private TargetMethod method_;
		private boolean forceStaticCall_ = false;

		public MethodElement(String name, SpecificCommonErrorOutput error) {
			this.name_ = name;
			this.error_ = error;
			this.pathToVariableDef_ = new BaseTrackers.JavaVariablePath();
		}

		public SpecificCommonErrorOutput getError() {
			return error_;
		}

		public boolean doAddTargetVarRequirementDirect(MacroCallDataBlock macroCallBlock, TargetElementAccess baseEnvironment) {
			error_.generalSyntaxError("Inappropriate method use");
			return false;
		}

		public boolean doAddTargetVarRequirementCombination(
				MacroCallDataBlock.CallParameterDataBlock.TargetVariableDataBlock.ConstructedDataBlock macroCallBlock, TargetElementAccess baseEnvironment) {
			method_.createCallExpressionFinal(macroCallBlock.getChainDetailsParameter(), pathToVariableDef_);
			return true;
		}

		public TargetCallArgumentsPatternPeer addTargetCallArgumentsForArguments() {
			if (arguments_ == null) {
				this.arguments_ = new ArgumentBucket(this.error_);
			}
			return this.arguments_;
		}

		public void end() {
		}

		public TargetType resolveFinal(TargetType currentType, AccessPool pool, GeneralDataTracker generalTracker, CodeBlockEnvironment baseEnvironment) {
			boolean argumentsOkay = true;
			if (arguments_ != null) {
				argumentsOkay &= this.arguments_.basicBuild(baseEnvironment);
			}

			try {
				method_ = pool.getMethod(currentType, this.name_, this.pathToVariableDef_);
				checkStatic(generalTracker, false);
				if (argumentsOkay) {
					return method_.getReturnTargetTypeReferenceQuietFinal().getBaseType();
				} else {
					return null;
				}
			} catch (ParsingException e) {
				e.updateError(this.error_);
				return null;
			}

		}

		public void construct(BasicCallChainFormBlock target, CodeBlockEnvironment codeEnvironment) {
			// TODO: may not need getCopy()
			ArgumentSetDetailsFormBlock argumentsSetCode = method_.createCallExpressionFinal(target, pathToVariableDef_.getCopy());
			if (this.arguments_ != null) {
				this.arguments_.finish(codeEnvironment, argumentsSetCode);
			}
		}

		private final void checkStatic(GeneralDataTracker generalTracker, boolean allowStatic) {
			if (generalTracker.isRequiresStatic()) {
				generalTracker.setRequiresStatic(false);
				if (!method_.isStatic()) {
					this.error_.invalidConstructionError("instance method", this.name_, "require a static method");
				}
			} else {
				if (method_.isStatic()) {
					if (allowStatic) {
						generalTracker.setRequiresStatic(false);
						forceStaticCall_ = true;
					} else {
						this.error_.invalidConstructionError("static method", this.name_, "require an instance method");
					}
				}
			}
		}

		public TargetType resolveFirstInFinal(boolean allowLocalVariables, AccessPool pool, GeneralDataTracker generalTracker, CodeBlockEnvironment codeEnvironment) {
			if (arguments_ != null) {
				boolean succeeded = this.arguments_.basicBuild(codeEnvironment);
				if (!succeeded) {
					return null;
				}
			}

			try {
				method_ = pool.getFirstMethod(codeEnvironment, this.name_, this.pathToVariableDef_);
				checkStatic(generalTracker, true);
				return method_.getReturnTargetTypeReferenceQuietFinal().getBaseType();
			} catch (ParsingException e) {
				e.updateError(this.error_);
				return null;
			}
		}

	}

	private final static class GeneralDataTracker {
		private boolean requiresStatic_ = false;

		public void setRequiresStatic(boolean value) {
			this.requiresStatic_ = value;
		}

		public boolean isRequiresStatic() {
			return this.requiresStatic_;
		}

	}

}
