package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.GlobalClassReference;
import org.ffd2.skeletonx.compile.java.GlobalMethodReference;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.MacroConstruction;
import org.ffd2.skeletonx.compile.java.ScopeContainer;
import org.ffd2.skeletonx.compile.java.TargetMethod;
import org.ffd2.skeletonx.compile.java.TargetType;
import org.ffd2.skeletonx.compile.java.TargetVariable;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock;

import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;

public class DualIndirectLayerType implements LayerType {
	// private final JavaVariablePath pathToVariableDef_;
	// private final FoundationNode relatedFoundationNode_;
	private final SLayer parentLayer_;
	private final SLayer childLayer_;
	private final IndirectionType indirectionType_;

	/**
	 * Variable based layers share the space of their parent layers, but any
	 * references to things in their target scaffolds must go through a variable
	 * chain
	 * 
	 * @param pathToVariableDef
	 *          the chain to the variable that this Variable is connected too
	 */
	private DualIndirectLayerType(
			ImplementationBlockSkeletonManager blockSkeleton, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker, IndirectionType indirectionType, FileEnvironment environment) {
		this.childLayer_ = new SLayer(this, childMaker, blockSkeleton, parentLayer.getRelatedTracker(), environment);
		// this.pathToVariableDef_ = pathToVariableDef;
		this.parentLayer_ = parentLayer;
		this.indirectionType_ = indirectionType;
		indirectionType.notifyOfChildLayer(this.childLayer_);
	}

	public IRecordVariable getLayerStrictBuilderVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable) {
		this.indirectionType_.addLinkToParentLayer(pathToVariable);
		return parentLayer_.getLayerStrictBuilderVariableQuiet(name, pathToVariable);
	}

	public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
		return this.indirectionType_.getRelatedFoundationNode(pathToFoundationLayer);
	}

	public String getFullName() {
		return "[Dual indirection]" + indirectionType_.getFullNameComponent();
	}

	public SLayer getAncestorNodeSLayerBackwards(String name, BaseTrackers.JavaVariablePath pathToParentLayer) throws ParsingException {
		return this.indirectionType_.getParentNodeLayer(name, pathToParentLayer, this.parentLayer_);
	}

	public SLayer getAncestorVariableSLayerBackwards(String name, JavaVariablePath pathToParentLayer) throws ParsingException {
		return this.indirectionType_.getParentVariableLayer(name, pathToParentLayer, this.parentLayer_, this.childLayer_);

	}

	public String toString() {
		return "Dual:" + this.indirectionType_.getFullNameComponent();
	}

	public SLayer getChildLayer() {
		return this.childLayer_;
	}

	public ChainDefinitionBlock getChainDefinition(String name) throws ParsingException {
		return parentLayer_.getChainDefinition(name);
	}

	public MappingDefinitionBlock getMappingDefinition(String name) throws ParsingException {
		return parentLayer_.getMappingDefinition(name);
	}

	public KeyDefinitionBlock getKeyDefinition(String name) throws ParsingException {
		return parentLayer_.getKeyDefinition(name);
	}

	public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
		this.indirectionType_.setLocalChildTypeAsPeerLayer(localChildBaseSkeleton, targetVariablePathToFoundationChildrenChainTracker);
	}

	public FoundationNode getRootFoundationNodeBasic() {
		return parentLayer_.getRootFoundationNodeBasic();
	}

	//
	// public static final DualIndirectLayerType
	// buildVariableBased(BaseTrackers.JavaVariablePath pathToVariableDef,
	// FoundationNode relatedFoundationNode, SLayer parentLayer,
	// NonFoundationChildSLayerMaker childMaker,IRecordVariable relatedVariable,
	// FileEnvironment environment) {
	// return new DualIndirectLayerType(parentLayer.getBlockSkeleton(),
	// parentLayer,childMaker, new
	// VariableBasedIndirection(pathToVariableDef,relatedVariable,new
	// FoundationVariableSpecifics(relatedFoundationNode)),environment);
	// }
	public static final DualIndirectLayerType buildVariableBased(BaseTrackers.JavaVariablePath pathToVariableDef, SLayer relatedLayer, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker, IRecordVariable relatedVariable, FileEnvironment environment) {
		/*if (pathToVariableDef.isEmpty()) {
			Thread.dumpStack();
		}*/
		return new DualIndirectLayerType(parentLayer.getBlockSkeleton(), parentLayer, childMaker, new VariableBasedIndirection(pathToVariableDef, relatedVariable, new GeneralSLayerVariableSpecifics(relatedLayer)), environment);
	}

	public static final DualIndirectLayerType buildPeerBased(SLayer relatedLayer, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker, FileEnvironment environment) {
		return new DualIndirectLayerType(parentLayer.getBlockSkeleton(), parentLayer, childMaker, new PeerBasedIndirection(relatedLayer), environment);
	}

	public static final DualIndirectLayerType buildLocalNodeChildBased(LocalChild childRef, FoundationNode relatedFoundationNode, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker, FileEnvironment environment) {
		return new DualIndirectLayerType(childRef.getSkeletonManager(), parentLayer, childMaker, new LocalChildBasedIndirection(childRef, relatedFoundationNode), environment);
	}

	public static final DualIndirectLayerType buildLocalNodeChildBased(LocalChild childRef, SLayer relatedLayer, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker, FileEnvironment environment) {
		return new DualIndirectLayerType(childRef.getSkeletonManager(), parentLayer, childMaker, new LocalChildBasedIndirectionWithLayer(childRef, relatedLayer), environment);
	}

	// -----------------------------
	public void buildPathToUpperLayerBack(SLayer ancestor, JavaVariablePath path) {
		this.indirectionType_.addLinkToParentLayer(path);
		this.parentLayer_.buildPathToUpperLayer(ancestor, path);

	}

	private final ImplementationBlockSkeletonManager getBaseSkeletonsForPathSearce() {
		// TODO Check use child layer, not parent
		return this.childLayer_.getBlockSkeleton();
	}

	public boolean buildPathToFoundationNode(FoundationNode target, BaseTrackers.JavaVariablePath path) {
		int mark = path.mark();
		if (!this.indirectionType_.checkBuildPathToFoundationNode(getBaseSkeletonsForPathSearce(), target, path)) {
			path.unmark(mark);
			this.indirectionType_.addLinkToParentLayer(path);
			return parentLayer_.buildPathToFoundationNode(target, path);
		} else {
			return true;
		}
	}

	public boolean buildPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path) {
		int mark = path.mark();
		// Debug.println("Checking:",this.indirectionType_);
		if (this.indirectionType_.checkBuildPathToPeerSLayer(getBaseSkeletonsForPathSearce(), target, path)) {
			return true;
		} else {
			path.unmark(mark);
			this.indirectionType_.addLinkToParentLayer(path);
			return parentLayer_.buildPathToPeerLayer(target, path);
		}
	}

	public boolean buildHorizontalPathToPeerSkeleton(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
		int mark = path.mark();
		if (this.indirectionType_.checkBuildPathToPeerBlock(base, target, path)) {
			return true;
		} else {
			path.unmark(mark);
			return false;
		}
	}

	public boolean buildHorizontalPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path) {
		int mark = path.mark();
		if (this.indirectionType_.checkBuildPathToPeerSLayer(getBaseSkeletonsForPathSearce(), target, path)) {
			return true;
		} else {
			path.unmark(mark);
			return false;
		}
	}

	// public void addPathElementsToFoundationNodePeer(JavaVariablePath pathStore)
	// {
	// //Just add the variable chain to the Foundation Layer
	// this.pathToVariableDef_.transferTo(pathStore);
	// }

	public IRecordVariable getBuilderVariableQuiet(String name, JavaVariablePath pathToVariable, boolean isFirstLayer) {

		// Now we have two options
		// (1) check back along the Foundation nodes of the Scaffold heirarchy of
		// the base layer variable
		// (2) check back along the implementation layers.

		// If the variable is "local", or that it exists in the related foundation,
		// then we only need to append the chain
		// to the layer variable's reference
		// That is if $a is an A and A(C c) , and we are in @link $a { }, then $c
		// will be resolved to aRef_.cVarRef_
		final int pathMark = pathToVariable.mark();
		// Try via the FoundationNode tree first

		IRecordVariable local = this.indirectionType_.getIndirectedBuilderParameterQuiet(name, pathToVariable, isFirstLayer);

		// (if first layer, we check all the way back on the Foundation Heirarchy
		// first, otherwise, just the variables or the locally defined node
		if (local != null) {
			return local;
		} else {
			// Clear path, and try again

			pathToVariable.unmark(pathMark);

			this.indirectionType_.addLinkToParentLayer(pathToVariable);
			return this.parentLayer_.getBuilderVariableQuiet(name, pathToVariable, false);
		}
	}

	public boolean buildPathToType(TargetType type, JavaVariablePath pathStore, boolean isFirstLayer) {
		final int pathMark = pathStore.mark();

		boolean foundType = this.indirectionType_.buildPathToType(type, pathStore, isFirstLayer);
		if (foundType) {
			return true;
		}
		pathStore.unmark(pathMark);
		// (if first layer, we check all the way back on the Foundation Heirarchy
		// first, otherwise, just the variables or the locally defined node

		this.indirectionType_.addLinkToParentLayer(pathStore);
		return this.parentLayer_.buildPathToType(type, pathStore, false);
	}

	public TargetVariable getTargetVariableQuiet(boolean allowLocalVariables, TargetType outputClass, String variableName, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		final int pathMark = pathToDeclaration.mark();
		this.indirectionType_.addLinkToLocalFoundationLayer(pathToDeclaration);
		if (pathToDeclaration.isSameMark(pathMark)) {
			// Thread.dumpStack();
			// Debug.println("Link to foundation failed:",this.indirectionType_);
		}
		TargetVariable result = this.indirectionType_.getParallelLayer().getTargetVariableQuiet(allowLocalVariables, outputClass, variableName, pathToDeclaration, isFirstLayer);
		if (result != null) {
			return result;
		}
		pathToDeclaration.unmark(pathMark);

		this.indirectionType_.addLinkToParentLayer(pathToDeclaration);
		if (pathToDeclaration.isSameMark(pathMark)) {
			// Thread.dumpStack();
			// Debug.println("Link to parent failed:",this.indirectionType_);
			// Debug.println("Current path:",pathToDeclaration);
		}
		return parentLayer_.getTargetVariableQuiet(allowLocalVariables, outputClass, variableName, pathToDeclaration, false);
	}

	public TargetMethod getTargetMethodQuiet(TargetType outputClass, String name, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		final int pathMark = pathToDeclaration.mark();
		this.indirectionType_.addLinkToLocalFoundationLayer(pathToDeclaration);
		TargetMethod result = this.indirectionType_.getParallelLayer().getTargetMethodQuiet(outputClass, name, pathToDeclaration, isFirstLayer);
		if (result != null) {
			return result;
		}
		pathToDeclaration.unmark(pathMark);

		this.indirectionType_.addLinkToParentLayer(pathToDeclaration);
		return parentLayer_.getTargetMethodQuiet(outputClass, name, pathToDeclaration, false);
	}

	public TargetType getTargetTypeQuiet(String typeName, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		final int pathMark = pathToDeclaration.mark();
		this.indirectionType_.addLinkToLocalFoundationLayer(pathToDeclaration);
		TargetType result = this.indirectionType_.getParallelLayer().getTargetTypeQuiet(typeName, pathToDeclaration, isFirstLayer);
		if (result != null) {
			return result;
		}

		pathToDeclaration.unmark(pathMark);

		this.indirectionType_.addLinkToParentLayer(pathToDeclaration);
		return parentLayer_.getTargetTypeQuiet(typeName, pathToDeclaration, false);
	}

	public MacroConstruction getMacroConstructionFinal(String name, BaseTrackers.JavaVariablePath pathToContainingLayer) throws ItemNotFoundException {
		this.indirectionType_.addLinkToParentLayer(pathToContainingLayer);

		return parentLayer_.getMacroConstructionFinal(name, pathToContainingLayer);
	}

	public GlobalClassReference getGlobalClassReferenceQuiet(String className, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {

		final int pathMark = pathToDeclaration.mark();
		this.indirectionType_.addLinkToLocalFoundationLayer(pathToDeclaration);
		GlobalClassReference result = this.indirectionType_.getParallelLayer().getGlobalClassReferenceQuiet(className, pathToDeclaration, isFirstLayer);
		if (result != null) {
			return result;
		}
		pathToDeclaration.unmark(pathMark);

		this.indirectionType_.addLinkToParentLayer(pathToDeclaration);
		return parentLayer_.getGlobalClassReferenceQuiet(className, pathToDeclaration, false);
	}

	public GlobalMethodReference getGlobalMethodReferenceQuiet(GlobalClassReference baseClass, String methodName, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		final int pathMark = pathToDeclaration.mark();
		this.indirectionType_.addLinkToLocalFoundationLayer(pathToDeclaration);
		GlobalMethodReference result = this.indirectionType_.getParallelLayer().getGlobalMethodReferenceQuiet(baseClass, methodName, pathToDeclaration, isFirstLayer);
		if (result != null) {
			return result;
		}
		pathToDeclaration.unmark(pathMark);

		this.indirectionType_.addLinkToParentLayer(pathToDeclaration);
		return parentLayer_.getGlobalMethodReferenceQuiet(baseClass, methodName, pathToDeclaration, false);
	}

	// Last finished 20/01/11
	@Override
	public TargetVariable getLocalVariableQuiet(ScopeContainer relatedScope, String variableName, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		final int pathMark = pathToDeclaration.mark();
		this.indirectionType_.addLinkToLocalFoundationLayer(pathToDeclaration);
		TargetVariable result = this.indirectionType_.getParallelLayer().getLocalVariableQuiet(relatedScope, variableName, pathToDeclaration, isFirstLayer);
		if (result != null) {
			return result;
		}
		pathToDeclaration.unmark(pathMark);
		this.indirectionType_.addLinkToParentLayer(pathToDeclaration);
		return parentLayer_.getLocalVariableQuiet(relatedScope, variableName, pathToDeclaration, false);
	}

	// Last finished 20/01/11
	@Override
	public boolean isLocalVariableQuiet(ScopeContainer relatedScope, String variableName, boolean isFirstLayer) {
		if (isFirstLayer) {
			if (this.indirectionType_.getParallelLayer().isLocalTargetVariableQuiet(relatedScope, variableName, isFirstLayer)) {
				return true;
			}
		}
		return parentLayer_.isLocalTargetVariableQuiet(relatedScope, variableName, false);
	}

	// Last finished 20/01/11
	@Override
	public AncestorConnection getAncestorFoundationNodeQuiet(String foundationNodeName, JavaVariablePath pathToAncestorNodeParent, boolean isFirstLayer) {
		final int pathMark = pathToAncestorNodeParent.mark();
		this.indirectionType_.addLinkToLocalFoundationLayer(pathToAncestorNodeParent);
		AncestorConnection result = this.indirectionType_.getParallelLayer().getAncestorFoundationNodeQuiet(foundationNodeName, pathToAncestorNodeParent, isFirstLayer);
		if (result != null) {
			return result;
		}
		pathToAncestorNodeParent.unmark(pathMark);
		this.indirectionType_.addLinkToParentLayer(pathToAncestorNodeParent);
		return this.parentLayer_.getAncestorFoundationNodeQuiet(foundationNodeName, pathToAncestorNodeParent, false);
	}

	// ==========================================================================
	// IndirectionType classes
	private static interface IndirectionType {
		public String getFullNameComponent();

		public void notifyOfChildLayer(SLayer newChildLayer);

		public boolean checkBuildPathToFoundationNode(ImplementationBlockSkeletonManager baseSkeletons, FoundationNode target, BaseTrackers.JavaVariablePath path);

		public boolean checkBuildPathToPeerSLayer(ImplementationBlockSkeletonManager baseSkeletons, SLayer target, BaseTrackers.JavaVariablePath path);

		public boolean checkBuildPathToPeerBlock(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path);

		public void addLinkToParentLayer(BaseTrackers.JavaVariablePath pathStore);

		public void addLinkToLocalFoundationLayer(BaseTrackers.JavaVariablePath pathStore);

		public IRecordVariable getIndirectedBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer);

		public SLayer getParallelLayer(); // Get the layer of the

		public boolean buildPathToType(TargetType type, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer);

		public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker);

		public SLayer getParentNodeLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer) throws ParsingException;

		public SLayer getParentVariableLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer, SLayer myLayer) throws ParsingException;

		public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer);

	}

	// -------------------------------------------------------------------------
	private interface VariableBasedSpecifics {
		public boolean doFinishBuildPath(TargetType type, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer);

		public IRecordVariable getIndirectedBuilderParameterFinishQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer);

		public SLayer getParallelLayer();

		public FoundationNode doGetRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer);

		public SLayer doGetRelatedLayer();
	}

	private static final class GeneralSLayerVariableSpecifics implements VariableBasedSpecifics {
		private final SLayer relatedLayer_;

		public GeneralSLayerVariableSpecifics(SLayer relatedLayer) {
			this.relatedLayer_ = relatedLayer;

		}

		public FoundationNode doGetRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
			return relatedLayer_.getRelatedFoundationNode(pathToFoundationLayer);
		}

		public SLayer doGetRelatedLayer() {
			return relatedLayer_;
		}

		public boolean doFinishBuildPath(TargetType type, JavaVariablePath pathToVariable, boolean isFirstLayer) {
			return relatedLayer_.buildPathToType(type, pathToVariable, isFirstLayer);
		}

		public IRecordVariable getIndirectedBuilderParameterFinishQuiet(String name, JavaVariablePath pathToVariable, boolean isFirstLayer) {
			return this.relatedLayer_.getBuilderVariableQuiet(name, pathToVariable, isFirstLayer);
		}

		public SLayer getParallelLayer() {
			return this.relatedLayer_;
		}
	}

	private static final class FoundationVariableSpecifics implements VariableBasedSpecifics {
		private final FoundationNode relatedFoundationNode_;

		public FoundationVariableSpecifics(FoundationNode relatedNode) {
			this.relatedFoundationNode_ = relatedNode;

		}

		public FoundationNode doGetRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
			return relatedFoundationNode_;
		}

		public SLayer doGetRelatedLayer() {
			return relatedFoundationNode_.getRelatedLayerForLayerComponents();
		}

		public SLayer getParallelLayer() {
			return this.relatedFoundationNode_.getRelatedLayerForLayerComponents();
		}

		public boolean doFinishBuildPath(TargetType type, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			return relatedFoundationNode_.buildPathToType(type, pathToVariable, isFirstLayer);
		}

		public final IRecordVariable getIndirectedBuilderParameterFinishQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			return this.relatedFoundationNode_.getBuilderParameterQuiet(name, pathToVariable, isFirstLayer);
		}

	}

	private static final class PeerBasedIndirection implements IndirectionType {
		private final SLayer peerLayer_;

		public PeerBasedIndirection(SLayer peerLayer) {
			this.peerLayer_ = peerLayer;

		}

		public void notifyOfChildLayer(SLayer newChildLayer) {
			// newChildLayer.getBlockSkeleton().addIsPeer(peerLayer_.getBlockSkeleton());
		}

		public String getFullNameComponent() {
			return "{PEER}:" + peerLayer_.getFullName();
		}

		public boolean checkBuildPathToPeerBlock(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			SLayer checkLayer = this.peerLayer_;
			base.getPeerConnection().addPathToPeer(path);
			if (!checkLayer.buildHorizontalPathToPeerSkeleton(target, path)) {
				path.unmark(mark);
				return false;
			}
			return true;
		}

		public boolean checkBuildPathToPeerSLayer(ImplementationBlockSkeletonManager baseSkeletons, SLayer target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			// Debug.println("Doing check build path:",target);
			SLayer checkLayer = this.peerLayer_;

			// Debug.println("Check:",checkLayer);
			// addLinkToLocalFoundationLayer(path);
			baseSkeletons.getPeerConnection().addPathToPeer(path);
			if (!checkLayer.buildHorizontalPathToPeerLayer(target, path)) {
				path.unmark(mark);
				return false;
			}
			return true;
		}

		public boolean checkBuildPathToFoundationNode(ImplementationBlockSkeletonManager baseSkeletons, FoundationNode target, BaseTrackers.JavaVariablePath path) {
			return checkBuildPathToPeerSLayer(baseSkeletons, target.getGlobalSLayer(), path);
		}

		public String toString() {
			return "PB:" + this.peerLayer_;
		}

		public SLayer getParentNodeLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer) throws ParsingException {
			return parentLayer.getAncestorNodeSLayer(name, pathToParentLayer);
		}

		public SLayer getParentVariableLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer, SLayer myLayer) throws ParsingException {
			return parentLayer.getAncestorVariableSLayer(name, pathToParentLayer);
		}

		public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
			addLinkToLocalFoundationLayer(pathToFoundationLayer);
			return peerLayer_.getRelatedFoundationNode(pathToFoundationLayer);
		}

		public boolean buildPathToType(TargetType type, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			addLinkToLocalFoundationLayer(pathToVariable);
			return this.peerLayer_.buildPathToType(type, pathToVariable, isFirstLayer);
		}

		public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
			throw new RuntimeException("ERR: MESSED UP!");
		}

		public IRecordVariable getIndirectedBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			addLinkToLocalFoundationLayer(pathToVariable);
			return this.peerLayer_.getBuilderVariableQuiet(name, pathToVariable, isFirstLayer);
		}

		// public void addLinkToParentLayer(BaseTrackers.JavaVariablePath pathStore)
		// { }
		public SLayer getParallelLayer() {
			return this.peerLayer_;
		}

		public void addLinkToLocalFoundationLayer(BaseTrackers.JavaVariablePath pathStore) {
			// Just add the variable chain to the Foundation Layer
			// this.pathToVariableDef_.transferTo(pathStore);
			throw Debug.finishMeMarker();
		}

		public void addLinkToParentLayer(JavaVariablePath pathStore) {
			// TODO Auto-generated method stub

		}

	}

	private static final class VariableBasedIndirection implements IndirectionType {
		private final JavaVariablePath pathToVariableDef_;
		private final IRecordVariable variableToMe_;
		private final VariableBasedSpecifics specifics_;

		public VariableBasedIndirection(JavaVariablePath pathToVariableDef, IRecordVariable variableToMe, VariableBasedSpecifics specifics) {
			this.pathToVariableDef_ = pathToVariableDef;
			this.variableToMe_ = variableToMe;
			this.specifics_ = specifics;
		}

		public void notifyOfChildLayer(SLayer newChildLayer) {
		}

		public boolean checkBuildPathToPeerBlock(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			// Debug.println("Doing check build path:",target);
			SLayer checkLayer = specifics_.getParallelLayer();
			// Debug.println("Check:",checkLayer);
			// base.getPeerConnection().addPathToPeer(path);
			addLinkToLocalFoundationLayer(path);
			// Debug.println("Doing check build path:",target, " Path=",path);

			if (!checkLayer.buildHorizontalPathToPeerSkeleton(target, path)) {
				path.unmark(mark);
				Thread.dumpStack();

				return false;
			}
			return true;
		}

		public boolean checkBuildPathToPeerSLayer(ImplementationBlockSkeletonManager baseSkeletons, SLayer target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			// Debug.println("Doing check build path:",target);
			SLayer checkLayer = specifics_.getParallelLayer();

			// Debug.println("Check:",checkLayer);
			addLinkToLocalFoundationLayer(path);
			// baseSkeletons.getPeerConnection().addPathToPeer(path);
			if (!checkLayer.buildHorizontalPathToPeerLayer(target, path)) {
				path.unmark(mark);
				return false;
			}
			return true;
		}

		public boolean checkBuildPathToFoundationNode(ImplementationBlockSkeletonManager baseSkeletons, FoundationNode target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			FoundationNode check = specifics_.doGetRelatedFoundationNode(path);
			if (check != target) {
				path.unmark(mark);
				return false;
			}
			return true;
		}

		public String toString() {
			return "VB:$" + variableToMe_.getRoughVariableName();
		}

		public String getFullNameComponent() {
			return "$" + variableToMe_.getRoughVariableName();
		}

		public SLayer getParentNodeLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer) throws ParsingException {
			return parentLayer.getAncestorNodeSLayer(name, pathToParentLayer);
		}

		public SLayer getParentVariableLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer, SLayer myLayer) throws ParsingException {
			if (this.variableToMe_.isMatchesVariableName(name)) {
				return myLayer;
			} else {
				return parentLayer.getAncestorVariableSLayer(name, pathToParentLayer);

			}
		}

		public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {

			addLinkToLocalFoundationLayer(pathToFoundationLayer);
			return specifics_.doGetRelatedFoundationNode(pathToFoundationLayer);
		}

		public boolean buildPathToType(TargetType type, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {

			addLinkToLocalFoundationLayer(pathToVariable);
			// Try via the FoundationNode tree first
			return this.specifics_.doFinishBuildPath(type, pathToVariable, isFirstLayer);
		}

		public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
			throw new RuntimeException("ERR: MESSED UP!");
		}

		public IRecordVariable getIndirectedBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			addLinkToLocalFoundationLayer(pathToVariable);
			return this.specifics_.getIndirectedBuilderParameterFinishQuiet(name, pathToVariable, isFirstLayer);
		}

		public void addLinkToParentLayer(BaseTrackers.JavaVariablePath pathStore) {
		}

		public SLayer getParallelLayer() {
			return this.specifics_.getParallelLayer();
		}

		public void addLinkToLocalFoundationLayer(BaseTrackers.JavaVariablePath pathStore) {
			// Just add the variable chain to the Foundation Layer
			this.pathToVariableDef_.transferTo(pathStore);
		}
	}

	// -------------------------------------------------------------------------
	// private static final class VariableBasedIndirectionWithLayer implements
	// IndirectionType {
	// private final JavaVariablePath pathToVariableDef_;
	// private final SLayer relatedLa2yer_;
	// public VariableBasedIndirectionWithLayer(JavaVariablePath
	// pathToVariableDef, SLayer relatedLayer) {
	// this.pathToVariableDef_ = pathToVariableDef;
	// this.relatedLayer_ = relatedLayer;
	// }
	// public boolean buildPathToType(TargetType type,
	// BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
	// addLinkToLocalFoundationLayer(pathToVariable);
	// //Try via the FoundationNode tree first
	//
	// return relatedLayer_.buildPathToType(type, pathToVariable, isFirstLayer) ;
	// }
	// public void
	// setLocalChildTypeAsPeerLayer(JavaImplementationForm.BlockDataBlock.LocalChildRefDataBlock
	// localChildBaseSkeleton, BaseTrackers.JavaVariablePath
	// targetVariablePathToFoundationChildrenChainTracker) {
	// throw new RuntimeException("ERR: MESSED UP!");
	// }
	// public IRecordVariable getIndirectedBuilderParameterQuiet(String name,
	// BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
	// addLinkToLocalFoundationLayer(pathToVariable);
	//
	// //Try via the FoundationNode tree first
	//
	// return
	// this.relatedLayer_.getBuilderVariableQuiet(name,pathToVariable,isFirstLayer)
	// ;
	// }
	//
	// public void addLinkToParentLayer(BaseTrackers.JavaVariablePath pathStore) {
	//
	// }
	// public SLayer getParallelLayer() { return this.relatedLayer_; }
	// public void addLinkToLocalFoundationLayer(BaseTrackers.JavaVariablePath
	// pathStore) {
	// //Just add the variable chain to the Foundation Layer
	// this.pathToVariableDef_.transferTo(pathStore);
	// }
	// }
	// -------------------------------------------------------------------------

	// -------------------------------------------------------------------------
	private final static class LocalChildBasedIndirection implements IndirectionType {
		private final LocalChild childRef_;
		private final FoundationNode relatedFoundationNode_;

		public LocalChildBasedIndirection(LocalChild childRef, FoundationNode relatedFoundationNode) {
			this.childRef_ = childRef;
			this.relatedFoundationNode_ = relatedFoundationNode;
		}

		public String getFullNameComponent() {
			return this.childRef_.getChildName();
		}

		public boolean checkBuildPathToFoundationNode(ImplementationBlockSkeletonManager baseSkeletons, FoundationNode target, BaseTrackers.JavaVariablePath path) {
			if (target == relatedFoundationNode_) {
				this.childRef_.addGlobalFoundationNodeLink(path);
				return true;
			} else {
				return false;
			}
		}

		public boolean checkBuildPathToPeerBlock(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			base.getPeerConnection().addPathToPeer(path);
			// addLinkToLocalFoundationLayer(path);

			if (relatedFoundationNode_.getRelatedLayerForLayerComponents().buildHorizontalPathToPeerSkeleton(target, path)) {
				// this.childRef_.addGlobalFoundationNodeLink(path);
				return true;
			} else {
				path.unmark(mark);
				Thread.dumpStack();

				return false;
			}
		}

		public boolean checkBuildPathToPeerSLayer(ImplementationBlockSkeletonManager baseSkeletons, SLayer target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();

			// addLinkToLocalFoundationLayer(path);
			baseSkeletons.getPeerConnection().addPathToPeer(path);
			if (relatedFoundationNode_.getRelatedLayerForLayerComponents().buildHorizontalPathToPeerLayer(target, path)) {
				// this.childRef_.addGlobalFoundationNodeLink(path);
				return true;
			} else {
				path.unmark(mark);
				return false;
			}
		}

		public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
			addLinkToLocalFoundationLayer(pathToFoundationLayer);
			return relatedFoundationNode_;
		}

		public SLayer getParentVariableLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer, SLayer myLayer) throws ParsingException {
			this.childRef_.addLocalParentLink(pathToParentLayer);
			return parentLayer.getAncestorVariableSLayer(name, pathToParentLayer);
		}

		public SLayer getParentNodeLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer) throws ParsingException {
			if (this.childRef_.getChildName().equals(name)) {
				return childRef_.getChildLayer();
			} else {
				this.childRef_.addLocalParentLink(pathToParentLayer);
				return parentLayer.getAncestorNodeSLayer(name, pathToParentLayer);
			}
		}

		public String toString() {
			return "LocalChildFN:" + childRef_;
		}

		public boolean buildPathToType(TargetType type, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			addLinkToLocalFoundationLayer(pathToVariable);
			// Try via the FoundationNode tree first

			return relatedFoundationNode_.buildPathToType(type, pathToVariable, isFirstLayer);
		}

		public void addLinkToParentLayer(JavaVariablePath pathStore) {
			this.childRef_.addLocalParentLink(pathStore);
		}

		public IRecordVariable getIndirectedBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			addLinkToLocalFoundationLayer(pathToVariable);

			// Try via the FoundationNode tree first

			return this.relatedFoundationNode_.getBuilderParameterQuiet(name, pathToVariable, isFirstLayer);
		}

		public SLayer getParallelLayer() {
			return this.relatedFoundationNode_.getRelatedLayerForLayerComponents();
		}

		public void addLinkToLocalFoundationLayer(JavaVariablePath pathStore) {
			this.childRef_.addGlobalFoundationNodeLink(pathStore);
		}

		public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
			this.childRef_.setLocalChildTypeAsPeerLayer(localChildBaseSkeleton, targetVariablePathToFoundationChildrenChainTracker);
		}

		public void notifyOfChildLayer(SLayer newChildLayer) {
		}
	}

	// -------------------------------------------------------------------------
	private final static class LocalChildBasedIndirectionWithLayer implements IndirectionType {
		private final LocalChild childRef_;
		private final SLayer relatedLayer_;

		public LocalChildBasedIndirectionWithLayer(LocalChild childRef, SLayer relatedLayer) {
			this.childRef_ = childRef;
			this.relatedLayer_ = relatedLayer;
		}

		public void notifyOfChildLayer(SLayer newChildLayer) {
		}

		public boolean checkBuildPathToPeerBlock(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			base.getPeerConnection().addPathToPeer(path);
			// addLinkToLocalFoundationLayer(path);

			if (relatedLayer_.buildHorizontalPathToPeerSkeleton(target, path)) {
				return true;
			} else {
				path.unmark(mark);
				return false;
			}
		}

		public boolean checkBuildPathToPeerSLayer(ImplementationBlockSkeletonManager baseSkeletons, SLayer target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			// addLinkToLocalFoundationLayer(path);
			// this.childRef_.addGlobalFoundationNodeLink(path);
			baseSkeletons.getPeerConnection().addPathToPeer(path);

			if (relatedLayer_.buildHorizontalPathToPeerLayer(target, path)) {
				return true;
			} else {
				path.unmark(mark);
				return false;
			}
		}

		public boolean checkBuildPathToFoundationNode(ImplementationBlockSkeletonManager baseSkeletons, FoundationNode target, BaseTrackers.JavaVariablePath path) {
			int mark = path.mark();
			//this.childRef_.addGlobalFoundationNodeLink(path);
//			boolean worked = relatedLayer_.buildPathToFoundationNode(target,path);
//			if(worked) {
//				return true;
//			} 
//			path.unmark(mark);
			return false;
			//this.childRef_.addGlobalFoundationNodeLink(path);

			// TODO: not sure how to handle this!
			//Debug.println("Warning: may not be handling this right!");
			//return false;
		}

		public String getFullNameComponent() {
			return this.childRef_.getChildName();
		}

		public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
			addLinkToLocalFoundationLayer(pathToFoundationLayer);
			// Use the related one, as that is the one that connects with Foundation
			// Nodes
			return relatedLayer_.getRelatedFoundationNode(pathToFoundationLayer);
		}

		public String toString() {
			return "LocalChildSLayer:" + childRef_ + "," + this.relatedLayer_;
		}

		public boolean buildPathToType(TargetType type, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			addLinkToLocalFoundationLayer(pathToVariable);
			// Try via the FoundationNode tree first

			return relatedLayer_.buildPathToType(type, pathToVariable, isFirstLayer);
		}

		public void addLinkToParentLayer(JavaVariablePath pathStore) {
			this.childRef_.addLocalParentLink(pathStore);
		}

		public SLayer getParentNodeLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer) throws ParsingException {

			// if(relatedLayer_.)
			if (this.childRef_.getChildName().equals(name)) {
				return childRef_.getChildLayer();
			} else {
				// Hmm... maybe this isn't needed?
				this.childRef_.addLocalParentLink(pathToParentLayer);

				return parentLayer.getAncestorNodeSLayer(name, pathToParentLayer);
			}
		}

		public SLayer getParentVariableLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer, SLayer parentLayer, SLayer myLayer) throws ParsingException {
			this.childRef_.addLocalParentLink(pathToParentLayer);
			return parentLayer.getAncestorVariableSLayer(name, pathToParentLayer);
		}

		public IRecordVariable getIndirectedBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
			addLinkToLocalFoundationLayer(pathToVariable);

			// Try via the FoundationNode tree first

			return this.relatedLayer_.getBuilderVariableQuiet(name, pathToVariable, isFirstLayer);
		}

		public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
			this.childRef_.setLocalChildTypeAsPeerLayer(localChildBaseSkeleton, targetVariablePathToFoundationChildrenChainTracker);
		}

		public SLayer getParallelLayer() {
			return this.relatedLayer_;
		}

		public void addLinkToLocalFoundationLayer(JavaVariablePath pathStore) {
			this.childRef_.addGlobalFoundationNodeLink(pathStore);
		}
	}

}
