package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.GlobalClassReference;
import org.ffd2.skeletonx.compile.java.GlobalMethodReference;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.MacroConstruction;
import org.ffd2.skeletonx.compile.java.ScopeContainer;
import org.ffd2.skeletonx.compile.java.TargetMethod;
import org.ffd2.skeletonx.compile.java.TargetType;
import org.ffd2.skeletonx.compile.java.TargetVariable;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;

import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;

public interface LayerType {
	/**
	 * Called from basicBuild in SLayer
	 * @param nodeBuilder
	 * @param parentBlockSkeleton
	 * @param children
	 */
//	public void basicBuildNodeChildren(LinkupTracker nodeBuilder, ImplementationBlockSkeletonManager parentBlockSkeleton, AccessStoreX<NodeLayerChild> children);
		
	public SLayer getAncestorNodeSLayerBackwards(String name, BaseTrackers.JavaVariablePath pathToParentLayer) throws ParsingException;
	public SLayer getAncestorVariableSLayerBackwards(String name, BaseTrackers.JavaVariablePath pathToParentLayer) throws ParsingException;

	public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker);
//	public void checkVariableChildOkay(IRecordVariable referencingVariable) throws ParsingException;
	/**
	 * Go back up heirarchy looking for a builder parameter - if "first layer", branches off to check associated FounationNode (if not a Foundation Layer, otherwise doesn't matter)
	 * @param name
	 * @param pathToVariable
	 * @return
	 */
	public IRecordVariable getBuilderVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer);
	//Get variables only define within this layer stream - none of the parallel ones
	public IRecordVariable getLayerStrictBuilderVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable);

	public boolean buildPathToType(TargetType type, BaseTrackers.JavaVariablePath pathToDeclaration, boolean isFirstLayer);

  public ChainDefinitionBlock getChainDefinition(String name) throws ParsingException;
  public MappingDefinitionBlock getMappingDefinition(String name) throws ParsingException;
  public KeyDefinitionBlock getKeyDefinition(String name) throws ParsingException;
  
  public TargetVariable getTargetVariableQuiet(boolean allowLocalVariables,TargetType outputClass, String variableName, BaseTrackers.JavaVariablePath chainToVariable, boolean isFirstLayer);
  public TargetMethod getTargetMethodQuiet(TargetType outputClass, String name, BaseTrackers.JavaVariablePath pathToDeclaration, boolean isFirstLayer);
  public TargetType getTargetTypeQuiet( String typeName, BaseTrackers.JavaVariablePath pathToDeclaration, boolean isFirstLayer);
  
  //Macro Constructions defined at a root, currently, so no need for isFirstLayer stuff
  public MacroConstruction getMacroConstructionFinal(String name,BaseTrackers.JavaVariablePath pathToContainingLayer) throws ItemNotFoundException;
  	  
  public GlobalClassReference getGlobalClassReferenceQuiet(String className, BaseTrackers.JavaVariablePath pathToDeclaration, boolean isFirstLayer);
	public GlobalMethodReference getGlobalMethodReferenceQuiet( GlobalClassReference baseClass, String methodName, JavaVariablePath pathToVariable, boolean isFirstLayer);
	
  public TargetVariable getLocalVariableQuiet(ScopeContainer relatedScope,  String variableName, JavaVariablePath pathToDeclaration, boolean isFirstLayer);
  public boolean isLocalVariableQuiet(ScopeContainer relatedScope, String variableName, boolean isFirstLayer);
  
	public void buildPathToUpperLayerBack(SLayer ancestor, BaseTrackers.JavaVariablePath path);
	public boolean buildPathToFoundationNode(FoundationNode target, BaseTrackers.JavaVariablePath path);

	public boolean buildPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path);
	public boolean buildHorizontalPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path);
	public boolean buildHorizontalPathToPeerSkeleton(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path);

	
	
	public String getFullName();
	public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer);

	/**
	 * Get an access to the root FoundationNode in this total layer space (that is, the FoundationNode at the root of all Builder Type references).
	 * Implementations should either pass on to parent layer, or if a root layer - they should know the root node specifically.
	 * This is used by the MacroStuff for parameter specification (and is called from its basicBuild() level stuff)
	 * @return the root FoundationNode, for Type access
	 */
	public FoundationNode getRootFoundationNodeBasic();
	
	// Are the following two required?
	//public void addPathElementsToFoundationNodePeer(BaseTrackers.JavaVariablePath pathStore);
	
	public AncestorConnection getAncestorFoundationNodeQuiet(String foundationNodeName, BaseTrackers.JavaVariablePath pathToAncestorNodeParent, boolean isFirstLayer);
}
