package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.LinkupTracker;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
//import org.ffd2.skeletonx.skeleton.JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.solar.general.Debug;

public class LocalChild implements PeerConnection {
	private final JavaImplementationFormBlock.BlockDataBlock.IsLocalDataBlock localSkeleton_;
	private final ImplementationBlockSkeletonManager skeletonManager_;
	private final String childName_;
	private final DualIndirectLayerType builtLayerType_;
	private final JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localRefSkeleton_;

	/**
	 * LocalChild constructor
	 * 
	 * @param childName
	 *          the name of the child in the parent (eg, the name of the
	 *          FoundationNode of the child)
	 * @param localSkeleton
	 *          the skeleton ref to the child block, and its IsLocal block (which
	 *          is the setLinks(parent, foundationNode)
	 * @param parentLayer
	 *          the reference to the layer to this child layer
	 * @param pathToAncestor
	 *          the variable path from the parent layer to the AncestorConnection
	 *          parent FoundationNode
	 * @param ancestor
	 *          the connected FoundationNode parent, and the child link of the
	 *          FoundationNode peer that we are concerned with
	 */
	public LocalChild(
			String childName, FoundationNode relatedFoundationNode, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker,
			JavaImplementationFormBlock.BlockDataBlock.IsLocalDataBlock localSkeleton,
			JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localRefSkeleton,
			LinkupTracker relatedTracker, FileEnvironment environment) {
		this.childName_ = childName;
		this.localSkeleton_ = localSkeleton;
		this.localRefSkeleton_ = localRefSkeleton;
		this.skeletonManager_ = new ImplementationBlockSkeletonManager(localSkeleton.getParent(), relatedTracker, environment);
		this.skeletonManager_.setPeerConnection( this );
		this.builtLayerType_ = DualIndirectLayerType.buildLocalNodeChildBased(this, relatedFoundationNode, parentLayer, childMaker, environment);
	}

	public LocalChild(
			String childName, SLayer relatedPeerLayer, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker,
			JavaImplementationFormBlock.BlockDataBlock.IsLocalDataBlock localSkeleton,
			JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localRefSkeleton,

			LinkupTracker relatedTracker, FileEnvironment environment) {

		this.childName_ = childName;
		this.localSkeleton_ = localSkeleton;
		this.localRefSkeleton_ = localRefSkeleton;
		this.skeletonManager_ = new ImplementationBlockSkeletonManager(localSkeleton.getParent(), relatedTracker, environment);
		this.skeletonManager_.setPeerConnection(this);
			//Need to use the local ref bits- that points to local peer, and to current parent
		
		this.builtLayerType_ = DualIndirectLayerType.buildLocalNodeChildBased(this, relatedPeerLayer, parentLayer, childMaker, environment);
	}

	public void doAddAncestorTypeSearch(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock target, BaseTrackers.JavaVariablePath pathToParentLayer) {
		target.addAncestorTypeLocalChild(this.localRefSkeleton_, pathToParentLayer);
	}

	public JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock getLocalRefSkeleton() {
		return this.localRefSkeleton_;
	}

	public String getChildName() {
		return this.childName_;
	}

	public JavaImplementationFormBlock.BlockDataBlock.IsLocalDataBlock getIsLocalSkeleton() {
		return this.localSkeleton_;
	}

	public SLayer getChildLayer() {
		return this.builtLayerType_.getChildLayer();
	}

	public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
		localChildBaseSkeleton.addMacro(this.localRefSkeleton_, targetVariablePathToFoundationChildrenChainTracker);
	}

	public void addLocalParentLink(BaseTrackers.JavaVariablePath pathStore) {
		pathStore.addLocalParent(this.localSkeleton_);
	}

	public void addGlobalFoundationNodeLink(BaseTrackers.JavaVariablePath pathStore) {
		pathStore.addLocalGlobalPeer(this.localSkeleton_);
	}

	public ImplementationBlockSkeletonManager getSkeletonManager() {
		return this.skeletonManager_;
	}

	public void addPathToPeer(JavaVariablePath path) {
		path.addLocalGlobalPeer(localSkeleton_);
	}
}
