package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.solar.exceptions.ParsingException;

import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.CodeBlockEnvironment;
import org.ffd2.skeletonx.compile.java.IRecordTypeTarget;
import org.ffd2.skeletonx.compile.java.IRecordVariable.NameBlockDetails;
import org.ffd2.skeletonx.compile.java.IVariableType;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.BooleanBuilderVariableAccessFormHolder.BooleanBuilderVariableAccessFormBlock;
import org.ffd2.skeletonx.skeleton.DoubleBuilderVariableAccessFormHolder.DoubleBuilderVariableAccessFormBlock;
import org.ffd2.skeletonx.skeleton.IntArrayBuilderVariableAccessFormHolder.IntArrayBuilderVariableAccessFormBlock;
import org.ffd2.skeletonx.skeleton.IntBuilderVariableAccessFormHolder.IntBuilderVariableAccessFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock;
import org.ffd2.skeletonx.skeleton.NameDetailsFormHolder.NameDetailsFormBlock.ComplexDataBlock;
import org.ffd2.skeletonx.skeleton.*;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.StringBuilderVariableAccessFormHolder.StringBuilderVariableAccessFormBlock;

public class PrimitiveVariableTypes {
	public static final BuilderVariableType createIntArrayVariableType(final IntArrayBuilderVariableAccessFormBlock parameterSkeleton) {
		return new AbstractPrimitiveVariableType("int array") {
			public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
				throw ParsingException.createInvalidType("int[]", "String or int");
			}
			public void updateExpressionFinal(
					CodeBlockEnvironment environment, 
					ExpressionDetailsFormHolder.ExpressionDetailsFormBlock  expressionSkeleton, 
					JavaVariablePath variablePath) throws ParsingException {
				expressionSkeleton.addIntArrayParameter(parameterSkeleton, variablePath);
			}
			 public final void addAsMacroRecordParameter(final MacroCallDataBlock macroCallBlock,final JavaVariablePath variablePath,IVariableType type) throws ParsingException {
		     throw ParsingException.createInvalidType(type.getName(), "int[]"); 
		     
		   }
		};
	}
	public static final BuilderVariableType createIntVariableType(final IntBuilderVariableAccessFormBlock parameterSkeleton) {
		return new AbstractPrimitiveVariableType("int") {
			public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
				target.addNameElement().addIntParameter(variablePath, parameterSkeleton );
			}
			public void updateExpressionFinal(
					CodeBlockEnvironment environment, 
					ExpressionDetailsFormBlock  expressionSkeleton, 
					JavaVariablePath variablePath) throws ParsingException {
				expressionSkeleton.addIntParameter(parameterSkeleton, variablePath);
			}
			 public final void addAsMacroRecordParameter(final MacroCallDataBlock macroCallBlock,final JavaVariablePath variablePath,IVariableType type) throws ParsingException {
		     if(type==IVariableType.IntSource) {
		    	 macroCallBlock.addCallParameter().addIntSource(parameterSkeleton, variablePath);
		     } else {
		    	 throw ParsingException.createInvalidType(type.getName(), "int"); 
		     }
		   }
			
		};
	}
	public static final BuilderVariableType createStringVariableType(final StringBuilderVariableAccessFormBlock parameterSkeleton,final JavaParameterSettingsFormBlock.ParameterDataBlock.StringDataBlock specificBlockMayBeNull) {

		return new AbstractPrimitiveVariableType("string") {
				public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
					elementDetails.updateStringParameter(parameterSkeleton, target, variablePath);
				}
			  @Override public void getAsReference(IRecordTypeTarget target) { 
			  	if(specificBlockMayBeNull!=null) {
			  		target.setStringReference(specificBlockMayBeNull); 
			  	} else {
			  		target.setNotReference();
			  }
			  }

				public void updateExpressionFinal( CodeBlockEnvironment environment,  ExpressionDetailsFormBlock  expressionSkeleton,  JavaVariablePath variablePath) throws ParsingException {
					expressionSkeleton.addStringParameter(parameterSkeleton, variablePath);
				}
				 public final void addAsMacroRecordParameter(final MacroCallDataBlock macroCallBlock,final JavaVariablePath variablePath,IVariableType type) throws ParsingException {
			     if(type==IVariableType.StringSource) {
			    	 macroCallBlock.addCallParameter().addStringSource(parameterSkeleton, variablePath);
			     } else {
			    	 throw ParsingException.createInvalidType(type.getName(), "string"); 
			     }
			   }
			};
	}
	public static final BuilderVariableType createBooleanVariableType(final BooleanBuilderVariableAccessFormBlock parameterSkeleton) {

		return new AbstractPrimitiveVariableType("boolean") {
			public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
				throw ParsingException.createInvalidType("boolan", "string or int");
			}
			public void updateExpressionFinal( CodeBlockEnvironment environment,  ExpressionDetailsFormBlock  expressionSkeleton,  JavaVariablePath variablePath) throws ParsingException {
				expressionSkeleton.addBooleanParameter(parameterSkeleton, variablePath);
			}
			public final void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath,IVariableType type) throws ParsingException {
	    	throw ParsingException.createInvalidType(type.getName(), "boolean"); 
		  }
		};
	}
	public static final BuilderVariableType createDoubleVariableType(final DoubleBuilderVariableAccessFormBlock parameterSkeleton) {

		return new AbstractPrimitiveVariableType("double") {
			public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
				target.addNameElement().addDoubleParameter(variablePath, parameterSkeleton );
			}
			public void updateExpressionFinal( CodeBlockEnvironment environment,  ExpressionDetailsFormBlock  expressionSkeleton,  JavaVariablePath variablePath) throws ParsingException {
				expressionSkeleton.addDoubleParameter(parameterSkeleton, variablePath);
			}
			public final void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath,IVariableType type) throws ParsingException {
	    	throw ParsingException.createInvalidType(type.getName(), "double"); 
		  }
		};
		
	}
//----------------
	private static abstract class AbstractPrimitiveVariableType implements BuilderVariableType {
		private final String	typeName_;
		protected AbstractPrimitiveVariableType(String typeName) {
			this.typeName_ = typeName;
			
		}
	  public void getAsReference(IRecordTypeTarget target) { target.setNotReference(); }
	  public SLayer getAsFoundataionNodeReferenceVariableForType(BaseTrackers.JavaVariablePath pathToExtend) throws ParsingException {
	  	throw ParsingException.createInvalidType(this.typeName_, "scaffold reference");
	  }
	  public void addAsMacroRecordMappingParameter(
				MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath,
				MappingDefinitionBlock type) throws ParsingException {
	  	throw ParsingException.createInvalidType(this.typeName_, "mapping");
			
		}
	  public void addAsMacroRecordKeyParameter(
				MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath,
				KeyDefinitionBlock type) throws ParsingException {
	  	throw ParsingException.createInvalidType(this.typeName_, "key");
			
		}
    public void updateCallChainFinal(BasicCallChainFormBlock  target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
    	throw ParsingException.createGeneralSyntax("Non chain-element type");
    }
	}
}
